"
 COPYRIGHT (c) 1988-93 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

StandardSystemView subclass:#CalculatorView
       instanceVariableNames:'prevOperand thisOperand operator
                              displayView base hexDigitButtons nonOctalButtons'
       classVariableNames:''
       poolDictionaries:''
       category:'Demos-Graphics'
!

CalculatorView comment:'
 a simple (and stupid) demo calculator

 start with:
       CalculatorView start

 %W% %E%
 written spring/summer 89 by claus
'!

!CalculatorView class methodsFor:'instance creation'!

new
    ^ super
        label:'Calculator'
        icon:(Form fromFile:'Calc.xbm')
! !

!CalculatorView methodsFor:'initialization'!

initialize
    |w h butt ext space buttonFont displayFont|

    super initialize.
    space := 5.

    buttonFont := Font family:'helvetica' face:'bold' style:'roman' size:12.
    buttonFont isNil ifTrue: [
        buttonFont := self font
    ].
    displayFont := Font family:'helvetica' face:'bold' style:'roman' size:18.
    displayFont isNil ifTrue: [
        displayFont := self font
    ].

    "create a prototype (dummy) button to get minimum dimensions"
    butt := Button new.
    butt font:buttonFont.
    butt label:'WWW'.
    ext := butt extent.

    "make the button square"
    ext x:((ext x) max: (ext y)).
    ext y:(ext x).
    butt extent:ext.

    w := butt widthIncludingBorder.
    h := butt heightIncludingBorder.

    self createButtonsFromProto:butt separatedBy:space.
    self createDisplayWidth:((4 * w) + (3 * space))
                      height: ext y
                      font:displayFont space:space.

    self extent:(((4 * w) + (5 * space)) @ ((10 * h) + (12 * space))).
    self minExtent:(self extent).
    self maxExtent:(self extent).

    thisOperand := 0.
    prevOperand := 0.
    operator := #+.
    base := 10.

    "silently set to base10"
    self baseKey:10

    "CalculatorView start"
!

createDisplayWidth:w height:h font:font space:space
    |x y|

    x := space.
    y := space.
    displayView := Label origin:(x @ y) extent:(w @ h) in: self.
    displayView sizeFixed:true.
    displayView backgroundColor:(Color white).
    displayView level:-1.
    displayView label:'0'.
    displayView font:font.
    displayView adjust:#right
!

createButtonsFromProto:prototype separatedBy:space
    |x y ext w h theFont g|

    ext := prototype extent.
    w := prototype widthIncludingBorder.
    h := prototype heightIncludingBorder.
    theFont := prototype font.

    x := space.
    y := space + h + space + h + space.
    (Button origin:(x @ y) extent:ext in:self)
      sizeFixed:true;
           font:theFont;
           label:'OFF';
          action:[self offKey].

    x := x + w + space + w + space + w + space.
    (Button origin:(x @ y) extent:ext in:self)
      sizeFixed:true;
            font:theFont;
           label:'CLR';
          action:[self clearKey].

    g := RadioButtonGroup new.

    x := space.
    y := y + h + space.
    g add:((RadioButton origin:(x @ y) extent:ext in:self) font:theFont;
                     sizeFixed:true;
                         label:'DEC';
                        action:[self baseKey:10];
                toggleNoAction;
                      yourself).

    x := x + w + space.
    g add:((RadioButton origin:(x @ y) extent:ext in:self) font:theFont;
                     sizeFixed:true;
                         label:'HEX';
                        action:[self baseKey:16];
                      yourself).
    x := x + w + space.
    g add:((RadioButton origin:(x @ y) extent:ext in:self) font:theFont;
                     sizeFixed:true;
                         label:'OCT';
                        action:[self baseKey:8];
                      yourself).
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
                     sizeFixed:true;
                         label:' = ';
                        action:[self eqKey].

    x := space.
    y := y + h + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' + '; action:[self opKey:#+].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' - '; action:[self opKey:#-].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' * '; action:[self opKey:#*].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' / '; action:[self opKey:#//].

    hexDigitButtons := OrderedCollection new.
    nonOctalButtons := OrderedCollection new.

    x := space.
    y := y + h + space + h + space.
    hexDigitButtons add:((Button origin:(x @ y) extent:ext in:self) font:theFont;
                              sizeFixed:true;
                                  label:' C '; 
                                 action:[self numKey:12];
                               yourself).
    x := x + w + space.
    hexDigitButtons add:((Button origin:(x @ y) extent:ext in:self) font:theFont;
                              sizeFixed:true;
                                  label:' D '; 
                                 action:[self numKey:13];
                               yourself).
    x := x + w + space.
    hexDigitButtons add:((Button origin:(x @ y) extent:ext in:self) font:theFont;
                              sizeFixed:true;
                                  label:' E '; 
                                 action:[self numKey:14];
                               yourself).
    x := x + w + space.
    hexDigitButtons add:((Button origin:(x @ y) extent:ext in:self) font:theFont;
                              sizeFixed:true;
                                  label:' F '; 
                                 action:[self numKey:15];
                               yourself).

    x := space.
    y := y + h + space.
    nonOctalButtons add:((Button origin:(x @ y) extent:ext in:self) font:theFont;
                              sizeFixed:true;
                                  label:' 8 '; 
                                 action:[self numKey:8];
                               yourself).
    x := x + w + space.
    nonOctalButtons add:((Button origin:(x @ y) extent:ext in:self) font:theFont;
                              sizeFixed:true;
                                  label:' 9 '; 
                                 action:[self numKey:9];
                               yourself).
    x := x + w + space.
    hexDigitButtons add:((Button origin:(x @ y) extent:ext in:self) font:theFont;
                              sizeFixed:true;
                                  label:' A '; 
                                 action:[self numKey:10];
                               yourself).
    x := x + w + space.
    hexDigitButtons add:((Button origin:(x @ y) extent:ext in:self) font:theFont;
                              sizeFixed:true;
                                  label:' B '; 
                                 action:[self numKey:11];
                               yourself).

    x := space.
    y := y + h + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' 4 '; action:[self numKey:4].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' 5 '; action:[self numKey:5].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' 6 '; action:[self numKey:6].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' 7 '; action:[self numKey:7].

    x := space.
    y := y + h + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' 0 '; action:[self numKey:0].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' 1 '; action:[self numKey:1].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' 2 '; action:[self numKey:2].
    x := x + w + space.
    (Button origin:(x @ y) extent:ext in:self) font:theFont;
       sizeFixed:true;
           label:' 3 '; action:[self numKey:3]
!

opError
    thisOperand := 0.
    displayView label:'Error'
!

updateDisplay
    displayView label:(thisOperand printStringRadix:base)
! !

!CalculatorView methodsFor:'button actions'!

numKey:value
    (value < base) ifTrue: [
        thisOperand := (thisOperand * base) + value.
        self updateDisplay
    ]
!

opKey:selector
    prevOperand := thisOperand.
    operator := selector.
    thisOperand := 0
!

eqKey
    |result|
    ((operator == #/) and: [thisOperand = 0]) ifTrue: [
        self opError
    ] ifFalse: [
        result := prevOperand perform:operator with:thisOperand.
        prevOperand := thisOperand.
        thisOperand := result.
        self updateDisplay
    ]
!

clearKey
    thisOperand := 0.
    self updateDisplay
!

offKey
    displayView := nil.
    self unrealize.
    self destroy
!

baseKey:value
    base := value.
    base == 16 ifFalse:[
        hexDigitButtons do:[:butt | butt disable]
    ] ifTrue:[
        hexDigitButtons do:[:butt | butt enable]
    ].
    base == 8 ifTrue:[
        nonOctalButtons do:[:butt | butt disable]
    ] ifFalse:[
        nonOctalButtons do:[:butt | butt enable]
    ].
    self updateDisplay
! !
