StandardSystemView subclass:#Example10
         instanceVariableNames:'menu myView openBox'
         classVariableNames:''
         poolDictionaries:''
         category:'Demos-Tutorial'
!

Example10 comment:'

Tutorial example - a complete setup for a scrollable textview with a menu.
The open function has been implemented.
Notice - this is almost an editor !!

    Example10 start

%W% %E%
written spring 92 by claus
'!

!Example10 methodsFor:'initialize / release'!

initialize
    "setup view"

    |menu scrollingView|

    super initialize.

    self label:'Tutorial Example 10'.

    menu := PullDownMenu in:self.
    menu labels:#('file' 'edit').

    menu at:'file'
            putLabels:#('new' 
                        'open ..' 
                        '-' 
                        'quit')
            selectors:#(doNew doOpen 
                        nil destroy)
             receiver:self.

    menu at:'edit'
            putLabels:#('copy' 
                        'cut' 
                        'paste')
            selectors:#(copySelection cut paste)
             receiver:self.

    scrollingView := ScrollableView for:EditTextView in:self.
    scrollingView origin:(0 @ menu height)
                  extent:[width @ (height - menu height)].
    myView := scrollingView scrolledView.

    myView contents:'
This example shows some more user interface components.

It allows editing, both via a pop-up menu (right button) and via
a pull-down menu (''edit''-menu). 

The most interresting new thing here is the open panel (a
FileSelectionBox) which pops up when then ''open''-entry in the
''file'' menu is selected.

Finish this example by selecting ''quit'' in the ''file''-menu.'

   "Example10 start"

! !

!Example10 methodsFor:'release'!

destroy
    "destroy boxes when this view is destroyed"
    openBox notNil ifTrue:[
        openBox destroy
    ].
    super destroy
! !

!Example10 methodsFor:'user interaction'!

doNew
    myView contents:nil
!


openFile:aFileName
    |stream text|

    stream := FileStream readonlyFileNamed:aFileName.
    stream isNil ifTrue:[
        self notify:('cannot open ' , aFileName).
        ^ self
    ].
    text := stream contents.
    myView contents:text
!

doOpen

    "create the open-box when we come here the first time ..."
    openBox isNil ifTrue:[
        openBox := FileSelectionBox new.
        openBox action:[:string | self openFile:string].
        openBox title:'opening file:'.
        openBox okText:'open'
    ].
    "and launch it"
    openBox showAtPointer
!

copySelection
    myView copySelection
!

cut
    myView cut
!

paste
    myView paste
! !
