Point subclass: #Point3D
        instanceVariableNames: 'z '
        classVariableNames: 'RandomSource '
        poolDictionaries: ''
        category: 'Graphics-Primitives'!

!Point3D methodsFor: 'accessing'!
z
        ^ z!

z: value
        ^ z _ value! !

!Point3D methodsFor: 'converting'!

asProjectedPoint
        "Translate to 2D (integral) coordinates with some rotation.
        Answer integer coordinates cause bitblt likes it that way."

        | newX newY|
        newX _ x.
        newY _ y - (1.125 * z) truncated.
        ^ newX @ newY
!

middle: otherPoint displaced: range
        "Answer a point in the middle, randomly displaced.
        Keep coordinates integral for speed only."

        | newX newY newZ |
        newX _ (x + otherPoint x) // 2.
        newY _ (y + otherPoint y) // 2.
        newZ _ (z + otherPoint z) // 2.
        newZ _ newZ + (RandomSource next - 0.5 * range) truncated.
        ^ Point3D x: newX y: newY z: newZ
! !

!Point3D methodsFor: 'printing'!

printOn: aStream 
        "The receiver prints on aStream in terms of infix notation."

        x printOn: aStream.
        aStream nextPut: $@.
        y printOn: aStream.
        aStream nextPut: $@.
        z printOn: aStream
! !

!Point3D methodsFor: 'private'!

setX: xValue setY: yValue setZ: zValue

        x _ xValue.
        y _ yValue.
        z _ zValue
! !

!Point3D class methodsFor: 'class initialization'!

initialize
        RandomSource _ Random new

        "Point3D initialize"
! !

!Point3D class methodsFor: 'instance creation'!

x: xValue y: yValue z: zValue
        "Answer an instance of me with coordinates xValue, yValue, and zValue."

        ^self new setX: xValue setY: yValue setZ: zValue
! !

Point3D initialize!

" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
