'From Smalltalk/X, Version:2.8.1 on 16-nov-1993 at 14:25:32'!

View subclass:#HistogrammView
         instanceVariableNames:'labels values colors sep framed usedHeight showLabels
                labelPosition showValues'
         classVariableNames:''
         poolDictionaries:''
         category:'Views-Presentation'
!

!HistogrammView class methodsFor:'examples'!

example1
    |v top|

    v := HistogrammView new.
    v labels:#(1992 1993 1994).
    v values:#(50 75 25).
    v colors:(Array with:Color red with:Color green with:Color blue).

    top := StandardSystemView label:'chewing gum sales'.
    top addSubView:v in:(0.0 @ 0.0 corner:1.0 @ 1.0) borderWidth:0.
    top open

    "HistogrammView example1"
!

example2
    |v top|

    v := HistogrammView new.
    v labels:#(1992 1993 1994).
    v values:#(50 75 25).
    v colors:(Array with:Color red with:Color green with:Color blue).
    v showValues:false.

    top := StandardSystemView label:'chewing gum sales'.
    top addSubView:v in:(0.0 @ 0.0 corner:1.0 @ 1.0) borderWidth:0.
    top open

    "HistogrammView example2"
!

example3
    |v top|
    v := HistogrammView new.
    v labels:#('50''s' '60''s' '70''s' '80''s' '90''s (exp)').
    v values:#(10 16 23 18 27).
    v labelPosition:#bottom.
    v showValues:true.

    top := StandardSystemView label:'chocolade sales (1000s)'.
    top addSubView:v in:(0.0 @ 0.0 corner:1.0 @ 1.0) borderWidth:0.
    top open

    "HistogrammView example3"
! !

!HistogrammView methodsFor:'drawing'!

redraw
    |nRect rectWidth max x oneSpace allSpace|

    "get number of histogramm rectangles"

    nRect := values size.

    "actual width per rectangle"
    oneSpace := device horizontalPixelPerMillimeter * sep.
    allSpace := oneSpace * (nRect + 1).
    rectWidth := (width - allSpace) // nRect.

    "find max. value"
    max := values inject:0 into:[:currentMax :current | current max:currentMax].

    self fill:Color white.
    x := oneSpace.
    values keysAndValuesDo:[:index :value |
        |rH color|

        rH := (height * usedHeight * (value / max)) rounded.
        (colors notNil and:[(colors at:index) notNil]) ifTrue:[
            color := (colors at:index)
        ] ifFalse:[
            color := Color grey.
        ].
        self paint:color.
        self fillRectangleX:x rounded y:(height - rH) width:rectWidth rounded height:rH.
        framed ifTrue:[
            self paint:Color black.
            self displayRectangleX:x rounded y:(height - rH) width:rectWidth rounded height:rH
        ].
        (labels notNil and:[(labels at:index) notNil]) ifTrue:[
            showLabels ifTrue:[
                |l xOffset xPosition yPosition|

                l := (labels at:index) printString.
                xOffset := (rectWidth - (font widthOf:l)) / 2.
                xPosition := (x + xOffset) rounded.

                labelPosition == #top ifTrue:[
                    yPosition := font ascent.
                    self paint:Color black
                ] ifFalse:[
                    yPosition := height - (font descent * 2).
                    color brightness < 0.5 ifTrue:[
                        self paint:Color white
                    ] ifFalse:[
                        self paint:Color black
                    ].
                ].
                self displayString:l
                                 x:xPosition
                                 y:yPosition.
            ].
        ].
        showValues ifTrue:[
            |l xOffset xPosition yPosition|

            l := value printString.
            xOffset := (rectWidth - (font widthOf:l)) / 2.
            xPosition := (x + xOffset) rounded.
            yPosition := height - rH - (font descent * 2).
            self paint:color.
            self displayString:l
                             x:xPosition
                             y:yPosition.
        ].
        x := x + rectWidth + oneSpace
    ]
! !

!HistogrammView methodsFor:'initialization'!

initialize
    super initialize.
    framed := true.
    sep := 1.
    usedHeight := 0.75.
    showLabels := true.
    labelPosition := #top.
    showValues := true.
! !

!HistogrammView methodsFor:'accessing'!

labels:aLabelArray
    labels := aLabelArray
!

values:aValueArray
    values := aValueArray
!

colors:aColorArray
    colors := aColorArray
!

separatingSpace:mm
    sep := mm
!

framed:aBoolean
    framed := aBoolean
!

showValues:aBoolean
    showValues := aBoolean
!

labelPosition:aSymbol
    labelPosition := aSymbol
! !
