Object subclass:#BlockExamples
         instanceVariableNames:''
         classVariableNames:''                        
         poolDictionaries:''                     
         category:'Examples-Blocks'                       
!

!BlockExamples class methodsFor:'recursive blocks'!

recursiveExample1
    "will not work on PPS before V2.3 and some Smalltalk/V versions"

    |fBlock|

    fBlock := [:n |
        n<= 0 ifTrue:[1]
              ifFalse:[n * (fBlock value:(n - 1))]].

    ^ (0 to:3) collect:fBlock

    "BlockExamples recursiveExample1
     -> should return OrderedCollection(1 1 2 6)"
!

recursiveExample2
    "will not work on PPS before V2.3 and some Smalltalk/V versions"

    |fBlock|

    fBlock := [:n |
        n<= 0 ifTrue:[1]
              ifFalse:[(fBlock value:(n - 1)) * n]].

    ^ (0 to:3) collect:fBlock.

    "BlockExamples recursiveExample2 
     -> should return OrderedCollection(1 1 2 6)"
! !

!BlockExamples class methodsFor:'context example'!

contextExample1
    "will not work on PPS before 2.3 and some Smalltalk/V versions"

    | collectionOfBlocks |

    collectionOfBlocks := self contextExample1Helper.
    ^ collectionOfBlocks collect:[:j | j value]

    "BlockExamples contextExample1  
     -> should return OrderedCollection(1 2 3 4)"
!

contextExample1Helper
    "helper for contextExample1; return a collection of blocks"

    |c|

    c:=OrderedCollection new. 
    1 to:4 do:[:i|
      c add:[i]     "a block returning i"
    ].
    ^ c             "a collection of blocks returning i', i'', i''' and i''''"
! !

!BlockExamples class methodsFor:'closure example'!

closureExample1
    "this example shows that blocks can live and access method
     locals even when their defining method has already left"

    | blocks incBlock decBlock getBlock|

    "get the blocks"

    blocks := self closureExample1Helper.
    incBlock := blocks at:1.
    decBlock := blocks at:2.
    getBlock := blocks at:3.

    Transcript show:'variable value == '; showCr:getBlock value.
    Transcript showCr:'increment twice ..'. 
    incBlock value. incBlock value.
    Transcript show:'variable value == '; showCr:getBlock value.
    Transcript showCr:'decrement ..'. 
    decBlock value.
    Transcript show:'variable value == '; showCr:getBlock value.

    "BlockExamples closureExample1"
!

closureExample1Helper
    "helper for returnExample1; return a returning block"

    |variable|

    variable := 0.
    ^ Array with:[variable := variable + 1]
            with:[variable := variable - 1]
            with:[variable]
! !

!BlockExamples class methodsFor:'return example'!

returnExample1
    "this example shows that blocks can live even when their
     defining method has already left"

    | block |

    "get the block, which contains a return from an already
     left method"

    block := self returnExample1Helper.
    Transcript showCr:'one'.
    block value.
    Transcript showCr:'two'.

    "BlockExamples returnExample1"
!

returnExample1Helper
    "helper for returnExample1; return a returning block"

    ^ [ 
        Transcript showCr:'b-1'.
        ^ nil.
        Transcript showCr:'b-2'.
      ]
!

returnExample2
    | block |

    "get the block, which contains a return from an already
     left method. the return is performed in a nested block"
    block := self returnExample2Helper.
    Transcript showCr:'one'.
    block value.
    Transcript showCr:'two'.

    "BlockExamples returnExample2"
!

returnExample2Helper
    "helper for returnExample2; return a block with a return from
     a nested block"

    ^ [
        Transcript showCr:'b1-1'.
        [
          Transcript showCr:'b2-1'.
          [
            Transcript showCr:'b3-1'.
            ^ nil.
            Transcript showCr:'b3-2'.
          ] value.
          Transcript showCr:'b2-2'
        ] value.
        Transcript showCr:'b1-2'.
    ]
!

returnExample3
    "this example shows how blocks can be used to
     implement some catch&throw mechanism"

    | block |

    block := self returnExample3Helper.
    Transcript showCr:'one'.
    block value:[^ self].
    Transcript showCr:'two'.

    "BlockExamples returnExample3"
!

returnExample3Helper
    "helper for returnExample3; return a block, which evaluates its
     argument block in a nested block"

    ^ [:exitBlock |
        Transcript showCr:'b1-1'.
        [:exitBlock |
          Transcript showCr:'b2-1'.
          [:exitBlock |
            Transcript showCr:'b3-1'.
            exitBlock value.
            Transcript showCr:'b3-2'.
          ] value:exitBlock.
          Transcript showCr:'b2-2'
        ] value:exitBlock.
        Transcript showCr:'b1-2'.
    ]
! !
