StandardSystemView subclass:#Example11
         instanceVariableNames:'menu myView fileBox currentFileName'
         classVariableNames:''
         poolDictionaries:''
         category:'Examples-Tutorial'
!

Example11 comment:'

Tutorial example - a complete setup for a scrollable textview with a menu.
The open and save functions have been implemented.
All texts have been internationalized.

This is a full blown editor (in one page of code) !!

written summer 92 by claus
'!

!Example11 methodsFor:'initialize / release'!

initialize
    "setup view"

    |menu scrollingView|

    super initialize.

    self label:'Tutorial Example 11'.

    menu := PullDownMenu in:self.
    menu labels:(resources array:#('file' 'edit')).

    menu at:1
            putLabels:(resources array:#('new'
                                         'open ...'
                                         '-'
                                         'save'
                                         'save as ...'
                                         '-'
                                         'quit')) 
            selectors:#(doNew 
                        doOpen
                        nil 
                        doSave
                        doSaveAs
                        nil 
                        destroy)
             receiver:self.

    menu at:2
            putLabels:(resources array:#('copy' 'cut' 'paste')) 
            selectors:#(copySelection cut paste)
             receiver:self.

    scrollingView := ScrollableView for:EditTextView in:self.
    scrollingView origin:(0 @ menu height)
                  extent:[width @ (height - menu height)].
    myView := scrollingView scrolledView.

    myView contents:'
This example shows some more user interface components.

It allows editing, both via a pop-up menu (right button) and via
a pull-down menu (''edit''-menu). And there is a possibility to
save the text after it has been edited.

The very interresting thing here is that all user-interface texts
have been internationalized.
(open file ''Smalltalk.rs'' to see how such resources are defined.)

Finish this example by selecting ''quit'' in the ''file''-menu.'

   "Example11 start"

! !

!Example11 methodsFor:'release'!

destroy
    "destroy box when this view is destroyed"
    fileBox notNil ifTrue:[
        fileBox destroy
    ].
    super destroy
!

openFile:aFileName
    |stream text|

    stream := FileStream readonlyFileNamed:aFileName.
    stream isNil ifTrue:[
        self notify:(resources string:'cannot open %1' with:aFileName).
        ^ self
    ].
    text := stream contents.
    stream close.

    myView contents:text.
    currentFileName := aFileName
!

saveFile:aFileName
    |stream text|

    stream := FileStream newFileNamed:aFileName.
    stream isNil ifTrue:[
        self notify:(resources string:'cannot create %1' with:aFileName).
        ^ self
    ].
    stream nextPutAll:(myView contents).
    stream close
! !

!Example11 methodsFor:'user interaction'!

doNew
    currentFileName := nil.
    myView contents:nil
!

doOpen
    fileBox isNil ifTrue:[
        fileBox := FileSelectionBox new
    ].
    fileBox action:[:string | self openFile:string].
    fileBox title:(resources string:'open file').
    fileBox okText:(resources string:'open').
    fileBox showAtPointer
!

doSave
    currentFileName isNil ifTrue:[
        ^ self doSaveAs
    ].
    self saveFile:currentFileName
!

doSaveAs
    fileBox isNil ifTrue:[
        fileBox := FileSelectionBox new
    ].
    fileBox action:[:string | self saveFile:string].
    fileBox title:(resources string:'save contents into:').
    fileBox okText:(resources string:'save').
    fileBox showAtPointer
!

copySelection
    myView copySelection
!

cut
    myView cut
!

paste
    myView paste
! !
