"       NAME            Paths
        AUTHOR          scaletti@uxc.cso.uiuc.edu (Kurt J. Hebel)
        FUNCTION        Reimplementation of the Path hierarchy
        ST-VERSION      2.2?
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1
        DATE    3 Jun 1987
"

DisplayObject subclass: #AbstractPath
        instanceVariableNames: 'form'
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Graphics-Display Objects'
!

AbstractPath comment:
'AbstractPath is an abstract superclass which defines the protocol common to
 all Paths.  Paths know how to display themselves, and how to transform themselves.
 They display using the instance variable ''form'' as the writing tool.

The basic protocol allows access to the set of points in the Path, and includes
support for the protocols defined in the Blue Book, pg. 401.

Modifications (from original manchester goody):

    claus: since Forms are of limited use as drawing writing tools on color
           displays, allow the use of a drawing pen instead of a form.
'!


!AbstractPath methodsFor: 'point accessing'!

add: aPoint
        "Add this point to the end of the path."
        self subclassResponsibility.!

pointAt: index
        "Return the point at the index given. This assumes that the points
         in this Path are represented as an OrderedCollection."
        ^ self points at: index!

pointAt: index put: aPoint
        "Set the point at the index given.  This assumes that the points
         in this Path are represented as an OrderedCollection."
        ^ self points at: index put: aPoint!

points
        "Return the collection of points for this Path."
        ^ self subclassResponsibility!

points: aCollection
        "Set the collection of points for this Path."
        self subclassResponsibility.! !

!AbstractPath methodsFor: 'accessing'!

form
        "Answer the receiver's form.  If the form is nil, 
         then set it to a black dot of size 1."

        form == nil ifTrue: [form _ DrawingPen "Form" dotOfSize: 1].
        ^ form
!

form: aForm 
        "Make the argument, aForm, the receiver's form."
        form _ aForm.
!

offset
        "There are basically two kinds of display objects in the system:  those that,
 when asked to transform themselves, create a new object;  and those that side
 effect themselves by maintaining a record of the transformation request (typically
 an offset).  Path, like Rectangle and Point, is a display object of the first
 kind."
        self shouldNotImplement! !

!AbstractPath methodsFor: 'transforming'!

scaleBy: aPoint 
        "Answer with a new Path scaled by aPoint.  Does not effect the current data
 in this Path."
        self subclassResponsibility.!

translateBy: aPoint 
        "Answer a new instance of Path whose elements are translated by aPoint.  Does
 not effect the elements of this Path."
        self subclassResponsibility.! !

!AbstractPath methodsFor: 'displaying'!

displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule:
 ruleInteger mask: aForm 
        "See comment in DisplayObject|displayOn:at:clippingBox:rule:mask:."
        self subclassResponsibility.!

displayOn: aDisplayMedium transformation: displayTransformation clippingBox:
 clipRectangle
        "See comment in DisplayObject|displayOn:transformation:clippingBox:"
        self displayOn: aDisplayMedium
                transformation: displayTransformation
                clippingBox: clipRectangle
                rule: Form over
                mask: Form black!

displayOn: aDisplayMedium transformation: aTransformation clippingBox: clipRect
 rule: anInteger mask: aForm
        "See comment in DisplayObject|displayOn:transformation:clippingBox:rule:mask:."

        (aTransformation applyTo: self)
                displayOn: aDisplayMedium
                at: 0 @ 0
                clippingBox: clipRect
                rule: anInteger
                mask: aForm.! !

!AbstractPath methodsFor: 'display box access'!

computeBoundingBox
        "Compute the smallest Rectangle that covers the Path."
        self subclassResponsibility.! !

!AbstractPath methodsFor: 'Blue Book Compatibility'!

at: index
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self pointAt: index!

at: index put: aPoint
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self pointAt: index put: aPoint!

first
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self points first!

collect: aBlock
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self points collect: aBlock!

do: aBlock
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self points do: aBlock!

isEmpty
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self points isEmpty!

removeAllSuchThat: aBlock
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self points: (self points reject: aBlock)!

select: aBlock
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self points select: aBlock!

size
        "This message is obsolete -- provided for compatibility with the Blue Book."

        ^ self points size
! !

" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
