"
 COPYRIGHT (c) 1990 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

StandardSystemView subclass:#Animation
       instanceVariableNames:'halted delay phase xpos forms myBlock
                              standalone'
       classVariableNames:''
       poolDictionaries:''
       category:'Demos'
!

Animation comment:'

walking man demo

$Header: /claus/smalltalk/st2.6/clients/Animation/RCS/Animation.st,v 1.1 93/10/25 10:42:46 claus Exp $
written spring 90 by claus
'!

!Animation class methodsFor:'startup'!

start
    |anim|

    anim := self
                origin:0 @ (Display height - 64)
                extent:(Display width @ 64)
                label:'Animation'
                icon:(Form fromFile:'Animation.icon' resolution:100)
                minExtent:(100 @ 64)
                maxExtent:(Display width @ 64).

    anim realize.
    ^ anim

    "Animation start"
! !

!Animation methodsFor:'initialization'!

initialize
    super initialize.

    standalone := false.
    halted := false.
    delay := 0.2.
    xpos := 0.
    phase := 1.
    self initForms.
    self viewBackground:background.
    myBlock := [self cycle]
!

initializeMiddleButtonMenu
    |labels|

    labels := resources array:#('faster'
                                'slower'
                                '-'
                                'stop'
                                'step'
                                'go'
                                '-'
                                'quit').

    self middleButtonMenu:(PopUpMenu
                                labels:labels
                             selectors:#(faster
                                         slower
                                         nil
                                         stop
                                         step
                                         go
                                         nil
                                         destroy)
                                receiver:self
                                     for:self)
!

initForms
    |form fileName|

    forms isNil ifTrue:[
        forms := Array new:8.
        1 to:(forms size) do:[:i |
            fileName := 'man' , (i printString) , '.xbm'.
            'reading ' print. fileName printNewline.
            form := Form fromFile:fileName resolution:100.
            form isNil ifTrue:[
                self notify:('cannot read bitmap ', fileName).
                ^ self
            ].
            forms at:i put:form
        ]
    ]
!

realize
    super realize.
    self enableKeyEvents.
    Processor addTimedBlock:myBlock after:delay
! !

!Animation methodsFor:'drawing'!

cycle
    self step.
    halted ifFalse:[
        Processor addTimedBlock:myBlock after:delay
    ]
!

step
    "clear previous form - draw new one"

    self clearRectangle:(Rectangle left:xpos top:0 width:32 height:64).

    xpos := xpos + 4.
    (xpos > self width) ifTrue:[
        xpos := 0
    ].
    phase := phase + 1.
    (phase > (forms size)) ifTrue:[
        phase := phase - (forms size)
    ].
    self drawPicture:phase at:xpos.
!

drawForm:whichForm atX:x y:y
    |form|
    form := forms at:whichForm.
    form notNil ifTrue:[
        self displayOpaqueForm:form x:x y:y
    ]
!

drawPicture:whichForm at:xpos
    self drawForm:whichForm atX:xpos y:0.
    device synchronizeOutput
! !

!Animation methodsFor:'control'!

faster
    (delay > 0.0001) ifTrue:[
        Processor removeTimedBlock:myBlock.
        delay := (delay / 2) .
        Processor addTimedBlock:myBlock after:delay.
    ]
!

slower
    Processor removeTimedBlock:myBlock.
    delay := delay * 2.
    Processor addTimedBlock:myBlock after:delay.
!

stop
    halted ifFalse:[
        Processor removeTimedBlock:myBlock.
        halted := true
    ]
!

go
    halted ifTrue:[
        Processor addTimedBlock:myBlock after:delay.
        halted := false
    ]
! !

!Animation methodsFor:'events'!

redraw
    self drawPicture:phase at:xpos
!

destroy
    self stop.
    super destroy.
    Processor removeTimedBlock:myBlock
!

keyPress:key x:x y:y
    (key == $f) ifTrue: [
        ^ self faster
    ].
    (key == $s) ifTrue: [
        ^ self slower
    ].
    (key == $h) ifTrue:[
        ^ self stop
    ].
    (key == $g) ifTrue:[
        ^ self go
    ].
    (key == $q) ifTrue:[
        ^ self destroy
    ]
! !
