"       NAME            Paths
        AUTHOR          scaletti@uxc.cso.uiuc.edu (Kurt J. Hebel)
        FUNCTION        Reimplementation of the Path hierarchy
        ST-VERSION      2.2?
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1
        DATE    3 Jun 1987
"

AbstractPath subclass: #Arc
        instanceVariableNames: 'centerPoint radiusPoint beginAngle endAngle'
        classVariableNames: 'CirclePointTable ScaleFactor'
        poolDictionaries: ''
        category: 'Graphics-Paths'!
Arc comment:
'This class implements general ellipsoidal arcs given a center point, a radius
 point (which specifies the x and y radius) and the starting and ending angles
 of the arc, in degrees.'!


!Arc methodsFor: 'accessing'!

beginAngle
        "Answer the beginning angle of this Arc."
        ^ beginAngle!

beginAngle: aNumber
        "Set the beginning angle of this Arc."
        beginAngle _ aNumber.!

beginAngle: number1 endAngle: number2
        "Set the beginning and ending angle of this Arc."
        beginAngle _ number1.
        endAngle _ number2.!

center
        "Answer the point at the center of the receiver."
        ^ centerPoint!

center: aPoint 
        "Set aPoint to be the receiver's center."
        centerPoint _ aPoint.!

center: point1 radius: point2 
        "The receiver is defined by a point at the center and a radius point which gives
         the radius in the x and y directions."

        centerPoint _ point1.
        radiusPoint _ point2 asPoint.!

endAngle
        "Answer the ending angle of this Arc."
        ^ endAngle!

endAngle: aNumber
        "Set the ending angle of this Arc."
        endAngle _ aNumber.!

radius
        "Answer the receiver's radius point."
        ^ radiusPoint!

radius: aPoint 
        "Set aPoint to be the receiver's radius point specifying the x and y radii."

        radiusPoint _ aPoint asPoint.! !

!Arc methodsFor: 'transforming'!

scaleBy: aPoint 
        "Answer with a new Path scaled by aPoint.  Does not effect the current data
         in this Path."
        | anArc |

        anArc _ self species new.
        anArc form: self form.
        anArc beginAngle: beginAngle endAngle: endAngle.
        anArc center: centerPoint * aPoint.
        anArc radius: radiusPoint * aPoint.
        ^ anArc!

translateBy: aPoint 
        "Answer with a new Path scaled by aPoint.  Does not effect the current data
         in this Path."
        | anArc |

        anArc _ self species new.
        anArc form: self form.
        anArc beginAngle: beginAngle endAngle: endAngle.
        anArc center: centerPoint + aPoint.
        anArc radius: radiusPoint.
        ^ anArc! !

!Arc methodsFor: 'displaying'!

displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger mask: aForm
        "Display the receiver as a series of interpolating lines to make an arc.  Also,
         see comment in DisplayObject|displayOn:at:clippingBox:rule:mask:."

        | deltaAngle aLinearFit startAngle finishAngle |

        "Compute the starting and finishing angles."
        startAngle _ beginAngle rounded \\ 360.
        beginAngle = 360 ifTrue: [startAngle _ 360].
        finishAngle _ endAngle rounded \\ 360.
        endAngle = 360 ifTrue: [finishAngle _ 360].

        "Compute the change in angle between each line segment."
        deltaAngle _ (10 / (radiusPoint x max: radiusPoint y)) radiansToDegrees truncated + 1.
        startAngle > finishAngle ifTrue: [deltaAngle _ deltaAngle negated].

        "Set up the LinearFit for plotting."
        aLinearFit _ LinearFit new.
        aLinearFit form: self form.

        "Add the Arc points to LinearFit."
        startAngle to: finishAngle by: deltaAngle do: [ :angle |
            aLinearFit add: (CirclePointTable at: angle + 1) * radiusPoint // ScaleFactor + centerPoint
	].
        aLinearFit add: (CirclePointTable at: finishAngle + 1) * radiusPoint // ScaleFactor + centerPoint.

        aLinearFit
                displayOn: aDisplayMedium
                at: aPoint
                clippingBox: clipRect
                rule: anInteger
                mask: aForm.! !

!Arc methodsFor: 'display box access'!

computeBoundingBox
        "Compute the smallest Rectangle that covers all of the points in the Path."

        ^ centerPoint - radiusPoint corner: centerPoint + radiusPoint! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!Arc class methodsFor: 'class initialization'!

initialize
        "Initialize the circle point lookup table."
        | radians |

        ScaleFactor _ 2 raisedTo: SmallInteger maxBits - 1.
        CirclePointTable _ Array new: 361.
        0 to: 360 do: [ :degree |
                radians _ degree degreesToRadians.
                CirclePointTable at: degree + 1 put: radians cos @ radians sin * ScaleFactor
	].

        "Arc initialize."! !

Arc initialize!


" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
