"       NAME            Arrow (was DirectedArc)
        AUTHOR          N.Haddley <neil@comp.lancs.ac.uk>
        CONTRIBUTOR     Alan Wills <alan@cs.man.ac.uk>
        FUNCTION        Line with arrowheads
        ST-VERSIONS     VI2.2
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    global
        VERSION         1.1
        DATE            16/06/90
SUMMARY

APPLICATION : Allows a line to be drawn between two specified points decorated
              with an arrowhead showing direction of relationship. Arrowheads
              can be placed at the center, two thirds of the way down or at
              the end of the arc. Bi-directional arcs can be created by 
              drawing an arc from A to B then redrawing the same arc from B to
              A.

claus: renamed to Arrow from DirectedArc
"!

Line subclass: #Arrow
        instanceVariableNames: 'type arrowHeadLength '
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Graphics-Paths'!


!Arrow methodsFor: 'displaying'!

displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger mask: aForm 
        "Draws a directed arc, made up of three lines ."

        | start finish arrowHeadPoint angle point grad 
                xDis yDis pos endPoint headForm l |

        self size < 2 ifTrue: [self error: 'a line must have two points'].
        start _ self beginPoint.
        finish _ self endPoint.

        "check arc type and calculate location of arrow head"
        type = #centered
                ifTrue: [arrowHeadPoint _ start + (finish - start / 2)]
                ifFalse: [type = #displaced
                        ifTrue: [arrowHeadPoint _ start + (finish - start / 1.35)]
                        ifFalse: [type = #extreme
                                ifTrue: [arrowHeadPoint _ finish]
                                ifFalse: [self error: 
        'Arrow type must be #centered, #displaced or #extreme']]].

        "draw the base line of the arc on the DisplayMedium"
        super displayOn: aDisplayMedium at: aPoint clippingBox: clipRect
                   rule: anInteger mask: aForm. 

        angle _ 2.75.
        point _ arrowHeadPoint - start.
        point x = 0
                ifTrue: [grad _ 9999999]
                ifFalse: [grad _ point y / point x].
        xDis _ arrowHeadLength * (angle - grad arcTan) cos.
        yDis _ arrowHeadLength * (angle - grad arcTan) sin.
        point x = 0
                ifTrue: [point y > 0
                        ifTrue: [pos _ point x + xDis @ (point y - yDis)]
                        ifFalse: [pos _ point x - xDis @ (point y + yDis)]]
                ifFalse: [point x > 0
                        ifTrue: [pos _ point x + xDis @ (point y - yDis)]
                        ifFalse: [pos _ point x - xDis @ (point y + yDis)]].
        l := Line new.
        l beginPoint:arrowHeadPoint.
        l endPoint:start + pos.
        l form:(DrawingPen dotOfSize:(form width)).
        l displayOn: aDisplayMedium at: aPoint clippingBox: clipRect 
               rule: anInteger mask: aForm. 

        xDis _ arrowHeadLength * (angle + grad arcTan) cos.
        yDis _ arrowHeadLength * (angle + grad arcTan) sin.
        point x = 0
                ifTrue: [point y > 0
                        ifTrue: [pos _ point x + xDis @ (point y + yDis)]
                        ifFalse: [pos _ point x - xDis @ (point y - yDis)]]
                ifFalse: [point x > 0
                        ifTrue: [pos _ point x + xDis @ (point y + yDis)]
                        ifFalse: [pos _ point x - xDis @ (point y - yDis)]].
        l endPoint:start + pos.
        l displayOn: aDisplayMedium at: aPoint clippingBox: clipRect 
               rule: anInteger mask: aForm 
! !

!Arrow methodsFor: 'private'!

initialize
        "default settings for Arrow type and arrow head length"

        super initialize.
        self type: #extreme.
        self arrowHeadLength: 10! !

!Arrow methodsFor: 'accessing'!

arrowHeadLength: aNumber
        "set arrow head length to desired number"

        arrowHeadLength _ aNumber!

type: aType
        "set directed arc type to aType (#centered, #displaced or #extreme)"

        type _ aType! !

!Arrow class methodsFor: 'instance creation'!

new
        ^super new initialize! !


" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
