"
 COPYRIGHT (c) 1988 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Object subclass:#Behavior
       instanceVariableNames:'superclass otherSuperclasses
                              selectors methods
                              instSize flags'
       classVariableNames:''
       poolDictionaries:''
       category:'Kernel-Classes'
!

Behavior comment:'

COPYRIGHT (c) 1988 by Claus Gittinger
              All Rights Reserved

NOTICE: layout known by compiler and runtime system; be careful when changing

$Header: /claus/smalltalk/st2.6/libbasic/RCS/Behavior.st,v 1.11 94/02/25 12:54:13 claus Exp $
written Dec 88 by claus
'!

!Behavior class methodsFor:'documentation'!

documentation
"
    Every class in the system inherits from Behavior (via Class, ClassDescription);
    so here is where most of the class messages end up being implemented.
    (to answer a FAQ: 'Point basicNew' will be done here :-)

    Beginners should keep in mind, that all classes are instances of Behavior
    thus, you will find the above mentioned 'basicNew:' method under the 'instance'-
    methods of Behavior - NOT under the class methods.

    Behavior provides minimum support for all classes - additional stuff is
    found in ClassDescription and Class. Behaviors provides the basic (and minimum
    needed) stuff for classes to work. However, Behavior does not provide all the
    (symbolic) information needed to compile methods for a class.

    In contrast to other ST implementations, the methods have been separated
    from the selectors (there is no Dictionary, but two separate Arrays)
    - this avoids the need for knowledge about Dictionaries in the runtime library (VM)
    (lookup and search in these is seldom anyway, so the added benefit from using a 
     hashed dictionary is almost void).

    Instance variables:

        superclass        <Class>           the receivers superclass
        otherSuperclasses <Array of Class>  experimental: other superclasses
        selectors         <Array of Symbol> the selectors for which inst-methods are defined here
        methods           <Array of Method> the inst-methods corresponding to the selectors
        instSize          <SmallInteger>    the number of instance variables
        flags             <SmallInteger>    special flag bits coded in a number

    flag bits:

    2r 0000 0000 1111      index type:
                            0 - not indexed
                            1 - byte indexed
                            2 - short indexed
                            3 - long indexed
                            4 - pointer indexed
                            5 - weak pointer indexed
                            6 - float indexed
                            7 - double indexed

                            8-15 reserved for future extensions

    2r 0000 0001 0000       reserved for alien pointers extension
    2r 0000 0010 0000       class has subclasses - not set/used currently

    2r 0000 0100 0000       instances are kind-of blocks (alien ptr in code)
    2r 0000 1000 0000       instances are kind-of methods (alien ptr in code)

    2r 0001 0000 0000       instances are kind-of contexts
    2r 0010 0000 0000       instances are kind-of block contexts

    2r 0100 0000 0000       reserved for C++ extension (inst is proxy for alien C++ object)
    2r 1000 0000 0000       reserved for objectiveC extension (inst is proxy for alien OBJC object)

    other bits are reserved for future extensions.
    Dont depend on these bit positions - they may change without notice; 
    use access methods (such as isFloats) to get this kind of information.

    NOTICE: layout known by compiler and runtime system; be careful when changing
"
! !

!Behavior class methodsFor:'queries'!

isBuiltInClass
    "this class is known by the run-time-system"

    ^ true
! !

!Behavior class methodsFor:'creating new classes'!

new
    "creates and return a new class"

    |newClass|

    newClass := self basicNew.
    newClass setSuperclass:Object
                 selectors:(Array new:0)
                   methods:(Array new:0)
                  instSize:0
                     flags:0.
    ^ newClass
! !

!Behavior methodsFor:'initialization'!

initialize
    "to catch initialize for classes which do not"

    ^ self
!

reinitialize
    "to catch reinitialize for classes which do not"

    ^ self
! !

!Behavior methodsFor:'copying'!

deepCopy
    "return a deep copy of the receiver
     - return the receiver here - time will show if this is ok"

    ^ self
!

deepCopyUsing:aDictionary
    "return a deep copy of the receiver
     - return the receiver here - time will show if this is ok"

    ^ self
!

simpleDeepCopy
    "return a deep copy of the receiver
     - return the receiver here - time will show if this is ok"

    ^ self
! !

!Behavior methodsFor:'creating an instance of myself'!

uninitializedNew
    "same as new - only redefined in ByteArray"

    ^ self basicNew
!

uninitializedNew:anInteger
    "same as new:anInteger - only redefined in ByteArray"

    ^ self basicNew:anInteger
!

new
    "return an instance of myself without indexed variables"

    ^ self basicNew
!

new:anInteger
    "return an instance of myself with anInteger indexed variables"

    ^ self basicNew:anInteger
!

basicNew
    "return an instance of myself without indexed variables.
     If the receiver-class has indexed instvars, the new object will have
     a basicSize of zero - i.e. 'aClass basicNew' is equivalent to 'aClass basicNew:0'.

     ** Do not redefine this method in any class **"

%{  /* NOCONTEXT */

    OBJ new();
    OBJ newobj;
    int instsize;
    REGISTER int nInstVars;

    /*
     * the following ugly code is nothing more than a new() followed
     * by a nilling of the new instance.
     * unrolled for a bit more speed since this is one the central object allocation
     * methods in the system
     */
    nInstVars = _intVal(_INST(instSize));
    instsize = OHDR_SIZE + nInstVars * sizeof(OBJ);

    newobj = newNextPtr;
    if (((char *)newobj + instsize) < newEndPtr) {
        _objPtr(newobj)->o_size = instsize;
        o_allFlags(newobj) = 0;
        _objPtr(newobj)->o_space = allocSpace;
        if (instsize & (ALIGN-1)) {
            newNextPtr = (char *)newobj + (instsize & ~(ALIGN-1)) + ALIGN;
        } else {
            newNextPtr = (char *)newobj + instsize;
        }

        /* 
         * save nil-check in this case
         * stupid: c-compilers should find this out themselfes ... 
         */
        goto ok;
    } else {
        PROTECT_CONTEXT
        newobj = new(instsize, SENDER);
        UNPROTECT_CONTEXT
    }
        
    if (newobj != nil) {
ok:
        _InstPtr(newobj)->o_class = self;

        if (nInstVars) {
#if defined(memset4)
            memset4(_InstPtr(newobj)->i_instvars, nil, nInstVars);
#else
# if defined(FAST_MEMSET) && ! defined(NEGATIVE_ADDRESSES)
            /*
             * knowing that nil is 0
             */
            memset(_InstPtr(newobj)->i_instvars, 0, instsize - OHDR_SIZE);
# else
            REGISTER OBJ *op;

            op = _InstPtr(newobj)->i_instvars;
#  ifdef FAST_MEMSET_WORDS_UNROLLED
            while (nInstVars > 8) {
                *op = nil;
                *(op+1) = nil;
                *(op+2) = nil;
                *(op+3) = nil;
                *(op+4) = nil;
                *(op+5) = nil;
                *(op+6) = nil;
                *(op+7) = nil;
                op += 8;
                nInstVars -= 8;
            }
#  endif
            do {
                *op++ = nil;
            } while (--nInstVars);
# endif
#endif
        }
        RETURN ( newobj );
    }
%}
.
    "
     memory allocation failed.
     When we arrive here, there was no memory, even after
     a garbage collect. This means, that the VM wanted to
     get some more memory from the Operatingsystem, which
     was not kind enough to give some
    "
    ObjectMemory allocationFailureSignal raise.
    ^ nil
!

basicNew:anInteger
    "return an instance of myself with anInteger indexed variables.
     If the receiver-class has no indexed instvars, this is only allowed
     if the argument, anInteger is zero.
     ** Do not redefine this method in any class **"

%{  /* NOCONTEXT */

    OBJ newobj;
    INT instsize, nInstVars, nindexedinstvars;
    INT flags;
#if ! defined(FAST_MEMSET) || defined(NEGATIVE_ADDRESSES)
    REGISTER char *cp;
    short *sp;
    long *lp;
#endif
    REGISTER OBJ *op;
    float *fp;
    double *dp;
    extern OBJ new();

    if (_isSmallInteger(anInteger)) {
        nindexedinstvars = _intVal(anInteger);
        if (nindexedinstvars >= 0) {
            PROTECT_CONTEXT
            nInstVars = _intVal(_INST(instSize));
            flags = _intVal(_INST(flags)) & ARRAYMASK;
            switch (flags) {
                case BYTEARRAY:
                    instsize = OHDR_SIZE + nInstVars * sizeof(OBJ) + nindexedinstvars * sizeof(char);
                    _qNew(newobj, instsize, SENDER);
                    UNPROTECT_CONTEXT
                    if (newobj == nil) {
                        break;
                    }
                    _InstPtr(newobj)->o_class = self;
#if defined(FAST_MEMSET) && ! defined(NEGATIVE_ADDRESSES)
                    /*
                     * knowing that nil is 0
                     */
                    memset(_InstPtr(newobj)->i_instvars, 0, instsize - OHDR_SIZE);
#else
                    op = _InstPtr(newobj)->i_instvars;
                    while (nInstVars--)
                        *op++ = nil;
                    cp = (char *)op;
                    while (nindexedinstvars >= sizeof(long)) {
                        *(long *)cp = 0;
                        cp += sizeof(long);
                        nindexedinstvars -= sizeof(long);
                    }
                    while (nindexedinstvars--)
                        *cp++ = '\0';
#endif
                    RETURN ( newobj );
                    break;

                case WORDARRAY:
                    instsize = OHDR_SIZE + nInstVars * sizeof(OBJ) + nindexedinstvars * sizeof(short);
                    _qNew(newobj, instsize, SENDER);
                    UNPROTECT_CONTEXT
                    if (newobj == nil) {
                        break;
                    }
                    _InstPtr(newobj)->o_class = self;
#if defined(FAST_MEMSET) && ! defined(NEGATIVE_ADDRESSES)
                    /*
                     * knowing that nil is 0
                     */
                    memset(_InstPtr(newobj)->i_instvars, 0, instsize - OHDR_SIZE);
#else
                    op = _InstPtr(newobj)->i_instvars;
                    while (nInstVars--)
                        *op++ = nil;
                    sp = (short *)op;
                    while (nindexedinstvars--)
                        *sp++ = 0;
#endif
                    RETURN ( newobj );
                    break;

               case LONGARRAY:
                    instsize = OHDR_SIZE + nInstVars * sizeof(OBJ) + nindexedinstvars * sizeof(long);
                    _qAlignedNew(newobj, instsize, SENDER);
                    UNPROTECT_CONTEXT
                    if (newobj == nil) {
                        break;
                    }
                    _InstPtr(newobj)->o_class = self;
#if defined(memset4) && ! defined(NEGATIVE_ADDRESSES)
                    /*
                     * knowing that nil is 0
                     */
                    memset4(_InstPtr(newobj)->i_instvars, 0, nInstVars + nindexedinstvars);
#else
# if defined(FAST_MEMSET) && ! defined(NEGATIVE_ADDRESSES)
                    /*
                     * knowing that nil is 0
                     */
                    memset(_InstPtr(newobj)->i_instvars, 0, instsize - OHDR_SIZE);
# else
                    op = _InstPtr(newobj)->i_instvars;
                    while (nInstVars--)
                        *op++ = nil;
                    lp = (long *)op;
                    while (nindexedinstvars--)
                        *lp++ = 0;
# endif
#endif
                    RETURN ( newobj );
                    break;

               case FLOATARRAY:
                    instsize = sizeof(struct floatArray) + 
                               nInstVars * sizeof(OBJ) + 
                               (nindexedinstvars - 1) * sizeof(float);

                    _qNew(newobj, instsize, SENDER);
                    UNPROTECT_CONTEXT
                    if (newobj == nil) {
                        break;
                    }
                    _InstPtr(newobj)->o_class = self;
                    op = _InstPtr(newobj)->i_instvars;
                    while (nInstVars--)
                        *op++ = nil;
                    fp = (float *)op;
                    while (nindexedinstvars--)
                        *fp++ = 0.0;
                    RETURN ( newobj );
                    break;

               case DOUBLEARRAY:
                    instsize = sizeof(struct doubleArray) + 
                               nInstVars * sizeof(OBJ) + 
                               (nindexedinstvars - 1) * sizeof(double);

                    _qAlignedNew(newobj, instsize, SENDER);
                    UNPROTECT_CONTEXT
                    if (newobj == nil) {
                        break;
                    }
                    _InstPtr(newobj)->o_class = self;
                    op = _InstPtr(newobj)->i_instvars;
                    while (nInstVars--)
                        *op++ = nil;
#ifdef NEED_DOUBLE_ALIGN
                    /*
                     * care for double alignment
                     */
                    if ((INT)op & (ALIGN-1)) {
                        *op++ = nil;
                    }
#endif
                    dp = (double *)op;
                    while (nindexedinstvars--)
                        *dp++ = 0.0;
                    RETURN ( newobj );
                    break;

                case WKPOINTERARRAY:
                case POINTERARRAY:
                    nInstVars += nindexedinstvars;
                    instsize = OHDR_SIZE + nInstVars * sizeof(OBJ);
                    _qAlignedNew(newobj, instsize, SENDER);
                    UNPROTECT_CONTEXT
                    if (newobj == nil) {
                        break;
                    }
                    _InstPtr(newobj)->o_class = self;
#if defined(memset4)
                    memset4(_InstPtr(newobj)->i_instvars, nil, nInstVars);
#else
# if defined(FAST_MEMSET) && ! defined(NEGATIVE_ADDRESSES)
                    /*
                     * knowing that nil is 0
                     */
                    memset(_InstPtr(newobj)->i_instvars, 0, instsize - OHDR_SIZE);
# else
                    op = _InstPtr(newobj)->i_instvars;
                    while (nInstVars--)
                        *op++ = nil;
# endif
#endif
                    RETURN ( newobj );
                    break;

                default:
                    /*
                     * new:n for non-variable classes only allowed if
                     * n == 0
                     */
                    if (nindexedinstvars == 0) {
                        instsize = OHDR_SIZE + nInstVars * sizeof(OBJ);
                        _qAlignedNew(newobj, instsize, SENDER);
                        UNPROTECT_CONTEXT
                        if (newobj == nil) {
                            break;
                        }
                        _InstPtr(newobj)->o_class = self;
                        if (nInstVars) {
#if defined(memset4)
                            memset4(_InstPtr(newobj)->i_instvars, nil, nInstVars);
#else
# if defined(FAST_MEMSET) && ! defined(NEGATIVE_ADDRESSES)
                            /*
                             * knowing that nil is 0
                             */
                            memset(_InstPtr(newobj)->i_instvars, 0, instsize - OHDR_SIZE);
# else
                            op = _InstPtr(newobj)->i_instvars;
                            do {
                                *op++ = nil;
                            } while (--nInstVars);
# endif
#endif
                        }
                        RETURN ( newobj );
                    }
                    break;
            }
        }
    }
%}.
    "
     arrive here if something went wrong ...
     figure out what it was
    "

    (anInteger isMemberOf:SmallInteger) ifFalse:[
        self error:'argument to new: must be Integer'.
	^ nil
    ].
    (anInteger < 0) ifTrue:[
        self error:'bad (negative) argument to new:'.
	^ nil
    ].
    self isVariable ifFalse:[
        self error:'class has no indexed instvars - cannot create with new:'.
	^ nil
    ].
    "
     memory allocation failed.
     When we arrive here, there was no memory, even after
     a garbage collect. This means, that the VM wanted to
     get some more memory from the Operatingsystem, which
     was not kind enough to give some
    "
    ObjectMemory allocationFailureSignal raise.
    ^ nil
!

readFrom:aStream
    "read an objects printed representation from the argument, aStream 
     and return it. 

     The read object must be a kind of myself 
     - to get any object, use 'Object readFrom:...',
       to get any number, use 'Number readFrom:...' and so on.
     This is the reverse operation to 'storeOn:'.

     WARNING: storeOn: does not handle circular references and multiple 
              references to the same object - use #storeBinary: for this."

    |newObject|

    newObject := self compiler evaluate:aStream.
    (newObject isKindOf:self) ifFalse:[
        self error:('expected ' , self name)
    ].
    ^ newObject

    "|s|
     s := WriteStream on:String new.
     #(1 2 3 4) storeOn:s.
     Object readFrom:(ReadStream on:s contents)  
    "
!

readFromString:aString
    "create an object from its printed representation
     (i.e. recreate what was stored using storeOn: or storeString).
     See warning in Behavior>>readFrom:"

    ^ self readFrom:(ReadStream on:aString)
! !

!Behavior methodsFor:'autoload check'!

isLoaded
    "return true, if the class has been loaded; 
     redefined in Autoload; see comment there"

    ^ true
!

autoload
    "force autoloading - do nothing here; 
     redefined in Autoload; see comment there"

    ^ self
! !

!Behavior class methodsFor:'flag bit constants'!

flagNotIndexed
    "return the flag code for non-indexed instances"

    ^ 0
! 

flagBytes
    "return the flag code for byte-valued indexed instances"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( BYTEARRAY ) );
%}
! 

flagWords
    "return the flag code for word-valued indexed instances (i.e. 2-byte)"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( WORDARRAY ) );
%}
! 

flagLongs
    "return the flag code for long-valued indexed instances (i.e. 4-byte)"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( LONGARRAY ) );
%}
! 

flagFloats
    "return the flag code for float-valued indexed instances (i.e. 4-byte reals)"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( FLOATARRAY ) );
%}
! 

flagDoubles
    "return the flag code for double-valued indexed instances (i.e. 8-byte reals)"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( DOUBLEARRAY ) );
%}
! 

flagPointers
    "return the flag code for pointer indexed instances (i.e. Array of object)"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( POINTERARRAY ) );
%}
! 

maskIndexType
    "return a mask to extract all index-type bits"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( ARRAYMASK ) );
%}
! 

flagBlock
    "return the flag code which marks Block-type instances"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( BLOCK_INSTS ) );
%}
! 

flagMethod
    "return the flag code which marks Method-type instances"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( METHOD_INSTS ) );
%}
! 

flagContext
    "return the flag code which marks Context-type instances"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( CONTEXT_INSTS ) );
%}
! 

flagBlockContext
    "return the flag code which marks BlockContext-type instances"

%{  /* NOCONTEXT */
    /* this is defined as a primitive to get defines from stc.h */

    RETURN ( _MKSMALLINT( BCONTEXT_INSTS ) );
%}
! !

!Behavior methodsFor:'accessing'!

superclass
    "return the receivers superclass"

    ^ superclass
!

selectorArray 
    "return the receivers selector array.
     Notice: this is not compatible with ST-80."

    ^ selectors
!

methodArray
    "return the receivers method array.
     Notice: this is not compatible with ST-80."

    ^ methods
!

methodDictionary
    "return the receivers method dictionary 
     - since no dictionary is actually present, create one for compatibility."

    |dict|

    dict := IdentityDictionary new.
    1 to:selectors size do:[:index |
        dict at:(selectors at:index) put:(methods at:index)
    ].
    ^ dict
!

instSize
    "return the number of instance variables of the receiver.
     This includes all superclass instance variables."

    ^ instSize
!

flags
    "return the receivers flag bits"

    ^ flags
!

isVariable
    "return true, if instances have indexed instance variables"

    "this could be defined as:
        ^ (flags bitAnd:(Behavior maskIndexType)) ~~ 0
     "

%{  /* NOCONTEXT */

    RETURN ( (_intVal(_INST(flags)) & ARRAYMASK) ? true : false ); 
%}
!

isFixed
    "return true, if instances do not have indexed instance variables"

    "this could be defined as:
        ^ self isVariable not
    "

%{  /* NOCONTEXT */

    RETURN ( (_intVal(_INST(flags)) & ARRAYMASK) ? false : true ); 
%}
!

isBits
    "return true, if instances have indexed byte or short instance variables.
     Ignore long, float and double arrays, since ST-80 code using isBits are probably
     not prepared to handle them correctly."

%{  /* NOCONTEXT */

    REGISTER int flags;

    RETURN ( (((flags = (_intVal(_INST(flags)) & ARRAYMASK)) == BYTEARRAY)
             || (flags == WORDARRAY)) ? true : false ); 
%}
!

isBytes
    "return true, if instances have indexed byte instance variables"

    "this could be defined as:
        ^ (flags bitAnd:(Behavior maskIndexType)) == Behavior flagBytes
    "
%{  /* NOCONTEXT */

    RETURN ( ((_intVal(_INST(flags)) & ARRAYMASK) == BYTEARRAY) ? true : false ); 
%}
!

isWords
    "return true, if instances have indexed short instance variables"

    "this could be defined as:
        ^ (flags bitAnd:(Behavior maskIndexType)) == Behavior flagWords
    "
%{  /* NOCONTEXT */

    RETURN ( ((_intVal(_INST(flags)) & ARRAYMASK) == WORDARRAY) ? true : false ); 
%}
!

isLongs
    "return true, if instances have indexed long instance variables"

    "this could be defined as:
        ^ (flags bitAnd:(Behavior maskIndexType)) == Behavior flagLongs
    "
%{  /* NOCONTEXT */

    RETURN ( ((_intVal(_INST(flags)) & ARRAYMASK) == LONGARRAY) ? true : false ); 
%}
!

isFloats
    "return true, if instances have indexed float instance variables"

    "this could be defined as:
        ^ (flags bitAnd:(Behavior maskIndexType)) == Behavior flagFloats
    "
%{  /* NOCONTEXT */

    RETURN ( ((_intVal(_INST(flags)) & ARRAYMASK) == FLOATARRAY) ? true : false ); 
%}
!

isDoubles
    "return true, if instances have indexed double instance variables"

    "this could be defined as:
        ^ (flags bitAnd:(Behavior maskIndexType)) == Behavior flagDoubles
    "
%{  /* NOCONTEXT */

    RETURN ( ((_intVal(_INST(flags)) & ARRAYMASK) == DOUBLEARRAY) ? true : false ); 
%}
!

isPointers
    "return true, if instances have pointer instance variables 
     i.e. are either non-indexed or have indexed pointer variables"

    "QUESTION: should we ignore WeakPointers ?"

%{  /* NOCONTEXT */

    REGISTER int flags;

    flags = _intVal(_INST(flags)) & ARRAYMASK;
    switch (flags) {
        default:
            /* normal objects */
            RETURN ( true );

        case BYTEARRAY:
        case WORDARRAY:
        case LONGARRAY:
        case FLOATARRAY:
        case DOUBLEARRAY:
            RETURN (false );

        case WKPOINTERARRAY:
            /* what about those ? */
            RETURN (true );
    }
%}
!

superclass:aClass
    "set the superclass - this actually creates a new class,
     recompiling all methods for the new one. The receiving class stays
     around anonymous to allow existing instances some life.
     This may change in the future (adjusting existing instances)"

    "must flush caches since lookup chain changes"
    ObjectMemory flushCaches.

"
    superclass := aClass
"
    "for correct recompilation, just create a new class ..."

    aClass subclass:(self name)
           instanceVariableNames:(self instanceVariableString)
           classVariableNames:(self classVariableString)
           poolDictionaries:''
           category:self category
!

addSuperclass:aClass
    "EXPERIMENTAL: add aClass to the set of classes, from which instances
     inherit protocol."

    "first, check if the class is abstract - 
     allows abstract mixins are allowed in the current implementation"

    aClass instSize == 0 ifFalse:[
        self error:'only abstract mixins allowed'.
        ^ self
    ].
    otherSuperclasses isNil ifTrue:[
        otherSuperclasses := Array with:aClass
    ] ifFalse:[
        otherSuperclasses := otherSuperclasses copyWith:aClass
    ].
    ObjectMemory flushCaches
!

removeSuperclass:aClass
    "EXPERIMENTAL: add aClass to the set of classes, from which instances
     inherit protocol."

    otherSuperclasses notNil ifTrue:[
        otherSuperclasses := otherSuperclasses copyWithout:aClass.
        otherSuperclasses isEmpty ifTrue:[
            otherSuperclasses := nil
        ].
        ObjectMemory flushCaches
    ].
!

selectors:selectorArray methods:methodArray
    "set both selector array and method array of the receiver,
     and flush caches"

    ObjectMemory flushCaches.
    selectors := selectorArray.
    methods := methodArray
! !

!Behavior methodsFor:'queries'!

isBehavior
    "return true, if the receiver is describing another objects behavior,
     i.e. is a class. Defined to avoid the need to use isKindOf:"

    ^ true

    "True isBehavior"
    "true isBehavior"
!

canBeSubclassed
    "return true, if its allowed to create subclasses of the receiver.
     This method is redefined in SmallInteger and UndefinedObject, since
     instances are detected by their pointer-fields, i.e. they do not have
     a class entry (you dont have to understand this :-)"

    ^ true
!

hasMultipleSuperclasses
    "Return true, if this class inherits from other classes 
     (beside its primary superclass). 
     This method is a preparation for a future multiple inheritance extension 
     - currently it is not supported by the VM"

    ^ otherSuperclasses notNil
!

superclasses
    "return a collection of the receivers immediate superclasses.
     This method is a preparation for a future multiple inheritance extension 
     - currently it is not supported by the VM"

    otherSuperclasses notNil ifTrue:[
        ^ (Array with:superclass) , otherSuperclasses
    ].
    ^ Array with:superclass

    "String superclasses"
!

allSuperclasses
    "return a collection of the receivers accumulated superclasses"

    |aCollection theSuperClass|

    theSuperClass := superclass.
    theSuperClass notNil ifTrue:[
        aCollection := OrderedCollection new.
        [theSuperClass notNil] whileTrue:[
            aCollection add:theSuperClass.
            theSuperClass := theSuperClass superclass
        ]
    ].
    ^ aCollection

    "String allSuperclasses"
!

withAllSuperclasses
    "return a collection containing the receiver and all
     of the receivers accumulated superclasses"

    |aCollection theSuperClass|

    aCollection := OrderedCollection with:self.
    theSuperClass := superclass.
    [theSuperClass notNil] whileTrue:[
        aCollection add:theSuperClass.
        theSuperClass := theSuperClass superclass
    ].
    ^ aCollection

    "String withAllSuperclasses"
!

subclasses
    "return a collection of the direct subclasses of the receiver"

    |newColl|

    newColl := OrderedCollection new.
    self subclassesDo:[:aClass |
        newColl add:aClass
    ].
    ^ newColl

    "Collection subclasses"
!

allSubclasses
    "return a collection of all subclasses (direct AND indirect) of
     the receiver. There will be no specific order, in which entries
     are returned."

    |newColl|

    newColl := OrderedCollection new.
    self allSubclassesDo:[:aClass |
        newColl add:aClass
    ].
    ^ newColl

    "Collection allSubclasses"
!

allSubclassesInOrder
    "return a collection of all subclasses (direct AND indirect) of
     the receiver. Higher level subclasses will come before lower ones."

    |newColl|

    newColl := OrderedCollection new.
    self allSubclassesInOrderDo:[:aClass |
        newColl add:aClass
    ].
    ^ newColl

    "Collection allSubclassesInOrder"
!

withAllSubclasses
    "return a collection containing the receiver and 
     all subclasses (direct AND indirect) of the receiver"

    |newColl|

    newColl := OrderedCollection with:self.
    self allSubclassesDo:[:aClass |
        newColl add:aClass
    ].
    ^ newColl

    "Collection withAllSubclasses"
!

isSubclassOf:aClass
    "return true, if I am a subclass of the argument, aClass"

    |theClass|

    theClass := superclass.
    [theClass notNil] whileTrue:[
        (theClass == aClass) ifTrue:[^ true].
        theClass := theClass superclass
    ].
    ^ false

    "String isSubclassOf:Collection"
    "LinkedList isSubclassOf:Array"
    "1 isSubclassOf:Number"     "will fail since 1 is no class"
!

allInstances
    "return a collection of all my instances"

    |coll|

    coll := OrderedCollection new.
    self allInstancesDo:[:anObject |
        coll add:anObject
    ].
    ^ coll 

    "ScrollBar allInstances"
!

allDerivedInstances
    "return a collection of all instances of myself and 
     instances of all subclasses of myself"

    |coll|

    coll := OrderedCollection new.
    self allDerivedInstancesDo:[:anObject |
        (anObject isKindOf:self) ifTrue:[
            coll add:anObject
        ]
    ].
    ^ coll 

    "View allDerivedInstances"
!

instanceCount
    "return the number of instances of myself"

    |count|

    count := 0.
    ObjectMemory allObjectsDo:[:anObject |
        (anObject class == self) ifTrue:[
            count := count + 1
        ]
    ].
    ^ count

    "View instanceCount"
!

derivedInstanceCount
    "return the number of instances of myself and of subclasses"

    |count|

    count := 0.
    ObjectMemory allObjectsDo:[:anObject |
        (anObject isKindOf:self) ifTrue:[
            count := count + 1
        ]
    ].
    ^ count

    "View derivedInstanceCount"
!

selectorIndex:aSelector
    "return the index in the arrays for given selector aSelector"

    ^ selectors identityIndexOf:aSelector startingAt:1
!

compiledMethodAt:aSelector
    "return the method for given selector aSelector or nil.
     Only methods in the receiver - not in the superclass chain are tested."

    |index|

    index := selectors identityIndexOf:aSelector startingAt:1.
    (index == 0) ifTrue:[^ nil].
    ^ methods at:index

    "Object compiledMethodAt:#=="
    "(Object compiledMethodAt:#==) category"
!

sourceCodeAt:aSelector
    "return the methods source for given selector aSelector or nil.
     Only methods in the receiver - not in the superclass chain are tested."

    |method|

    method := self compiledMethodAt:aSelector.
    method isNil ifTrue:[^ nil].
    ^ method source

    "True sourceCodeAt:#ifTrue:"
    "Object sourceCodeAt:#=="
    "Behavior sourceCodeAt:#sourceCodeAt:"
!

lookupMethodFor:aSelector
    "return the method, which would be executed if aSelector was sent to
     an instance of the receiver. I.e. the selector arrays of the receiver
     and all of its superclasses are searched for aSelector.
     Return the method, or nil if instances do not understand aSelector.
     EXPERIMENTAL: take care of multiple superclasses."

    |m cls|

    cls := self.
    [cls notNil] whileTrue:[
        m := cls compiledMethodAt:aSelector.
        m notNil ifTrue:[^ m].
        cls hasMultipleSuperclasses ifTrue:[
            cls superclasses do:[:aSuperClass |
                m := aSuperClass lookupMethodFor:aSelector.
                m notNil ifTrue:[^ m].
            ].
            ^ nil
        ] ifFalse:[
            cls := cls superclass
        ]
    ].
    ^ nil
!

cachedLookupMethodFor:aSelector
    "return the method, which would be executed if aSelector was sent to
     an instance of the receiver. I.e. the selector arrays of the receiver
     and all of its superclasses are searched for aSelector.
     Return the method, or nil if instances do not understand aSelector.
     This interface provides exactly the same information as #lookupMethodFor:,
     but uses the lookup-cache in the VM for faster search. 
     However, keep in mind, that doing a lookup through the cache also adds new
     entries and can thus slow down the system by polluting the cache with 
     irrelevant entries. (do NOT loop over all objects calling this method).
     Does NOT (currently) handle MI"

%{  /* NOCONTEXT */
    extern OBJ lookup();

    RETURN ( lookup(self, aSelector, SENDER) );
%}

    "String cachedLookupMethodFor:#="
    "String cachedLookupMethodFor:#asOrderedCollection"
!

hasMethods
    "return true, if there are any (local) methods in this class"

    ^ (methods size ~~ 0)

    "True hasMethods"
    "True class hasMethods"
!

implements:aSelector
    "return true, if the receiver implements aSelector.
     (i.e. implemented in THIS class - NOT in a superclass).
     Dont use this method to check if someone responds to a message -
     use #canUnderstand: on the class or #respondsTo: on the instance
     to do this."

    ^ (selectors identityIndexOf:aSelector startingAt:1) ~~ 0

    "True implements:#ifTrue:"
    "True implements:#=="
!

canUnderstand:aSelector
    "return true, if the receiver or one of its superclasses implements aSelector.
     (i.e. true if my instances understand aSelector)"

    ^ (self lookupMethodFor:aSelector) notNil

    "True canUnderstand:#ifTrue:"
    "True canUnderstand:#=="
    "True canUnderstand:#do:"
!

whichClassImplements:aSelector
    "return the class in the inheritance chain, which implements the method
     for aSelector; return nil if none.
     EXPERIMENTAL: handle multiple superclasses"

    |cls|

    cls := self.
    [cls notNil] whileTrue:[
        (cls implements:aSelector) ifTrue:[^ cls].
        cls hasMultipleSuperclasses ifTrue:[
            cls superclasses do:[:aSuperClass |
                |implementingClass|

                implementingClass := aSuperClass whichClassImplements:aSelector.
                implementingClass notNil ifTrue:[^ implementingClass].
            ].
            ^ nil
        ] ifFalse:[
            cls := cls superclass
        ]
    ].
    ^ nil

    "String whichClassImplements:#=="
    "String whichClassImplements:#collect:"
!

inheritsFrom:aClass
    "return true, if the receiver inherits methods from aClass"

    ^ self isSubclassOf:aClass

    "True inheritsFrom:Object"
    "LinkedList inheritsFrom:Array"
!

selectorForMethod:aMethod
    "Return the selector for given method aMethod."

    |index|

    index := methods identityIndexOf:aMethod startingAt:1.
    (index == 0) ifTrue:[^ nil].
    ^ selectors at:index
!

containsMethod:aMethod
    "Return true, if aMethod is a method of myself"

    ^ (methods identityIndexOf:aMethod startingAt:1) ~~ 0
! !

!Behavior methodsFor:'private accessing'!

setSuperclass:sup selectors:sels methods:m instSize:i flags:f
    "set some inst vars. 
     this method is for private use only - there will be not recompilation
     and no change record written here; 
     Do NOT use it."

    superclass := sup.
    selectors := sels.
    methods := m.
    instSize := i.
    flags := f
!

setSuperclass:aClass
    "set the superclass of the receiver.
     this method is for private use only - there will be not recompilation
     and no change record written here; 
     Do NOT use it."

    superclass := aClass
!

setOtherSuperclasses:anArrayOfClasses
    "EXPERIMENTAL: set the other superclasses of the receiver.
     this method is for private use only - there will be not recompilation
     and no change record written here; 
     Do NOT use it."

    otherSuperclasses := anArrayOfClasses
!

instSize:aNumber
    "set the instance size.
     this method is for private use only - there will be not recompilation
     and no change record written here; 
     Do NOT use it."

    instSize := aNumber
!

flags:aNumber
    "set the flags.
     this method is for private use only - there will be not recompilation
     and no change record written here; 
     Do NOT use it."

    flags := aNumber
!

setSelectors:anArray
    "set the selector array of the receiver.
     this method is for private use only - there will be not recompilation
     and no change record written here; 
     Do NOT use it."

    selectors := anArray
!

setMethodDictionary:anArray
    "set the method array of the receiver.
     this method is for private use only - there will be not recompilation
     and no change record written here; 
     Do NOT use it."

    methods := anArray
! !

!Behavior methodsFor:'compiler interface'!

compiler
    "return the compiler to use for this class - 
     this can be redefined in special classes, to get classes with
     Lisp, Prolog, ASN1, Basic :-) or whatever syntax."

    ^ Compiler
! !

!Behavior methodsFor:'enumeration'!

allInstancesDo:aBlock
    "evaluate aBlock for all of my instances"

    ObjectMemory allObjectsDo:[:anObject |
        (anObject class == self) ifTrue:[
            aBlock value:anObject
        ]
    ]

    "StandardSystemView allInstancesDo:[:v | Transcript showCr:(v name)]"
!

allDerivedInstancesDo:aBlock
    "evaluate aBlock for all of my instances and all instances of subclasses"

    ObjectMemory allObjectsDo:[:anObject |
        (anObject isKindOf:self) ifTrue:[
            aBlock value:anObject
        ]
    ]

    "StandardSystemView allDerivedInstancesDo:[:v | Transcript showCr:(v name)]"
!

subclassesDo:aBlock
    "evaluate the argument, aBlock for all immediate subclasses"

    Smalltalk allBehaviorsDo:[:aClass |
        (aClass superclass == self) ifTrue:[
            aBlock value:aClass
        ]
    ]

    "Collection subclassesDo:[:c | Transcript showCr:(c name)]"
!

allSubclassesDo:aBlock
    "evaluate aBlock for all of my subclasses.
     There is no specific order, in which the entries are enumerated."

    Smalltalk allBehaviorsDo:[:aClass |
        (aClass isSubclassOf:self) ifTrue:[
            aBlock value:aClass
        ]
    ]

    "Collection allSubclassesDo:[:c | Transcript showCr:(c name)]"
!

allSubclassesInOrderDo:aBlock
    "evaluate aBlock for all of my subclasses.
     Higher level subclasses will be enumerated before the deeper ones"

    self subclassesDo:[:aClass |
        aBlock value:aClass.
        aClass allSubclassesInOrderDo:aBlock
    ]

    "Collection allSubclassesInOrderDo:[:c | Transcript showCr:(c name)]"
!

allSuperclassesDo:aBlock
    "evaluate aBlock for all of my superclasses"

    |theClass|

    theClass := superclass.
    [theClass notNil] whileTrue:[
        aBlock value:theClass.
        theClass := theClass superclass
    ]

    "String allSuperclassesDo:[:c | Transcript showCr:(c name)]"
! !

!Behavior methodsFor:'binary storage'!

readBinaryFrom:aStream
    "read an objects binary representation from the argument,
     aStream and return it. 
     The read object must be a kind of myself 
     - to get any object, use 'Object readBinaryFrom:...',
       to get any number, use 'Number readBinaryFrom:...' and so on.
     This is the reverse operation to 'storeBinaryOn:'. "

    |newObject|

    newObject := (BinaryInputManager new:1024) readFrom:aStream.
    (newObject isKindOf:self) ifFalse:[
        self error:('expected ' , self name)
    ].
    ^ newObject

    "|s|
     s := WriteStream on:ByteArray new.
     #(1 2 3 4) storeBinaryOn:s.
     Object readBinaryFrom:(ReadStream on:s contents)  
    "
!

storeBinaryDefinitionOn: stream manager: manager
    "classes will store the name only and restore by looking for
     that name in the Smalltalk dictionary."

    | myName |

    myName := self name.
    stream
        "nextWordPut: (format bitAnd: 16rFFFF);"
        "nextWordPut:" nextNumber:2 put:myName size.
    myName do:[:c| 
        stream nextPut:c asciiValue
    ]
!

binaryDefinitionFrom:stream manager:manager
    "read the definition on an empty instance (of my class) from stream.
     All pointer instances are left nil, while all bits are read in here.
     return the new object."

    |obj t
     basicSize "{ Class: SmallInteger }" |

    self isPointers ifTrue: [
        stream next. "skip instSize"
        self isVariable ifTrue: [
            ^ self basicNew:(stream nextNumber:3)
        ].
        ^ self basicNew
    ].

    basicSize := stream nextNumber:4.
    obj := self basicNew:basicSize.

    self isBytes ifTrue: [
        stream nextBytes:basicSize into:obj
"
        1 to:basicSize do:[:i |
            obj basicAt:i put:stream next
        ]
"
    ] ifFalse: [
        self isWords ifTrue: [
            1 to:basicSize do:[:i |
                obj basicAt:i put: stream nextNumber:2
            ]
        ] ifFalse:[
            self isLongs ifTrue: [
                1 to:basicSize do:[:i |
                    obj basicAt:i put: stream nextNumber:4
                ]
            ] ifFalse:[
                self isFloats ifTrue: [
                    "could do it in one big read on machines which use IEEE floats ..."
                    1 to:basicSize do:[:i |
                        t := Float basicNew.
                        Float readBinaryIEEESingleFrom:stream into:t.
                        obj basicAt:i put: t
                    ]
                ] ifFalse:[
                    self isDoubles ifTrue: [
                        "could do it in one big read on machines which use IEEE doubles ..."
                        1 to:basicSize do:[:i |
                            t := Float basicNew.
                            Float readBinaryIEEEDoubleFrom:stream into:t.
                            obj basicAt:i put: t
                        ]
                    ]
                ]
            ]
        ]
    ].
    ^obj
! !
