TreeGraphView subclass:#ClassTreeGraphView
       instanceVariableNames:''
       classVariableNames:''
       poolDictionaries:''
       category:'Demos-Graphics'
!

ClassTreeGraphView comment:'
 a small application showing an ObjectView / TreeView use

 must fileIn TreeView.st, TreeGraphView.st first,
 then start with:
     ClassTreeGraphView start

 %W% %E%
 written spring 91 by claus

 This is one of the rare cases, where I like to have multiple inheritance
 (TreeGraphView and ClassTreeView).
'!

!ClassTreeGraphView class methodsFor:'startup'!

start
    |aDemo topView v|

    topView := StandardSystemView 
                    label:'Class hierarchy'
                    icon:(Form fromFile:'Demo.icon' resolution:100).
    v  := HVScrollableView for:self in:topView.
    v origin:(0 @ 0) extent:(1.0 @ 1.0).

    topView open

    "ClassTreeGraphView start"
! !

!ClassTreeGraphView methodsFor:'initialization'!

initialize
    topNode := Object.
    super initialize.
!

initializeMiddleButtonMenu
    |labels|

    labels := resources array:#(
                        'browse'
                        'browse hierarchy'
                        '-'
                        'category'
                        'comment'
                        'definition'
                       ).

    self middleButtonMenu:(PopUpMenu
                                labels:labels
                             selectors:#(startBrowser
                                         startHierarchyBrowser
                                         nil
                                         showCategory
                                         showComment
                                         showDefinition
                                        )
                                receiver:self
                                     for:self)
! !

!ClassTreeGraphView methodsFor:'generating the class-tree picture'!

childrenOf:aClass do:aBlock
    |subclasses|

    subclasses := aClass subclasses.
    (subclasses size ~~ 0) ifTrue:[
        subclasses := subclasses asSortedCollection:[:a :b | a name < b name].
        subclasses do:aBlock
    ]
!

textFor:aClass
    ^ aClass name
! !

!ClassTreeGraphView methodsFor:'user interaction'!

startBrowser
    self withSelectedElementDo:[:theClass |
         SystemBrowser browseClass:theClass
    ]
!

startHierarchyBrowser
    self withSelectedElementDo:[:theClass |
         SystemBrowser browseClassHierarchy:theClass
    ]
!

showCategory
    self withSelectedElementDo:[:theClass |
         self notify:(theClass category)
    ]
!

showComment
    self withSelectedElementDo:[:theClass |
         self notify:(theClass comment)
    ]
!

showDefinition
    |s|

    self withSelectedElementDo:[:theClass |
        s := WriteStream on:String new.
        theClass fileOutDefinitionOn:s.
        self notify:(s contents)
    ]
! !
