"
 COPYRIGHT (c) 1993 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

View subclass:#ClockView
         instanceVariableNames:'lastTime timeBlock
                                xTable yTable showSeconds
                                bgColor fgColor'
         classVariableNames:''
         poolDictionaries:''
         category:'Demos'
!

ClockView comment:'
 xclock replacement

 :-) not that I thought this clock is any better than xclock
     or that the world needs clocks ...

     its showing how easy it is to program in Smalltalk - 
     this took less than an hour to program - compare with xclock code ....

 $Header: /claus/smalltalk/st2.6/clients/Clock/RCS/ClockView.st,v 1.2 94/01/13 00:34:08 claus Exp $
 written spring 91 by claus
'!

!ClockView methodsFor:'initialization'!

initialize
    |angle ta|

    super initialize.

    bgColor := White.
    fgColor := Black.
    self viewBackground:White.

    showSeconds := true.

    "initialize table of (normalized) x/y coordinates of ticks"
    xTable := Array new:60.
    yTable := Array new:60.
    angle := 0.
    1 to:60 do:[:index |
        angle := angle + 6. "360 / 60"
        ta := (360 - angle) + 90.
        xTable at:index put:((ta degreesToRadians) cos).
        yTable at:index put:((ta degreesToRadians) sin negated)
    ]
!

initializeMiddleButtonMenu
    |labels|

    labels := resources array:#('seconds').
    self middleButtonMenu:(PopUpMenu
                                    labels:labels
                                 selectors:#(toggleSeconds)
                                    receiver:self
                                         for:self)
!

initEvents
    super initEvents.

    "trigger timeout block in a second"
    timeBlock := [self showTime].
    Processor addTimedBlock:timeBlock after:2
! !

!ClockView methodsFor:'misc'!

showSeconds:aBoolean
    showSeconds := aBoolean.
    shown ifTrue:[
        Processor removeTimedBlock:timeBlock.
        lastTime := nil.
        self clear.
        self redraw.
        Processor addTimedBlock:timeBlock after:1
    ]
!

toggleSeconds
    "user wants to toggle seconds flag"

    showSeconds := showSeconds not.
    Processor removeTimedBlock:timeBlock.
    lastTime := nil.
    self clear.
    self redraw.
    Processor addTimedBlock:timeBlock after:1
! !

!ClockView methodsFor:'drawing'!

showTime
    "executed every second (by timedBlock)"

    |now anyChange deltaT|

    shown ifFalse:[^ self].

    now := Time now.
    lastTime isNil ifTrue:[
        anyChange := true
    ] ifFalse:[
        showSeconds ifTrue:[
            anyChange := now ~= lastTime
        ] ifFalse:[
            anyChange := now hourInDay ~= lastTime hourInDay.
            anyChange ifFalse:[
                anyChange := now minuteInDay ~= lastTime minuteInDay
            ]
        ]
    ].
    anyChange ifTrue:[
        self updateArms.
        device synchronizeOutput
    ].
    showSeconds ifTrue:[
        deltaT := 1
    ] ifFalse:[
        deltaT := 30
    ].
    Processor addTimedBlock:timeBlock after:deltaT
!

redraw
    lastTime isNil ifTrue:[
        lastTime := Time now
    ].
    self redrawTicks.
    self redrawArms
!

updateArms
    "clear previous arms - draw new arms"

    |hr min sec lastSec lastHr lastMin now tm ts th|

    now := Time now.
    lastTime notNil ifTrue:[
        "a little optimization - avoid redrawing minute-
         and hour arms if second-arms distance is big enough"

        showSeconds ifTrue:[
            lastHr := lastTime hourInDay.
            lastMin := lastTime minuteInDay.
            lastSec := lastTime secondInDay.

            hr := now hourInDay.
            min := now minuteInDay.
            sec := now secondInDay.

            ((min == lastMin) and:[hr == lastHr]) ifTrue:[
                "only second has not changed"
                (hr >= 12) ifTrue:[
                    hr := hr - 12
                ].
                hr := hr * 5.
                hr := hr + (min * 5 / 60) asInteger.

                min < 30 ifTrue:[
                    tm := min + 60
                ] ifFalse:[
                    tm := min
                ].
                sec < 30 ifTrue:[
                    ts := sec + 60
                ] ifFalse:[
                    ts := sec
                ].
                hr < 30 ifTrue:[
                    th := hr + 60
                ] ifFalse:[
                    th := hr
                ].

                (((tm - ts) abs > 5) and:[(th - ts) abs > 5]) ifTrue:[
                    "second is more than 5 ticks away from others -
                     no need to draw other arms"

                    lastMin < 30 ifTrue:[
                        tm := lastMin + 60
                    ] ifFalse:[
                        tm := lastMin
                    ].
                    lastSec < 30 ifTrue:[
                        ts := lastSec + 60
                    ] ifFalse:[
                        ts := lastSec
                    ].
                    lastHr < 30 ifTrue:[
                        th := lastHr + 60
                    ] ifFalse:[
                        th := lastHr
                    ].
                    "if lastsecond was more than 5 away from others,
                     just draw seconds-arm"
                    (((tm - ts) abs > 5) and:[(th - ts) abs > 5]) ifTrue:[
                        self paint:bgColor.
                        self drawArm:lastSec width:1 len:0.8.
                        lastTime := now.
                        self paint:fgColor.
                        self drawArm:sec width:1 len:0.8.
                        ^ self
                    ]
                ]
            ]
        ].

        self paint:bgColor.
        self drawArmsFor:lastTime
    ].
    lastTime := now.
    self paint:fgColor.
    self drawArmsFor:lastTime
!

redrawArms
    "draw current arms"

    self drawArmsFor:lastTime
!

drawArmsFor:aTime
    "draw arms for a given time in current paint"

    |sec min hr hrTick|

    aTime isNil ifTrue:[^ self].
    hr := aTime hourInDay.
    min := aTime minuteInDay.
    sec := aTime secondInDay.
    (hr >= 12) ifTrue:[
        hr := hr - 12
    ].
    hrTick := hr * 5.
    hrTick := hrTick + (min * 5 / 60) asInteger.
    self drawArm:hrTick width:3 len:0.5.
    self drawArm:min width:3 len:0.8.
    showSeconds ifTrue:[
        self drawArm:sec width:1 len:0.8
    ]
!

drawArm:tick width:w len:l
    "draw an arm of the clock; argument tick specifies minute (0..59); 
     width is width in pixel, len is relative len - 1.0 is full"

    |index p1x p1y p2x p2y|

    index := tick.
    (index == 0) ifTrue:[
        index := 60
    ].
    p1x := (width - w) // 2.
    p1y := height // 2.
    p2x := p1x + (((width / 2) * (xTable at:index)) * l) + 0.5.
    p2y := p1y + (((height / 2) * (yTable at:index)) * l) + 0.5.
    self lineWidth:w.
    self displayLineFromX:p1x y:p1y
                      toX:(p2x asInteger) y:(p2y asInteger)
!

redrawTicks
    |cx cy p1x p1y p2x p2y nx ny long drawShorts tickStart|

    self lineWidth:0.
    self paint:fgColor.

    cx := width // 2.
    cy := height // 2.
    drawShorts := (width >= 100) and:[height >= 100].

    1 to:60 do:[:tick |

        "is it a long tick ?"

        long := (tick \\ 5) == 0.

        (drawShorts or:[long]) ifTrue:[

            "get normalized x/y of tick-end"

            nx := (width / 2) * (xTable at:tick).
            ny := (height / 2) * (yTable at:tick).

            "if long, tick goes from 0.85 to 0.95;
             otherwise from 0.90 to 0.95"

            long ifTrue:[
                tickStart := 0.85
            ] ifFalse:[
                tickStart := 0.9
            ].

            p1x := cx + (nx * tickStart).
            p1y := cy + (ny * tickStart).
            p2x := cx + (nx * 0.95).
            p2y := cy + (ny * 0.95).
            self displayLineFromX:(p1x asInteger) y:(p1y asInteger)
                              toX:(p2x asInteger) y:(p2y asInteger)
        ]
    ]
! !

!ClockView methodsFor:'events'!

update:what with:arg
    (what == #rectangle) ifTrue:[
        self redraw
    ]
!

sizeChanged
    "have to clear & redraw when my size changes"

    super sizeChanged.
    self clear.
    self redraw
! !

!ClockView methodsFor:'user actions'!

destroy
    "remove myself"

    Processor removeTimedBlock:timeBlock.
    super destroy
! !
