"
 COPYRIGHT (c) 1993 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

HorizontalSlider subclass:#ColorSlider
         instanceVariableNames:'bgPattern colors labelString baseColor nColors'
         classVariableNames:''
         poolDictionaries:''
         category:'Views-Interactors'
!

!ColorSlider methodsFor:'initialization'!

sizeChanged:how
    super sizeChanged:how.
    bgPattern := Form width:width height:height depth:device depth.
    self setBgPattern
!

initialize
    super initialize.
    nColors := 40.
    self is3D ifTrue:[
        level := -1
    ].
    bgPattern := Form width:width height:height depth:device depth.
!

recreate
    super recreate.
    bgPattern := Form width:width height:height depth:device depth
! !

!ColorSlider methodsFor:'drawing'!

drawThumbBackgroundInX:x y:y width:w height:h
    "draw part of the thumbs background; defined as a separate
     method, to allow drawing of arbitrary patterns under thumb."

    self copyFrom:bgPattern
                x:x y:y 
              toX:x y:y
            width:w height:h
! !

!ColorSlider methodsFor:'private'!

setBgPattern
    ""
    |n x1 x2|

    n := colors size.
    n == 0 ifTrue:[
        "no colors - at least clear bgPattern"
        bgPattern paint:Color black.
        bgPattern fillRectangleX:0 y:0 width:width height:height.
    ].
    n to:1 by:-1 do:[:i |
        x1 := (width / n * (i - 1)) rounded.
        x2 := (width / n * i) rounded.
        bgPattern paint:(colors at:i).
        bgPattern fillRectangleX:x1
                               y:0
                           width:(x2 - x1 + 1)
                          height:height
    ].
    labelString notNil ifTrue:[
        (colors at:1) greyIntensity > 50 ifTrue:[
            bgPattern paint:Color black
        ] ifFalse:[
            bgPattern paint:Color white
        ].

        bgPattern displayString:labelString x:10 
                                            y:(height // 2 + font ascent - (font height // 2)).
    ]
! !

!ColorSlider methodsFor:'accessing'!

colors
    ^ colors
!

baseColor
    ^ baseColor
!

label:aString
    labelString := aString.
    shown ifTrue:[
        self setBgPattern.
        self redraw
    ]
!

inverseColor:aColor
    "set bg pattern based on aColor, smoothing from 100 to 0"

    |fr fg fb|

    baseColor := aColor.
    aColor notNil ifTrue:[
        colors := Array new:nColors.
        fr := aColor red / (nColors - 1).
        fg := aColor green / (nColors - 1).
        fb := aColor blue / (nColors - 1).
        1 to:nColors do:[:i |
            colors at:(nColors - i + 1)
                  put:(Color red:(fr * (i - 1))
                           green:(fg * (i - 1))
                            blue:(fb * (i - 1)))
        ].
    ].
    shown ifTrue:[
        self setBgPattern.
        self redraw
    ]
!

color:aColor
    "set bg pattern based on aColor, smoothing from 0 to 100"

    |fr fg fb|

    baseColor ~= aColor ifTrue:[
        baseColor := aColor.
        aColor notNil ifTrue:[
            colors := Array new:nColors.
            fr := aColor red / (nColors - 1).
            fg := aColor green / (nColors - 1).
            fb := aColor blue / (nColors - 1).
            1 to:nColors do:[:i |
                colors at:i put:(Color red:(fr * (i - 1))
                                     green:(fg * (i - 1))
                                      blue:(fb * (i - 1)))
            ].
        ].
        shown ifTrue:[
            self setBgPattern.
            self redraw
        ]
    ]
! !
