"
 COPYRIGHT (c) 1993 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

StandardSystemView subclass:#ColorWheel
       instanceVariableNames:'nSegment nRing colors action position markColor
                              saveUnderMark'
       classVariableNames:''
       poolDictionaries:''
       category:'Demos'
!

ColorWheel comment:'
 for color choosing.

 $Header: /claus/smalltalk/st2.6/clients/ColorPanel/RCS/ColorWheel.st,v 1.1 93/10/25 10:44:05 claus Exp $
 written summer 93 by claus
'!

!ColorWheel class methodsFor:'startup'!

start
    |aDemo w|

    aDemo := self 
                label:'ColorWheel'
                icon:(Form fromFile:'ColorWheel.xbm' resolution:100)
                minExtent:(200 @ 200)
                maxExtent:(1000 @ 1000).

    aDemo extent:(200 @ 200).
    aDemo numberOfSegments:36 numberOfRings:5.
    aDemo realize.
    ^ aDemo

    "ColorWheel start"
! !

!ColorWheel methodsFor:'accessing'!

colors
    "return the collection of colors used in this wheel"

    ^ colors
!

action:aBlock
    "set the actionblock to be evaluated with the choosen
     color"

    action := aBlock
!

numberOfSegments:nseg numberOfRings:nring
    "allocate the colors for nseg segments in nring rings and
     keep refs to those in colors.
     Because this takes a long time, we look for an existing wheel
     first and attach to its colors if possible."

    |delta percent clr hue saturation otherColors|

    self class allInstances do:[:aWheel |
        aWheel device == device ifTrue:[
            otherColors := aWheel colors.
            otherColors size == nRing ifTrue:[
                (otherColors at:1) size == nSegment ifTrue:[
                    colors := otherColors.
                    ^ self
                ]
            ]
        ]
    ].

    nRing := nring.
    nSegment := nseg.
    colors := Array new:nRing.
    1 to:nRing do:[:r |
        colors at:r put:(Array new:nSegment)
    ].
    hue := 0.
    1 to:nSegment do:[:segment |
        saturation := 0.
        1 to:nRing do:[:ring |
            clr := (Color hue:hue
                          light:50
                          saturation:saturation).
            clr := clr on:device.
            (colors at:ring) at:segment put:clr.
            saturation := saturation + (100 / (nRing - 1)).
        ].
        hue := hue + (360 / nSegment)
    ]
!

position:aPoint
    |oldPos x y w h|

    aPoint = position ifTrue:[^ self].

    realized ifFalse:[
        position := aPoint.
        ^ self
    ] .
    position notNil ifTrue:[
        oldPos := position.
        position := nil.
        x := oldPos x * width + width // 2 - 6.
        y := oldPos y * height + height // 2 - 6.
        w := 12.
        h := 12.
        saveUnderMark notNil ifTrue:[
            self foreground:(Color colorId:1) background:(Color colorId:0).
            self copyFrom:saveUnderMark x:0 y:0 toX:x y:y width:12 height:12
        ] ifFalse:[
            self clearRectangleX:x y:y width:w height:h.
            self redrawX:x y:y width:w height:h.
        ]
    ].
    position := aPoint.
    x := position x * width + width // 2 - 5.
    y := position y * height + height // 2 - 5.

    saveUnderMark := Form width:12 height:12 depth:(device depth) on:device.
    saveUnderMark foreground:(Color colorId:1) background:(Color colorId:0).
    saveUnderMark copyFrom:self x:x-1 y:y-1 toX:0 y:0 width:12 height:12.

    self paint:markColor.
    self displayArcX:x y:y w:10 h:10 from:0 angle:360
!

color:aColor
    aColor brightness >= 0.5 ifTrue:[
        markColor := Black
    ] ifFalse:[
        markColor := White
    ].
    self position:(self positionFromColor:aColor) 
! !

!ColorWheel methodsFor:'initialization'!

initialize
    super initialize.
    action := [:color | color printNewline].
    nSegment := 36.
    nRing := 5.
    position := nil
!

initEvents
    self enableButtonEvents.
    self enableButtonMotionEvents
! !

!ColorWheel methodsFor:'realization'!

create
    |clr|

    super create.

    colors isNil ifTrue:[
        self numberOfSegments:nSegment numberOfRings:nRing
    ].

    1 to:nRing do:[:ring |
        1 to:nSegment do:[:segment |
            clr := (colors at:ring) at:segment.
            clr := clr on:device.
            (colors at:ring) at:segment put:clr
        ]
    ]
! !

!ColorWheel methodsFor:'drawing'!

redraw
    |angle startAngle center off p2 w h cx cy|

    angle := 360 / nSegment.
    cx := ((width - 1) / 2) rounded.
    cy := ((height - 1) / 2) rounded.

    nRing to:1 by:-1 do:[:ring |
        startAngle := 0.
        1 to:nSegment do:[:segment |
            self paint:((colors at:ring) at:segment).
            w := ((width - 1) / nRing * ring) rounded.
            h := ((height - 1) / nRing * ring) rounded.

            self fillArcX:cx-(w // 2) y:cy-(h // 2)
                        w:w h:h
                     from:startAngle angle:angle.
            startAngle := startAngle + angle.
        ]
    ].

    "one circle around that thing"
    self paint:Black.
    startAngle := 0.
    self displayArcX:0 y:0
                   w:width-1 h:height-1
                from:0 angle:360.

    position notNil ifTrue:[
        self paint:markColor.
        self displayArcX:position x * width + width // 2 - 5
                       y:position y * height + height // 2 - 5
                       w:10 h:10 from:0 angle:360
    ]
! !

!ColorWheel methodsFor:'private'!

colorFromX:x y:y
    "given an x/y position, return corresponding color"

    |angle cx cy pX pY a b dist t|

    "translate to center-origin"

    a := (width - 1 - 2) / 2.0.
    b := (height - 1 - 2) / 2.0.

    cx := a rounded.
    cy := b rounded.

    pX := x - cx.
    pY := (y - cy) negated.

    "scale to 0..1"

    pX := pX / a.
    pY := pY / b.

    "get distance"

    pX := (pX min:1) max:-1.
    pY := (pY min:1) max:-1.

    dist := (pX squared + pY squared) sqrt.
    dist := dist min:1.

" uncommenting this will threshhold saturation values ...
dist := (dist * nRing) rounded / nRing.
"

    pX = 0 ifTrue:[
        angle := pY > 0 ifTrue:[90] ifFalse:[270].
    ] ifFalse:[
        angle := (pY / pX) arcTan radiansToDegrees.
        pX < 0 ifTrue:[
            angle := angle + 180
        ] ifFalse:[
            pY < 0 ifTrue:[
                angle := angle + 360
            ]
        ]
    ].

    ^ Color hue:angle light:50 saturation:(dist * 100.0)
!

positionFromColor:aColor
    "given a color, return corresponding x/y position (normalized)"

    |angle cx cy pX pY a b dist t h|


    "translate to center-origin"

    a := (width - 1 - 2) / 2.0.
    b := (height - 1 - 2) / 2.0.

    cx := a rounded.
    cy := b rounded.  "center"

    "distance (0..1) from saturation"

    dist := aColor saturation / 100.0.

    h := aColor hue.
    h isNil ifTrue:[
        "achromatic - center"
        ^ 0@0
    ].
    h := h degreesToRadians.

    pX := h cos.
    pY := h sin negated.

    pX := pX * dist.
    pY := pY * dist.

    ^ pX @ pY
! !

!ColorWheel methodsFor:'events'!

sizeChanged:how
    super sizeChanged:how.
    saveUnderMark := nil
!

buttonPress:button x:x y:y
    |color|

    button == 1 ifFalse:[
        ^ super buttonPress:button x:x y:y
    ].
    color := self colorFromX:x y:y.
    action notNil ifTrue:[
        action value:color
    ].
    self color:(self colorFromX:x y:y)
!

buttonMotion:button x:x y:y
    |color|

    self buttonPress:1 x:x y:y
!

buttonRelease:button x:x y:y
"
    self color:(self colorFromX:x y:y)
"
! !
