"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DisplayObject subclass:#Connection
       instanceVariableNames:'srcGate dstGate srcPinNr dstPinNr'
       classVariableNames:''
       poolDictionaries:''
       category:'Interface-LogicTool'
!

Connection comment:'
 represents connections

 $Header: /claus/smalltalk/st2.6/clients/LogicTool/RCS/Connect.st,v 1.1 94/01/13 00:45:35 claus Exp $
 written summer 89 by claus
'!

!Connection class methodsFor:'instance creation'!

from:gate1 pin:pin1Nr to:gate2 pin:pin2Nr
    ^ (self basicNew) from:gate1 pin:pin1Nr
                        to:gate2 pin:pin2Nr
! !

!Connection methodsFor:'testing'!

isHitBy:hitPoint withDelta:hitDelta
    "return true, if connection is hit by point"

    |originX originY cornerX cornerY
     p1 p2 hx hy dx dy expy|

    "quick boundary check"
    (super isHitBy:hitPoint withDelta:hitDelta) ifFalse:[
        ^ false
    ].

    p1 := self findOrigin.
    p2 := self findCorner.

    originX := p1 x.
    originY := p1 y.
    cornerX := p2 x.
    cornerY := p2 y.

    "look, if hitpoint is on line"
    dx := cornerX - originX. "deltaX"
    dy := cornerY - originY. "deltaY"

    "if its a horizontal/vertical line - we're done"
    (dx == 0) ifTrue:[^ true].
    (dy == 0) ifTrue:[^ true].

    hx := hitPoint x - originX.
    hy := hitPoint y - originY.

    expy := (hx * dy) // dx.

    "for the given x, then the point on the line is x/expy.
     now, look if hitpoint is within a circle with radius delta
     around this point
    "
    hx := 0. hy := hy - expy.
    (hy * hy) <= (hitDelta * hitDelta) ifTrue:[^ true].
    ^ false
!

canBeMoved
    "no - connections are moved when gates are moved"

    ^ false
! !

!Connection methodsFor:'private'!

srcPosition
    ^ (srcGate outputPinLocation:srcPinNr)
!

dstPosition
    ^ (dstGate inputPinLocation:dstPinNr)
!

findOrigin
    ^ (srcGate outputPinLocation:srcPinNr) 
      min:(dstGate inputPinLocation:dstPinNr)
!

findCorner
    ^ (srcGate outputPinLocation:srcPinNr) 
      max:(dstGate inputPinLocation:dstPinNr)
!

recomputeFrame
    |origin corner left right top bot|

    srcGate notNil ifTrue:[
        dstGate notNil ifTrue:[
            origin := self findOrigin.
            corner := self findCorner.
            top := (origin y min:corner y) - 1.
            bot := (origin y max:corner y) + 1.
            left := (origin x min:corner x) - 1.
            right := (origin x max:corner x) + 1.
            frame := Rectangle left:left top:top right:right bottom:bot
        ]
    ]
! !

!Connection methodsFor:'drawing'!

drawIn:aGC offset:anOffset
    |offs|

    offs := anOffset - aGC viewOrigin.
    aGC paint:(Color black).
    aGC displayLineFrom:(self srcPosition + offs) to:(self dstPosition + offs)
!

drawSelectedIn:aGC offset:anOffset
    self drawIn:aGC offset:anOffset
! !

!Connection methodsFor:'accessing'!

srcPinNr
    ^ srcPinNr
!

dstPinNr
    ^ dstPinNr
!

srcGate
    ^ srcGate
!

dstGate
    ^ dstGate
!

srcGate:gate1 dstGate:gate2
    srcGate := gate1.
    dstGate := gate2.
    self recomputeFrame
!

fromNamedGate:gate1 pin:pin1Nr toNamedGate:gate2 pin:pin2Nr
    srcGate := gate1.
    dstGate := gate2.
    srcPinNr := pin1Nr.
    dstPinNr := pin2Nr
!

from:gate1 pin:pin1Nr to:gate2 pin:pin2Nr
    srcGate := gate1.
    dstGate := gate2.
    srcPinNr := pin1Nr.
    dstPinNr := pin2Nr.
    self recomputeFrame
! !

!Connection methodsFor:'signals'!

propagate
    dstGate input:dstPinNr value:(srcGate output:srcPinNr)
! !

!Connection methodsFor:'storing'!

storeOn:aStream
    aStream nextPut:$(.
    aStream nextPutAll:(self class name).
    aStream nextPutAll:' new'.
    aStream nextPutAll:' fromNamedGate:'. (srcGate name) storeOn:aStream.
    aStream nextPutAll:' pin:'. srcPinNr storeOn:aStream.
    aStream nextPutAll:' toNamedGate:'. (dstGate name) storeOn:aStream.
    aStream nextPutAll:' pin:'. dstPinNr storeOn:aStream.
    aStream nextPut:$)
! !
