"       NAME            Paths
        AUTHOR          scaletti@uxc.cso.uiuc.edu (Kurt J. Hebel)
        FUNCTION        Reimplementation of the Path hierarchy
        ST-VERSION      2.2?
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1
        DATE    3 Jun 1987
"

AbstractPath subclass: #Curve
        instanceVariableNames: 'firstPoint secondPoint thirdPoint'
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Graphics-Paths'!
Curve comment:
'A subclass of AbstractPath that is a conic section determined by three points
 that interpolates the first and the third and is tangent to the angle formed
 by the three points at the first and third points.'!


!Curve methodsFor: 'point accessing'!

add: aPoint
        "Add this point to the end of the path."
        firstPoint isNil ifTrue: [firstPoint _ aPoint. ^ aPoint].
        secondPoint isNil ifTrue: [secondPoint _ aPoint. ^ aPoint].
        thirdPoint isNil ifTrue: [thirdPoint _ aPoint. ^ aPoint].
        self indexOutOfRangeError.!

firstPoint
        "Answer the first point of the receiver."
        ^ firstPoint!

firstPoint: aPoint 
        "Set the first point of the receiver.  Answer the argument, aPoint."
        firstPoint _ aPoint. ^ aPoint!

pointAt: index
        "Return the point at the index given."
        index = 1 ifTrue: [^ firstPoint].
        index = 2 ifTrue: [^ secondPoint].
        index = 3 ifTrue: [^ thirdPoint].
        self indexOutOfRangeError.!

pointAt: index put: aPoint
        "Set the point at the index given."
        index = 1 ifTrue: [firstPoint _ aPoint. ^ aPoint].
        index = 2 ifTrue: [secondPoint _ aPoint. ^ aPoint].
        index = 3 ifTrue: [thirdPoint _ aPoint. ^ aPoint].
        self indexOutOfRangeError.!

points
        "Return the collection of points for this Path."
        ^ Array with: firstPoint with: secondPoint with: thirdPoint!

points: aCollection
        "Set the collection of points for this Path."
        firstPoint _ nil.
        secondPoint _ nil.
        thirdPoint _ nil.
        aCollection do: [ :aPoint | self add: aPoint].!

secondPoint
        "Answer the second point of the receiver."
        ^ secondPoint!

secondPoint: aPoint 
        "Set the second point of the receiver."
        secondPoint _ aPoint. ^ aPoint!

thirdPoint
        "Answer the third point of the receiver."
        ^ thirdPoint!

thirdPoint: aPoint 
        "Set the third point of the receiver."
        thirdPoint _ aPoint. ^ aPoint! !

!Curve methodsFor: 'transforming'!

scaleBy: aPoint 
        "Answer with a new Path scaled by aPoint.  Does not effect the current data
 in this Path."
        | aCurve |

        aCurve _ self species new.
        aCurve form: self form.
        aCurve firstPoint: firstPoint * aPoint.
        aCurve secondPoint: secondPoint * aPoint.
        aCurve thirdPoint: thirdPoint * aPoint.
        ^ aCurve!

translateBy: aPoint 
        "Answer a new instance of Path whose elements are translated by aPoint.  Does
 not effect the elements of this Path."
        | aCurve |

        aCurve _ self species new.
        aCurve form: self form.
        aCurve firstPoint: firstPoint + aPoint.
        aCurve secondPoint: secondPoint + aPoint.
        aCurve thirdPoint: thirdPoint + aPoint.
        ^ aCurve! !

!Curve methodsFor: 'displaying'!

displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger mask:
 aForm
        "Display the receiver as a series of interpolating lines to make a curve.  Also,
 see comment in DisplayObject|displayOn:at:clippingBox:rule:mask:."
        | aLinearFit pa pb numberSegments |

        "Create a LinearFit for displaying the Curve."
        aLinearFit _ LinearFit new.
        aLinearFit form: self form.

        "Compute the number of line segments to use to approximate the curve."
        pa _ secondPoint - firstPoint.
        pb _ thirdPoint - secondPoint.
        numberSegments _ 5 max: pa x abs + pa y abs + pb x abs + pb y abs // 20.

        "Add all of the points necessary for the path."
        aLinearFit add: firstPoint.
        1 to: numberSegments do: [ :seg | 
                aLinearFit add:
                        ((pa * seg // numberSegments + firstPoint) * (numberSegments - seg)
                        + ((pb * (seg - 1) // numberSegments + secondPoint) * (seg - 1)))
                        // (numberSegments - 1)].
        aLinearFit add: thirdPoint.

        "Display the LinearFit which is an approximation of this curve."
        aLinearFit
                displayOn: aDisplayMedium
                at: aPoint
                clippingBox: clipRect
                rule: anInteger
                mask: aForm.! !

!Curve methodsFor: 'display box access'!

computeBoundingBox
        "Compute the smallest Rectangle that covers all of the points in the Path."

        | minPt maxPt |

        minPt _ (firstPoint min: secondPoint) min: thirdPoint.
        maxPt _ (firstPoint max: secondPoint) max: thirdPoint.
        ^ minPt corner: maxPt! !

!Curve methodsFor: 'private'!

indexOutOfRangeError
        self error: 'Curves only have 3 points.  You tried to access more.'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
