"
 COPYRIGHT (c) 1992 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DisplayMedium subclass:#DeviceDrawable
       instanceVariableNames:'device deviceId drawableId gcId
                              realized'
       classVariableNames:''
       poolDictionaries:''
       category:'Graphics-Display Objects'
!

DeviceDrawable comment:'

COPYRIGHT (c) 1992 by Claus Gittinger
              All Rights Reserved

$Header: /claus/smalltalk/st2.6/libview/RCS/DevDraw.st,v 1.7 94/01/09 21:51:31 claus Exp $

totally rewritten (from XDrawable) summer 92 by claus
'!

!DeviceDrawable class methodsFor:'documentation'!

documentation
"
    I represent any drawable on a device (i.e. Bitmaps, Pixmaps, RootWindow and Windows in Xs world).
    My instance variables are mainly caching device-related stuff (such as font- and color-Ids)
    to avoid needless message traffic. This class is abstract, no direct instances of it
    exist in the system.
    All real work is done by my device, most drawing requests are simply forwarded to it.

    Instance variables:

    device                  <Device>        the device this drawable is on
    deviceId                <SmallInteger>  cached (device id)
    drawableId              <SmallInteger>  my drawableId on the device
    gcId                    <SmallInteger>  my gcs ID on the device
    realized                <Boolean>       true if visible (i.e. mapped)
                                            - for bit/pixmaps this is always true
"
! !

!DeviceDrawable class methodsFor:'instance creation'!

new
    "notice: you should not create drawables with new, since this makes
     multiscreen applications hard to port (due to the defaulted display).
     eventually this will trigger an error - so be prepared"

"
    'Warning: DeviceDrawables (' print. self name print. ') should not be created with new' printNewline.
"
    ^ self on:Display
!

on:aDevice
    "create a new drawable on aDevice"

    |newDrawable|

    newDrawable := self basicNew.

    "set display before its initialized - so it can do something useful there"
    newDrawable device:aDevice.
    newDrawable initialize.
    ^ newDrawable
! !

!DeviceDrawable methodsFor:'initialization'!

initialize
    "setup everything for later use; actual work is done in
     initColors and initFont, which are usually redefined."

    "make shure Workstation is initialized - just a check - will vanish soon"
    Display isNil ifTrue:[
        'DEVDRAW: Display not initialized when first DeviceDrawable created' errorPrintNewline.
        Workstation initialize
    ].

    super initialize.
    self initColors.

    realized := false
!

initColors
!

initGC
    "since we do not need a gc-object for the drawable until something is
     really drawn, none is created.
     This method is sent, when the first drawing happens"

    |fgId bgId p id|

    gcId := device gcFor:drawableId.

    foreground isNil ifTrue:[
        foreground := Black
    ].
    background isNil ifTrue:[
        background := White
    ].

    "get device colors from the device indep. colors"
    foreground := foreground on:device.
    background := background on:device.

    fgId := foreground colorId.
    bgId := background colorId.

    "mhmh the following is a kludge ...."
    fgId isNil ifTrue:[
        (foreground greyIntensity >= 50) ifTrue:[
            fgId := device whitepixel
        ] ifFalse:[
            fgId := device blackpixel
        ]
    ].
    bgId isNil ifTrue:[
        (background greyIntensity >= 50) ifTrue:[
            bgId := device whitepixel
        ] ifFalse:[
            bgId := device blackpixel
        ]
    ].

    "now, this is something the device can work with ..."
    device setForeground:fgId background:bgId in:gcId.

    "switch to paint"
    p := paint.
    paint := nil.
    self paint:p.

    ((lineWidth ~~ 0) 
    or:[(lineStyle ~~ #solid) 
    or:[(capStyle ~~ #butt)
    or:[joinStyle ~~ #miter]]]) ifTrue:[
        device setLineWidth:lineWidth 
                      style:lineStyle
                        cap:capStyle
                       join:joinStyle
                         in:gcId
    ].

    mask notNil ifTrue:[
        (mask depth == 1) ifTrue:[
            device setBitmapMask:(mask id) in:gcId
        ] ifFalse:[
            device setPixmapMask:(mask id) in:gcId
        ].
        maskOrigin notNil ifTrue:[
            device setMaskOriginX:maskOrigin x y:maskOrigin y in:gcId
        ]
    ].
    (function ~~ #copy) ifTrue:[device setFunction:function in:gcId].

    "get device font - now done when first draw string occurs"

"
    font := font on:device.
"
"
    id := font fontId.
    id notNil ifTrue:[
        device setFont:id in:gcId
    ]
"
!

destroy
    "when the drawable is destroyed, the associated GC must be destroyed with it"

    gcId notNil ifTrue:[
        device destroyGC:gcId.
        gcId := nil
    ]
!

recreate
    "sent after a snapin, reinit draw stuff for new device"

    gcId := nil.
    foreground notNil ifTrue:[
        foreground := foreground on:device
    ].
    background notNil ifTrue:[
        background := background on:device
    ].
    paint notNil ifTrue:[
        paint := paint on:device
    ].
    bgPaint notNil ifTrue:[
        bgPaint := bgPaint on:device
    ]
!

prepareForReinit
    "kludge - clear drawableId and gcId 
     needed after snapin"

    gcId := nil.
    drawableId := nil
!

reinitialize
    'reinit of ' errorPrint. self classNameWithArticle errorPrint.
    ' failed' errorPrintNewline
! !

!DeviceDrawable methodsFor:'accessing'!

device
    "return the device, the receiver is associated with"

    ^ device
!

device:aDevice
    "set the device"

    deviceId := aDevice id.
    device := aDevice
!

id
    "return the id of the dravable on the device"

    ^ drawableId
!

gcId
    "return the receivers graphic context id on the device"

    ^ gcId
!

paint:aColor
    "set the drawing painting color, which may be a real color or
     a dithered one"

    |dither map pixelId p|

    (aColor ~~ paint) ifTrue:[
        aColor notNil ifTrue:[
            paint := aColor.
            gcId notNil ifTrue:[
                paint := paint on:device.
                pixelId := paint colorId.
                pixelId notNil ifTrue:[
                    "a real (undithered) color"
                    mask notNil ifTrue:[
                        mask := nil.
                        device setBitmapMask:nil in:gcId
                    ]. 
                    (paint ~~ foreground) ifTrue:[
                        foreground := paint.
                        device setForeground:pixelId in:gcId
                    ].
                    ^ self
                ].
                "a dithered color"
                dither := paint ditherForm.
                (dither depth == 1) ifTrue:[
                    "a simple 0/1 bitmap"
                    map := dither colorMap.
                    "temporary (kludgy) fix for destroyed paint"
                    p := paint.
                    self foreground:(map at:2) background:(map at:1).
                    paint := p
                ] ifFalse:[
                    (dither depth ~~ device depth) ifTrue:[
                        self error:'bad dither depth'.
                        ^ self
                    ]
                ].
                self mask:dither
            ]
        ]
    ]
!
    
paint:fgColor on:bgColor
    |fgId bgId|

    ((fgColor ~~ paint) or:[bgColor ~~ bgPaint]) ifTrue:[
        fgColor notNil ifTrue:[
            paint := fgColor
        ].
        bgColor notNil ifTrue:[
            bgPaint := bgColor
        ].
        gcId notNil ifTrue:[
            paint := paint on:device.
            bgPaint := bgPaint on:device.
            fgId := paint colorId.
            fgId notNil ifTrue:[
                mask notNil ifTrue:[
                    mask := nil.
                    device setBitmapMask:nil in:gcId
                ]. 
                bgId := bgPaint colorId.
                bgId notNil ifTrue:[
                    "the common case, both are real colors"
                    (paint ~~ foreground) ifTrue:[
                        foreground := paint.
                        (bgPaint ~~ background) ifTrue:[
                            background := bgPaint.
                            device setForeground:fgId background:bgId in:gcId.
                            ^ self
                        ].
                        device setForeground:fgId in:gcId.
                        ^ self
                    ].
                    (bgPaint ~~ background) ifTrue:[
                        background := bgPaint.
                        device setBackground:bgId in:gcId.
                    ].
                    ^ self
                ].
                "bgPaint is dithered, setup paint here, leave bgPaint
                 till next opaque draw comes around."

                (paint ~~ foreground) ifTrue:[
                    foreground := paint.
                    device setForeground:fgId in:gcId
                ].
                ^ self

            ].
            "either paint or bgPaint (or both) are dithered colors,
             setup for paint, leave bg-problem till next opaque draw
             comes around."
            self paint:paint
        ]
    ]
!

backgroundPaint:aColor
    (aColor ~~ bgPaint) ifTrue:[
        aColor notNil ifTrue:[
            bgPaint := aColor.
            gcId notNil ifTrue:[
                bgPaint := aColor on:device.
                bgPaint colorId notNil ifTrue:[
                    background := bgPaint.
                    device setBackground:(bgPaint colorId) in:drawableId.
                    ^ self
                ].
                self paint:paint on:aColor
            ]
        ]
    ].
!

foreground:aColor
    "set the foreground color for drawing - aColor must be a real color"

    |fgId|

    (aColor ~~ foreground) ifTrue:[
        aColor notNil ifTrue:[
            foreground := aColor.
            gcId notNil ifTrue:[
                (foreground isMemberOf:SmallInteger) ifTrue:[
                    fgId := foreground
                ] ifFalse:[
                    foreground := foreground on:device.
                    fgId := foreground colorId.
                ].

                "mhmh the following is a kludge ...."
                fgId isNil ifTrue:[
                    (foreground greyIntensity >= 50) ifTrue:[
                        fgId := device whitepixel
                    ] ifFalse:[
                        fgId := device blackpixel
                    ]
                ].
                device setForeground:fgId in:gcId.
                paint := foreground
            ]
        ]
    ]
!

background:aColor
    "set the background color for drawing - aColor must be a real color"

    |bgId|

    (aColor ~~ background) ifTrue:[
        aColor notNil ifTrue:[
            background := aColor.
            gcId notNil ifTrue:[
                background := background on:device.
                bgId := background colorId.

                "mhmh the following is a kludge ...."
                bgId isNil ifTrue:[
                    (background greyIntensity >= 50) ifTrue:[
                        bgId := device whitepixel
                    ] ifFalse:[
                        bgId := device blackpixel
                    ]
                ].
                device setBackground:bgId in:gcId
            ]
        ]
    ]
!

foreground:fgColor background:bgColor
    "set both foreground and background colors - these must be real colors"

    |fgPixel bgPixel|

    ((fgColor ~~ foreground) or:[bgColor ~~ background]) ifTrue:[
        fgColor notNil ifTrue:[
            foreground := fgColor
        ].
        bgColor notNil ifTrue:[
            background := bgColor
        ].
        gcId notNil ifTrue:[
            foreground := foreground on:device.
            background := background on:device.
            fgPixel := foreground colorId.
            bgPixel := background colorId.

            "mhmh the following is a kludge ...."
            fgPixel isNil ifTrue:[
                (foreground greyIntensity >= 50) ifTrue:[
                    fgPixel := device whitepixel
                ] ifFalse:[
                    fgPixel := device blackpixel
                ]
            ].
            bgPixel isNil ifTrue:[
                (background greyIntensity >= 50) ifTrue:[
                    bgPixel := device whitepixel
                ] ifFalse:[
                    bgPixel := device blackpixel
                ]
            ].
            device setForeground:fgPixel background:bgPixel in:gcId.
            paint := foreground
        ]
    ]
!

function:aSymbol
    "set the drawing function if it has changed"

    (aSymbol ~~ function) ifTrue:[
        function := aSymbol.
        gcId notNil ifTrue:[
            device setFunction:aSymbol in:gcId
        ]
    ]
!

foreground:aColor function:fun
    "set the foreground color and function for drawing"

    |fgPixel|

    ((aColor ~~ foreground) or:[fun ~~ function]) ifTrue:[
        foreground := aColor.
        function := fun.
        gcId notNil ifTrue:[
            foreground := foreground on:device.
            fgPixel := foreground colorId.

            "mhmh the following is a kludge ...."
            fgPixel isNil ifTrue:[
                (foreground greyIntensity >= 50) ifTrue:[
                    fgPixel := device whitepixel
                ] ifFalse:[
                    fgPixel := device blackpixel
                ]
            ].
            device setForeground:fgPixel in:gcId.
            device setFunction:fun in:gcId.
            paint := foreground
        ]
    ]
!

foreground:fgColor background:bgColor function:fun
    "set foreground, background colors and function"

    self foreground:fgColor background:bgColor.
    self function:fun
!

lineWidth:aNumber
    "set the line width for drawing if it has changed"

    (aNumber ~~ lineWidth) ifTrue:[
        lineWidth := aNumber.
        gcId notNil ifTrue:[
            device setLineWidth:aNumber 
                          style:lineStyle
                            cap:capStyle
                           join:joinStyle
                             in:gcId
        ]
    ]
!

lineStyle:aSymbol
    "set the style in which lines are drawn -
     aSymbol may be #solid, #dashed, #doubleDashed"

    (aSymbol ~~ lineStyle) ifTrue:[
        lineStyle := aSymbol.
        gcId notNil ifTrue:[
            device setLineWidth:lineWidth 
                          style:aSymbol
                            cap:capStyle
                           join:joinStyle
                             in:gcId
        ]
    ]
!

capStyle:aSymbol
    "set the style in which the endpoints of lines
     are drawn - aSymbol may be #notLast, #butt, #round, #projecting"

    (aSymbol ~~ capStyle) ifTrue:[
        capStyle := aSymbol.
        gcId notNil ifTrue:[
            device setLineWidth:lineWidth 
                          style:lineStyle
                            cap:aSymbol
                           join:joinStyle
                             in:gcId
        ]
    ]
!

joinStyle:aSymbol
    "set the style in which 2 lines are connected in polygons -
     aSymbol may be #miter, #bevel, #round"

    (aSymbol ~~ joinStyle) ifTrue:[
        joinStyle := aSymbol.
        gcId notNil ifTrue:[
            device setLineWidth:lineWidth 
                          style:lineStyle
                            cap:capStyle
                           join:aSymbol
                             in:gcId
        ]
    ]
!

font:aFont
    "set the font for drawing if it has changed"

    |id|

    (aFont ~~ font) ifTrue:[
        aFont notNil ifTrue:[
            font := aFont.
            gcId notNil ifTrue:[
                font := font on:device.
                id := font fontId.
                id notNil ifTrue:[
                    device setFont:id in:gcId
                ]
            ]
        ]
    ]
!

mask:aForm
    "set the mask form for drawing"

    (aForm ~~ mask) ifTrue:[
        mask := aForm.
        gcId notNil ifTrue:[
            (mask == nil) ifTrue:[
                device setBitmapMask:nil in:gcId
            ] ifFalse:[
                (mask depth == 1) ifTrue:[
                    device setBitmapMask:(aForm id) in:gcId
                ] ifFalse:[
                    device setPixmapMask:(aForm id) in:gcId
                ]
            ]
        ]
    ]
!

setMaskOrigin:aPoint
    "set the origin of the pattern"

    (maskOrigin isNil or:[maskOrigin ~= aPoint]) ifTrue:[
        maskOrigin := aPoint.
        gcId notNil ifTrue:[
            device setMaskOriginX:aPoint x y:aPoint y in:gcId
        ]
    ]
!

setMaskOriginX:orgX y:orgY
    "set the origin of the pattern"

    (maskOrigin isNil or:[
     ((orgX ~~ maskOrigin x) or:[orgY ~~ maskOrigin y]) ]) ifTrue:[
        maskOrigin := orgX  @ orgY.
        gcId notNil ifTrue:[
            device setMaskOriginX:orgX y:orgY in:gcId
        ]
    ]
!

clipX:x y:y width:w height:h
    "set the clipping rectangle for drawing"

    clipRect notNil ifTrue:[ 
        (clipRect left == x) ifTrue:[
            (clipRect top == y) ifTrue:[
                (clipRect width == w) ifTrue:[
                    (clipRect height == h) ifTrue:[
                        ^ self
                    ]
                ]
            ]
        ]
    ].
    clipRect := Rectangle left:x top:y width:w height:h.
    gcId notNil ifTrue:[
        device setClipX:x y:y width:w height:h in:gcId
    ]
!

clipRect:aRectangle
    "set the clipping rectangle for drawing;
     a nil argument turn off clipping (i.e. whole view is drawable)"

    aRectangle isNil ifTrue:[
        clipRect isNil ifTrue:[^ self].
        gcId notNil ifTrue:[
            device noClipIn:gcId
        ]
    ] ifFalse:[
        clipRect notNil ifTrue:[
            (clipRect = aRectangle) ifTrue:[^ self]
        ].
        gcId notNil ifTrue:[
            device setClipX:aRectangle left
                           y:aRectangle top
                       width:aRectangle width
                      height:aRectangle height
                          in:gcId
        ]
    ].
    clipRect := aRectangle
!

clipRect
    "return the clipping rectangle for drawing"

    clipRect isNil ifTrue:[^ 0@0 extent:width@height].
    ^ clipRect
!

setGraphicsExposures:aBoolean
    "want to if aBoolean is true - or dont want to be notified
     of graphics exposures"

    gcId notNil ifTrue:[
        device setGraphicsExposures:aBoolean in:gcId
    ]
    
!

at:aPoint
    "return the pixel at the coordinate given by aPoint"

    ^ device
        getPixelX:(aPoint x)
                y:(aPoint y)
             from:drawableId
!

at:aPoint put:aPixel
    "set a pixel"

    |oldFg|

    oldFg := foreground.
    self foreground:aPixel.
    self displayPointX:(aPoint x) y:(aPoint y).
    self foreground:oldFg
! !

!DeviceDrawable methodsFor:'queries'!

horizontalPixelPerMillimeter:millis
    "return the number of pixels (not reounded) per millimeter"

    ^ device horizontalPixelPerMillimeter * millis
!

verticalPixelPerMillimeter:millis
    "return the number of pixels (not reounded) per millimeter"

    ^ device verticalPixelPerMillimeter * millis
!

horizontalPixelPerMillimeter
    "return the (unrounded) number of pixels per millimeter"

    ^ device horizontalPixelPerMillimeter
!

verticalPixelPerMillimeter
    "return the (unrounded) number of pixels per millimeter"

    ^ device verticalPixelPerMillimeter
!

horizontalIntegerPixelPerMillimeter
    "return the (rounded) number of pixels per millimeter"

    ^ device horizontalPixelPerMillimeter rounded
!

verticalIntegerPixelPerMillimeter
    "return the (rounded) number of pixels per millimeter"

    ^ device verticalPixelPerMillimeter rounded
!

horizontalPixelPerInch
    "return the number of horizontal pixels per inch of the display"

    ^ device horizontalPixelPerMillimeter * 25.4
!

verticalPixelPerInch
    "return the number of vertical pixels per inch of the display"

    ^ device verticalPixelPerMillimeter * 25.4
! !

!DeviceDrawable methodsFor:'evaluating in another context'!

withForeground:fgColor do:aBlock
    "evaluate aBlock with changed foreground.
     This can be reimplemented here in a faster way."

    |oldFg|

    oldFg := foreground.
    self foreground:fgColor.
    aBlock value.
    self foreground:oldFg
!

withForeground:fgColor background:bgColor do:aBlock
    "evaluate aBlock with changed foreground and background.
     This can be reimplemented here in a faster way."

    |oldFg oldBg|

    oldFg := foreground.
    oldBg := background.
    self foreground:fgColor background:bgColor.
    aBlock value.
    self foreground:oldFg background:oldBg
!

reverseDo:aBlock
    "evaluate aBlock with foreground and background interchanged.
     This can be reimplemented here in a faster way."

    |oldFg oldBg|

    oldFg := foreground on:device.
    oldBg := background on:device.
    self foreground:background background:foreground.
    aBlock value.
    self foreground:oldFg background:oldBg
!

xoring:aBlock
    "evaluate aBlock with function xoring"

    |fgPixel bgPixel oldFunction|

    fgPixel := (Black on:device) colorId.
    bgPixel := (White on:device) colorId.

    gcId isNil ifTrue:[
        self initGC
    ].
    oldFunction := function.
    device setForeground:(fgPixel bitXor:bgPixel)
              background:bgPixel
                      in:gcId.
    device setFunction:#xor in:gcId.
    aBlock value.
    paint := nil.        "invalidate"
    foreground := Black.   
    background := White.
    device setForeground:fgPixel
              background:bgPixel
                      in:gcId.
    device setFunction:oldFunction in:gcId.
    function := oldFunction
! !

!DeviceDrawable methodsFor:'drawing'!

displayString:aString x:x y:y
    "draw a string at the coordinate x/y - draw foreground only"

    |id|

    gcId isNil ifTrue:[
        self initGC
    ].

    font device ~~ device ifTrue:[
        font := font on:device.
    ].
    id := font fontId.
    id isNil ifTrue:[
        "hook for alien fonts"
        font displayString:aString x:x y:y in:self
    ] ifFalse:[
        device setFont:id in:gcId.
        device displayString:aString x:x y:y in:drawableId with:gcId
    ]
!

displayString:aString from:index1 to:index2 x:x y:y
    "draw a substring at the coordinate x/y - draw foreground only"

    |id|

    gcId isNil ifTrue:[
        self initGC
    ].

    font device ~~ device ifTrue:[
        font := font on:device.
    ].
    id := font fontId.
    id isNil ifTrue:[
        "hook for alien fonts"
        font displayString:aString from:index1 to:index2 x:x y:y in:self
    ] ifFalse:[
        device setFont:id in:gcId.
        device displayString:aString from:index1 to:index2
                           x:x y:y in:drawableId with:gcId
    ]
!

displayOpaqueString:aString x:x y:y
    "draw a string at the coordinate x/y - draw foreground and background.
     Assuming that device can only draw in device colors, we have to handle
     the case where paint and/or bgPaint are dithered colors."

    |easy w h savedPaint id|

    gcId isNil ifTrue:[
        self initGC
    ].

    font device ~~ device ifTrue:[
        font := font on:device.
    ].
    id := font fontId.
    id notNil ifTrue:[
        device setFont:id in:gcId.

        "if bgPaint or paint is not a real Color, we have to do it the hard way ..."

        bgPaint notNil ifTrue:[
            easy := true.
            bgPaint := bgPaint on:device.
            bgPaint colorId isNil ifTrue:[
                easy := false
            ] ifFalse:[
                paint colorId isNil ifTrue:[
                    easy := false
                ]
            ].
            easy ifTrue:[
                self foreground:paint background:bgPaint.
                device
                    displayOpaqueString:aString x:x y:y
                                     in:drawableId with:gcId.
                ^ self
            ].

            "hard case: fill first, then draw foreground"
            w := font widthOf:aString.
            h := font height.

            "first fill using bgPaint"
            savedPaint := paint.
            self paint:bgPaint.
            self fillRectangleX:x y:(y - font ascent) width:w height:h.
            self paint:savedPaint.

            "then draw using fgPaint"
            device displayString:aString x:x y:y
                              in:drawableId with:gcId.
        ] ifFalse:[
            device
                displayOpaqueString:aString x:x y:y
                                 in:drawableId with:gcId
        ]
    ] ifFalse:[
        "hook for alien fonts"
        font displayOpaqueString:aString x:x y:y in:self
    ]
!

displayOpaqueString:aString from:index1 to:index2 x:x y:y
    "draw a substring at the coordinate x/y - draw foreground and background.
     Assuming that device can only draw in device colors, we have to handle
     the case where paint and/or bgPaint are dithered colors"

    |easy w h savedPaint id|

    gcId isNil ifTrue:[
        self initGC
    ].

    font device ~~ device ifTrue:[
        font := font on:device.
    ].
    id := font fontId.
    id notNil ifTrue:[
        device setFont:id in:gcId.

        "if bgPaint or paint is not a real Color, we have to do it the hard way ..."
    
        bgPaint notNil ifTrue:[
            easy := true.
            bgPaint := bgPaint on:device.
            bgPaint ditherForm notNil ifTrue:[
                easy := false
            ] ifFalse:[
                paint ditherForm notNil ifTrue:[
                    easy := false
                ]
            ].
            easy ifTrue:[
                self foreground:paint background:bgPaint.
                device displayOpaqueString:aString from:index1 to:index2
                                         x:x y:y in:drawableId with:gcId
            ] ifFalse:[
                "fill first, "
                w := font widthOf:aString from:index1 to:index2.
                h := font height.
                savedPaint := paint.
                self paint:bgPaint.
                self fillRectangleX:x y:(y - font ascent) width:w height:h.
                self paint:savedPaint.
    
                "then draw using fgPaint"
                id isNil ifTrue:[
                    "hook for alien fonts"
                    font displayString:aString from:index1 to:index2
                                     x:x y:y in:self
                ] ifFalse:[
                    device displayString:aString from:index1 to:index2
                                       x:x y:y in:drawableId with:gcId.
                ]
            ]
        ] ifFalse:[
            device displayOpaqueString:aString from:index1 to:index2
                                     x:x y:y in:drawableId with:gcId
        ]
    ] ifFalse:[
        "hook for alien fonts"
        font displayOpaqueString:aString from:index1 to:index2 x:x y:y in:self
    ]
!

displayPointX:x y:y
    "draw a point"

    gcId isNil ifTrue:[
        self initGC
    ].
    device displayPointX:x y:y in:drawableId with:gcId
!

displayLineFromX:x0 y:y0 toX:x1 y:y1
    "draw a line"

    gcId isNil ifTrue:[
        self initGC
    ].
    device displayLineFromX:x0 y:y0 toX:x1 y:y1 in:drawableId with:gcId
!

displayRectangleX:x y:y width:w height:h
    "draw a rectangle"

    "I asked myself many times if we should draw w/h or (w-1)/(h-1) bits -
     this one seems mathematically incorrect but allows to draw and fill
     a rectangle using the same extents.
     I'm not certain if is the right thing to do ..."

    gcId isNil ifTrue:[
        self initGC
    ].
    device displayRectangleX:x y:y width:(w - 1) height:(h - 1) in:drawableId with:gcId
!

displayPolygon:aPolygon
    "draw a polygon"

    gcId isNil ifTrue:[
        self initGC
    ].
    device displayPolygon:aPolygon in:drawableId with:gcId
!

copyFrom:aDrawable x:srcX y:srcY toX:dstX y:dstY width:w height:h
    "copy from aDrawable into the receiver;
     the source may be the receiver as well - in this case its a scroll"

    |id|

    id := aDrawable id.

    "temporary ...
     this fixes a problem after restart on another display,
     when a file-bitmap wasnt found. In this case, the id of the
     bitmap will be nil. This will be fixed soon."

    id isNil ifTrue:[
        'DEVDRAW: invalid bitmap copy - ignored' errorPrintNewline.
        ^ self
    ].

    gcId isNil ifTrue:[
        self initGC
    ].

    "this is a workaround for a bug (or a feature ?) in many X-servers;
     copy is only copy if fg==1 and bg==0"

    ((aDrawable == self) and:[function == #copy]) ifTrue:[
        device setForeground:1 background:0 in:gcId.
        background := nil.
        foreground := nil.
        paint := nil
    ].

    (aDrawable depth == 1) ifTrue:[
        device
            copyPlaneFromId:id
                     x:srcX y:srcY
                    to:drawableId
                     x:dstX y:dstY
                 width:w
                height:h
                  with:gcId
    ] ifFalse:[
        device
            copyFromId:id
                     x:srcX y:srcY
                    to:drawableId
                     x:dstX y:dstY
                 width:w
                height:h
                  with:gcId
    ]
!

displayOpaqueForm:aForm x:x y:y
    "draw a form"

    |id|

    id := aForm id.

    "temporary ..."
    id isNil ifTrue:[
        'DEVDRAW: invalid form draw - ignored' errorPrintNewline.
        ^ self
    ].
    gcId isNil ifTrue:[
        self initGC
    ].
    (aForm depth == 1) ifTrue:[
        device
            copyPlaneFromId:id
                          x:0 y:0
                         to:drawableId
                          x:x y:y
                      width:(aForm width)
                     height:(aForm height)
                       with:gcId
    ] ifFalse:[
        device
            copyFromId:id
                     x:0 y:0
                    to:drawableId
                     x:x y:y
                 width:(aForm width)
                height:(aForm height)
                  with:gcId
    ]
!

displayArcX:x y:y w:width h:height from:startAngle angle:angle
    "draw an arc"

    gcId isNil ifTrue:[
        self initGC
    ].
    device
          displayArcX:x y:y
                    w:width h:height
                 from:startAngle angle:angle
                   in:drawableId with:gcId
!

fillArcX:x y:y w:width h:height from:startAngle angle:angle
    "draw a filled arc"

    gcId isNil ifTrue:[
        self initGC
    ].
    device
          fillArcX:x
                 y:y
                 w:width
                 h:height
              from:startAngle
             angle:angle
                in:drawableId
              with:gcId
!

fillRectangleX:x y:y width:w height:h
    "draw a filled rectangle"

    gcId isNil ifTrue:[
        self initGC
    ].
    device
        fillRectangleX:x y:y width:w height:h
                    in:drawableId with:gcId
!

fillPolygon:aPolygon
    "draw a filled polygon"

    gcId isNil ifTrue:[
        self initGC
    ].
    device
        fillPolygon:aPolygon
                 in:drawableId
               with:gcId
! !
