"
 COPYRIGHT (c) 1991 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DisplayObject subclass:#DirectoryViewObject
         instanceVariableNames:'name font origin activeAction passiveAction icon mask pathName'
         classVariableNames:'All0Color All1Color'
         poolDictionaries:''
         category:'Interface-DirectoryView'
!

!DirectoryViewObject class methodsFor:'initialization'!

initialize
    All0Color := Color colorId:0.
    All1Color := Color colorId:-1

    "DirectoryViewObject initialize"
! !

!DirectoryViewObject class methodsFor:'specification'!

dragOutline
    "if true, dragging is done by drawing outline only;
     if false, dragging is done by full draw (fast servers only)
     - can be redefined in subclasses"
    ^ true
!

iconFileBaseName
    ^ self name
!

maskFileName
    ^ self iconFileBaseName , '_m'
!

iconFileName
    ^ self iconFileBaseName 
!

new
    |newObject|
    newObject := super new.
    newObject initialize.
    ^ newObject
! !

!DirectoryViewObject methodsFor:'accessing'!

name
    ^ name
!

name:aString
    name := aString
!

action:aBlock
    activeAction := aBlock
!

font
    ^ font
!

font:aFont
    font := aFont
!

icon
    ^ icon
!

icon:aForm
    icon := aForm
!

mask
    ^ mask
!

mask:aForm
    mask := aForm
!

icon:iconForm mask:maskForm
    icon := iconForm.
    mask := maskForm
!

hasFixedSize
    ^ true
!

origin
    ^ origin
!

origin:aPoint
    origin := aPoint.
    self recomputeFrame
!

moveTo:newOrigin
    origin := newOrigin.
    self recomputeFrame
!

recomputeFrame
    |myWidth myHeight|

    icon isNil ifTrue:[
        myWidth := font widthOf:name.
        myHeight := font height
    ] ifFalse:[
        myWidth := (icon width) max: (font widthOf:name).
        myHeight := icon height + 2 + font height.
    ].
    frame := Rectangle origin:origin
                       extent:(myWidth @ myHeight)
!

pathName:aString
    pathName := aString.
    name := OperatingSystem baseNameOf:aString
!

pathName
    ^ pathName
!

asDirectoryViewObject
    ^ self
! !

!DirectoryViewObject methodsFor:'drawing'!

drawSelectedIn:aView offset:anOffset
    self drawIn:aView offset:anOffset with:Color white
                                       and:Color black
!

drawIn:aView offset:anOffset
    self drawIn:aView offset:anOffset with:Color black
                                       and:Color white
!

drawQuickIn:aView offset:anOffset
    "drawQuick does not mask out - can only be used if objects frame
     does not intersect any other"

    |offs picWidth picHeight textWidth
     topLeftX topLeftY myWidth picture|

    offs := anOffset - aView viewOrigin.

    picture := self icon.
    picWidth := picture width.
    picHeight := picture height.
    textWidth := font widthOf:name.
    myWidth := picWidth max: textWidth.
    topLeftX := origin x + offs x.
    topLeftY := origin y + offs y.

    aView displayOpaqueForm:picture
                       x:(topLeftX + ((myWidth - picWidth) // 2))
                       y:topLeftY.

    "
     text is overpainted
    "
    name notNil ifTrue:[
        (textWidth < myWidth) ifTrue:[
            topLeftX := topLeftX + ((myWidth - textWidth) // 2)
        ].
        aView displayOpaqueString:name 
                             x:topLeftX
                             y:(topLeftY + picHeight + 2 + (font ascent))
   ]
!

drawOutlineIn:aView offset:anOffset
    |offs picWidth picHeight textWidth
     topLeftX topLeftY myWidth picture leftOffs|

    offs := anOffset - aView viewOrigin.

    picture := self icon.
    picture isNil ifTrue:[
        picWidth := 0.
        picHeight := 0
    ] ifFalse:[
        picWidth := picture width.
        picHeight := picture height + 2
    ].
    font := font on:(aView device).
    textWidth := font widthOf:name.

    myWidth := picWidth max:textWidth.
    topLeftX := origin x + offs x.
    topLeftY := origin y + offs y.

    leftOffs := (myWidth - picWidth) // 2.
    icon notNil ifTrue:[
        aView displayRectangle:(Rectangle
                                origin:(topLeftX + leftOffs) @ topLeftY
                                extent:picWidth @ picHeight)
    ].
    name notNil ifTrue:[
        (textWidth < myWidth) ifTrue:[
            topLeftX := topLeftX + ((myWidth - textWidth) // 2)
        ].
"this one draws the string:"
        aView displayString:name 
                          x:topLeftX
                          y:(topLeftY + picHeight + (font ascent))

"or, this one a rectangle around:"
"
        aView displayRectangle:(Rectangle
                               origin:topLeftX @ (topLeftY + 2 + picHeight)
                               extent:textWidth @ textHeight)
"
    ]
!

drawIn:aView offset:anOffset with:fg and:bg
    |offs picWidth picHeight textWidth
     topLeftX topLeftY myWidth picture mask
     fgColor bgColor leftOffs|

    offs := anOffset - aView viewOrigin.

    All0Color := All0Color on:(aView device).
    All1Color := All1Color on:(aView device).

    fgColor := fg on:aView device.
    bgColor := bg on:aView device.

    picture := self icon.
    picture isNil ifTrue:[
        picWidth := 0.
        picHeight := 0
    ] ifFalse:[
        picWidth := picture width.
        picHeight := picture height + 2
    ].
    font := font on:aView device.
    textWidth := font widthOf:name.

    mask := self mask.
    myWidth := picWidth max:textWidth.
    topLeftX := origin x + offs x.
    topLeftY := origin y + offs y.

    leftOffs := (myWidth - picWidth) // 2.
    picture notNil ifTrue:[
        mask notNil ifTrue:[
            "mask out first - then or in picture; if no mask,
             overpaint picture"

            "clear mask-bits to zero"
            aView foreground:All0Color background:All1Color.
            aView function:#and.
            aView displayOpaqueForm:mask
                               x:(topLeftX + leftOffs)
                               y:topLeftY.

            "draw mask-bits with bg-color"
            (bgColor colorId == 0) ifFalse:[
                aView function:#or.
                aView foreground:bgColor background:All0Color.
                aView displayOpaqueForm:mask
                               x:(topLeftX + leftOffs)
                               y:topLeftY
            ].

            "clear picture-bits to zero"
            aView foreground:All0Color background:All1Color.
            aView function:#and.
            aView displayOpaqueForm:picture
                               x:(topLeftX + leftOffs)
                               y:topLeftY.

            "draw picture-bits with fg-color"
            (fgColor colorId == 0) ifFalse:[
                aView function:#or.
                aView foreground:fgColor background:All0Color.
                aView displayOpaqueForm:picture
                               x:(topLeftX + leftOffs)
                               y:topLeftY
            ]
        ] ifFalse:[
            aView foreground:fgColor background:bgColor.
            aView displayOpaqueForm:picture
                               x:(topLeftX + leftOffs)
                               y:topLeftY
        ]
    ].

    aView function:#copy.
    aView paint:fgColor.
    aView foreground:fgColor background:bgColor.
    aView paint:fgColor on:bgColor.

    "
     text is overpainted
    "
    name notNil ifTrue:[
        (textWidth < myWidth) ifTrue:[
            topLeftX := topLeftX + ((myWidth - textWidth) // 2)
        ].
        aView displayOpaqueString:name 
                             x:topLeftX
                             y:(topLeftY + picHeight + font ascent)
    ]
! !

!DirectoryViewObject methodsFor:'user interaction'!

activate
    activeAction notNil ifTrue:[
        activeAction value:pathName
    ]
! !

!DirectoryViewObject methodsFor:'storing'!

storeOn:aStream
    aStream nextPut:$(.
    aStream nextPutAll:(self class name).
    aStream nextPutAll:' new origin:'.
    origin storeOn:aStream.
    aStream nextPutAll:' name:'.
    name storeOn:aStream.
    aStream nextPut:$)
! !

!DirectoryViewObject methodsFor:'queries'!

isHitBy:aPoint withDelta:delta
    "separate checks on picture frame and file-name frame"

    "check overall boundary first"
    (super isHitBy:aPoint withDelta:delta) ifFalse:[^ false].

    "check icons frame"
    (self iconIsHitBy:aPoint withDelta:delta) ifTrue:[^ true].

    "check labels frame"
    (self labelIsHitBy:aPoint withDelta:delta) ifTrue:[^ true].

    ^ false
!

intersects:aRectangle
    "check for icon or text"

    |iw ih ix|

    (frame intersects:aRectangle) ifFalse:[^ false].
    iw := icon width.
    ih := icon height.
    ix := frame origin x + ((frame width - iw) // 2).
    ((Rectangle origin:(ix @ frame origin y)
                extent:(iw @ ih)) intersects:aRectangle) ifTrue:[^ true].

    ^ false
!

iconIsHitBy:aPoint withDelta:delta
    "checks hit on icon"

    |org left right top bott px py d2 iw|

    icon isNil ifTrue:[^ false].

    "check icons frame"
    iw := icon width.
    org := frame origin.
    left := org x - delta + ((frame width - iw) // 2).

    px := aPoint x.
    (px < left) ifTrue:[^ false].

    d2 := delta * 2.
    right := left + iw + d2.
    (px > right) ifTrue:[^ false].

    top := org y - delta.
    py := aPoint y.
    (py < top) ifTrue:[^ false].

    bott := top + icon height + d2.
    (py > bott) ifTrue:[^ false].

    ^ true
!

labelIsHitBy:aPoint withDelta:delta
    "checks hit on label"

    |org top bott py iconHeight|

    "check icons frame"
    org := frame origin.

    icon isNil ifTrue:[
        iconHeight := 0
    ] ifFalse:[
        iconHeight := icon height
    ].

    top := org y + iconHeight + 2 - delta.
    py := aPoint y.
    (py < top) ifTrue:[^ false].

    bott := top + font height + (delta * 2).
    (py > bott) ifTrue:[^ false].

    ^ true
! !

DirectoryViewObject initialize!
