"
 COPYRIGHT (c) 1991 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ObjectView subclass:#DirectoryView
         instanceVariableNames:'currentDirectoryName currentDirectory fileAction dirAction
                systemIconDefs userIconDefs localIconDefs systemActionDefs
                userActionDefs localActionDefs propertyBox settingBox sortBy
                collectExposes outstandingExposes checkBlock
                timeOfLastCheck checkDelta warnBox anyObscured bgAction'
         classVariableNames:'KnownForms'
         poolDictionaries:''
         category:'Interface-DirectoryView'
!

!DirectoryView class methodsFor:'defaults'!

hitDelta
    "when clicking an object, allow for hitDelta pixels around object;
     0 is exact."
    ^ 0
!

defaultExtentOn:aDisplay
    ^ (aDisplay width // 2) @ (aDisplay height // 2)
!

icon
    ^ (Form fromFile:'DirObj.xbm' resolution:100)
! !

!DirectoryView class methodsFor:'startup'!

startOn:pathName
    |topView scrollView dirView|

    topView := StandardSystemView new.
    topView extent:(self defaultExtentOn:Display).
    topView icon:self icon.
    scrollView := ScrollableView for:self in:topView.
    scrollView origin:(0.0 @ 0.0) extent:(1.0 @ 1.0).
    dirView := scrollView scrolledView.
    pathName notNil ifTrue:[
        dirView showContentsOf:pathName
    ].
    topView open.
    ^ dirView

    "DirectoryView startOn:'.'"
    "ArchiveView startOn:'../SRCDISTRIB/upd.tar'"
!

start
    ^ self startOn:nil

    "DirectoryView start"
! !

!DirectoryView class methodsFor:'cleanup'!

releaseRememberedForms
    KnownForms := nil

    "DirectoryView releaseRememberedForms"
! !

!DirectoryView methodsFor:'misc queries'!

heightOfContents
    "answer the height of the document in pixels
     in this case, the max y extent is returned"

    |maxY|

    maxY := 0.
    contents do:[:something |
        (something frame bottom > maxY) ifTrue:[
            maxY := something frame bottom
        ]
    ].
    ^ maxY + 8

!

isObscured:anObject
    "return true, if anObject is obscured by some other"

    anyObscured ifTrue:[
        ^ super isObscured:anObject
    ].
    ^ false
!

rectangleForScroll
    "no need to waste time looking for objects; need full width always"

    ^ Rectangle left:0 top:0 width:width height:height
!

widthOfContents
    "answer the height of the document in pixels
     in this case, the max y extent is returned"

    |maxX|

    maxX := 0.
    contents do:[:something |
        (something frame right > maxX) ifTrue:[
            maxX := something frame right
        ]
    ].
    ^ maxX
! !

!DirectoryView methodsFor:'private'!

XXwaitForExpose
    "a kludge to catch graphicsExpose events all at once:
     do not wait for every copy, but count outstanding events and wait for
     all of them in waitForAllExposes; see reorgContents"

    collectExposes ifTrue:[
        outstandingExposes := outstandingExposes + 1
    ]
!

getFormFromFile:fileName
    "this one looks for already known forms - to not create too many
     forms"

    |index aForm image|

    KnownForms notNil ifTrue:[
        aForm := KnownForms at:fileName ifAbsent:[nil].
        aForm notNil ifTrue:[
            ^ aForm
        ]
    ].
    (fileName endsWith:'.tiff') ifTrue:[
        image := Image fromFile:fileName.
        image notNil ifTrue:[
           aForm := image asFormOn:device
        ]
    ] ifFalse:[
        aForm := Form fromFile:fileName resolution:100
    ].
    aForm notNil ifTrue:[
        KnownForms isNil ifTrue:[
            KnownForms := Dictionary new
        ].
        KnownForms at:fileName put:aForm
    ].
    ^ aForm
!

checkIfDirectoryHasChanged
    "every checkDelta secs, check if directoy has changed and update view if so"

    realized ifTrue:[
        (currentDirectory notNil) ifTrue:[
            (currentDirectory timeOfLastChange > timeOfLastCheck) ifTrue:[
                 self updateContents
            ] ifFalse:[
                 Processor addTimedBlock:checkBlock after:checkDelta
            ]
        ]
    ]
!

maximumObjectBounds
    |maxX maxY w h frame|

    "find the widest width and highest height"
    maxX := 0.
    maxY := 0.
    contents do:[:obj |
        frame := obj frame.
        w := frame width.
        h := frame height.
        maxX := maxX max:w.
        maxY := maxY max:h
    ].
    ^ maxX @ maxY
!

reorgContents
    "reorganize elements to make them fit nicely"

    |frame oldOrigin newOrigin movingUp
     xPos yPos deltaY spaceX spaceY t w h index newOrigins
     max|

    self cursor:Cursor wait.

    anyObscured := false.

    "find the widest width and highest height"
    max := self maximumObjectBounds.

    deltaY := 0.
    spaceX := max x + 8 "font widthOf:'wwwwwwwwwwwwwww'" .
    spaceY := 8.
    xPos := 8. 
    yPos := spaceY.
    movingUp := false.
    newOrigins := OrderedCollection new.

    "to make visible effect better, first get all new origins without any update"
    contents do:[:obj |
        frame := obj frame.
        w := frame width.
        t := (spaceX - w) // 2.
        oldOrigin := obj origin.
        newOrigin := ((xPos + t) @ yPos).
        (newOrigin = oldOrigin) ifTrue:[
            newOrigins add:nil
        ] ifFalse:[
            newOrigins add:newOrigin.
            (newOrigin y < oldOrigin y) ifTrue:[
                movingUp := true
            ]
        ].

        "advance to the right -"
        xPos := xPos + spaceX.
        ((xPos + spaceX) >= width) ifTrue:[
            "right of view reached - advance to next row"
            xPos := 8.
            yPos := yPos + deltaY + spaceY.
            deltaY := 0
        ].
        "keep track of highest object in this row for next row advance"
        deltaY := deltaY max:(frame height)
    ].

    "then update"
    shown ifTrue:[
        "must react on previous exposes first"
        [self exposeEventPending] whileTrue:[
            device dispatchEventFor:drawableId withMask:(device eventMaskFor:#expose)
        ].
        collectExposes := true.
        movingUp ifTrue:[
            index := 1.
            contents do:[:obj |
                newOrigin := newOrigins at:index.
                newOrigin notNil ifTrue:[
                    self moveObject:obj to:newOrigin.
               ].
                index := index + 1
            ]
        ] ifFalse:[
            index := newOrigins size.
            contents reverseDo:[:obj |
                newOrigin := newOrigins at:index.
                newOrigin notNil ifTrue:[
                    self moveObject:obj to:newOrigin
                ].
                index := index - 1
            ]
        ].
    ].
    "sorted := true."
    self contentsChanged.
    self cursor:Cursor normal.
!

updateContents
    "read in currentDirectory - show pictures"

    |files|

    self removeAllWithoutRedraw.
    self label:currentDirectoryName.
    self topView iconName:(currentDirectory baseName).

    currentDirectory isNil ifTrue:[
        currentDirectory := FileDirectory directoryNamed:currentDirectoryName
    ].
    timeOfLastCheck := Time now.
    files := currentDirectory asText asSortedCollection.
    files do:[:aFileName |
        |obj|

        (aFileName = '.') ifFalse:[
            obj := self objectForFile:aFileName.
            obj font:font.
            obj pathName:(currentDirectoryName , '/' , aFileName).
            obj origin:0@0.
            self addWithoutRedraw:obj.
        ]
    ].

    self showObjects.
    Processor addTimedBlock:checkBlock after:checkDelta.
!

directoryChanged
    Processor removeTimedBlock:checkBlock.
    localIconDefs := self readIconDefinitionsFrom:'.DeskTop.defs'.
    localActionDefs := self readActionDefinitionsFrom:'.DeskTop.defs'.
    self topView label:currentDirectoryName.
    self removeAllWithoutRedraw.
    self scrollToTop. 
    self updateContents
!

XXwaitForAllExposes
    [outstandingExposes > 0] whileTrue:[
        super waitForExpose.
        outstandingExposes := outstandingExposes - 1
    ]
!

showAlert:aMessage
    warnBox isNil ifTrue:[
        warnBox := WarningBox new
    ].
    warnBox title:aMessage.
    warnBox showAtPointer
!

accessRightsOfSelectionAsNumber

    selection isNil ifTrue:[^ nil].
    ^ currentDirectory accessModeOf:(selection name)
!

changeAccessRightOfSelection:bitMask to:aBoolean
    |mode|

    selection isNil ifTrue:[^ nil].
    mode := currentDirectory accessModeOf:(selection name).
    aBoolean ifTrue:[
        mode := mode bitOr:bitMask
    ] ifFalse:[
        mode := mode bitAnd:(bitMask bitInvert)
    ].
    ^ currentDirectory changeAccessModeOf:(selection name) to:mode
!

showObjects
    "reorganize and show pictures"

    |files obj frame
     xPos yPos max deltaY spaceX spaceY t w h|

    self cursor:Cursor wait.

    "find the widest width and highest height"
    max := self maximumObjectBounds.

    deltaY := max y.
    spaceX := max x + 8 "font widthOf:'wwwwwwwwwwwwwww'" .
    spaceY := 8.
    xPos := 8. 
    yPos := spaceY.

    contents do:[:obj |
        frame := obj frame.
        w := frame width.
        t := (spaceX - w) // 2.
            obj origin:(xPos + t @ yPos).
            xPos := xPos + spaceX.
            ((xPos + spaceX) >= width) ifTrue:[
                xPos := 8.
                yPos := yPos + deltaY + spaceY.
            ].
    ].
    "sorted := true."
    anyObscured := false.
    self contentsChanged.
    shown ifTrue:[
        self redraw
    ].
    self cursor:Cursor normal
!

objectForFile:aFileName
    |type mode index sz fName iconFileName val
     icon mask actionBlock newObject|

    type := currentDirectory typeOf:aFileName.
    (type == #regular) ifTrue:[
        (currentDirectory isExecutable:aFileName) ifTrue:[
            mode := #executable
        ]
    ].

    "get an icon-filename"
    localIconDefs notNil ifTrue:[
        index := 1.
        sz := localIconDefs size.
        [index <= sz] whileTrue:[
            val := (localIconDefs at:index) value:aFileName
                                            value:type
                                            value:mode.
            (val isKindOf:String) ifTrue:[
                fName := val.
                index := sz  "exit loop"
            ].
            index := index + 1
        ]
    ].
    fName isNil ifTrue:[
        userIconDefs notNil ifTrue:[
            index := 1.
            sz := userIconDefs size.
            [index <= sz] whileTrue:[
                val := (userIconDefs at:index) value:aFileName
                                               value:type
                                               value:mode.
                (val isKindOf:String) ifTrue:[
                    fName := val.
                    index := sz  "exit loop"
                ].
                index := index + 1
            ]
        ]
    ].
    fName isNil ifTrue:[
        systemIconDefs notNil ifTrue:[
            index := 1.
            sz := systemIconDefs size.
            [index <= sz] whileTrue:[
                val := (systemIconDefs at:index) value:aFileName
                                                 value:type
                                                 value:mode.
                (val isKindOf:String) ifTrue:[
                    fName := val.
                    index := sz  "exit loop"
                ].
                index := index + 1
            ]
        ]
    ].
    fName isNil ifTrue:[
        (type == #directory) ifTrue:[
            fName := 'DirObj'
        ] ifFalse:[
            fName := 'FileObj'
        ]
    ].
    iconFileName := fName.
    icon := self getFormFromFile:(iconFileName , '.xbm').
    mask := self getFormFromFile:(fName , '_m.xbm').

    icon isNil ifTrue:[
        (type == #directory) ifTrue:[
            icon := self getFormFromFile:'DirObj.xbm'.
            mask := self getFormFromFile:'DirObj_m.xbm'
        ] ifFalse:[
            icon := self getFormFromFile:'FileObj.xbm'.
            mask := self getFormFromFile:'FileObj_m.xbm'
        ]
    ].
    (type == #directory) ifTrue:[
        newObject := (DirectoryObject new) icon:icon mask:mask
    ] ifFalse:[
        newObject := (FileObject new) icon:icon mask:mask
    ].

    "get actionblock"
    (type == #directory) ifTrue:[
        actionBlock := dirAction
    ] ifFalse:[
        localActionDefs notNil ifTrue:[
            index := 1.
            sz := localActionDefs size.
            [index <= sz] whileTrue:[
                val := (localActionDefs at:index) value:aFileName
                                                  value:type
                                                  value:mode.
                (val isKindOf:BlockNode) ifTrue:[
                    actionBlock := val.
                    index := sz  "exit loop"
                ].
                index := index + 1
            ]
        ].
        actionBlock isNil ifTrue:[
            userActionDefs notNil ifTrue:[
                index := 1.
                sz := userActionDefs size.
                [index <= sz] whileTrue:[
                    val := (userActionDefs at:index) value:aFileName
                                                     value:type
                                                     value:mode.
                    (val isKindOf:BlockNode) ifTrue:[
                        actionBlock := val.
                        index := sz  "exit loop"
                    ].
                    index := index + 1
                ]
            ]
        ].
        actionBlock isNil ifTrue:[
            systemActionDefs notNil ifTrue:[
                index := 1.
                sz := systemActionDefs size.
                [index <= sz] whileTrue:[
                    val := (systemActionDefs at:index) value:aFileName
                                                       value:type
                                                       value:mode.
                    (val isKindOf:BlockNode) ifTrue:[
                        actionBlock := val.
                        index := sz  "exit loop"
                    ].
                    index := index + 1
                ]
            ]
        ].
        "none specified - set default action"
        actionBlock isNil ifTrue:[
            actionBlock := fileAction
        ]
    ].
    newObject action:actionBlock.
    ^ newObject
! !

!DirectoryView methodsFor:'initializing'!

readDefinitionsFrom:aFileName for:aKeyWord
    |definitions skipping aStream chunk value|

    aStream := FileStream readonlyFileNamed:aFileName.
    aStream notNil ifTrue:[
        definitions := OrderedCollection new.
        skipping := true.
        [aStream atEnd] whileFalse:[
            chunk := aStream nextChunk.
            chunk notNil ifTrue:[
                value := Compiler evaluate:chunk.
                skipping ifTrue:[
                    (value = aKeyWord) ifTrue:[
                        skipping := false
                    ]
                ] ifFalse:[
                    (value isKindOf:BlockNode) ifTrue:[
                        definitions add:value
                    ] ifFalse:[
                        skipping := true
                    ]
                ]
            ]
        ].
        aStream close
    ].
    ^ definitions
!

readActionDefinitionsFrom:aFileName
    ^ self readDefinitionsFrom:aFileName for:'actions'
!

readIconDefinitionsFrom:aFileName
    ^ self readDefinitionsFrom:aFileName for:'icons'
!

initialize
    collectExposes := false.
    outstandingExposes := 0.
    checkBlock := [self checkIfDirectoryHasChanged].
    checkDelta := 5.

    super initialize.

    anyObscured := false.
    canDragOutOfView := true.

    self initializeDefinitions.

    self setupActions.

    doublePressAction := [:pressPoint | self doubleClick:pressPoint].

    pressAction := [:aPoint | self startSelectOrMove:aPoint].
    shiftPressAction := [:aPoint | self startSelectMoreOrMove:aPoint].

    self setupContentsSpecific

    "DirectoryView start"
!

initializeDefinitions
    |homeDir|

    systemIconDefs := self readIconDefinitionsFrom:
                                        '/usr/lib/smalltalk/DeskTop.defs'.
    systemActionDefs := self readActionDefinitionsFrom:
                                        '/usr/lib/smalltalk/DeskTop.defs'.
    homeDir := OperatingSystem getHomeDirectory.
    homeDir notNil ifTrue:[
        userIconDefs := self readIconDefinitionsFrom:
                                        (homeDir , '/.DeskTop.defs').
        userActionDefs := self readActionDefinitionsFrom:
                                        (homeDir , '/.DeskTop.defs')
    ]
!

initializeMiddleButtonMenu
    self middleButtonMenu:(PopUpMenu
                                labels:#(
                                         'open'
                                         'show'
                                         'fileIn'
                                         '-'
                                         'remove'
"
                                         'rename'
                                         'properties'
"
                                         '-'
                                         'execute'
                                         'print'
"
                                         '-'
                                         'settings'
"
                                       )
                             selectors:#(open
                                         show
                                         fileIn
                                         nil
                                         remove
"
                                         rename
                                         properties
"
                                         nil
                                         execute
                                         print
"
                                         nil
                                         settings
"
                                        )
                                receiver:self
                                     for:self)
!

setupActions
    fileAction := [:aFileName | self executeFile:aFileName].
    dirAction := [:aFileName | self changeDirectory:aFileName]
!

initializeFullScreen
    |rootView|

    self initialize.
    drawableId notNil ifTrue:[
        'too late to change to rootWindow' printNewline.
        ^ nil
    ].
    rootView := DisplayRootView new.
    rootView isWindowManagerRunning ifTrue:[
        'another window manager is running' printNewline.
        ^ false
    ].
    width := device width.
    height := device height.
    realized := true.
    shown := true.
    drawableId := device rootWindowFor:self.
    self initGC
!

realize
    super realize.
    self privateRealize
!

fullScreen
    "make me running in fullscreen mode"

    |rootView|

    rootView := DisplayRootView new.
    rootView isWindowManagerRunning ifTrue:[^ nil].
    drawableId := rootView id. "a kludge"
    self enableButtonEvents.
    self enableButtonMotionEvents
!

privateRealize
    self directoryChanged
!

setupContentsSpecific 
    currentDirectory := FileDirectory directoryNamed:'.'.
    currentDirectory notNil ifTrue:[
        currentDirectoryName := currentDirectory pathName
    ].
    self topView label:currentDirectoryName.
    self topView iconName:(currentDirectory baseName)
!

showContentsOf:pathName
    self changeDirectory:pathName
! !

!DirectoryView methodsFor:'layout manipulation'!

moveObject:anObject to:newOrigin
    "move anObject to newOrigin, aPoint. Redefined to recompute the anyObscured
     flag."

    |objectsIntersectingFrame frame|

    super moveObject:anObject to:newOrigin.
    anyObscured ifFalse:[
        frame := self frameOf:anObject.
        objectsIntersectingFrame := self objectsIntersecting:frame.
        objectsIntersectingFrame size == 1 ifFalse:[
            anyObscured := true
        ]
    ]
! !

!DirectoryView methodsFor:'user interaction'!

startObjectMove:something at:aPoint
    "redefined since here, all clocked upon objects are brought to front"

    self toFront:something.
    super startObjectMove:something at:aPoint
!

doubleClick:aPoint
    |anObject|

    anObject := self findObjectAtVisible:aPoint.
    (anObject ~~ selection) ifTrue:[self unselect].
    anObject notNil ifTrue:[
        self cursor:Cursor wait.
        anObject activate.
        self cursor:Cursor normal
    ]
!

changeDirectory:pathName
    |newCurrentDirectory|

    newCurrentDirectory := FileDirectory directoryNamed:pathName.
    newCurrentDirectory notNil ifTrue:[
        currentDirectory := newCurrentDirectory.
        currentDirectoryName := currentDirectory pathName.
        self directoryChanged
    ]
!

reorg
    self updateContents
!

executeFile:name
    |nm|

    name notNil ifTrue:[
        ((currentDirectory typeOf:name) == #regular) ifTrue:[
            (currentDirectory isExecutable:name) ifTrue:[
                OperatingSystem executeCommand:('(cd ' , currentDirectoryName , '; ' , name , ') &').
                ^ self
            ]
        ].
        name size > 40 ifTrue:[
            nm := OperatingSystem baseNameOf:name
        ] ifFalse:[
            nm := name
        ].
        self showAlert:(resources string:'cannot execute %1' with:nm)
    ]
!

edit
    "start a document editor on the selected file"

    self cursor:Cursor wait.
    self selectionDo:[:anObject |
        EditTextView startOn:(anObject pathName) 
    ].
    self cursor:Cursor normal
!

settings
    "start a box to change settings"

    |label|

    label := resources string:'settings'.
    settingBox isNil ifTrue:[
        settingBox := DialogBox label:label for:self 
                                names:resources array:
                                      #(
                                        'sort by'
                                       )
                               fields:#(
                                        ( EditField  0 sortBy   sortBy:)
                                       )
    ].
    settingBox showAtPointer
!

properties
    "start a box to view/change file properties"

    |label|

    (selection isNil or:[selection isKindOf:Collection]) ifTrue:[
        self notify:'you must select exactly one object'.
        ^ self
    ].
    label := resources string:'properties'.
    propertyBox isNil ifTrue:[
        propertyBox := DialogBox label:label for:self 
                                 names:(resources array:#(
                                         'access'  
                                         'owner' 
                                         'read'  
                                         'write' 
                                         'execute'  
                                         'group'  
                                         'read' 
                                         'write'  
                                         'execute'  
                                         'others'  
                                         'read'  
                                         'write' 
                                         'execute'  
                                        ))
                                fields:#(
                                         (   nil          0 nil      nil)
                                         (   nil          0 nil      nil)
                                         (   CheckToggle  0 rOwner   rOwner:)
                                         (   CheckToggle  0 wOwner   wOwner:)
                                         (   CheckToggle  0 xOwner   xOwner:)
                                         (   nil          0 nil      nil)
                                         (   CheckToggle  0 rGroup   rGroup:)
                                         (   CheckToggle  0 wGroup   wGroup:)
                                         (   CheckToggle  0 xGroup   xGroup:)
                                         (   nil          0 nil      nil)
                                         (   CheckToggle  0 rOther   rOther:)
                                         (   CheckToggle  0 wOther   wOther:)
                                         (   CheckToggle  0 xOther   xOther:)
                                        )
    ].
    propertyBox showAtPointer
!

remove
    "remove the selected file"

    |name ok anythingRemoved|

    anythingRemoved := false.
    self selectionDo:[:anObject |
        name := anObject name.
        name notNil ifTrue:[
            ok := currentDirectory remove:name.
            ok ifFalse:[
                "was not able to remove it"
                self showAlert:(resources string:'cannot remove %1' with:name)
            ] ifTrue:[
                self removeObject:anObject.
                anythingRemoved := true
            ]
        ]
    ].
    anythingRemoved ifTrue:[
        timeOfLastCheck := Time now.
        self reorgContents "updateContents"
    ]
!

open
    "open selection i.e. start a document editor if its a file,
     start directory view of directory"

    |name type|

    self cursor:Cursor wait.
    self selectionDo:[:anObject |
        name := anObject name.
        type := currentDirectory typeOf:name.
        (type == #regular) ifTrue:[
            anObject activate
        ].
        (type == #directory) ifTrue:[
            DirectoryView startOn:(anObject pathName)
        ]
    ].
    self cursor:Cursor normal
!

show
    "start a document editor on the selected file(s)"

    |fileName|

    self cursor:Cursor wait.
    self selectionDo:[:anObject |
        fileName := anObject pathName.
        ((currentDirectory typeOf:fileName) == #regular) ifTrue:[
            (EditTextView startOn:fileName) readOnly
        ]
    ].
    self cursor:Cursor normal
!

fileIn
    "load the selected file(s)"

    |fileName aStream prevState|

    self cursor:Cursor wait.
    self selectionDo:[:anObject |
        fileName := anObject name.
        ((currentDirectory typeOf:fileName) == #regular) ifTrue:[
            aStream := FileStream readonlyFileNamed:fileName 
                                                 in:currentDirectory.
            aStream isNil ifFalse:[
                prevState := Class noUpdateChanges.
                aStream fileIn.
                Class updateChanges:prevState.
                aStream close
            ]
        ]
    ].
    self cursor:Cursor normal
! !

!DirectoryView methodsFor:'accessing'!

label:aString
    "label change"

    superView notNil ifTrue:[
        superView label:currentDirectoryName
    ] ifFalse:[
        super label:currentDirectoryName
    ]
!

currentDirectory:aPathName
    currentDirectoryName := aPathName.
    currentDirectory := FileDirectory directoryNamed:currentDirectoryName
! !

!DirectoryView methodsFor:'group selection'!

endRectangleDrag
    "objectView selects objects fully contained in lasso - 
     here we want to select all objects touched by lasso"

    self xoring:[self displayRectangle:groupRectangleFrame].
    self cursor:Cursor normal.
    self selectAllIntersecting:(groupRectangleFrame + viewOrigin)

! !

!DirectoryView methodsFor:'events'!

sizeChanged:how
    self reorgContents.
    super sizeChanged:how

!

destroy
    Processor removeTimedBlock:checkBlock.
    warnBox notNil ifTrue:[warnBox destroy].
    propertyBox notNil ifTrue:[propertyBox destroy].
    settingBox notNil ifTrue:[settingBox destroy].
    super destroy
! !

!DirectoryView methodsFor:'drag & drop'!

move:something to:aPoint inAlienViewId:aViewId
    "move something into another view (which is not a smalltalk
     view)"

    self notify:'cannot move object(s) to alien views'
!

canDrop:something
    "return true, if anObjectOrCollection can be
     dropped in the receiver."

    (something isKindOf:Collection) ifTrue:[
        "everything must be a DirectoryViewObject"
        something do:[:anObject |
            (anObject isKindOf:DirectoryViewObject) ifFalse:[
                (anObject respondsTo:#asDirectoryViewObject) ifFalse:[
                    ^ false
                ]
            ]
        ].
        ^ true
    ].
    (something isKindOf:DirectoryViewObject) ifTrue:[^ true].
    ^ something respondsTo:#asDirectoryViewObject

!

move:something to:aPoint in:aView
    "move something into another view"

    |nm|

    (aView canDrop:something) ifFalse:[
        nm := aView name.
        (nm ~~ aView class name) ifTrue:[
            self notify:('cannot drop object(s) in '
                         , aView name)
        ] ifFalse:[
            self notify:('cannot drop object(s) in '
                         , aView classNameWithArticle)
        ].
        ^ self
    ].
    aView drop:something at:aPoint from:self 
    with:[:takenObject |
        "takeover was successful - remove it here"

        'takeover of: ' print. takenObject pathName print.
        ' ok' printNewline
    ]
    ifFail:[:failedObject |
        "takeover was not successful"

        'takeover of: ' print. failedObject pathName print.
        ' failed' printNewline
    ]
!

dropObject:anObject at:aPoint
    "Takeover the object - return true if successfull"

    |oldPathName newPathName moveOk|

    oldPathName := anObject pathName.
    newPathName := currentDirectory pathName , '/' , anObject name.
    moveOk := true.
    (OperatingSystem linkFile:oldPathName to:newPathName) ifTrue:[
        (OperatingSystem removeFile:oldPathName) ifTrue:[^ true]
    ].
    OperatingSystem removeFile:newPathName.

    "move failed - try mv command"

    (OperatingSystem executeCommand:('mv ' , oldPathName, ' ' , newPathName))
    ifTrue:[
        ^ true
    ].

    "is it a directory ?"

    (OperatingSystem isDirectory:oldPathName) ifTrue:[
        (OperatingSystem executeCommand:
                           ('mvdir ' , oldPathName, ' ' , newPathName))
        ifTrue:[
            ^ true
        ]
    ].

    "try copying"

    (OperatingSystem isDirectory:oldPathName) ifTrue:[^ false].
    (OperatingSystem executeCommand:('cp ' , oldPathName, ' ' , newPathName))
    ifTrue:[
        (OperatingSystem removeFile:oldPathName) ifTrue:[^ true].
        (OperatingSystem executeCommand:('rm -f ' , oldPathName))
        ifTrue:[^ true].
        OperatingSystem executeCommand:('rm -f ' , newPathName)
    ].
    ^ false
!

drop:something at:aPoint from:aView with:aBlock ifFail:failBlock
    "another view wants me to take an object. Take the object
     and evaluate aBlock for each successful takeover. Evaluate
     failBlock if something went wrong."

    |o|

    (something isKindOf:Collection) ifTrue:[
        "everything must be a DirectoryViewObject"
        something do:[:anObject |
            (anObject isKindOf:DirectoryViewObject) ifTrue:[
                o := anObject
            ] ifFalse:[
                o := anObject asDirectoryViewObject
            ].
            (self dropObject:o at:aPoint) ifTrue:[aBlock value:anObject]
                                         ifFalse:[failBlock value:anObject]
        ]
    ] ifFalse:[
        (something isKindOf:DirectoryViewObject) ifTrue:[
            o := something
        ] ifFalse:[
            o := something asDirectoryViewObject
        ].
        (self dropObject:o at:aPoint) ifTrue:[aBlock value:something]
                                     ifFalse:[failBlock value:something]
    ].
    self updateContents.
    aView updateContents
! !

!DirectoryView methodsFor:'property-dialog'!

rOwner
    ^ self accessRightsOfSelectionAsNumber bitTest:8r400
!

rOwner:aBoolean
    ^ self changeAccessRightOfSelection:8r400 to:aBoolean
!

wOwner
    ^ self accessRightsOfSelectionAsNumber bitTest:8r200
!

wOwner:aBoolean
    ^ self changeAccessRightOfSelection:8r200 to:aBoolean
!

xOwner
    ^ self accessRightsOfSelectionAsNumber bitTest:8r100
!

xOwner:aBoolean
    ^ self changeAccessRightOfSelection:8r100 to:aBoolean
!

rGroup
    ^ self accessRightsOfSelectionAsNumber bitTest:8r040
!

rGroup:aBoolean
    ^ self changeAccessRightOfSelection:8r040 to:aBoolean
!

wGroup
    ^ self accessRightsOfSelectionAsNumber bitTest:8r020
!

wGroup:aBoolean
    ^ self changeAccessRightOfSelection:8r020 to:aBoolean
!

xGroup
    ^ self accessRightsOfSelectionAsNumber bitTest:8r010
!

xGroup:aBoolean
    ^ self changeAccessRightOfSelection:8r010 to:aBoolean
!

rOther
    ^ self accessRightsOfSelectionAsNumber bitTest:8r004
!

rOther:aBoolean
    ^ self changeAccessRightOfSelection:8r004 to:aBoolean
!

wOther
    ^ self accessRightsOfSelectionAsNumber bitTest:8r002
!

wOther:aBoolean
    ^ self changeAccessRightOfSelection:8r002 to:aBoolean
!

xOther
    ^ self accessRightsOfSelectionAsNumber bitTest:8r001
!

xOther:aBoolean
    ^ self changeAccessRightOfSelection:8r001 to:aBoolean
! !
