"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DrawObject subclass:#DrawGroup
       instanceVariableNames:'groupedObjects'
       classVariableNames:''
       poolDictionaries:''
       category:'Applications-DrawTool'
!

DrawGroup comment:'
COPYRIGHT (c) 1989-93 by Claus Gittinger
              All Rights Reserved


this class represents grouped objects

$Header: /claus/smalltalk/st2.6/clients/DrawTool/RCS/DrawGroup.st,v 1.3 93/12/16 11:40:47 claus Exp $
written summer 89 by claus
'!

!DrawGroup methodsFor:'copying'!

copy
    "answer a copy - since neither a shallow copy nor a deep copy
     is not good here, answer a deep copy of shallow copies"

    |aCopy sz newGroupedObjects|

    aCopy := super copy.
    sz := groupedObjects size.
    newGroupedObjects := OrderedCollection new:sz.
    1 to:sz do:[:index |
        newGroupedObjects at:index put:(groupedObjects at:index) copy
    ].
    aCopy groupedObjects:newGroupedObjects.
    ^ aCopy
! !

!DrawGroup methodsFor:'accessing'!

groupedObjects
    ^ groupedObjects
!

groupedObjects:aCollection
    groupedObjects := aCollection.
    self computeBoundingBox
! !

!DrawGroup methodsFor:'adding objects'!

add:anObject
    groupedObjects isNil ifTrue:[
        groupedObjects := OrderedCollection new.
        lineWidth := 0.
        frame := anObject frame
    ].
    groupedObjects add:anObject.
    origin isNil ifTrue:[
        origin := anObject origin.
        corner := anObject corner
    ] ifFalse:[
        origin := origin min:(anObject origin).
        corner := corner max:(anObject corner)
    ].
    frame := frame merge:(anObject frame).
    lineWidth := lineWidth max:(anObject lineWidth).
    anObject fillPattern notNil ifTrue:[fillPattern := anObject fillPattern].
    anObject linePattern notNil ifTrue:[linePattern := anObject linePattern]
! !

!DrawGroup methodsFor:'inside check'!

isHitBy:hitPoint withDelta:hitDelta
    "return true, if object group is hit by point"

    groupedObjects isNil ifTrue:[^ false].

    "quick boundary check"
    (super isHitBy:hitPoint withDelta:hitDelta) ifFalse:[^ false].

    groupedObjects do:[:object |
        (object isHitBy:hitPoint withDelta:hitDelta) ifTrue:[^ true]
    ].
    ^ false
! !

!DrawGroup methodsFor:'moving and resizing'!

computeBoundingBox
    |f|
    groupedObjects do:[:anObject |
        f isNil ifTrue:[
            f := anObject frame
        ] ifFalse:[
            f := f merge:(anObject frame)
        ]
    ].
    frame := f
!

moveTo:newOrigin
    |delta|

    delta := newOrigin - origin.
    groupedObjects do:[:theObject |
        theObject moveTo:(theObject origin + delta)
    ].
    super moveTo:newOrigin
!

origin:newOrigin corner:newCorner
    |newSize oldSize scaleX scaleY p nx ny nOrigin nCorner|

    scaleX := 1.
    newSize := (newCorner x) - (newOrigin x).
    oldSize := (corner x) - (origin x).
    (oldSize ~= newSize) ifTrue:[
        scaleX := newSize / oldSize
    ].
    scaleY := 1.
    newSize := (newCorner y) - (newOrigin y).
    oldSize := (corner y) - (origin y).
    (oldSize ~= newSize) ifTrue:[
        scaleY := newSize / oldSize
    ].

    (scaleX ~~ 1) ifTrue:[
        (scaleY ~~ 1) ifTrue:[
            groupedObjects do:[:theObject |
                p := theObject origin.
                nx := p x.
                ny := p y.

                "move to 0@0 - origin"
                nx := nx - origin x.
                ny := ny - origin y.

                "scale to new size"
                nx := (nx * scaleX) asInteger.
                ny := (ny * scaleY) asInteger.

                "move to new - origin"
                nx := nx + newOrigin x.
                ny := ny + newOrigin y.

                nOrigin := nx @ ny.

                p := theObject corner.
                nx := p x.
                ny := p y.

                "move to 0@0 - origin"
                nx := nx - origin x.
                ny := ny - origin y.

                "scale to new size"
                nx := (nx * scaleX) asInteger.
                ny := (ny * scaleY) asInteger.

                "move to new - origin"
                nx := nx + newOrigin x.
                ny := ny + newOrigin y.

                nCorner := nx @ ny.

                theObject origin:nOrigin corner:nCorner
            ]
        ]
    ].
    super origin:newOrigin corner:newCorner
! !

!DrawGroup methodsFor:'changing attributes'!

fillPattern:aFormOrNil
    groupedObjects do:[:theObject |
        theObject fillPattern:aFormOrNil
    ].
    fillPattern := aFormOrNil
!

linePattern:aFormOrNil
    groupedObjects do:[:theObject |
        theObject linePattern:aFormOrNil
    ].
    linePattern := aFormOrNil
!

lineWidth:aNumber
    groupedObjects do:[:theObject |
        theObject lineWidth:aNumber
    ].
    lineWidth := aNumber
!

foreground:aColor
    groupedObjects do:[:theObject |
        theObject foreground:aColor
    ]
!

background:aColor
    groupedObjects do:[:theObject |
        theObject background:aColor
    ]
!

fontFamily:aString
    groupedObjects do:[:theObject |
        theObject fontFamily:aString
    ]
!

fontFace:aString
    groupedObjects do:[:theObject |
        theObject fontFace:aString
    ]
!

fontStyle:aString
    groupedObjects do:[:theObject |
        theObject fontStyle:aString
    ]
!

fontSize:aNumber
    groupedObjects do:[:theObject |
        theObject fontSize:aNumber
    ]
! !

!DrawGroup methodsFor:'drawing'!

drawIn:aView offset:anOffset
    groupedObjects do:[:object |
        object drawIn:aView offset:anOffset
    ]
!

drawOutlineIn:aView offset:anOffset
    groupedObjects do:[:object |
        object drawOutlineIn:aView offset:anOffset
    ]
! !

!DrawGroup methodsFor:'storing'!

storeOn:aStream
    |first|

    first := true.
    aStream nextPut:$(.
    aStream nextPutAll:(self class name).
    aStream nextPutAll:' new '.
    groupedObjects notNil ifTrue:[
        groupedObjects do:[:object |
            first ifTrue:[
                first := false
            ] ifFalse:[
                aStream nextPutAll:';'.
            ].
            aStream nextPutAll:'add:'.
            object storeOn:aStream
        ]
    ].
    aStream nextPut:$)
! !
