"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DrawObject subclass:#DrawLine
       instanceVariableNames:'p1 p2'
       classVariableNames:''
       poolDictionaries:''
       category:'Applications-DrawTool-Objects'
!

DrawLine comment:'
COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

this class represent line segments

$Header: /claus/smalltalk/st2.6/clients/DrawTool/RCS/DrawLine.st,v 1.2 93/12/16 11:40:51 claus Exp $
written summer 89 by claus
'!

!DrawLine methodsFor:'accessing'!

firstPoint:aPoint
    p1 := aPoint.
    self computeBoundingBox
!

secondPoint:aPoint
    p2 := aPoint.
    self computeBoundingBox
!

mmP1:newP1 mmP2:newP2
    "set new start/endPoint; arguments given in millimeter"

    p1 := (newP1 x * Display horizontalPixelPerMillimeter) rounded asInteger
          @
          (newP1 y * Display verticalPixelPerMillimeter) rounded asInteger.
    p2 := (newP2 x * Display horizontalPixelPerMillimeter)  rounded asInteger
          @
          (newP2 y * Display verticalPixelPerMillimeter) rounded asInteger.
    self computeBoundingBox
!
moveTo:newOrigin
    p1 := p1 + (newOrigin - origin).
    p2 := p2 + (newOrigin - origin).
    self computeBoundingBox
!

origin:origin corner:corner
    p1 := origin.
    p2 := corner.
    self computeBoundingBox
!

flipHorizontal
    self flipVertical.
!

flipVertical
    |np1 np2|

    np1 := p1 x @ p2 y.
    np2 := p2 x @ p1 y.
    p1 := np1.
    p2 := np2.
    self computeBoundingBox
! !

!DrawLine methodsFor:'testing'!

isHitBy:hitPoint withDelta:hitDelta
    "answer true, if line is hit by point"

    |startX startY endX endY 
     hx hy dx dy expy lw delta|

    "quick boundary check"
    (super isHitBy:hitPoint withDelta:hitDelta) ifFalse:[
        ^ false
    ].

    lw := (lineWidth // 2) + 1.
    delta := hitDelta + lw.

    startX := p1 x.
    startY := p1 y.
    endX := p2 x.
    endY := p2 y.

    "look, if hitpoint is on line"
    dx := endX - startX.
    dx == 0 ifTrue:[^ true].

    dy := endY - startY.
    dy == 0 ifTrue:[^ true].

    hx := hitPoint x - startX.
    hy := hitPoint y - startY.
    expy := (hx * dy) // dx.   "expected x for given y"

    "for the given x, then line-point is x/expy.
     now, look if hitpoint is within a circle with radius delta
     around this point
    "
    hx := 0. hy := hy - expy.
    (hy * hy) <= (delta * delta) ifTrue:[^ true].
    ^ false
! !

!DrawLine methodsFor:'frame computation'!

computeBoundingBox
    |minx miny maxx maxy lw|

    p1 isNil ifTrue:[^ self].

    minx := p1 x.
    maxx := p2 x.
    (maxx < minx) ifTrue:[
        minx := p2 x.
        maxx := p1 x
    ].
    miny := p1 y.
    maxy := p2 y.
    maxy < miny ifTrue:[
        miny := p2 y.
        maxy := p1 y
    ].
    origin := minx @ miny.
    corner := maxx @ maxy.
    lw := (lineWidth // 2) + 1.
    minx := minx - lw.
    maxx := maxx + lw.
    miny := miny - lw.
    maxy := maxy + lw.
    frame := Rectangle left:minx right:maxx top:miny bottom:maxy.
! !

!DrawLine methodsFor:'drawing'!

drawIn:aGC offset:anOffset
    |offs org oldStyle|

    offs := anOffset - aGC viewOrigin.

    (lineWidth == 1) ifTrue:[
        aGC lineWidth:0
    ] ifFalse:[
        aGC lineWidth:lineWidth
    ].
    oldStyle :=  aGC lineStyle.
    (lineStyle notNil and:[lineStyle ~~ oldStyle]) ifTrue:[
        aGC lineStyle:lineStyle
    ].

    org := origin + offs.
    aGC setMaskOrigin:org.
    aGC paint:foreground on:background.

    linePattern notNil ifTrue:[
        linePattern == 1 ifTrue:[
            aGC displayLineFrom:(p1 + offs) to:(p2 + offs)
        ] ifFalse:[
            aGC withPattern:linePattern do:[
                aGC displayLineFrom:(p1 + offs) to:(p2 + offs)
            ]
        ]
    ].
    "leave gc in some known state"
    lineWidth > 1 ifTrue:[
        aGC lineWidth:0.
    ].
    lineStyle ~~ oldStyle ifTrue:[
        aGC lineStyle:oldStyle
    ].
!

drawOutlineIn:aGC offset:anOffset
    |offs|

    offs := anOffset - aGC viewOrigin.
"
    aGC paint:Color black.
"
    aGC displayLineFrom:(p1 + offs) to:(p2 + offs)
! !

!DrawLine methodsFor:'handles'!

selectorForHandle:aHandle
    aHandle = p1 ifTrue:[^ #firstPoint:].
    aHandle = p2 ifTrue:[^ #secondPoint:].
    ^ nil
!

handlesDo:aBlock
    aBlock value:p1.
    aBlock value:p2
! !

!DrawLine methodsFor:'storing'!

storeInstvarsOn:aStream
    |pixelPerMmH pixelPerMmV mmPoint|

    super storeInstvarsOn:aStream.
    pixelPerMmH := Display horizontalPixelPerMillimeter.
    pixelPerMmV := Display verticalPixelPerMillimeter.
    mmPoint := (p1 x / pixelPerMmH) @ (p1 y / pixelPerMmV).
    aStream nextPutAll:'; mmP1:'. mmPoint storeOn:aStream.
    mmPoint := (p2 x / pixelPerMmH) @ (p2 y / pixelPerMmV).
    aStream nextPutAll:' mmP2:'. mmPoint storeOn:aStream.
! !
