"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DisplayObject subclass:#DrawObject
       instanceVariableNames:'origin corner
                              fillPattern linePattern lineWidth lineStyle
                              foreground background'
       classVariableNames:''
       poolDictionaries:''
       category:'Applications-DrawTool'
!

DrawObject comment:'

COPYRIGHT (c) 1989-93 by Claus Gittinger
              All Rights Reserved

abstract superclass for all objects handled by drawView

$Header: /claus/smalltalk/st2.6/clients/DrawTool/RCS/DrawObj.st,v 1.3 94/01/13 00:42:07 claus Exp $
written summer 89 by claus
'!

!DrawObject class methodsFor:'DrawView interface'!

icon
    "take <classname>.xbm as default bitmap file"

    ^ Form fromFile:((self name) , '.xbm') resolution:100
!

needsAllMovePoints
    "if false, objects are defined by indiviual point-input;
     if true, they are defined by just moving the pointer (free-hand draw)"

    ^ false
!

needsMultiPointCreate
    "if false, objects are defined by origin-corner;
     if true, they are defined by adding points"

    ^ false
!

needsKeyboardInput
    "if false, objects are defined dragging;
     if true, they are defined by adding keyboard input"

    ^ false
! !

!DrawObject methodsFor:'initialization'!

initialize
    lineWidth := 0.
    foreground := Color black.
    background := Color white.
    linePattern := 1.           "marks as solid"
    fillPattern := nil          "default is unfilled"
! !

!DrawObject methodsFor:'frame computations'!

computeBoundingBox
    "recompute the frame - must be done after every size/position change.
     The frame may be larger than the actual real estate used 
     but NEVER smaller for correct clipping"

    |halfLineWidth|

    lineWidth even ifTrue:[
        halfLineWidth := lineWidth // 2
    ] ifFalse:[
        "must always add 1 since display cannot draw half a pixel "
        halfLineWidth := (lineWidth // 2) + 1
    ].
    frame := Rectangle origin:(origin - halfLineWidth)
                       corner:(corner + halfLineWidth)
! !

!DrawObject methodsFor:'queries'!

hasFixedSize
    "answer true, if object cannot be resized"

    ^ false
! !

!DrawObject methodsFor:'accessing'!

fillPattern:newFillPattern linePattern:newLinePattern lineWidth:newLineWidth foreground:newFg background:newBg
    "set all of fill, line, lineWidth, fg and bg colors"

    fillPattern := newFillPattern.
    linePattern := newLinePattern.
    lineWidth := newLineWidth.
    foreground := newFg.
    background := newBg.
    self computeBoundingBox

!

fillPattern
    "answer the fill pattern form"

    ^ fillPattern
!

fillPattern:aFormOrNil
    "set the fill pattern form"

    fillPattern := aFormOrNil
!

linePattern
    "answer the line pattern form"

    ^ linePattern
!

linePattern:aFormOrNil
    "set the line pattern form"

    linePattern := aFormOrNil
!

lineWidth
    "answer the line width"

    ^ lineWidth
!

lineWidth:aNumber
    "set the line width"

    lineWidth := aNumber.
    self computeBoundingBox
!

lineStyle
    "answer the line style"

    ^ lineStyle
!

lineStyle:aStyleSymbol 
    "set the line style"

    lineStyle := aStyleSymbol.
!

foreground
    "answer the foreground color"

    ^ foreground
!

foreground:aColor
    "set the foreground color"

    foreground := aColor
!

background
    "answer the background color"

    ^ background
!

background:aColor
    "set the background color"

    background := aColor
!

foreground:fgColor background:bgColor
    "set foreground and background colors"

    foreground := fgColor.
    background := bgColor
!

fontFamily
    "answer the font family"

    ^ nil
!

fontFamily:aString
    "set the font family"

    ^ self
!

fontFace
    "answer the font face"

    ^ nil
!

fontFace:aString
    "set the font face"

    ^ self
!

fontStyle
    "answer the font style"

    ^ nil
!

fontStyle:aString
    "set the font style"

    ^ self
!

fontSize
    "answer the font size"

    ^ nil
!

fontSize:aString
    "set the font size"

    ^ self
!

groupedObjects
    "ungrouping me does nothing"

    ^ self
! !

!DrawObject methodsFor:'moving/resizing'!

flipHorizontal
    ^ self 
!

flipVertical
    ^ self 
!

mmOrigin:newOrigin mmCorner:newCorner
    "set new origin/corner; arguments given in millimeter"

    origin := (newOrigin x * Display horizontalPixelPerMillimeter) rounded asInteger
              @
              (newOrigin y * Display verticalPixelPerMillimeter) rounded asInteger.
    corner := (newCorner x * Display horizontalPixelPerMillimeter)  rounded asInteger
              @
              (newCorner y * Display verticalPixelPerMillimeter) rounded asInteger.
    self computeBoundingBox
!

mmOrigin:aPoint
    "set new origin; argument given in millimeter"

    |pixPoint|

    pixPoint := (aPoint x * Display horizontalPixelPerMillimeter)
                @
                (aPoint y * Display verticalPixelPerMillimeter).
    self origin:pixPoint corner:corner
!

mmCorner:aPoint
    "set new corner; argument given in millimeter"

    |pixPoint|

    pixPoint := (aPoint x * Display horizontalPixelPerMillimeter)
                @
                (aPoint y * Display verticalPixelPerMillimeter).
    self origin:origin corner:pixPoint
!

origin:newOrigin corner:newCorner
    "set new origin/corner; arguments given in pixels"

    origin := newOrigin.
    corner := newCorner.
    self computeBoundingBox
!

origin:aPoint
    "set new origin; argument given in pixels"

    self origin:aPoint corner:corner
!

corner:aPoint
    "set new corner; argument given in pixels"

    self origin:origin corner:aPoint
!

top:aPoint
    self origin:((origin x) @ (aPoint y))
         corner:corner
!

topRight:aPoint
    self origin:((origin x) @ (aPoint y))
         corner:((aPoint x) @ (corner y))
!

right:aPoint
    self origin:origin
         corner:((aPoint x) @ (corner y))
!

bottom:aPoint
    self origin:origin
         corner:((corner x) @ (aPoint y))
!

bottomLeft:aPoint
    self origin:((aPoint x) @ (origin y))
         corner:((corner x) @ (aPoint y))
!

left:aPoint
    self origin:((aPoint x) @ (origin y))
         corner:corner
!

moveTo:newOrigin
    |width height|
    width := corner x - origin x.
    height := corner y - origin y.
    self origin:newOrigin
         corner:(newOrigin x + width) @ (newOrigin y + height)
! !

!DrawObject methodsFor:'drawing'!

drawOutlineIn:aGC offset:anOffset
    |offs halfLw nOrg nCorner|

"
    aGC paint:Color black on:Color white.
"
    offs := anOffset - aGC viewOrigin.

    nOrg := origin + offs.
    nCorner := corner + offs.

    lineWidth > 2 ifTrue:[
        "draw twice"
        halfLw := lineWidth // 2.
        aGC displayRectangle:(Rectangle origin:(nOrg - halfLw) corner:(nCorner + halfLw)).
        aGC displayRectangle:(Rectangle origin:(nOrg + halfLw) corner:(nCorner - halfLw)).
    ] ifFalse:[
        aGC displayRectangle:(Rectangle origin:nOrg corner:nCorner)
    ]
! !

!DrawObject methodsFor:'handles'!

handlesDo:aBlock
    "evaluate the argument, aBlock for every handle"

    |halfHeight halfWidth top left right bottom hCenter vCenter|

    top := origin y.
    left := origin x.
    bottom := corner y.
    right := corner x.

    aBlock value:origin.                "top-left"
    aBlock value:corner.                "bottom-right"
    aBlock value:(left @ bottom).       "bottom-left"
    aBlock value:(right @ top).         "top-right"

    halfHeight := (bottom - top) // 2.
    vCenter := top + halfHeight.
    aBlock value:(left @ vCenter).   "center left"
    aBlock value:(right @ vCenter).     "center right"

    halfWidth := (right - left) // 2.
    hCenter := left + halfWidth.
    aBlock value:(hCenter @ top).       "top-center"
    aBlock value:(hCenter @ bottom)     "bottom-center"
!

selectorForHandle:aHandle
    |halfWidth halfHeight top left bottom right hCenter vCenter hx hy|

    top := origin y.
    bottom := corner y.
    left := origin x.
    right := corner x.
    hx := aHandle x.
    hy := aHandle y.

    (hx == left) ifTrue:[
        (hy == top) ifTrue:[^ #origin:].
        (hy == bottom) ifTrue:[^ #bottomLeft:]
    ].

    (hx == right) ifTrue:[
        (hy == bottom) ifTrue:[^ #corner:].
        (hy == top) ifTrue:[^ #topRight:]
    ].

    ((hx == left) or:[hx == right]) ifTrue:[
        halfHeight := (bottom - top) // 2.
        vCenter := top + halfHeight.

        (hy == vCenter) ifTrue:[
            (hx == left) ifTrue:[^ #left:].
            ^ #right:
        ]
    ].

    ((hy == top) or:[hy == bottom]) ifTrue:[
        halfWidth := (right - left) // 2.
        hCenter := left + halfWidth.

        (hx == hCenter) ifTrue:[
            (hy == top) ifTrue:[^ #top:].
            ^ #bottom:
        ]
    ].

    ^ nil
! !

!DrawObject methodsFor:'storing'!

storePattern:aPattern on:aStream
    |pIndex|

    pIndex := (DrawTool listOfPatterns) indexOf:aPattern.
    pIndex == 0 ifTrue:[
        (aPattern = Form black) ifTrue:[
            aStream nextPutAll:'(Form black)'.
            ^ self
        ].
        (aPattern = Form white) ifTrue:[
            aStream nextPutAll:'(Form white)'.
            ^ self
        ].
        aStream nextPutAll:'nil'
    ] ifFalse:[
        aStream nextPutAll:'(DrawTool listOfPatterns at:'.
        pIndex storeOn:aStream.
        aStream nextPutAll:')'
    ]
!

storeInstvarsOn:aStream
    |mmPoint pixelPerMmH pixelPerMmV|

    pixelPerMmH := Display horizontalPixelPerMillimeter.
    pixelPerMmV := Display verticalPixelPerMillimeter.
    mmPoint := (origin x / pixelPerMmH) @ (origin y / pixelPerMmV).
    aStream nextPutAll:'; mmOrigin:'. mmPoint storeOn:aStream.
    mmPoint := (corner x / pixelPerMmH) @ (corner y / pixelPerMmV).
    aStream nextPutAll:' mmCorner:'. mmPoint storeOn:aStream.
    aStream cr.
    aStream nextPutAll:'; fillPattern:'. 
    self storePattern:fillPattern on:aStream.
    aStream nextPutAll:'; linePattern:'. 
    self storePattern:linePattern on:aStream.
    aStream cr.
    aStream nextPutAll:'; lineWidth:'. lineWidth storeOn:aStream.
    aStream cr.
    aStream nextPutAll:'; foreground:('. foreground storeOn:aStream.
    aStream nextPutAll:'); background:('. background storeOn:aStream.
    aStream nextPutAll:')'.
    aStream cr
!

storeOn:aStream
    aStream nextPut:$(.
    aStream nextPutAll:(self class name).
    aStream nextPutAll:' new yourself'.
    self storeInstvarsOn:aStream.
    aStream nextPut:$)
! !
