Object subclass: #DrawingPen
        instanceVariableNames: 'lineWidth lineStyle joinStyle capStyle
                                paint dashPaint'
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Graphics-Support'
!

DrawingPen comment:
'Drawing pens are used in place of Forms as drawing forms for Display objects.
They replace the drawing form (that was used in pre V4 ST-80).'!

"
|l p v|
v := View new.
v extent:(500@500).
v realize. v device synchronizeOutput.

f := DrawingPen lineWidth:5 lineStyle:#dashed capStyle:#butt joinStyle:#miter.
f paint:(Color grey).

l := Line new.
l beginPoint:10 @ 10.
l endPoint:(100 @ 10).
l form:f
"!

!DrawingPen class methodsFor: 'instance creation'!

new
    "return a new (default) drawing pen"

    ^ self basicNew initialize
!

lineWidth:lineWidth lineStyle:lineStyle joinStyle:joinStyle capStyle:capStyle
    "return a new drawing pen"

    |newPen|

    newPen := self new.
    newPen lineWidth:lineWidth.
    newPen lineStyle:lineStyle.
    newPen capStyle:capStyle.
    newPen jopinStyle:joinStyle.
    ^ newPen
!

dotOfSize:size
    ^ (self new) lineWidth:size
! !

!DrawingPen methodsFor: 'private'!

initialize
    lineWidth := 1.
    lineStyle := #solid.
    capStyle := #butt.
    joinStyle := #miter.
    paint := Color black.
    dashPaint := Color white
! !

!DrawingPen methodsFor: 'accessing'!

width
    "return the lineWidth - for compatibility to old Form-style"

    ^ lineWidth
!

lineWidth
    "return the lineWidth"

    ^ lineWidth
!

lineWidth:aNumber
    "set the lineWidth of the pen"

    lineWidth := aNumber
!

lineStyle
    "return the lineStyle"

    ^ lineStyle
!

lineStyle:aStyleSymbol
    "set the lineStyle of the pen"

    lineStyle := aStyleSymbol
!

joinStyle
    "return the joinStyle"

    ^ joinStyle
!

joinStyle:aJoinSymbol
    "set the joinStyle of the pen"

    joinStyle := aJoinSymbol
!

capStyle
    "return the capStyle"

    ^ capStyle
!

capStyle:aCapSymbol
    "set the capStyle of the pen"

    capStyle := aCapSymbol
!

paint:p dashPaint:dp
    paint := p.
    dashPaint := dp.
!

paint 
    "return the color"

    ^ paint
!

paint:aColor
    "set the color of the pen"

    paint := aColor
!

dashPaint
    "return the alternate color (for dashes) of the pen"

    ^ dashPaint
!

dashPaint:aColor
    "set the alternate color (for dashes) of the pen"

    dashPaint := aColor
! !
