"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DrawObject subclass:#DrawPolygon
         instanceVariableNames:'originalPoints scaledPoints originalOrigin originalCorner
                helperBitmap'
         classVariableNames:''
         poolDictionaries:''
         category:'Applications-DrawTool-Objects'
!

DrawPolygon comment:'

COPYRIGHT (c) 1989-93 by Claus Gittinger
              All Rights Reserved

this class represents polygons

$Header: /claus/smalltalk/st2.6/clients/DrawTool/RCS/DrawPoly.st,v 1.3 94/01/13 00:42:12 claus Exp $
written summer 89 by claus
'!

!DrawPolygon class methodsFor:'interface to DrawView'!

icon
    ^ Form fromFile:'DrawPoly.xbm' resolution:100
!

needsMultiPointCreate
    ^ true
! !

!DrawPolygon methodsFor:'inside check'!

computeBoundingBox
    "funny: miterd lines can move away from endpoints -
     redefined to be save on X"

    |sixLineWidths|

    (lineWidth == 0) ifTrue:[
        sixLineWidths := 6
    ] ifFalse:[
        sixLineWidths := lineWidth * 6
    ].
    frame := Rectangle left:(origin x - sixLineWidths)
                      right:(corner x + sixLineWidths)
                        top:(origin y - sixLineWidths)
                     bottom:(corner y + sixLineWidths)
!

isHitBy:hitPoint withDelta:hitDelta
    "answer true, if polygon is hit by point"

    |hw hh index line newPoly|

    "quick boundary check"
    (super isHitBy:hitPoint withDelta:hitDelta) ifFalse:[^ false].

    "if helper bitmap already exists, check it first - its faster"

    helperBitmap notNil ifTrue:[
        ((helperBitmap at:(hitPoint - origin)) == 1) ifTrue:[^ true]
    ].

    "first, we walk the outlines and check them by using DrawLine's algorithm"
    index := 1.
    [index < (scaledPoints size)] whileTrue:[
        line := DrawLine new origin:(scaledPoints at:index)
                          corner:(scaledPoints at:(index + 1)).
        line lineWidth:lineWidth.
        (line isHitBy:hitPoint withDelta:hitDelta) ifTrue:[^ true].
        index := index + 1
    ].
    fillPattern isNil ifTrue:[^ false].

    "hard part, must check interior of polygon"
    "since I'm lazy, I (mis)-use the view stuff to compute this:
     fill the polygon into a temp-bitmap and check, if the pixel
     at hitPoint is on ..."

    helperBitmap isNil ifTrue:[
        newPoly := Polygon new:(scaledPoints size).
        1 to:(newPoly size) do:[:index |
            newPoly at:index put:((scaledPoints at:index) - origin)
        ].
        hw := corner x - origin x.
        hh := corner y - origin y.
        helperBitmap := Form width:hw height:hh.
        helperBitmap fill:(Color colorId:0).
        helperBitmap paint:(Color colorId:1).
        helperBitmap fillPolygon:newPoly
    ].
    ((helperBitmap at:(hitPoint - origin)) == 1) ifTrue:[^ true].

    ^ false
! !

!DrawPolygon methodsFor:'creation from DrawView'!

firstPoint:aPoint
    originalPoints := OrderedCollection new.
    originalPoints add:aPoint
!

addPoint:aPoint
    "try to compress if slope has not changed"

    |index pLast pPrev dxLast dyLast dx dy|

    index := originalPoints size.
    (index > 2) ifTrue:[
        pLast := originalPoints at:index.
        pPrev := originalPoints at:(index - 1).
        dx := aPoint x - pLast x.
        dy := aPoint y - pLast y.
        dxLast := pLast x - pPrev x.
        dyLast := pLast y - pPrev y.
        ((dy == 0) or:[dyLast == 0]) ifTrue:[
            (dy == dyLast) ifTrue:[
                originalPoints at:index put:aPoint.
                ^ self
            ]
        ] ifFalse:[
            (dx / dy) = (dxLast / dyLast) ifTrue:[
                originalPoints at:index put:aPoint.
                ^ self
            ]
        ]
    ].
    originalPoints add:aPoint
!

lastPointAt:aPoint
    originalPoints at:(originalPoints size) put:aPoint
!

finalPoint
    "creation finished - minimize the polygon"

    |point pts|

    pts := originalPoints.
    originalPoints := Polygon new:(pts size).
    1 to:(pts size) do:[:i |
        originalPoints at:i put:(pts at:i).
        pts at:1 put:nil
    ].
    helperBitmap := nil.
    scaledPoints := Polygon new:(originalPoints size).
    originalOrigin := originalPoints at:1.
    originalCorner := originalPoints at:1.
    1 to:(originalPoints size) do:[:index |
        point := originalPoints at:index.
        originalOrigin := originalOrigin min:point.
        originalCorner := originalCorner max:point.
        scaledPoints at:index put:point
    ].
    origin := originalOrigin x @ originalOrigin y.
    corner := originalCorner x @ originalCorner y
!

finalPoint:aPoint
    self lastPointAt:aPoint.
    self finalPoint
! !

!DrawPolygon methodsFor:'moving and resizing'!

moveTo:newOrigin
    |delta|

    delta := newOrigin - origin.
    1 to:(scaledPoints size) do:[:index |
        scaledPoints at:index
                    put:((scaledPoints at:index) + delta)
    ].
    super moveTo:newOrigin
!

origin:newOrigin corner:newCorner

    |newSize oldSize scaleX scaleY nx ny p|

    scaledPoints notNil ifTrue:[
        helperBitmap := nil.
        scaleX := 1.
        newSize := (newCorner x) - (newOrigin x).
        oldSize := (originalCorner x) - (originalOrigin x).
        (oldSize ~= newSize) ifTrue:[
            scaleX := newSize / oldSize
        ].
        scaleY := 1.
        newSize := (newCorner y) - (newOrigin y).
        oldSize := (originalCorner y) - (originalOrigin y).
        (oldSize ~= newSize) ifTrue:[
            scaleY := newSize / oldSize
        ].

        (scaleX ~~ 1) ifTrue:[
            (scaleY ~~ 1) ifTrue:[
                1 to:(originalPoints size) do:[:index |
                    p := originalPoints at:index.
                    nx := p x.
                    ny := p y.

                    "move to 0@0 - origin"
                    nx := nx - originalOrigin x.
                    ny := ny - originalOrigin y.

                    "scale to new size"
                    nx := (nx * scaleX) asInteger.
                    ny := (ny * scaleY) asInteger.

                    "move to new - origin"
                    nx := nx + newOrigin x.
                    ny := ny + newOrigin y.

                    scaledPoints at:index put:(nx @ ny)
                ]
            ]
        ]
    ].
    super origin:newOrigin corner:newCorner
! !

!DrawPolygon methodsFor:'drawing'!

drawIn:aGC offset:anOffset
    |srcPoly drawPoly offs org|

    offs := anOffset - aGC viewOrigin.
    srcPoly := scaledPoints.
    srcPoly isNil ifTrue:[
        srcPoly := originalPoints
    ].
    ((offs x = 0) and:[offs y = 0]) ifTrue:[
        drawPoly := srcPoly
    ] ifFalse:[
        drawPoly := Polygon new:(srcPoly size).
        1 to:(srcPoly size) do:[:index |
            drawPoly at:index put:((srcPoly at:index) + offs)
        ]
    ].
    (lineWidth == 1) ifTrue:[
        aGC lineWidth:0
    ] ifFalse:[
        aGC lineWidth:lineWidth
    ].
    org := origin + offs.
    aGC setMaskOrigin:org.
    aGC paint:foreground on:background.

    fillPattern notNil ifTrue:[
        fillPattern == 1 ifTrue:[
            aGC fillPolygon:drawPoly
        ] ifFalse:[
            aGC withPattern:fillPattern do:[
                aGC fillPolygon:drawPoly
            ]
        ].
        linePattern notNil ifTrue:[
            linePattern == 1 ifTrue:[
                aGC displayPolygon:drawPoly
            ] ifFalse:[
                drawPoly := drawPoly copyWith:(drawPoly at:1).
                aGC withPattern:linePattern do:[
                    aGC displayPolygon:drawPoly
                ]
            ]
        ]
    ] ifFalse:[
        linePattern notNil ifTrue:[
            linePattern == 1 ifTrue:[
                aGC displayPolygon:drawPoly
            ] ifFalse:[
                aGC withPattern:linePattern do:[
                    aGC displayPolygon:drawPoly
                ]
            ]
        ]
    ].
    "leave gc in some known state"
    lineWidth > 1 ifTrue:[
        aGC lineWidth:0.
    ].
!

drawOutlineIn:aGC offset:anOffset
    |offs org r srcPoly drawPoly|

    offs := anOffset - aGC viewOrigin.
"
    aGC paint:Color black.
"
    "if there are too many points, some servers become very
     slow - the number is arbitrary"

    (originalPoints size > 1000) ifTrue:[
        origin notNil ifTrue:[
            org := origin + offs.
            r := Rectangle origin:org corner:(corner + offs).
            aGC displayRectangle:r.
            ^ self
        ].
    ].

    srcPoly := scaledPoints.
    srcPoly isNil ifTrue:[
        srcPoly := originalPoints
    ].
    ((offs x == 0) and:[offs y == 0]) ifTrue:[
        drawPoly := srcPoly
    ] ifFalse:[
        drawPoly := Polygon new:(srcPoly size).
        1 to:(srcPoly size) do:[:index |
            drawPoly at:index put:((srcPoly at:index) + offs)
        ]
    ].
    fillPattern notNil ifTrue:[
        "closed polygon as outline looks better :-)"
        drawPoly := drawPoly copyWith:(drawPoly at:1)
    ].
    aGC displayPolygon:drawPoly
! !

!DrawPolygon methodsFor:'storing'!

storeInstvarsOn:aStream
    super storeInstvarsOn:aStream.
    aStream nextPutAll:'; firstPoint:'.
    (scaledPoints at:1) storeOn:aStream.
    2 to:(scaledPoints size) do:[:i |
        aStream nextPutAll:'; addPoint:'.
        (scaledPoints at:i) storeOn:aStream
    ].
    aStream nextPutAll:'; finalPoint'
! !
