"
 COPYRIGHT (c) 1990 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DrawRectangle subclass:#DrawRoundRectangle
         instanceVariableNames:'ovalWidth ovalHeight'
         classVariableNames:''
         poolDictionaries:''
         category:'Applications-DrawTool-Objects'
!

DrawRoundRectangle comment:'

COPYRIGHT (c) 1990-93 by Claus Gittinger
              All Rights Reserved

I represent rectangles with round corners

$Header: /claus/smalltalk/st2.6/clients/DrawTool/RCS/DrawRRect.st,v 1.2 93/12/16 11:40:58 claus Exp $
written spring 90 by claus
'!

!DrawRoundRectangle class methodsFor:'interface to DrawView'!

icon
    ^ Form fromFile:'DrawRRect.xbm' resolution:100
! !

!DrawRoundRectangle class methodsFor:'defaults'!

defaultOvalWidth
    ^ 0.25
!

defaultOvalHeight
    ^ 0.25
! !

!DrawRoundRectangle class methodsFor:'instance creation'!

new
    ^ super new ovalWidth:(self defaultOvalWidth)
               ovalHeight:(self defaultOvalHeight)
! !

!DrawRoundRectangle methodsFor:'private'!

ovalWidth:w ovalHeight:h
    ovalWidth := w.
    ovalHeight := h
! !

!DrawRoundRectangle methodsFor:'inside check'!

isHitBy:hitPoint withDelta:hitDelta
    "return true, if round rectangle is hit by point"

    (super isHitBy:hitPoint withDelta:hitDelta) ifFalse:[^ false].

    "should check for arcs here - maybe later"

    ^ true
! !

!DrawRoundRectangle methodsFor:'drawing'!

fillLeft:left top:top width:w height:h wCorner:wCorn hCorner:hCorn in:aView
    |right bot wCornHalf hCornHalf wCornHalfRest hCornHalfRest|

    right := left + w.
    bot := top + h.

    wCornHalf := wCorn // 2.
    hCornHalf := hCorn // 2.

    wCorn odd ifTrue:[
        wCornHalfRest := 1
    ] ifFalse:[
        wCornHalfRest := 0
    ].
    hCorn odd ifTrue:[
        hCornHalfRest := 1
    ] ifFalse:[
        hCornHalfRest := 0
    ].

    aView fillArcX:left y:top
                 w:wCorn h:hCorn
              from:90 angle:90.

    "top rect"
    aView fillRectangleX:(left + wCornHalf)
                       y:top
                   width:(w - wCorn + wCornHalfRest)
                  height:hCornHalf.

    "top right arc"
    aView fillArcX:(right - wCorn) y:top
                 w:wCorn h:hCorn
              from:0 angle:90.

    "big middle rect"
    aView fillRectangleX:left
                       y:(top + hCornHalf)
                   width:w
                  height:(h - hCorn + hCornHalfRest).

    "bottom left arc"
    aView fillArcX:left y:(bot - hCorn)
                 w:wCorn h:hCorn
              from:180 angle:90.

    "bottom rect"
    aView fillRectangleX:(left + wCornHalf)
                       y:(bot - hCornHalf)
                   width:(w - wCorn + wCornHalfRest)
                  height:hCornHalf.

    "bottom right arc"
    aView fillArcX:(right - wCorn) y:(bot - hCorn)
                 w:wCorn h:hCorn
              from:270 angle:90
!

drawLeft:left top:top width:w height:h wCorner:wCorn hCorner:hCorn in:aView
    |right bot wCornHalf hCornHalf wCornHalfRest hCornHalfRest|

    right := left + w.
    bot := top + h.

    wCornHalf := wCorn // 2.
    hCornHalf := hCorn // 2.

    wCorn odd ifTrue:[
        wCornHalfRest := 1
    ] ifFalse:[
        wCornHalfRest := 0
    ].
    hCorn odd ifTrue:[
        hCornHalfRest := 1
    ] ifFalse:[
        hCornHalfRest := 0
    ].

    "top line"
    aView displayLineFromX:(left + wCornHalf) y:top
                       toX:(right - wCornHalf) y:top.

    "top left arc"
    aView displayArcX:left y:top w:wCorn h:hCorn from:90 angle:90.

    "left line"
    aView displayLineFromX:left y:(top + hCornHalf)
                       toX:left y:(bot - hCornHalf).

    "top right arc"
    aView displayArcX:(right - wCorn) y:top w:wCorn h:hCorn from:0 angle:90.

    "bottom line"
    aView displayLineFromX:(left + wCornHalf) y:bot
                       toX:(right - wCornHalf) y:bot.

    "bottom right arc"
    aView displayArcX:(right - wCorn) y:(bot - hCorn) w:wCorn h:hCorn
                 from:270 angle:90.

    "right line"
    aView displayLineFromX:right y:(top + hCornHalf)
                       toX:right y:(bot - hCornHalf).

    "bottom left arc"
    aView displayArcX:left y:(bot - hCorn) w:wCorn h:hCorn from:180 angle:90
!

drawIn:aGC offset:anOffset
    |offs org corn w h wCorn hCorn top left|

    offs := anOffset - aGC viewOrigin.
    org := origin + offs.
    corn := corner + offs.
    w := corner x - origin x.
    h := corner y - origin y.
    left := org x.
    top := org y.
    wCorn := (w * ovalWidth) rounded asInteger.
    hCorn := (h * ovalHeight) rounded asInteger.

    (lineWidth == 1) ifTrue:[
        aGC lineWidth:0
    ] ifFalse:[
        aGC lineWidth:lineWidth
    ].
    aGC setMaskOrigin:org.
    aGC paint:foreground on:background.

    fillPattern notNil ifTrue:[
        fillPattern == 1 ifTrue:[
            self fillLeft:left top:top width:w height:h 
                                     wCorner:wCorn hCorner:hCorn in:aGC
        ] ifFalse:[
            aGC withPattern:fillPattern do:[
                self fillLeft:left top:top width:w height:h 
                                         wCorner:wCorn hCorner:hCorn in:aGC
            ]
        ]
    ].
    linePattern notNil ifTrue:[
        linePattern == 1 ifTrue:[
            self drawLeft:left top:top width:w height:h 
                                     wCorner:wCorn hCorner:hCorn in:aGC
        ] ifFalse:[
            aGC withPattern:linePattern do:[
                self drawLeft:left top:top width:w height:h 
                                         wCorner:wCorn hCorner:hCorn in:aGC
            ]
        ]
    ].
    "leave gc in some known state"
    lineWidth > 1 ifTrue:[
        aGC lineWidth:0.
    ].
!

drawOutlineIn:aGC offset:anOffset
    |savedLineWidth savedFillPattern savedLinePattern
     savedOrigin savedCorner savedFg savedBg|

    savedLineWidth := lineWidth.
    savedFillPattern := fillPattern.
    savedLinePattern := linePattern.
    savedFg := foreground.
    savedBg := background.

    foreground := aGC foreground.
    background := aGC background.

    lineWidth <= 3 ifTrue:[
        lineWidth := 1.
        fillPattern := nil.
        linePattern := 1.
        self drawIn:aGC offset:anOffset.
    ] ifFalse:[
        savedOrigin := origin.
        savedCorner := corner.
        lineWidth := 1.
        fillPattern := nil.
        linePattern := 1.
        origin := origin - (savedLineWidth // 2).
        corner := corner + (savedLineWidth // 2).
        self drawIn:aGC offset:anOffset.
        origin := savedOrigin + (savedLineWidth // 2).
        corner := savedCorner - (savedLineWidth // 2).
        self drawIn:aGC offset:anOffset.
        origin := savedOrigin.
        corner := savedCorner.
    ].
    lineWidth := savedLineWidth.
    fillPattern := savedFillPattern.
    linePattern := savedLinePattern.
    foreground := savedFg.
    background := savedBg
! !
