"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Object subclass:#DrawTool
         instanceVariableNames:'topView drawView menu commandPanel formMenu
                                ruler gridShown
                                fileBox currentFileName resources buttonGroup'
         classVariableNames:'ColorList PatternListInMenu PatternList'
         poolDictionaries:''
         category:'Applications-DrawTool'
!

DrawTool comment:'

COPYRIGHT (c) 1989-93 by Claus Gittinger
              All Rights Reserved

an interface to a drawView

$Header: /claus/smalltalk/st2.6/clients/DrawTool/RCS/DrawTool.st,v 1.3 94/01/13 00:42:30 claus Exp $
written summer 89 by claus
'!

!DrawTool class methodsFor:'defaults'!

commandPanelWidth
    ^ 60
!

listOfColorNames
    ^ #('white' 'yellow' 'red' 'magenta' 'blue' 'cyan' 'green' 'black')
!

listOfPatternFileNames
    ^ #('nopattern.xbm' 'white.xbm'
        'grey94.xbm' 'grey88.xbm' 'grey75.xbm'
        'grey50.xbm' 'grey25.xbm' 'grey12.xbm' 'grey6.xbm'
        'black.xbm'
        'grey75_2.xbm' 'grey50_2.xbm' 'grey25_2.xbm'
        'grey50_3.xbm'
        'hline1.xbm' 'hline2.xbm' 'hline3.xbm'
        'vline1.xbm' 'vline2.xbm' 'vline3.xbm'
        'd1line1.xbm' 'd1line2.xbm' 'd1line3.xbm'
        'd2line1.xbm' 'd2line2.xbm' 'd2line3.xbm'
        'brick1.xbm' 'brick2.xbm'
        'wave1.xbm' 'wave2.xbm'
        'cross1.xbm' 'cross2.xbm')
!

listOfObjectClasses
    |classArray|

    classArray := Array new:7.
    classArray at:1 put:DrawText.
    classArray at:2 put:DrawLine.
    classArray at:3 put:DrawRectangle.
    classArray at:4 put:DrawRoundRectangle.
    classArray at:5 put:DrawEllipse.
    classArray at:6 put:DrawPolygon.
    classArray at:7 put:DrawCurve.
    ^ classArray
!

listOfColors
    ColorList isNil ifTrue:[
        ColorList := OrderedCollection new.
        self listOfColorNames do:[:colorName |
            ColorList add:(Color name:colorName)
        ]
    ].
    ^ ColorList
!

listOfPatternsInMenu
    |form|

    PatternListInMenu isNil ifTrue:[
        PatternListInMenu := OrderedCollection new.
        PatternList := OrderedCollection new.
        self listOfPatternFileNames do:[:fileName |
            form := Form fromFile:fileName resolution:100.
            PatternListInMenu add:form.
            PatternList add:form
        ].
        PatternList at:1 put:nil
    ].
    ^ PatternListInMenu
!

listOfPatterns
    PatternList isNil ifTrue:[
        self listOfPatternsInMenu
    ].
    ^ PatternList
! !

!DrawTool class methodsFor:'startup'!

start
    "startup a DrawTool window with emty contents"

    ^ self new start
    "DrawTool start"
!

startOn:aFileName
    "startup a DrawTool window, initialize contents from aFileName"

    ^ self new startOn:aFileName
    "DrawTool startOn:'aFileName'"
! !

!DrawTool methodsFor:'startup'!

start
    |frame panelWidth myname pointerForm protoButton|

    resources := ResourcePack for:self class.
    gridShown := false.

    myname := self class name.
    topView := StandardSystemView
                label:myname
                icon:(Form fromFile:(myname , '.xbm') resolution:100)
                minExtent:(100 @ 100).

    menu := PullDownMenu in:topView.
    View3D ifFalse:[
        menu showSeparatingLines:true
    ].

    ruler := Ruler in:topView.

    "get this form first to see how wide commandPanel should be"
    pointerForm := Form fromFile:'Pointer.xbm' resolution:100.
    pointerForm isNil ifTrue:[
        self warn:'cannot find (some of) my bitmaps'.
        panelWidth := 80
    ] ifFalse:[
        protoButton := (RadioButton new) form:pointerForm.
        panelWidth := protoButton width + (Display horizontalPixelPerMillimeter * 2) rounded.
    ].

    commandPanel := VerticalPanelView
                        origin:(0 @ ((menu height * 2) + ruler height))
                        extent:[panelWidth
                                @
                                (topView height 
                                 - (menu height * 2) - ruler height)]
                        in:topView.
    commandPanel layout:#top.
    commandPanel level:0.
    commandPanel borderWidth:0.

    frame := HVScrollableView for:DrawView in:topView.
    frame origin:(panelWidth @ (menu height * 2 + ruler height))
          extent:((topView width - panelWidth)
                  @
                  (topView height - (menu height * 2) - ruler height - frame borderWidth)).
    frame extent:[(topView width - panelWidth)
                  @
                  (topView height - (menu height * 2) - ruler height)].
    drawView := frame scrolledView.

    drawView changeFillPattern:nil.
    drawView changeLinePattern:(Form black "fromFile:'black.xbm'").
    drawView changeLineWidth:1.

    ruler origin:((drawView originRelativeTo:topView) x
                  @ (menu height * 2))
          extent:(drawView width
                  @
                  ruler height).
    ruler origin:[(drawView originRelativeTo:topView) x
                  @ (menu height * 2)]
          extent:[drawView width
                  @
                  ruler height].
    ruler level:-1.

    "a kludge for now - redefine hScrollbars action.
     actually, scrollbars should notify dependents to make this easier"

    frame horizontalScrollBar scrollAction:[:pos |
        drawView scrollHorizontalToPercent:pos.
        ruler viewOrigin:(drawView viewOrigin x @ 0).
        ruler redraw.
    ].
    frame horizontalScrollBar scrollLeftAction:[
        drawView scrollLeft.
        ruler viewOrigin:(drawView viewOrigin x @ 0).
        ruler redraw.
    ].
    frame horizontalScrollBar scrollRightAction:[
        drawView scrollRight.
        ruler viewOrigin:(drawView viewOrigin x @ 0).
        ruler redraw.
    ].

    self initMenu.
    buttonGroup := RadioButtonGroup new:0.

   buttonGroup add:
        (RadioButton form:pointerForm
                   action:[drawView noCreate]
                       in:commandPanel) turnOn.

    (self class listOfObjectClasses) do:[:elClass |
        buttonGroup add:(RadioButton form:(elClass icon)
                                   action:[drawView createObjectWithClass:elClass]
                                       in:commandPanel).
    ].

    topView open.
    ruler superViewChangedSize
!

startOn:aFileName
    |aStream baseName|

    self start.

    "read file without redraw - done when file has to be read before
     first expose is handled - avoids double redraw"

    baseName := OperatingSystem baseNameOf:aFileName.
    topView label:baseName , ' - ' , (OperatingSystem directoryNameOf:aFileName).
    topView iconName:baseName.

    aStream := FileStream readonlyFileNamed:aFileName.
    aStream notNil ifTrue:[
        drawView withoutRedrawFileInContentsFrom:aStream.
        aStream close.
        currentFileName := aFileName
    ]
! !

!DrawTool methodsFor:'menu definition'!

initMenu
    |labels f family style face size|

    labels := resources array:#(
                               'file'
                               'edit'
                               'font' 
                               'form'
                               'extra'
                               'pattern'
                               'line'
                               'pen' 
                               'foreground'
                               'background').
    menu labels:labels.

    menu at:1
            putLabels:(resources array:#(
                                        'new' 
                                        'open' 
                                        'import' 
                                        '-' 
                                        'save' 
                                        'save as ...' 
                                        '-' 
                                        'print'
                                        '-' 
                                        'quit'))
            selectors:#(doNew doOpen doImport nil 
                        doSave doSaveAs nil doPrint
                        nil finish)
             receiver:self.

    menu at:2
            putLabels:(resources array:#(
                                        'copy' 
                                        'cut' 
                                        'paste'))
            selectors:#(copySelection deleteSelection pasteBuffer)
             receiver:drawView.

    menu at:3
            putLabels:(resources array:#(
                                        'larger' 
                                        'smaller' 
                                        '-'
                                        'normal' 
                                        'bold' 
                                        'italic' 
                                        'bold italic' 
                                        '-'
                                        'font panel'))
            selectors:#(largerFont
                        smallerFont
                        nil
                        normalFont
                        boldFont
                        italicFont
                        boldItalicFont
                        nil
                        showFontPanel)
             receiver:drawView.

    (Language == #english) ifTrue:[
        menu at:4
                putLabels:(resources array:#(
                                            'grid on' 
                                            'show grid'
                                            '-'
                                            'millimeter'
                                            'hide ruler'))
                selectors:#(alignOn showGrid nil millimeterMetric scaleOff)
                 receiver:self
    ] ifFalse:[
        menu at:4
                putLabels:(resources array:#(
                                            'grid on' 
                                            'show grid'
                                            '-'
                                            'inches'
                                            'hide ruler'))
                selectors:#(alignOn showGrid nil inchMetric scaleOff)
                 receiver:self
    ].
    formMenu := menu menuAt:4.

    menu at:5
            putLabels:(resources array:#(
                                        'to front' 
                                        'to back'
                                        '-' 
                                        'flip horizontal' 
                                        'flip vertical' 
                                        'rotate cw' 
                                        'rotate ccw' 
                                        '-' 
                                        'align left' 
                                        'align right' 
                                        'align top' 
                                        'align bottom' 
                                        '-' 
                                        'select all' 
                                        '-' 
                                        'group' 
                                        'ungroup'))
            selectors:#(selectionToFront selectionToBack
                        nil flipHorizontal flipVertical
                            rotate90 rotate270
                        nil selectionAlignLeft selectionAlignRight
                            selectionAlignTop selectionAlignBottom
                        nil selectAll
                        nil group ungroup)
             receiver:drawView.

    menu at:6
            putMenu:(PatternMenu
                        patterns:(self class listOfPatternsInMenu)
                        selectors:#changeFillPattern:
                        args:(self class listOfPatterns)
                        receiver:drawView
                        for:menu).

    menu at:7
            putLabels:#('1' 
                        '2' 
                        '3' 
                        '4' 
                        '5' 
                        '6' 
                        '7' 
                        '8' 
                        '9' 
                        '10' 
                        '15' 
                        '20')
            selectors:#changeLineWidth:
                 args:#(1 2 3 4 5 6 7 8 9 10 15 20)
             receiver:drawView.

    menu at:8
            putMenu:(PatternMenu
                        patterns:(self class listOfPatternsInMenu)
                        selectors:#changeLinePattern:
                        args:(self class listOfPatterns)
                        receiver:drawView
                        for:menu).

    menu at:9
            putMenu:(PatternMenu
                        patterns:(self class listOfColors)
                        selectors:#changeForeground:
                        args:(self class listOfColors)
                        receiver:drawView
                        for:menu).

    menu at:10
            putMenu:(PatternMenu
                        patterns:(self class listOfColors)
                        selectors:#changeBackground:
                        args:(self class listOfColors)
                        receiver:drawView
                        for:menu).

    family := resources at:'FONTFAMILY' default:'helvetica'.
    face := resources at:'FONTFACE' default:'medium'.
    style := resources at:'FONTSTYLE' default:'roman'.
    size := resources at:'FONTSIZE' default:14.
    f := Font family:family
                face:face
               style:style
                size:size.
    f notNil ifTrue:[
        menu font:f.
        1 to:10 do:[:i |
            (menu menuAt:i) font:f
        ]
    ]

! !

!DrawTool methodsFor:'user interaction'!

finish
    topView destroy.
    fileBox notNil ifTrue:[fileBox destroy. fileBox := nil].
!

doNew
    drawView removeAll
!

openFile:aFileName
    |aStream baseName|

    aStream := FileStream readonlyFileNamed:aFileName.
    aStream notNil ifTrue:[
        drawView fileInContentsFrom:aStream.
        aStream close.
        currentFileName := aFileName.
        baseName := OperatingSystem baseNameOf:aFileName.
        topView label:baseName , ' - ' , (OperatingSystem directoryNameOf:aFileName).
        topView iconName:baseName
    ]
!

doOpen
    fileBox isNil ifTrue:[
        fileBox := FileSelectionBox
                        title:''
                        okText:''
                        abortText:(resources string:'abort')
                        action:[nil].
        fileBox pattern:'*.draw'.
    ].
    fileBox title:(resources string:'open file:').
    fileBox action:[:fileName | self openFile:fileName].
    fileBox okText:(resources string:'open').

    currentFileName notNil ifTrue:[
        (currentFileName startsWith:'/') ifTrue:[
            fileBox directory:(OperatingSystem directoryNameOf:currentFileName)
        ]
    ].
    fileBox showAtPointer
!

doClose
    ^ self
!

saveAs:aFileName
    |aStream|

    aStream := FileStream newFileNamed:aFileName.
    aStream notNil ifTrue:[
        drawView storeContentsOn:aStream.
        aStream close
    ].
    currentFileName := aFileName
!

doSave
    currentFileName notNil ifTrue:[
        self saveAs:currentFileName
    ] ifFalse:[
        self doSaveAs
    ]
!

doSaveAs
    fileBox isNil ifTrue:[
        fileBox := FileSelectionBox
                        title:''
                        okText:''
                        abortText:(resources string:'abort')
                        action:[nil].
        fileBox pattern:'*.draw'.
    ].
    fileBox title:(resources string:'save as:').
    fileBox action:[:fileName | self saveAs:fileName].
    fileBox okText:(resources string:'save').
    fileBox showAtPointer
!

import:aFileName
    |img c|

    c := drawView cursor.
    drawView cursor:Cursor wait.
    [
        ObjectMemory allocationFailureSignal handle:[:ex |
             self warn:'no memory for image'.
             ex return
        ] do:[
            img := Image fromFile:aFileName.
        ].
        img notNil ifTrue:[
            drawView addSelected:(DrawImage new image:img).
        ]
    ] valueNowOrOnUnwindDo:[
        drawView cursor:c
    ].
!

doImport
    fileBox isNil ifTrue:[
        fileBox := FileSelectionBox
                        title:''
                       okText:''
                    abortText:(resources string:'abort')
                       action:[nil].
        fileBox pattern:'*'.
    ].
    fileBox title:(resources string:'Filename of image:').
    fileBox action:[:fileName | self import:fileName].
    fileBox okText:(resources string:'import').
    fileBox showAtPointer
!

doPrint
    |gc printStream|

    (Smalltalk includesKey:#PSGraphicsContext) ifFalse:[
        ^ self warn:'not implemented (need PSGraphicsContext)'
    ].
    PSGraphicsContext isNil ifTrue:[
        ^ self warn:'not implemented (need PSGraphicsContext)'
    ].
    (Printer == PostscriptPrinterStream) ifFalse:[
        ^ self warn:'currently only postscript printing supported'
    ].

    printStream := PipeStream writingTo:(Printer printCommand).

    gc := PSGraphicsContext on:printStream extent:(drawView widthOfContents @ drawView heightOfContents).
    [
        drawView redrawObjectsOn:gc
    ] valueNowOrOnUnwindDo:[gc close].
!

scaleOn
    formMenu labelAt:5 put:(resources string:'hide ruler').
    formMenu selectorAt:5 put:#scaleOff.
    drawView showScale
!

scaleOff
    formMenu labelAt:5 put:(resources string:'show ruler').
    formMenu selectorAt:5 put:#scaleOn.
    drawView hideScale
!

millimeterMetric
    formMenu labelAt:4 put:(resources name:'inches').
    formMenu selectorAt:4 put:#inchMetric.
    ruler metric:#mm.
    drawView millimeterMetric.
    ^ self
!

inchMetric    
    formMenu labelAt:4 put:(resources string:'millimeter').
    formMenu selectorAt:4 put:#millimeterMetric.
    ruler metric:#inch.
    drawView inchMetric.
    ^ self
!

alignOn
    formMenu labelAt:1 put:(resources string:'grid off').
    formMenu selectorAt:1 put:#alignOff.
    drawView alignOn
!

alignOff
    formMenu labelAt:1 put:(resources string:'grid on').
    formMenu selectorAt:1 put:#alignOn.
    drawView alignOff
!

showGrid
    formMenu labelAt:2 put:(resources string:'hide grid').
    formMenu selectorAt:2 put:#hideGrid.
    drawView showGrid.
    gridShown := true
!

hideGrid
    formMenu labelAt:2 put:(resources string:'show grid').
    formMenu selectorAt:2 put:#showGrid.
    drawView hideGrid.
    gridShown := false
! !
