"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ObjectView subclass:#DrawView
         instanceVariableNames:'createFillPattern createLinePattern createLineWidth
                createFontFamily createFontFace createFontStyle createFontSize
                createForeground createBackground pressAction2 releaseAction2
                resizedObject resizeSelector movedHandle createdObject
                createClass firstDrag startPoint prevPoint
                handlePositionBeforeMove frameBeforeHandleMove fontPanel textCursor'
         classVariableNames:''
         poolDictionaries:''
         category:'Applications-DrawTool'
!

DrawView comment:'
COPYRIGHT (c) 1989-93 by Claus Gittinger
              All Rights Reserved

$Header: /claus/smalltalk/st2.6/clients/DrawTool/RCS/DrawView.st,v 1.3 94/01/13 00:42:36 claus Exp $
written summer 89 by claus
'!

!DrawView class methodsFor:'defaults'!

hitDelta
    "region around edges and handles which snaps onto it when clicked upon"

    ^ (Display horizontalPixelPerMillimeter * 0.5) rounded asInteger
!

handleSize
    "size of blob drawn for handles"

    ^ (Display horizontalPixelPerMillimeter * 1.2) rounded asInteger
! !

!DrawView methodsFor:'initialization'!

initialize
    super initialize.
    createFillPattern := nil.
    createLinePattern := Form black "fromFile:'black.xbm'" .
    createLineWidth := 1.
    createForeground := Black.
    createBackground := White.
    createFontFamily := 'times'.
    createFontFace := 'medium'.
    createFontStyle := 'roman'.
    createFontSize := 14.
    self setupStandardActions.
    textCursor := Cursor text
!

setupStandardActions
    "set actions for pointing"

    pressAction := [:pressPoint | self startSelectOrMove:pressPoint].
    shiftPressAction := [:pressPoint | self startSelectMoreOrMove:pressPoint].
    motionAction := [:movePoint | true].
    releaseAction := [true].
    keyPressAction := [:key | self doKeyInput:key].
    pressAction2 := [:pressPoint | true].
    releaseAction2 := [true]
!

initEvents
    super initEvents.
    self enableKeyEvents
!

destroy
    fontPanel notNil ifTrue:[fontPanel destroy. fontPanel := nil].
    super destroy
! !

!DrawView methodsFor:'selection & handles'!

handleFor:aPoint
    |hsize centerX centerY|

    hsize := self class handleSize.
    centerX := aPoint x - self viewOrigin x.
    centerY := aPoint y - self viewOrigin y.
    ^ Rectangle left:(centerX - hsize)
                 top:(centerY - hsize)
               right:(centerX + hsize)
              bottom:(centerY + hsize)
!

handle:handlePoint isHitBy:aPoint
    ^ (self handleFor:handlePoint) containsPoint:aPoint
!

selectionHandlesDo:aBlock
    self forEach:selection do:[:theObject |
        theObject handlesDo:[:theHandle |
            aBlock value:theObject value:theHandle
        ]
    ]
!

drawHandle:aPoint
    |hsize halfSize offsX offsY|

    offsX := self viewOrigin x.
    offsY := self viewOrigin y.
    hsize := self class handleSize.
    halfSize := hsize // 2.
    self fillRectangleX:(aPoint x - offsX - halfSize)
                      y:(aPoint y - offsY - halfSize)
                  width:hsize height:hsize
!

drawHandlesFor:anObject
    |hsize halfSize offsX offsY|

    offsX := self viewOrigin x.
    offsY := self viewOrigin y.
    hsize := self class handleSize.
    halfSize := hsize // 2.
    anObject handlesDo:[:point |
        self fillRectangleX:(point x - offsX - halfSize)
                          y:(point y - offsY - halfSize)
                      width:hsize height:hsize
    ]
!

invertHandle:aHandle
    self xoring:[self drawHandle:aHandle]
!

invertHandlesOf:aSelection
    aSelection notNil ifTrue:[
        self xoring:[
            self forEach:aSelection do:[:anObject |
                self drawHandlesFor:anObject
            ]
        ]
    ]
!

showSelection
    self invertHandlesOf:selection
!

hideSelection
    self invertHandlesOf:selection
!

show:anObject
    "since objectView's show goes either to showSelected or showUnselected,
     we have to redefine this here to draw every object normal; then
     when draw operation is finished, show handles"

    anObject drawIn:self
!

showSelected:anObject
    self invertHandlesOf:anObject
!

showUnselected:anObject
    anObject drawIn:self
!

object:anObject hasHandleAt:aPoint
    |found|

    found := false.
    anObject handlesDo:[:handlePoint |
        (self handle:handlePoint isHitBy:aPoint) ifTrue:[
            found := true
        ]
    ].
    ^ found
!

findObjectHandleAt:aPoint
    |objectFound|
    contents do:[:object |
        (self object:object hasHandle:aPoint) ifTrue:[
            objectFound := object
        ]
    ].
    ^ objectFound
! !

!DrawView methodsFor:'drawing'!

redraw
    "have to redefine this here because of handles"

    self hideSelection.
    super redraw.
    self showSelection
!

redrawObjectsIn:aRectangle
    "have to redefine this here because of handles"

    self hideSelection.
    super redrawObjectsIn:aRectangle.
    self showSelection
!

redrawObjectsInVisible:aRectangle
    "have to redefine this here because of handles"

    self hideSelection.
    super redrawObjectsInVisible:aRectangle.
    self showSelection
!

invertOutlineOf:anObject
    self xoring:[anObject drawDragIn:self]
! !

!DrawView methodsFor:'draw-object initialization'!

setupCreatedObject
    createdObject fillPattern:createFillPattern
                  linePattern:createLinePattern
                  lineWidth:createLineWidth
                  foreground:createForeground
                  background:createBackground
! !

!DrawView methodsFor:'interface to DrawTool'!

noCreate
    self setupStandardActions
!

createObjectWithClass:aClass
    self unselect.
    createClass := aClass.
    createClass needsKeyboardInput ifTrue:[
        "create text-like objects"
        pressAction := [:pressPoint | self startKeyInputCreate:pressPoint]
    ] ifFalse: [
        createClass needsMultiPointCreate ifTrue:[
            createClass needsAllMovePoints ifTrue:[
                "create free-hand drawings"
                pressAction := [:pressPoint | 
                                self startFreeHandCreate:pressPoint]
            ] ifFalse:[
                "create multi-point objects"
                pressAction := [:pressPoint | 
                                self startMultiPointCreate:pressPoint]
            ]
        ] ifFalse: [
            "create standard (origin-corner) objects"
            pressAction := [:pressPoint | self startCreate:pressPoint]
        ]
    ].
    shiftPressAction := pressAction
! !

!DrawView methodsFor:'select or move'!

startSelectOrMove:aPoint
    |anObject aHandle|
    "if there is already a selection, look at handles first"
    selection notNil ifTrue:[
        self selectionHandlesDo:[:handleObject :handlePoint |
            (self handle:handlePoint isHitBy:aPoint) ifTrue:[
                anObject := handleObject.
                aHandle := handlePoint
            ]
        ].
        "if we found a handle, this starts a handleMove"
        aHandle notNil ifTrue:[
            (selection isKindOf:Collection) ifFalse:[
                selection hasFixedSize ifFalse:[
                    self startHandleMove:aHandle handleObject:anObject
                ]
            ].
"
            ((selection isKindOf:Collection)
             or:[selection hasFixedSize not]) ifTrue:[
                self startHandleMove:aHandle handleObject:anObject
            ].
"
            ^ self
        ]
    ].
    "otherwise, this starts an objectSelect or objectMove"
    super startSelectOrMove:aPoint
!

startHandleMove:aHandle handleObject:anObject
    aHandle notNil ifTrue:[
        resizeSelector := anObject selectorForHandle:aHandle.
        resizeSelector notNil ifTrue:[
            motionAction := [:movePoint | self doHandleMove:movePoint].
            releaseAction := [self endHandleMove].
            movedHandle := aHandle.
            frameBeforeHandleMove := self frameOf:anObject.
            handlePositionBeforeMove := (movedHandle x) @ (movedHandle y)
        ]
    ]
!

doHandleMove:aPoint
    |newPoint|
    newPoint := self alignToGrid:aPoint.
    newPoint = lastButt ifTrue:[^ self].

    movedHandle notNil ifTrue:[
        resizedObject isNil ifTrue:[
            resizedObject := selection.
            self unselect.
            self invertHandle:movedHandle
        ] ifFalse:[
            "invert prev. drag"
            self invertOutlineOf:resizedObject
        ].
        resizedObject perform:resizeSelector with:(newPoint + viewOrigin).
        self invertHandle:movedHandle.
        movedHandle x:(newPoint x + viewOrigin x) y:(newPoint y + viewOrigin y).
        self invertHandle:movedHandle.
        self invertOutlineOf:resizedObject.
        lastButt := newPoint
    ]
!

endHandleMove
    |frameAfterHandleMove|

    movedHandle notNil ifTrue:[
        resizedObject notNil ifTrue:[
            self invertHandle:movedHandle.
            self invertOutlineOf:resizedObject 
        ].
        (movedHandle = handlePositionBeforeMove) ifFalse:[
            frameAfterHandleMove := self frameOf:resizedObject.
            (frameBeforeHandleMove isContainedIn:frameAfterHandleMove) ifFalse:[
                (frameAfterHandleMove isContainedIn:frameBeforeHandleMove) ifFalse:[
                    self redrawObjectsIn:(frameBeforeHandleMove merge:frameAfterHandleMove)
                ] ifTrue:[
                    "object became smaller"
                    self redrawObjectsIn:frameBeforeHandleMove.
                ]
            ] ifTrue:[
                "object became bigger"
                self redrawObjectsIn:frameAfterHandleMove
            ]
        ].
        self select:resizedObject.
        self setDefaultActions.
        movedHandle := nil.
        resizedObject := nil
    ]
!

startKeyInputCreate:aPoint
    startPoint := self alignToGrid:aPoint.
    createClass notNil ifTrue:[
        pressAction := [:pressPoint | self endKeyInputCreate].
        shiftPressAction := pressAction.
        motionAction := [:movePoint | true].
        releaseAction := [true].
        keyPressAction := [:key | self doKeyInputCreate:key].
        oldCursor := cursor.
        self cursor:textCursor.
        self unselect.
        createdObject := createClass family:createFontFamily
                                       face:createFontFace
                                      style:createFontStyle
                                       size:createFontSize.
        createdObject origin:(aPoint + viewOrigin)
    ] ifFalse:[
        createdObject := nil
    ]
!

doKeyInputCreate:aKey
    createdObject notNil ifTrue:[
        self invertOutlineOf:createdObject.
        createdObject keyInput:aKey.
        self invertOutlineOf:createdObject
    ]
!

endKeyInputCreate
    createdObject notNil ifTrue:[
        self invertOutlineOf:createdObject.
        self setupCreatedObject.
        contents add:createdObject.
        createdObject drawIn:self.
        self select:createdObject.
        self cursor:oldCursor.
        createdObject := nil
"
        self setupStandardActions
"
    ]
!

doKeyInput:key
    |theObject oldFrame|
    selection notNil ifTrue:[
        (selection isKindOf:Collection) ifFalse:[
            theObject := selection.
            oldFrame := self frameOf:theObject.
            theObject keyInput:key.
            self redrawObjectsIn:oldFrame.
            (self isObscured:theObject) ifFalse:[
                theObject drawIn:self
            ] ifTrue:[
                self redrawObjectsIn:(self frameOf:theObject)
            ]
        ]
    ]
!

startFreeHandCreate:aPoint
    startPoint := self alignToGrid:aPoint.
    createClass notNil ifTrue:[
        self unselect.
        createdObject := createClass new.
        createdObject firstPoint:(aPoint + viewOrigin).
        prevPoint := startPoint.
        device compressMotionEvents:false.
        releaseAction := [self endFreeHandCreate].
        motionAction := [:movePoint | self doDragFreeHandCreate:movePoint]
    ] ifFalse:[
        createdObject := nil
    ]
!

doDragFreeHandCreate:movePoint
    createdObject notNil ifTrue:[
        self paint:Black.
        self displayLineFrom:prevPoint to:movePoint.
        createdObject addPoint:(movePoint + viewOrigin).
        prevPoint := movePoint
    ]
!

endFreeHandCreate
    device compressMotionEvents:true.
    createdObject notNil ifTrue:[
        self invertOutlineOf:createdObject.
        createdObject finalPoint.
        self setupCreatedObject.
        contents add:createdObject.
        createdObject drawIn:self.
        self select:createdObject.
        createdObject := nil
    ]
!

startMultiPointCreate:aPoint
    |p|

    startPoint := self alignToGrid:aPoint.
    createClass notNil ifTrue:[
        self unselect.
        self enableMotionEvents.
        p := startPoint + viewOrigin.
        createdObject := createClass new.
        createdObject firstPoint:p.
        createdObject addPoint:p.
        createdObject lastPointAt:p.
        pressAction := [:pressPoint | self doMultiPointCreate:pressPoint].
        shiftPressAction := pressAction.
        motionAction := [:movePoint | self doDragLastMultiPoint:movePoint].
        releaseAction := [true].
        pressAction2 := [:pressPoint | self endMultiPointCreate:pressPoint].
        self invertOutlineOf:createdObject
    ] ifFalse:[
        createdObject := nil
    ]

!

doDragLastMultiPoint:movePoint
    |p|

    createdObject notNil ifTrue:[
        self invertOutlineOf:createdObject.
        p := (self alignToGrid:movePoint) + viewOrigin.
        createdObject lastPointAt:p.
        self invertOutlineOf:createdObject
    ]

!

doMultiPointCreate:aPoint
    |p|

    createdObject notNil ifTrue:[
        p := (self alignToGrid:aPoint) + viewOrigin.
        self invertOutlineOf:createdObject.
        createdObject lastPointAt:p.
        createdObject addPoint:p.
        self invertOutlineOf:createdObject
    ]

!

endMultiPointCreate:aPoint
    |p|

    createdObject notNil ifTrue:[
        self disableMotionEvents.
        self invertOutlineOf:createdObject.
        p := (self alignToGrid:aPoint) + viewOrigin.
        createdObject finalPoint:p.
        self setupCreatedObject.
        contents add:createdObject.
        createdObject drawIn:self.
        self select:createdObject.
        createdObject := nil
    ]
"
    self setupStandardActions
"

!

startCreate:aPoint
    |p|

    createClass notNil ifTrue:[
        startPoint := self alignToGrid:aPoint.
        p := startPoint + viewOrigin.
        self unselect.
        createdObject := createClass new.
        createdObject hasFixedSize ifTrue:[
            createdObject origin:p
        ] ifFalse:[
            createdObject origin:p corner:p
        ].
        self invertOutlineOf:createdObject.
        motionAction := [:movePoint | self doDragCreate:movePoint].
        releaseAction := [self endCreate]
    ] ifFalse:[
        createdObject := nil
    ]
!

doDragCreate:aPoint
    createdObject notNil ifTrue:[
        createdObject hasFixedSize ifFalse:[
            self invertOutlineOf:createdObject.
            createdObject origin:(startPoint + viewOrigin)
                          corner:((self alignToGrid:aPoint) + viewOrigin).
            self invertOutlineOf:createdObject
        ]
    ]
!

endCreate
    createdObject notNil ifTrue:[
        self invertOutlineOf:createdObject.
        self setupCreatedObject.
        contents add:createdObject.
        createdObject drawIn:self.
        self select:createdObject.
        createdObject := nil
"
        self setupStandardActions
"
    ]
! !

!DrawView methodsFor:'misc'!

rectangleForScroll
    "find the area occupied by objects - use to minimize scroll area;
     has to be redefined since ObjectView does not know anything about handles which
     have to be included in the area"

    |rect hs t|

    rect := super rectangleForScroll.
    rect notNil ifTrue:[
        hs := self class handleSize // 2 + 1.
        t := rect left - hs.
        (t < 0) ifTrue:[t := 0].
        rect left:t.

        t := rect right + hs.
        (t > width) ifTrue:[t := width].
        rect right:t.

        t := rect top - hs.
        (t < 0) ifTrue:[t := 0].
        rect top:t.

        t := rect bottom + hs.
        (t > width) ifTrue:[t := height].
        rect bottom:t
    ].
    ^ rect
! !

!DrawView methodsFor:'manipulating objects'!

move:something by:delta
    "move an object or a collection of objects by some delta.
     have to redefine this because of handles"

    self withSelectionHiddenDo:[
        super move:something by:delta
    ]

!

group
    "make selection a group-object"

    |oldCollection newObject|

    selection notNil ifTrue:[
        (selection isKindOf:Collection) ifTrue:[
            oldCollection := selection.
            self unselect.
            self removeWithoutRedraw:oldCollection.
            newObject := DrawGroup new.
            self forEach:oldCollection do:[:theObject |
                newObject add:theObject
            ].
            self addWithoutRedraw:newObject.
            self select:newObject
        ]
    ]
!

ungroup
    "ungroup selection into individual objects"

    |objectsToUngroup objectsInThisGroup newSelection|

    selection notNil ifTrue:[
        newSelection := OrderedCollection new.
        objectsToUngroup := selection.
        self unselect.
        self removeWithoutRedraw:objectsToUngroup.
        self forEach:objectsToUngroup do:[:theObject |
            objectsInThisGroup := theObject groupedObjects.
            self forEach:objectsInThisGroup do:[:o |
                newSelection add:o
            ]
        ].
        self addWithoutRedraw:newSelection.
        self select:newSelection
    ]
!

performSelection:aSelector 
    "for every object in the selection, send a message to self
     with the object as argument
     - a helper for manipulation methods"

    |objectsToChange|

    selection notNil ifTrue:[
        objectsToChange := selection.
        self withSelectionHiddenDo:[
            self forEach:objectsToChange do:[:object |
                self perform:aSelector with:object
            ]
        ]
    ]

!

performSelection:aSelector with:anArgument
    "for every object in the selection, send a message to self
     with the object and anArgument as arguments
     - a helper for manipulation methods"

    |objectsToChange|

    selection notNil ifTrue:[
        objectsToChange := selection.
        self withSelectionHiddenDo:[
            self forEach:objectsToChange do:[:object |
                self perform:aSelector with:object with:anArgument
            ]
        ]
    ]

!

flipHorizontal
    "flip all selected objects vertical"

    self cursor:Cursor wait.
    self performSelection:#flipObjectHorizontal:.
    self cursor:Cursor normal.
!

flipVertical
    "flip all selected objects horizontal"

    self cursor:Cursor wait.
    self performSelection:#flipObjectVertical:.
    self cursor:Cursor normal.
!

rotate90
    "rotate all selected objects by 90 degrees clockwise"

    self cursor:Cursor wait.
    self performSelection:#rotate90Object:.
    self cursor:Cursor normal.
!

rotate270
    "rotate all selected objects by 90 degrees counter clockwise"

    self cursor:Cursor wait.
    self performSelection:#rotate270Object:.
    self cursor:Cursor normal.
!

changeFillPattern:aForm
    "if anything is selected, change fillpattern of selection to be
     aColor; otherwise set the default fillpattern for newly created
     objects"

    selection notNil ifTrue:[
        self performSelection:#changeFillPatternOf:to: with:aForm
    ] ifFalse:[
        createFillPattern := aForm
    ]
!

changeLinePattern:aForm
    "if anything is selected, change linepattern of selection to be
     aColor; otherwise set the default linepattern for newly created
     objects"

    selection notNil ifTrue:[
        self performSelection:#changeLinePatternOf:to: with:aForm
    ] ifFalse:[
        createLinePattern := aForm
    ]
!

changeLineWidth:aNumber
    "if anything is selected, change linewidth of selection to be
     aColor; otherwise set the default linewidth for newly created
     objects"

    selection notNil ifTrue:[
        self performSelection:#changeLineWidthOf:to: with:aNumber
    ] ifFalse:[
        createLineWidth := aNumber
    ]
!

changeForeground:aColor
    "if anything is selected, change foreground of selection to be
     aColor; otherwise set the default foreground color for newly created
     objects"

    selection notNil ifTrue:[
        self performSelection:#changeForegroundOf:to: with:aColor
    ] ifFalse:[
        createForeground := aColor
    ]
!

changeBackground:aColor
    "if anything is selected, change background of selection to be
     aColor; otherwise set the default background color for newly created
     objects"

    selection notNil ifTrue:[
        self performSelection:#changeBackgroundOf:to: with:aColor
    ] ifFalse:[
        createBackground := aColor
    ]
!

changeFontFamily:aFontName
    "if anything is selected, change font family of selection to be
     aColor; otherwise set the default font family for newly created
     objects"

    selection notNil ifTrue:[
        self performSelection:#changeFontFamilyOf:to: with:aFontName
    ] ifFalse:[
        createFontFamily := aFontName
    ]
!

changeFontFace:aFaceName
    selection notNil ifTrue:[
        self performSelection:#changeFontFaceOf:to: with:aFaceName
    ] ifFalse:[
        createFontFace := aFaceName
    ]
!

changeFontStyle:aFontStyle
    selection notNil ifTrue:[
        self performSelection:#changeFontStyleOf:to: with:aFontStyle
    ] ifFalse:[
        createFontStyle := aFontStyle
    ]
!

changeFontSize:aFontSize
    selection notNil ifTrue:[
        self performSelection:#changeFontSizeOf:to: with:aFontSize
    ] ifFalse:[
        createFontSize := aFontSize
    ]
!

with:anObject doFrameChangingBlock:aBlock
    "evaluate the block, aBlock for anObject and redraw what is needed"

    |oldFrame|

    oldFrame := self frameOf:anObject.
    aBlock value:anObject.
    self redrawObjectsIn:oldFrame.
    (self isObscured:anObject) ifFalse:[
        anObject drawIn:self
    ] ifTrue:[
        self redrawObjectsIn:(self frameOf:anObject)
    ]
!

flipObjectHorizontal:anObject
    "flip the object horizontally"

    self with:anObject doFrameChangingBlock:[:object | object flipHorizontal]
!

flipObjectVertical:anObject
    "flip the object vertically"

    self with:anObject doFrameChangingBlock:[:object | object flipVertical]
!

rotate90Object:anObject
    "rotate the object by 90 degrees clockwise"

    self with:anObject doFrameChangingBlock:[:object | object rotate:90]
!

rotate270Object:anObject
    "rotate the object by 90 degrees counter clockwise"

    self with:anObject doFrameChangingBlock:[:object | object rotate:270]
!

changeFillPatternOf:anObject to:aForm
    |oldFrame|

    "no change ?"
    aForm == (anObject fillPattern) ifTrue:[^ self].

    oldFrame := anObject frame.
    anObject fillPattern:aForm.

    "if object had a fill pattern - and is now going to loose it
     we must redraw everything under it"

    aForm isNil ifTrue:[
        self redrawObjectsIn:oldFrame.
        ^ self
    ].

    "if object is not covered, we must only redraw it"

    (self isObscured:anObject) ifFalse:[
        anObject drawIn:self.
        ^ self
    ].

    "must redraw the object, and everything above it"

    anObject drawIn:self.
    self redrawObjectsAbove:anObject in:oldFrame
!

changeLinePatternOf:anObject to:aForm
    |oldFrame|

    "no change ?"
    aForm == (anObject linePattern) ifTrue:[^ self].

    oldFrame := anObject frame.
    anObject linePattern:aForm.

    "if object had a line pattern - and is now going to loose it
     we must redraw everything under it"

    aForm isNil ifTrue:[
        self redrawObjectsIn:oldFrame.
        ^ self
    ].

    "if object is not covered, we must only redraw it"

    (self isObscured:anObject) ifFalse:[
        anObject drawIn:self.
        ^ self
    ].

    "must redraw the object, and everything above it"

    anObject drawIn:self.
    self redrawObjectsAbove:anObject in:oldFrame
!

changeLineWidthOf:anObject to:aNumber
    |oldFrame oldLineWidth|

    oldLineWidth := anObject lineWidth.

    "no change ?"
    aNumber == oldLineWidth ifTrue:[^ self].

    oldFrame := anObject frame.

    anObject lineWidth:aNumber.
    anObject linePattern isNil ifTrue:[^ self].

    "if new lineWidth is smaller, new frame is <= oldFrame
     we must redraw everything under it"

    (oldLineWidth > aNumber) ifTrue:[
        self redrawObjectsIn:oldFrame.
        ^ self
    ].

    "if object is not covered, we must only redraw it"

    (self isObscured:anObject) ifFalse:[
        anObject drawIn:self.
        ^ self
    ].

    "must redraw the object, and everything above it"

    anObject drawIn:self.
    self redrawObjectsAbove:anObject in:(anObject frame) "oldFrame"
!

changeForegroundOf:anObject to:aColor
    "change the foreground of anObject to aColor"

    "no change ?"
    aColor == anObject foreground ifTrue:[^ self].

    anObject foreground:aColor.

    "if object is not covered, we must only redraw it"

    (self isObscured:anObject) ifFalse:[
        anObject drawIn:self.
        ^ self
    ].

    "must redraw the object, and everything above it"

    anObject drawIn:self.
    self redrawObjectsAbove:anObject in:(anObject frame)
!

changeBackgroundOf:anObject to:aColor

    "no change ?"
    aColor == anObject background ifTrue:[^ self].

    anObject background:aColor.

    "if object is not covered, we must only redraw it"

    (self isObscured:anObject) ifFalse:[
        anObject drawIn:self.
        ^ self
    ].

    "must redraw the object, and everything above it"

    anObject drawIn:self.
    self redrawObjectsAbove:anObject in:(anObject frame)
!

doWithRedraw:aBlock to:anObject
    "helper method for font-change operations"

    |oldFrame|

    oldFrame := anObject frame.

    aBlock value:anObject.

    "if new frame is smaller, we must redraw everything under it"

    ((oldFrame width > anObject frame width)
    or:[oldFrame height > anObject frame height]) ifTrue:[
        self redrawObjectsIn:oldFrame.
        ^ self
    ].

    "if object is not covered, and we have a fillPattern,
     we must only redraw it"

    (self isObscured:anObject) ifFalse:[
        anObject fillPattern notNil ifTrue:[
            anObject drawIn:self.
            ^ self
        ]
    ].

    "must redraw everything in new frame"

    anObject drawIn:self.
    self redrawObjectsAbove:anObject in:(anObject frame)
!

changeFontFamilyOf:anObject to:aFontFamily
    self with:anObject doFrameChangingBlock:[:object | 
        object fontFamily:aFontFamily
    ]
!

changeFontFaceOf:anObject to:aFontFace
    self with:anObject doFrameChangingBlock:[:object | 
        object fontFace:aFontFace
    ]
!

changeFontStyleOf:anObject to:aFontStyle
    self with:anObject doFrameChangingBlock:[:object | 
        object fontStyle:aFontStyle
    ]
!

changeFontSizeOf:anObject to:aFontSize
    self with:anObject doFrameChangingBlock:[:object | 
        object fontSize:aFontSize
    ]
!

largerFont
    selection notNil ifTrue:[
        self performSelection:#largerFont:
    ] ifFalse:[
        createFontSize := createFontSize + 1
    ]

!

smallerFont
    selection notNil ifTrue:[
        self performSelection:#smallerFont:
    ] ifFalse:[
        createFontSize := createFontSize - 1
    ]

!

normalFont
    selection notNil ifTrue:[
        self performSelection:#normalFont:
    ] ifFalse:[
        createFontFace := 'medium'.
        createFontStyle := 'normal'
    ]

!

boldFont
    selection notNil ifTrue:[
        self performSelection:#boldFont:
    ] ifFalse:[
        createFontFace := 'bold'.
        createFontStyle := 'normal'
    ]

!

italicFont
    selection notNil ifTrue:[
        self performSelection:#italicFont:
    ] ifFalse:[
        createFontFace := 'medium'.
        createFontStyle := 'italic'
    ]
!

boldItalicFont
    selection notNil ifTrue:[
        self performSelection:#boldItalicFont:
    ] ifFalse:[
        createFontFace := 'bold'.
        createFontStyle := 'italic'
    ]
!

showFontPanel
    |action|

    fontPanel isNil ifTrue:[
        fontPanel := FontPanel new 
    ].

    selection notNil ifTrue:[
        action := [:family :face :style :size |
                       self performSelection:#changeFontFamily:face:style:size:of:
                                        with:family
                                        with:face
                                        with:style
                                        with:size
                  ]
    ] ifFalse:[
        action := [:family :face :style :size |
                       createFontFamily := family.
                       createFontFace := face.
                       createFontStyle := style.
                       createFontSize := size
                  ]
    ].

    fontPanel action:action.
    fontPanel showAtPointer
!

largerFont:anObject
    self with:anObject doFrameChangingBlock:[:object | 
        object fontSize:(object fontSize + 1)
    ]
!

smallerFont:anObject
    self with:anObject doFrameChangingBlock:[:object | 
        object fontSize:(object fontSize - 1)
    ]
!

normalFont:anObject
    self with:anObject doFrameChangingBlock:[:object | 
        object fontFace:'medium'.
        object fontStyle:'roman'
    ]
!

boldFont:anObject
    self with:anObject doFrameChangingBlock:[:object | 
        object fontFace:'bold'.
        object fontStyle:'roman'
    ]
!

italicFont:anObject
    self with:anObject doFrameChangingBlock:[:object | 
        object fontFace:'medium'.
        object fontStyle:'italic'
    ]
!

boldItalicFont:anObject
    self with:anObject doFrameChangingBlock:[:object | 
        object fontFace:'bold'.
        object fontStyle:'italic'
    ]
!

changeFontFamily:family face:face style:style size:size of:anObject
    self with:anObject doFrameChangingBlock:[:object | 
        object fontFamily:family face:face style:style size:size
    ]
!

performSelection:aSelector with:a1 with:a2 with:a3 with:a4
    "for every object in the selection, send a message to self
     with the object and anArgument as arguments
     - a helper for manipulation methods"

    |objectsToChange args|

    selection notNil ifTrue:[
        args := Array new:5.
        args at:1 put:a1.
        args at:2 put:a2.
        args at:3 put:a3.
        args at:4 put:a4.
        objectsToChange := selection.
        self withSelectionHiddenDo:[
            self forEach:objectsToChange do:[:object |
                args at:5 put:object.
                self perform:aSelector withArguments:args
            ]
        ]
    ]
! !

!DrawView methodsFor:'event handling'!

buttonPress:button x:x y:y
    (button == 2) ifTrue:[
        pressAction2 notNil ifTrue:[
            lastButt := x @ y.
            pressAction2 value:lastButt.
            ^ self
        ]
    ].
    super buttonPress:button x:x y:y
!

buttonRelease:button x:x y:y
    (button == 2) ifTrue: [
        releaseAction2 notNil ifTrue:[
            releaseAction2 value.
            ^ self
        ]
    ].
    super buttonRelease:button x:x y:y
!

keyPress:key x:x y:y
    ((key == #Delete) or:[key == #Cut]) ifTrue: [
        selection notNil ifTrue:[
            self deleteSelection
        ].
        ^ self
    ].
    (key == #Copy) ifTrue: [
        selection notNil ifTrue:[
            self copySelection
        ].
        ^ self
    ].
    (key == #Paste) ifTrue: [
        self pasteBuffer.
        ^ self
    ].
    keyPressAction notNil ifTrue:[
        keyPressAction value:key
    ]
! !
