"       NAME            Ellipse
        AUTHOR          Wilf R. LaLonde / John R. Pugh
        FUNCTION        Ellipses
        ST-VERSION      X2.5
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1
        DATE    3 Jun 1987
        SUMMARY
From 'Inside Smalltalk vol1.'
"

Path subclass: #Ellipse
        instanceVariableNames: 'semiMajorAxisLength semiMinorAxisLength
                                center'
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Graphics-Paths'
!

Ellipse comment:
'This class implements general ellipses given a center point and semiMinorAxis
and semiMajorAxis.'
!

!Ellipse methodsFor: 'accessing'!

center
    ^ center
!

center:aPoint
    center := aPoint
!

semiMinorAxis 
    ^ semiMinorAxisLength
!

semiMinorAxis:aLength
    semiMinorAxisLength := aLength
!

semiMajorAxis
    ^ semiMajorAxisLength
!

semiMajorAxis:aLength
    semiMajorAxisLength := aLength
!

center:aPoint semiMajorAxis:majorLength semiMinorAxis:minorLength
    center := aPoint.
    semiMajorAxisLength := majorLength.
    semiMinorAxisLength := minorLength
! !

!Ellipse methodsFor: 'display box access'!

computeBoundingBox
    ^ center - (semiMajorAxisLength @ semiMinorAxisLength)
             + (form width // 2)
      @
      form width asPoint
             + (2 * (semiMajorAxisLength @ semiMinorAxisLength))
! !

!Ellipse methodsFor: 'displaying'!

displayOn:aDisplayMedium at:aPoint clippingBox:clipRect rule:anInteger mask:aForm 
    |nSegments line angle sin cos
     xn yn xn1 yn1 ratio inverseRatio quadrant|

    nSegments := 48.0.
    line := Line new form:self form.
    angle := 360.0 / nSegments.
    sin := (angle * (2 * Float pi / 360)) sin.
    cos := (angle * (2 * Float pi / 360.0)) cos.
    ratio := semiMajorAxisLength / semiMinorAxisLength.
    inverseRatio := semiMinorAxisLength / semiMajorAxisLength.
    xn := semiMajorAxisLength asFloat.
    yn := 0.0.
    nSegments truncated timesRepeat:[
        xn1 := xn * cos + (ratio * yn * sin).
        yn1 := yn * cos - (inverseRatio * xn * sin).
        line beginPoint:center + (xn truncated @ yn truncated).
        line endPoint:center + (xn1 truncated @ yn1 truncated).
        line displayOn:aDisplayMedium at:aPoint clippingBox:clipRect
                  rule:anInteger mask:aForm.
        xn := xn1.
        yn := yn1
    ]
!

displayOn:aDisplayMedium transformation:aTransformation clippingBox:clipRect
     rule:anInteger mask:aForm 
    |newCenter e|
    newCenter := aTransformation applyTo:self center.
    e := Ellipse new.
    e center:newCenter truncated.
    e semiMajorAxis:(semiMajorAxisLength * aTransformation scale x) truncated.
    e semiMinorAxis:(semiMinorAxisLength * aTransformation scale y) truncated.
    e form:self form.
    e displayOn:aDisplayMedium at:0@0 clippingBox:clipRect rule:anInteger mask:aForm 
! !


" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
