"
 COPYRIGHT (c) 1993 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ArrayedCollection subclass:#ExternalBytes
       instanceVariableNames:'address'
       classVariableNames:''
       poolDictionaries:''
       category:'System-Support'
!

ExternalBytes comment:'

COPYRIGHT (c) 1993 by Claus Gittinger
              All Rights Reserved

This class provides access to any memory in the system. Its main purpose
is to provide a base-class for classes referencing external data. Use with
great care - access is not checked for out-of-bounds or valid addresses.
Also notice, that this class is not available on other smalltalk systems,
making code using it very unportable.

$Header: /claus/smalltalk/st2.6/libbasic/RCS/ExtBytes.st,v 1.5 94/02/25 12:57:21 claus Exp $

written jan 93 by claus
'!

!ExternalBytes class methodsFor:'instance creation'!

address:aNumber
    "return a new ExternalBytes object to access bytes starting at aNumber"

    ^ self basicNew setAddress:aNumber
!

newForText:numberOfBytes
    "allocate some memory usable to put machine executable instructions into;
     return a corresponding ExternalBytes object or nil.
     These bytes are NOT subject to garbage collection; therefore someone
     must take care of freeing the memory when its no longer needed"

    |addr|
%{
#ifdef mips
# include <scnhdr.h>
#endif

#ifndef SCNROUND
# define SCNROUND 0
#endif

#if defined(CAN_USE_DATA_FOR_TEXT)
    int space;
    /*
     * this one should allocate some space, into which we can put
     * some code to be executed. You are in bad luck, if running on
     * a SepId machine or if data space cannot be executed.
     * all you can do then, is to look for any OS-hooks or not run
     * the ObjectFileLoader.
     */
    space = (int)malloc(_intVal(numberOfBytes) + SCNROUND);
    if (space <= 0) {
        RETURN ( nil );
    }
    if (SCNROUND) {
        space &= ~(SCNROUND - 1);
        space += SCNROUND;
# ifdef SUPERDEBUG
        printf("rounding address to: %x\n", space);
# endif
    }
    addr = _MKSMALLINT(space);
#endif
%}
.
    addr notNil ifTrue:[
        ^ self address:addr
    ].
    ^ nil
!

newForData:numberOfBytes
    "allocate some memory usable for data;
     return a corresponding ExternalBytes object or nil (if malloc fails)"

    |addr|
%{
    int space;
    /*
     * this one is not at all critical - simply return some space
     * to be used for data space of loaded-in binaries.
     */
    space = (int)malloc(_intVal(numberOfBytes));
    if (space >= 0) {
        addr = _MKSMALLINT(space);
    }
%}
.
    addr notNil ifTrue:[
        ^ self address:addr
    ].
    ^ nil
! !

!ExternalBytes methodsFor:'freeing'!

free
    "free a previously allocated piece of memory - be very careful, there
     are no checks done here. All dangers you usually have with malloc/free
     are present here ..."

%{
    /* at least, we check for double freeing the same chunk */
    if (_isSmallInteger(_INST(address))) {
        free((char *)(_intVal(_INST(address))));
        _INST(address) = nil;
    }
%}
! !

!ExternalBytes methodsFor:'printing'!

displayString
    ^ 'ExternalBytes at:' , (address printStringRadix:16)
! !

!ExternalBytes methodsFor:'private accessing'!

setAddress:aNumber
    "set the start address"

    address := aNumber
! !

!ExternalBytes methodsFor:'accessing'!

address
    "return the start address"

    ^ address
!

basicSize
    "we do not know how many bytes are valid"

    ^ 0
!

basicAt:index
    "return the byte at index, anInteger;
     this is the byte at (address + index - 1)"

%{  /* NOCONTEXT */

    unsigned char *cp;

    if (_isSmallInteger(_INST(address))
     && _isSmallInteger(index) ) {
        cp = (unsigned char *)(_intVal(_INST(address)));
        cp = cp + _intVal(index) - 1;
        RETURN ( _MKSMALLINT(*cp) );
    }
%}
.
    self primitiveFailed
!

basicAt:index put:value
    "set the byte at index, anInteger to value which must be 0..255"

%{  /* NOCONTEXT */

    unsigned char *cp;
    int val;

    if (_isSmallInteger(_INST(address))
     && _isSmallInteger(index)
     && _isSmallInteger(value)) {
        val = _intVal(value);
        if ((val >= 0) && (val <= 255)) {
            cp = (unsigned char *)(_intVal(_INST(address)));
            cp = cp + _intVal(index) - 1;
            *cp = val;
            RETURN ( value );
        }
    }
%}
.
    self primitiveFailed
!

byteAt:index
    "return the unsigned byte at index, anInteger"

    ^ self basicAt:index
!

byteAt:index put:aByteValuedInteger
    "set the byte at index"

    ^ self basicAt:index put:aByteValuedInteger
!

wordAt:index
    "return the unsigned short at index, anInteger. fetching is MSB-first"

    ^ ((self at:index) bitShift:8) + (self at:(index + 1))
!

wordAt:index put:aNumber
    "set the short at index, index to aNumber; storing is MSB-first"

    self at:index       put:((aNumber bitShift:-8) bitAnd:16rFF).
    self at:(index + 1) put:(aNumber bitAnd:16rFF).
    ^ aNumber
!

doubleWordAt:index
    "return the unsigned long at index, anInteger. fetching is MSB-first"

    |val|

    val := self at:index.
    val := (val * 256) + (self at:(index + 1)).
    val := (val * 256) + (self at:(index + 2)).
    val := (val * 256) + (self at:(index + 3)).
    ^ val
!

doubleWordAt:index put:aNumber
    "set the long at index, index to aNumber; storing is MSB-first"

    self at:index       put:((aNumber bitShift:-24) bitAnd:16rFF).
    self at:index       put:((aNumber bitShift:-16) bitAnd:16rFF).
    self at:index       put:((aNumber bitShift:-8) bitAnd:16rFF).
    self at:(index + 1) put:(aNumber bitAnd:16rFF).
    ^ aNumber
! !
