"
 COPYRIGHT (c) 1991 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

StandardSystemView subclass:#FileBrowser
         instanceVariableNames:'labelView filterField fileListView subView 
                                currentDirectory
                                queryBox yesNoBox
                                topFrame fileList
                                checkBlock checkDelta timeOfLastCheck
                                showLongList showVeryLongList showDotFiles'
         classVariableNames:''
         poolDictionaries:''
         category:'Interface-Browsers'
!

FileBrowser comment:'
COPYRIGHT (c) 1991 by Claus Gittinger
              All Rights Reserved

a simple (but useful) FileBrowser

$Header: /claus/smalltalk/st2.6/libwidg/RCS/FBrowser.st,v 1.13 94/01/17 13:47:56 claus Exp $

written 91 by claus
'!

!FileBrowser class methodsFor:'instance creation'!

startOn:aDirectoryPath
    "start a new FileBrowser in a pathname"

    ^ (self new currentDirectory:aDirectoryPath) open

    "FileBrowser startOn:'aDirectoryPath'"
!

new
    "start a new FileBrowser in the currentDirectory"

    |newBrowser resources|

    resources := self classResources.
    newBrowser := super label:(resources string:self name).
    newBrowser icon:(Form fromFile:(resources at:'ICON_FILE' default:'FBrowser.xbm')
                        resolution:100).
    ^ newBrowser
! !

!FileBrowser methodsFor:'initialization'!

initialize
    |frame org spacing v|

    super initialize.

    spacing := ViewSpacing.

    checkBlock := [self checkIfDirectoryHasChanged].
    checkDelta := 5.

    currentDirectory := FileDirectory directoryNamed:'.'.
    showLongList := resources at:'LONG_LIST' default:false.
    showDotFiles := resources at:'SHOW_DOT_FILES' default:false.

    labelView := Label in:self.
    labelView origin:((spacing // 2) @ (spacing // 2))
              extent:[((width // 4 * 3) - spacing - borderWidth)
                       @
                       (font height + font descent)].
    labelView adjust:#right.
    self initializeLabelMiddleButtonMenu.

    filterField := EditField in:self.
    filterField origin:[((width // 4 * 3) + (spacing // 2)) @ (spacing // 2)]
                extent:[((width // 4) - borderWidth - (filterField margin) - (spacing // 2))
                       @
                       (font height + font descent + (filterField margin * 2))].
    self initializeFilterPattern.
    filterField leaveAction:[:key | fileListView scrollToTop. self updateCurrentDirectory].

    self is3D ifTrue:[
        org := 0
    ] ifFalse:[
        org := -1
    ].
    frame := VariableVerticalPanel
                 origin:[org @ (labelView height + labelView origin y + spacing)]
                 extent:[width
                         @
                         (height - spacing - labelView height
                                 - borderWidth)]
                     in:self.

    topFrame := ScrollableView for:SelectionInListView in:frame.
    topFrame origin:(0.0 @ 0.0) corner:(1.0 @ 0.3).

    fileListView := topFrame scrolledView.
    fileListView action:[:lineNr | self fileSelect:lineNr].
    fileListView doubleClickAction:[:lineNr | self fileSelect:lineNr.
                                              self fileGet].
    fileListView multipleSelectOk:true.

    v := self initializeSubViewIn:frame.
    v origin:(0.0 @ 0.3) corner:(1.0 @ 1.0).
    subView := v scrolledView.
    (subView respondsTo:#directoryForFileDialog:) ifTrue:[
        subView directoryForFileDialog:currentDirectory
    ].

    ObjectMemory addDependent:self.
!

initializeFilterPattern
    "set an initial matchpattern - can be redefined in subclasses"

    filterField contents:'*'
!

initializeSubViewIn:frame
    "set up the contents view - can be redefined in subclasses for
     different views (SoundFileBrowser)"

    ^ ScrollableView for:CodeView in:frame
!

currentDirectory:aDirectoryPath
    "set the directory to be browsed"

    currentDirectory := FileDirectory directoryNamed:aDirectoryPath.
    (subView respondsTo:#directoryForFileDialog:) ifTrue:[
        subView directoryForFileDialog:currentDirectory
    ]
!

realize
    self initializeFileListMiddleButtonMenu.
    super realize.
    self updateCurrentDirectory
!

initializeLabelMiddleButtonMenu
    |labels|

    labelView  notNil ifTrue:[
        labels := resources array:#(
                                   'copy path'
                                   '-'                               
                                   'change directory ...'
                                   'change to home-directory'
                                  ).             

        labelView
            middleButtonMenu:(PopUpMenu 
                                    labels:labels
                                 selectors:#(
                                             copyPath
                                             nil
                                             changeCurrentDirectory
                                             changeToHomeDirectory
                                            )
                                  receiver:self
                                       for:labelView)
    ]
!

initializeFileListMiddleButtonMenu
    |labels|

    fileListView  notNil ifTrue:[
        labels := resources array:#(
                                           'spawn'                   
                                           'get contents'                    
                                           'show info'             
                                           'show full info'          
                                           'fileIn'                 
                                           '-'                               
                                           'update'                 
                                           '-'                               
                                           'execute unix command ...'                
                                           '-'                               
                                           'remove'                 
                                           'rename ...'                 
                                           '-'                               
                                           'display long list'           
                                           'show all files'           
                                           '-'                               
                                           'create directory ...'         
                                           'create file ...').             

        fileListView
            middleButtonMenu:(PopUpMenu 
                                    labels:labels
                                 selectors:#(fileSpawn
                                             fileGet
                                             fileGetInfo
                                             fileGetLongInfo
                                             fileFileIn
                                             nil
                                             updateCurrentDirectory
                                             nil
                                             fileExecute
                                             nil
                                             fileRemove
                                             fileRename
                                             nil
                                             changeDisplayMode
                                             changeDotFileVisibility
                                             nil
                                             newDirectory
                                             newFile)
                                  receiver:self
                                       for:fileListView)
    ]
! !

!FileBrowser methodsFor:'private'!

withCursor:aCursor do:aBlock
    |oldFileListCursor oldCodeViewCursor|

    oldFileListCursor := fileListView cursor.
    oldCodeViewCursor := subView cursor.

    fileListView cursor:aCursor.
    subView cursor:aCursor.

    aBlock valueNowOrOnUnwindDo:[
        fileListView cursor:oldFileListCursor.
        subView cursor:oldCodeViewCursor
    ]
!

showAlert:aString
    "show an alertbox, displaying the last Unix-error"

    self warn:(aString , 
               '\\(' ,
               OperatingSystem lastErrorString ,
               ')' ) withCRs
!

ask:question yesButton:yesButtonText action:aBlock
    "common method to ask a yes/no question"

    self ask:question yesButton:yesButtonText noButton:'abort' action:aBlock
!

ask:question yesButton:yesButtonText noButton:noButtonText action:aBlock
    "common method to ask a yes/no question"

    "cache the box"
    yesNoBox isNil ifTrue:[
        yesNoBox := YesNoBox new
    ].
    yesNoBox title:question withCRs.
    yesNoBox okText:(resources at:yesButtonText).
    yesNoBox noText:(resources at:noButtonText).
    yesNoBox okAction:aBlock.
    yesNoBox showAtPointer
!

askIfModified:question yesButton:yesButtonText action:aBlock
    "tell user, that code has been modified - let her confirm"

    (subView modified not or:[subView contentsWasSaved]) ifTrue:[
        aBlock value.
        ^ self
    ].
    self ask:question yesButton:yesButtonText action:aBlock
!

withoutHiddenFiles:aCollection
    "remove hidden files (i.e. those that start with '.') from
     the list in aCollection"

    |newCollection|

    newCollection := aCollection species new.
    aCollection do:[:fname |
        ((fname startsWith:'.') and:[(fname = '..') not]) ifTrue:[
            showDotFiles ifTrue:[
                newCollection add:fname
            ]
        ] ifFalse:[
            newCollection add:fname
        ]
    ].
    ^ newCollection
!

getInfoFile
    "get filename of a description-file (.dir.info);
     uncomment stuff below if you want this to also
     automatically show contents of README files."

    #( '.dir.info'
"you can add these if you like ..."
"
       'README'
       'ReadMe'
       'Readme'
       'readme' 
"
    ) do:[:f |
        (currentDirectory isReadable:f) ifTrue:[^ f].
    ].
    ^ nil
!

showInfo:info
    "show directory info when dir has changed"

    info notNil ifTrue:[
        self show:(self readFile:info)
    ] ifFalse:[
        self show:nil.
    ]
!

getSelectedFileName
    "returns the currently selected file; shows an error if
     multiple files are selected"

    |sel|

    sel := fileListView selection.
    (sel isKindOf:Collection) ifTrue:[
        self onlyOneSelection
    ] ifFalse:[
        sel notNil ifTrue:[
            ^ fileList at:sel
        ]
    ].
    ^ nil
!

getFileInfoString:longInfo
    "get stat info on selected file - return a string which can be
     shown in a box"

    |fileName fullPath text info stream fileOutput type modeBits modeString s|

    fileName := self getSelectedFileName.
    fileName isNil ifTrue:[^ nil].

    info := currentDirectory infoOf:fileName.
    info isNil ifTrue:[
        self showAlert:(resources string:'cannot get info of ''%1'' !!' with:fileName).
        ^ nil
    ].

    text := Text new.
    type := info at:#type.
    (longInfo and:[type == #regular]) ifTrue:[
        fullPath := currentDirectory pathName , '/' , fileName.
        stream := PipeStream readingFrom:('file ' , fullPath).
        stream notNil ifTrue:[
            fileOutput := stream contents asString.
            stream close.
            fileOutput := fileOutput copyFrom:(fileOutput indexOf:$:) + 1.
            fileOutput := fileOutput withoutSeparators
        ]
    ].

    s := (resources at:'type:   ').
    fileOutput isNil ifTrue:[
        s := s ,  type asString
    ] ifFalse:[
        s := s , 'regular (' , fileOutput , ')'
    ].
    text add:s.
    text add:(resources at:'size:   ') , (info at:#size) printString.

    modeBits := (info at:#mode).
    modeString := self getModeString:modeBits.
    longInfo ifTrue:[
        text add:((resources at:'access: ')
                  , modeString 
                  , ' (' , (modeBits printStringRadix:8), ')' )
    ] ifFalse:[
        text add:(resources at:'access: ') , modeString
    ].
    text add:(resources at:'owner:  ')
             , (OperatingSystem getUserNameFromID:(info at:#uid)).
    longInfo ifTrue:[
      text add:(resources at:'group:  ')
             , (OperatingSystem getGroupNameFromID:(info at:#gid)).
      text add:(resources at:'last access:       ')
             , (info at:#accessTime) asTime printString
             , ' ' 
             , (info at:#accessTime) asDate printString.
      text add:(resources at:'last modification: ')
             , (info at:#modificationTime) asTime printString
             , ' ' 
             , (info at:#modificationTime) asDate printString.

    ].
    ^ text asString
!

getModeString:modeBits
    "convert file-mode bits into a more user-friendly string.
     This is wrong here - should be moved into OperatingSystem."

    |bits modeString|

    bits := modeBits bitAnd:8r777.
    modeString := ''.

    #( nil 8r400 8r200 8r100 nil 8r040 8r020 8r010 nil 8r004 8r002 8r001 ) 
    with: #( 'owner: ' $r $w $x ' group: ' $r $w $x ' others: ' $r $w $x ) do:[:bitMask :access |
        bitMask isNil ifTrue:[
            modeString := modeString , (resources string:access)
        ] ifFalse:[
            (bits bitAnd:bitMask) == 0 ifTrue:[
                modeString := modeString copyWith:$-
            ] ifFalse:[
                modeString := modeString copyWith:access
            ]
        ]
    ].
    ^ modeString
!

checkIfDirectoryHasChanged
    "every checkDelta secs, check if directoy has changed and update view if so"

    |oldSelection nOld|

    shown ifTrue:[
        (currentDirectory notNil) ifTrue:[
            (OperatingSystem isReadable:(currentDirectory pathName)) ifTrue:[
                (currentDirectory timeOfLastChange > timeOfLastCheck) ifTrue:[
                    nOld := fileListView numberOfSelections.
                    oldSelection := fileListView selectionValue.
                    self updateCurrentDirectory.
                    nOld ~~ 0 ifTrue:[
                        nOld > 1 ifTrue:[
                            oldSelection do:[:element  |
                                fileListView addElementToSelection:element
                            ]
                        ] ifFalse:[
                            fileListView selectElement:oldSelection
                        ]
                    ]
                ] ifFalse:[
                    Processor removeTimedBlock:checkBlock.
                    Processor addTimedBlock:checkBlock after:checkDelta
                ]
            ]
        ]
    ]
!

sizePrintString:size
    "helper for update-directory to return a string with a files size.
     This one gives the size in byte, Kb or Mb depending on size.
     If you dont like this, just uncomment the first statement below."

    |unitString sizeString|

"
    ^ size printString.
"
    unitString := ''.
    size < (500 * 1024) ifTrue:[
        size < (1024) ifTrue:[
            sizeString := size printString
        ] ifFalse:[
            sizeString := (size * 10 // 1024 / 10.0) printString.
            unitString := 'Kb'
        ]
    ] ifFalse:[
        sizeString := (size * 10 // 1024 // 1024 / 10.0) printString.
        unitString := 'Mb'
    ].
    (sizeString includes:$.) ifFalse:[
        sizeString := sizeString , '  '
    ].
    ^ (sizeString printStringLeftPaddedTo:5) , unitString.
!

updateCurrentDirectory
    "update listView with directory contents"

    |files text len line info modeString typ
     prevUid prevGid nameString groupString matchPattern|

    self withCursor:Cursor wait do:[
        labelView label:(currentDirectory pathName).
        timeOfLastCheck := Time now.

        matchPattern := filterField contents.
        (matchPattern notNil and:[
         matchPattern isEmpty not and:[
         matchPattern ~= '*']]) ifTrue:[
            files := currentDirectory asText.
            files := files select:[:aName | 
                         ((currentDirectory typeOf:aName) == #directory)
                         or:[matchPattern match:aName]
                     ].
            files sort
        ] ifFalse:[
            files := currentDirectory asText sort.
        ].

        files size == 0 ifTrue:[
            self notify:('directory ', currentDirectory pathName, ' vanished').
            ^ self
        ].
        files := self withoutHiddenFiles:files.

        fileList := files.
        showLongList ifTrue:[
            text := OrderedCollection new.
            files do:[:aFileName |
                len := aFileName size.
                (len < 20) ifTrue:[
                    line := aFileName , (String new:(22 - len))
                ] ifFalse:[
                    "can happen on BSD only"
                    line := (aFileName copyTo:20) , '  '
                ].
                info := currentDirectory infoOf:aFileName.
                info isNil ifTrue:[
                    "not accessable - usually a symlink,
                     which is not there/not readable
                    "
                    text add:line , '?  bad symbolic link'
                ] ifFalse:[
                    typ := (info at:#type) at:1.
                    (typ == $r) ifFalse:[
                        line := line , typ asString , '  '
                    ] ifTrue:[
                        line := line , '   '
                    ].

                    modeString := self getModeString:(info at:#mode).
                    line := line , modeString , '  '.

                    ((info at:#uid) ~~ prevUid) ifTrue:[
                        prevUid := (info at:#uid).
                        nameString := OperatingSystem getUserNameFromID:prevUid.
                        nameString := nameString , (String new:(10 - nameString size))
                    ].
                    line := line , nameString.
                    ((info at:#gid) ~~ prevGid) ifTrue:[
                        prevGid := (info at:#gid).
                        groupString := OperatingSystem getGroupNameFromID:prevGid.
                        groupString := groupString , 
                                                (String new:(10 - groupString size))
                    ].
                    line := line , groupString.

                    (typ == $r) ifTrue:[
                        line := line , (self sizePrintString:(info at:#size))
                    ].
                    text add:line
                ].
            ].
            fileListView setContents:text
        ] ifFalse:[
            files := files collect:[:aName |
                        (((currentDirectory typeOf:aName) == #directory) and:[
                        (aName ~= '..') and:[aName ~= '.']]) ifTrue:[
                            aName , ' ...'
                        ] ifFalse:[
                            aName
                        ]
                     ].
            fileListView setContents:files
        ].
        Processor removeTimedBlock:checkBlock.
        Processor addTimedBlock:checkBlock after:checkDelta
    ]
!

setCurrentDirectory:aPathName
    "setup for another directory"

    |newDirectory info|

    aPathName isEmpty ifTrue:[^ self].
    (currentDirectory isDirectory:aPathName) ifTrue:[
        newDirectory := FileDirectory directoryNamed:aPathName in:currentDirectory.
        newDirectory notNil ifTrue:[
            currentDirectory := newDirectory.
            fileListView contents:nil.
            self updateCurrentDirectory.
            info := self getInfoFile.
            self showInfo:info.
            (subView respondsTo:#directoryForFileDialog:) ifTrue:[
                subView directoryForFileDialog:currentDirectory
            ]
        ]
    ]
!

readFile:fileName
    "read in the file, answer its contents as Text"

    |stream text msg|

    stream := FileStream readonlyFileNamed:fileName in:currentDirectory.
    stream isNil ifTrue:[
        msg := (resources string:'cannot read file ''%1'' !!' with:fileName).
        self showAlert:msg.
        ^ nil
    ].
    text := stream contents.
    stream close.
    ^ text
!

writeFile:fileName text:aString
    |stream msg|

    self withCursor:(Cursor write) do:[
        stream := FileStream newFileNamed:fileName in:currentDirectory.
        stream isNil ifTrue:[
            msg := (resources string:'cannot write file ''%1'' !!' with:fileName).
            self showAlert:msg
        ] ifFalse:[
            stream nextPutAll:aString.
            stream close.
            subView modified:false
        ]
    ]
!

doCreateDirectory:newName
    (currentDirectory createDirectory:newName) ifTrue:[
        self updateCurrentDirectory
    ] ifFalse:[
        self showAlert:(resources string:'cannot create directory ''%1'' !!' with:newName).
    ]
!

doCreateFile:newName
    |aStream|

    aStream := FileStream newFileNamed:newName in:currentDirectory.
    aStream notNil ifTrue:[
        aStream close.
        self updateCurrentDirectory
    ] ifFalse:[
        self showAlert:(resources string:'cannot create file ''%1'' !!' with:newName).
    ]
!

showFile:fileName
    "show contents of fileName in subView"

    |buffer s n ok|

    ((currentDirectory typeOf:fileName) == #regular) ifFalse:[
        "clicked on something else - ignore it ..."
        self show:(resources string:'''%1'' is not a regular file' with:fileName).
        ^ self
    ].
    "check if file is a text file"
    buffer := String new:100.
    s := FileStream readonlyFileNamed:fileName in:currentDirectory.
    s isNil ifTrue:[
        self showAlert:(resources string:'cannot read file ''%1'' !!' with:fileName).
        ^ nil
    ].

    n := s nextBytes:100 into:buffer.
    ok := true.
    1 to:n do:[:i |
        (buffer at:i) isPrintable ifFalse:[ok := false].
    ].
    ok ifFalse:[
        (self confirm:(resources string:'''%1'' seems to be a binary file - continue anyway ?' with:fileName))
        ifFalse:[^ self]
    ].
    self show:(self readFile:fileName).
    subView acceptAction:[:theCode |
        self writeFile:fileName text:theCode
    ]
!

show:something
    "show something in subview and undef acceptAction"

    subView contents:something.
    subView acceptAction:nil.
    subView modified:false
!

doFileGet
    "get selected file - show contents in subView"

    |fileName|

    self withCursor:(Cursor read) do:[
        fileName := self getSelectedFileName.
        fileName notNil ifTrue:[
            (currentDirectory isDirectory:fileName) ifTrue:[
                self doChangeCurrentDirectoryTo:fileName.
                self label:(resources string:self class name)
            ] ifFalse:[
                self showFile:fileName.
                self label:((resources string:self class name) , ': ' , fileName)
            ]
        ]
    ]
!

doExecuteCommand:command
    "execute a unix command"

    |stream output line lnr curs|

    AnimatedCursor notNil ifTrue:[
        curs := AnimatedCursor wheel
    ] ifFalse:[
        curs := Cursor wait
    ].
    curs showIn:subView while:[
        self label:((resources string:self class name) , ': executing ...').
        output := Text new.
        stream := PipeStream readingFrom:('cd '
                                          , currentDirectory pathName
                                          , '; '
                                          , command).
        stream notNil ifTrue:[
            subView list:nil.
            lnr := 1.
            [stream atEnd] whileFalse:[
                (OperatingSystem selectOn:(stream fileDescriptor) withTimeOut:0) notNil ifTrue:[
                    line := stream nextLine.
                    subView at:lnr put:line.
                    lnr := lnr + 1.
                    Processor evaluateTimeouts
                ].
                [device eventPending] whileTrue:[
                    device dispatchEvent
                ].
                Processor evaluateTimeouts
            ].
            stream close.
            self checkIfDirectoryHasChanged
        ].
        subView modified:false.
        self label:(resources string:self class name).
    ]
!

initialCommandFor:fileName into:aBox
    "set a useful initial command for execute box.

     XXX should be changed to take stuff from a config file
     XXX or from resources."

    ((currentDirectory typeOf:fileName) == #regular) ifTrue:[

        (currentDirectory isExecutable:fileName) ifTrue:[
            aBox initialText:(fileName , '<arguments>').
            ^ self
        ].

        "some heuristics - my personal preferences ...
         (actually this should come from a configfile)"

        (fileName endsWith:'akefile') ifTrue:[
            aBox initialText:'make target' selectFrom:6 to:11.
            ^ self
        ].
        (fileName endsWith:'.tar.Z') ifTrue:[
            aBox initialText:'zcat ' , fileName , ' | tar tvf -'.
            ^ self
        ].
        (fileName endsWith:'.taz') ifTrue:[
            aBox initialText:'zcat ' , fileName , ' | tar tvf -'.
            ^ self
        ].
        (fileName endsWith:'.tar') ifTrue:[
            aBox initialText:'tar tvf ' , fileName selectFrom:1 to:7.
            ^ self
        ].
        (fileName endsWith:'.zoo') ifTrue:[
            aBox initialText:'zoo -list ' , fileName selectFrom:1 to:9.
            ^ self
        ].
        (fileName endsWith:'.zip') ifTrue:[
            aBox initialText:'unzip -l ' , fileName selectFrom:1 to:8.
            ^ self
        ].
        (fileName endsWith:'.Z') ifTrue:[
            aBox initialText:'uncompress ' , fileName selectFrom:1 to:10.
            ^ self
        ].
        (fileName endsWith:'.uue') ifTrue:[
            aBox initialText:'uudecode ' , fileName selectFrom:1 to:8.
            ^ self
        ].
        (fileName endsWith:'.xbm') ifTrue:[
            aBox initialText:'bitmap ' , fileName selectFrom:1 to:6.
            ^ self
        ].
        ((fileName endsWith:'.ps') or:[fileName endsWith:'.PS']) ifTrue:[
            aBox initialText:'gv ' , fileName selectFrom:1 to:2.
            ^ self
        ].
        ((fileName endsWith:'.1') 
        or:[fileName endsWith:'.man']) ifTrue:[
            aBox initialText:'nroff -man ' , fileName selectFrom:1 to:10.
            ^ self
        ].
        aBox initialText:'<cmd> ' , fileName selectFrom:1 to:5
    ]
!

doFileExecute
    "setup and launch a querybox to ask for unix command"

    |fileName|

    queryBox isNil ifTrue:[
        queryBox := EnterBox new
    ].
    queryBox initialText:''.

    fileName := self getSelectedFileName.
    fileName notNil ifTrue:[
        self initialCommandFor:fileName into:queryBox.
"
        ((currentDirectory typeOf:fileName) == #regular) ifTrue:[
            (currentDirectory isExecutable:fileName) ifTrue:[
                queryBox initialText:(fileName , '<arguments>')
            ] ifFalse:[
                queryBox initialText:('<cmd> ' , fileName)
            ]
        ]
"
    ].
    queryBox title:(resources at:'execute unix command:').
    queryBox okText:(resources at:'execute').
    "queryBox abortText:(resources at:'abort')." 
    queryBox action:[:command |
        self doExecuteCommand:command
    ].
    queryBox showAtPointer
!

doChangeToHomeDirectory
    "go to home directory"

    self doChangeCurrentDirectoryTo:(OperatingSystem getHomeDirectory)
!

doChangeCurrentDirectoryTo:fileName
    "verify argument is name of a readable & executable directory
     and if so, go there"

    |msg|

    fileName notNil ifTrue:[
        (currentDirectory isDirectory:fileName) ifTrue:[
            (currentDirectory isReadable:fileName) ifTrue:[
                (currentDirectory isExecutable:fileName) ifTrue:[
                    ^ self setCurrentDirectory:fileName
                ].
                msg := (resources string:'cannot change directory to ''%1'' !!' with:fileName)
            ] ifFalse:[
                msg := (resources string:'cannot read directory ''%1'' !!' with:fileName)
            ]
        ] ifFalse:[
            msg := (resources string:'''%1'' is not a directory !!' with:fileName)
        ].
        self showAlert:msg
    ]
!

selectedFilesDo:aBlock
    |sel files|

    sel := fileListView selection.
    sel notNil ifTrue:[
        (sel isKindOf:Collection) ifTrue:[
            files := sel collect:[:index | fileList at:index].
            files do:[:aFile |
                aBlock value:aFile
            ]
        ] ifFalse:[
            aBlock value:(fileList at:sel)
        ]
    ]

!

doRename:oldName to:newName
    (oldName notNil and:[newName notNil]) ifTrue:[
        (oldName isBlank or:[newName isBlank]) ifFalse:[
            currentDirectory renameFile:oldName newName:newName.
            self updateCurrentDirectory.
"
            self checkIfDirectoryHasChanged
"
        ]
    ]
!

doRemove
    "remove the selected file(s) - no questions asked"

    |ok msg dir|

    self withCursor:(Cursor execute) do:[
        self selectedFilesDo:[:fileName |
            (currentDirectory isDirectory:fileName) ifTrue:[
                dir := FileDirectory directoryNamed:fileName in:currentDirectory.
                dir isEmpty ifFalse:[
                    self ask:(resources string:'directory ''%1'' is not empty\remove anyway ?' with:fileName)
                         yesButton:'remove'
                         action:[currentDirectory removeDirectory:fileName]
                ] ifTrue:[
                    currentDirectory removeDirectory:fileName
                ]
            ] ifFalse:[
                ok := currentDirectory remove:fileName.
                ok ifFalse:[
                    "was not able to remove it"
                    msg := (resources string:'cannot remove ''%1'' !!' with:fileName).
                    self showAlert:msg
                ] ifTrue:[
                    self updateCurrentDirectory.
"
                    self show:nil
"
                ]
            ]
        ]
    ]
!

onlyOneSelection
    "show a warning, that only one file must be selected for
     this operation"

    self warn:(resources at:'exactly one file must be selected !!')
! !

!FileBrowser methodsFor:'user interaction'!

fileSpawn
    "start another FileBrowser on the selected directory or
     on the same directory if none is selected."

    |any|

    any := false.
    self selectedFilesDo:[:fileName |
        (currentDirectory isDirectory:fileName) ifTrue:[
            self class startOn:(currentDirectory pathName , '/' , fileName).
            any := true
        ]
    ].
    any ifFalse:[
        self class startOn:currentDirectory pathName
    ]
!

copyPath
    "copy current path into cut & paste buffer"

    Smalltalk at:#CopyBuffer put:(currentDirectory pathName)
!

fileExecute
    "if text was modified show a queryBox,
     otherwise pop up execute box immediately"

    self askIfModified:(resources at:'contents has not been saved.\\Modifications will be lost when command is executed.') withCRs
             yesButton:(resources at:'execute')
             action:[self doFileExecute]
!

fileSelect:lineNr
    "selected a file - do nothing here"
    ^ self
!

fileGet
    "if text was modified show an queryBox,
     otherwise get it immediately"

    |fileName msg label|

    (subView modified not or:[subView contentsWasSaved]) ifTrue:[^ self doFileGet].
    fileName := self getSelectedFileName.
    fileName notNil ifTrue:[
        (currentDirectory isDirectory:fileName) ifTrue:[
            msg := (resources at:'contents has not been saved.\\Modifications will be lost when directory is changed.').
            label := 'change'.
        ] ifFalse:[
            msg := (resources at:'contents has not been saved.\\Modifications will be lost when new file is read.').
            label := 'get'.
        ].
        self ask:msg yesButton:label action:[self doFileGet]
    ]
!

filePrint
    |fileName inStream printStream line|

    self withCursor:(Cursor execute) do:[
        fileName := self getSelectedFileName.
        fileName notNil ifTrue:[
            ((currentDirectory typeOf:fileName) == #regular) ifTrue:[
                inStream := FileStream readonlyFileNamed:fileName
                                                      in:currentDirectory.
                inStream isNil ifFalse:[
                    printStream := PrinterStream new.
                    printStream notNil ifTrue:[
                        [inStream atEnd] whileFalse:[
                            line := inStream nextLine.
                            printStream nextPutAll:line.
                            printStream cr
                        ].
                        printStream close
                    ].
                    inStream close
                ]
            ]
        ].
        0 "compiler hint"
    ]
!

fileFileIn
    "fileIn the selected file(s)"

    |aStream upd curs|

    curs := Cursor wait.
    curs showIn:subView while:[
        self selectedFilesDo:[:fileName |
            ((currentDirectory typeOf:fileName) == #regular) ifTrue:[
                (fileName endsWith:'.o') ifTrue:[
                    Object abortSignal catch:[
                        ObjectFileLoader loadObjectFile:(currentDirectory pathName , '/' ,fileName)
                    ]
                ] ifFalse:[
                    aStream := FileStream readonlyFileNamed:fileName in:currentDirectory.
                    aStream isNil ifFalse:[
                        upd := Class updateChanges:false.
                        [
                            Smalltalk systemPath addFirst:(currentDirectory pathName).
                            aStream fileIn.
                            Smalltalk systemPath removeFirst
                        ] valueNowOrOnUnwindDo:[
                            Class updateChanges:upd.
                            aStream close
                        ]
                    ]
                ]
            ]
        ]
    ]
!

fileRemove
    "remove the selected file(s).
     Query if user really wants to remove the file.
     - should be enhanced, to look for a ~/.trash directory 
     and move files there if it exists (without asking in this case)."

    |sel q|

    sel := fileListView selection.
    sel notNil ifTrue:[
        (sel isKindOf:Collection) ifTrue:[
            q := resources string:'remove selected files ?'
        ] ifFalse:[
            q := resources string:'remove ''%1'' ?' with:fileListView selectionValue
        ].
        self ask:q yesButton:'remove' action:[self doRemove]
    ]
!

newDirectory
    "ask for and create a new directory"

    queryBox isNil ifTrue:[
        queryBox := EnterBox new
    ].
    queryBox initialText:''.
    queryBox title:(resources at:'create new directory:') withCRs.
    queryBox okText:(resources at:'create').
    "queryBox abortText:(resources at:'abort')." 
    queryBox action:[:newName | self doCreateDirectory:newName].
    queryBox showAtPointer
!

newFile
    "ask for and create a new file"

    | sel |

    queryBox isNil ifTrue:[
        queryBox := EnterBox new
    ].
    sel := subView selection.
    sel notNil ifTrue:[
        queryBox initialText:(sel asString)
    ] ifFalse:[
        queryBox initialText:''
    ].
    queryBox title:(resources at:'create new file:') withCRs.
    queryBox okText:(resources at:'create').
    "queryBox abortText:(resources at:'abort')."
    queryBox action:[:newName | self doCreateFile:newName].
    queryBox showAtPointer
!

fileRename
    "rename the selected file(s)"

    queryBox isNil ifTrue:[
        queryBox := EnterBox new
    ].
    queryBox okText:(resources at:'rename').
    "queryBox abortText:(resources at:'abort')."
    self selectedFilesDo:[:oldName |
        queryBox title:(resources string:'rename ''%1'' to:' with:oldName).
        queryBox initialText:oldName.
        queryBox action:[:newName | self doRename:oldName to:newName].
        queryBox showAtPointer
    ]
!

terminate
    "exit FileBrowser"

    self askIfModified:(resources at:'contents has not been saved.\\Modifications will be lost when FileBrowser is closed.') withCRs
         yesButton:(resources at:'close')
         action:[self destroy]
!

destroy
    "destroy view and boxes"

    ObjectMemory removeDependent:self.
    Processor removeTimedBlock:checkBlock.
    checkBlock := nil.
    yesNoBox notNil ifTrue:[yesNoBox destroy. yesNoBox := nil].
    queryBox notNil ifTrue:[queryBox destroy. queryBox := nil].
    super destroy
!

update:what
    (what == #aboutToExit) ifTrue:[
        "system wants to shut down this
         - if text was modified, ask user and save if requested."

        (subView modified and:[subView contentsWasSaved not]) ifTrue:[
            shown ifFalse:[
                self unrealize.
                self realize
            ].
            self raise.
            self ask:(resources at:'FileBrowser:\\contents has not been saved.\\Save before exiting ?') withCRs
                 yesButton:'save'
                 noButton:'don''t save'
                 action:[
                        subView acceptAction notNil ifTrue:[
                            subView accept
                        ] ifFalse:[
                            subView save
                        ]
                    ]
        ].
        ^ self
    ].
    super update:what
!

changeCurrentDirectory
    "if text was modified show a queryBox, 
     otherwise change immediately to directory"

    self askIfModified:(resources at:'contents has not been saved.\\Modifications will be lost when directory is changed.') withCRs
         yesButton:(resources at:'change')
         action:[self queryForDirectoryToChange]
!

changeToHomeDirectory
    "if text was modified show a queryBox, 
     otherwise change immediately to directory"

    self askIfModified:(resources at:'contents has not been saved.\\Modifications will be lost when directory is changed.') withCRs
         yesButton:(resources at:'change')
         action:[self doChangeToHomeDirectory]
!

queryForDirectoryToChange
    "query for new directory"

    queryBox isNil ifTrue:[
        queryBox := EnterBox new
    ].
    queryBox initialText:''.
    queryBox title:(resources at:'change directory to:') withCRs.
    queryBox okText:(resources at:'change').
    "queryBox abortText:(resources at:'abort')."
    queryBox action:[:newName | self doChangeCurrentDirectoryTo:newName].
    queryBox showAtPointer
!

fileGetInfo:longInfo
    "get info on selected file - show it in a box"

    |string|

    string := self getFileInfoString:longInfo.
    string notNil ifTrue:[
        self information:string
    ]
!

fileGetLongInfo
    "triggered by menu: show long stat-info"

    self fileGetInfo:true
!

fileGetInfo
    "triggered by menu: show short stat-info"

    self fileGetInfo:false
!

changeDisplayMode
    "toggle from long to short listing (and vice-versa)"

    |long short|

    long := (resources at:'display long list').
    short := (resources at:'display short list').

    showLongList := showLongList not.
    showLongList ifFalse:[
        fileListView middleButtonMenu labelAt:short put:long
    ] ifTrue:[
        fileListView middleButtonMenu labelAt:long put:short
    ].
    self updateCurrentDirectory
!

changeDotFileVisibility
    "turn on/off visibility of files whose name starts with '.'"

    |show dontShow|

    show := (resources at:'show all files').
    dontShow := (resources at:'hide hidden files').

    showDotFiles := showDotFiles not.
    showDotFiles ifFalse:[
        fileListView middleButtonMenu labelAt:dontShow put:show
    ] ifTrue:[
        fileListView middleButtonMenu labelAt:show put:dontShow
    ].
    self updateCurrentDirectory
! !
