"
 COPYRIGHT (c) 1990 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

View subclass:#FormEditView
       instanceVariableNames:'form
			      dotWidth dotHeight
			      zeroColor oneColor'
       classVariableNames:''
       poolDictionaries:''
       category:'Interface-Inspector'
!

FormEditView comment:'
 a view to edit forms i.e. bit- and pixmaps

 @(#)FormEditV.st	1.2 92/02/03
 written feb 90 by claus
'!

!FormEditView class methodsFor:'staring an editor'!

editForm:aForm
    |topView editView|

    topView := StandardSystemView new.
    editView := self in:topView.
    editView form:aForm.
    topView extent:(editView extent).
    topView label:'Form editor'.
    topView realize.
    ^ editView
! !
    
!FormEditView methodsFor:'initialization'!

initialize
    super initialize.
    dotWidth := 16.
    dotHeight := 16.
    zeroColor := Form white.
    oneColor := Form black.
    ^ self
!

initializeMiddleButtonMenu
    |labels|

    (Language == #german) ifTrue:[
	labels := #(beenden)
    ] ifFalse:[
	labels := #(quit)
    ].
    self middleButtonMenu:(PopUpMenu
				  labels:labels
			       selectors:#(doQuit)
				receiver:self
				     for:self)

! !

!FormEditView methodsFor:'realization'!

realize
    super realize.
    self enableButtonEvents
! !

!FormEditView methodsFor:'accessing'!

form:aForm
    form := aForm.
    realized ifFalse:[
	self extent:(dotWidth * aForm width)
		    @
		    (dotHeight * aForm height)
    ].
    self redraw
! !

!FormEditView methodsFor:'editing'!

invertPixelRow:row col:col
    |pixel|
    (form depth == 1) ifTrue:[
	pixel := form at:(col @ row).
	form at:(col @ row) put:(1 - pixel).
	self redrawRow:row col:col
    ]
! !

!FormEditView methodsFor:'redrawing'!

redrawRow:row col:col
    |pixel|
    pixel := form at:(col @ row).
    (form depth == 1) ifTrue:[
	(pixel == 0) ifTrue:[
	    pixel := zeroColor
	] ifFalse:[
	    pixel := oneColor
	]
    ].
    self fillRectangleX:(col * dotWidth)
	              y:(row * dotHeight)
		  width:dotWidth
		 height:dotHeight
		   with:pixel
!

redrawRow:row
    |ypos xpos formDepth pixel prevPixel prevXpos prevCount|
    ypos := row * dotHeight.
    formDepth := form depth.
    xpos := 0.
    prevCount := 0.
    0 to:(form width - 1) do:[:col |
	pixel := form at:(col @ row).
	(formDepth == 1) ifTrue:[
	    (pixel == 0) ifTrue:[
	        pixel := zeroColor
	    ] ifFalse:[
	        pixel := oneColor
	    ]
	].
	(pixel ~~ prevPixel) ifTrue:[
	    (prevCount ~~ 0) ifTrue:[
	        self fillRectangleX:prevXpos
			          y:ypos
			      width:(dotWidth * prevCount)
			     height:dotHeight
			       with:prevPixel
	    ].
	    prevCount := 0.
	    prevPixel := pixel.
	    prevXpos := xpos
	].
	prevCount := prevCount + 1.
	xpos := xpos + dotWidth
    ].

    (prevCount ~~ 0) ifTrue:[
        self fillRectangleX:prevXpos
		          y:ypos
		      width:(dotWidth * prevCount)
		     height:dotHeight
		       with:prevPixel
    ]
! 

redraw
    realized ifFalse:[^ self].
    form isNil ifTrue:[^ self].
    0 to:(form height - 1) do:[:row |
	self redrawRow:row
    ]
! !

!FormEditView methodsFor:'user interaction'!

doQuit
    self destroy
!

leftButtonPressX:x y:y
    |row col|
    row := y // dotHeight.
    col := x // dotWidth.
    self invertPixelRow:row col:col
! !
