GLXView subclass:#GLObjectDemoView 
         instanceVariableNames:'trans angleX angleY angleZ deltaTrans deltaRot minTrans maxTrans redrawBlock 
                                deltaT points rotX rotY rotZ transZ
                                objectId'
         classVariableNames:''
         poolDictionaries:''
         category:'Views-3D'
!

GLObjectDemoView comment:'
abstract class for rotating objects

start with:
    GLCubeDemoView start
    GLTetraDemoView start
'!

!GLObjectDemoView class methodsFor:'defaults'!

defaultExtent
    ^ 300 @ 300
! !

!GLObjectDemoView methodsFor:'initialization'!

initialize
    super initialize.
    type := #colorIndexDoubleBuffer.
    redrawBlock := [self animate].

    deltaT := 0.02.

    angleX := angleY := angleZ := 0.0.
    deltaRot := 10.0.
    trans := -5.0.
    deltaTrans := 0.2.
    minTrans := -20.0.
    maxTrans := -3.0.

    rotX := true.
    rotY := true.
    rotZ := true.
    transZ := true.

    self setupPoints
!

unrealize 
    Processor removeTimedBlock:redrawBlock.
    super unrealize.
    KnownObjects remove:objectId
!

realize

    super realize.

    self perspectiveFovy:450 aspect:1.0 near:1.0 far:200.0. 

    objectId := KnownObjects size + 1.
    KnownObjects add:objectId.
    self makeObject:objectId.
    self drawObject.
    self closeObject.

    Processor addTimedBlock:redrawBlock after:deltaT
!

initializeMiddleButtonMenu
    self middleButtonMenu:(PopUpMenu labels:#('\c rotate x'
                                              '\c rotate y'
                                              '\c rotate z'
                                              '-'
                                              '\c translate Z'
                                             )
                                  selectors:#(rotX: rotY: rotZ: nil transZ:)
                                   receiver:self
                                        for:self).
    middleButtonMenu checkToggleAt:#rotX: put:true.
    middleButtonMenu checkToggleAt:#rotY: put:true.
    middleButtonMenu checkToggleAt:#rotZ: put:true.
    middleButtonMenu checkToggleAt:#transZ: put:true.
! !

!GLObjectDemoView methodsFor:'menu actions'!

transZ:aBoolean
    transZ := aBoolean
!

rotX:aBoolean
    rotX := aBoolean
!

rotY:aBoolean
    rotY := aBoolean
!

rotZ:aBoolean
    rotZ := aBoolean
! !

!GLObjectDemoView methodsFor:'drawing'!

drawObject
    self subclassResponsibility
!

animate
    rotX ifTrue:[angleX := angleX + 3.0].
    rotY ifTrue:[angleY := angleY + 3.0].
    rotZ ifTrue:[angleZ := angleZ + 3.0].
    transZ ifTrue:[
        trans := trans + deltaTrans.
        ((trans > maxTrans) or:[trans < minTrans]) ifTrue:[
            deltaTrans := deltaTrans negated
        ].
    ].
    self redraw.
!

redraw 
    realized ifFalse:[^ self].

    self color:Black.
    self clear.

    self pushMatrix.

    self translateX:0.0 y:0.0 z:trans.
    self rotateX:angleX y:angleY z:angleZ.

    self callObject:objectId.

    self popMatrix.
    self swapBuffers.

    Processor addTimedBlock:redrawBlock after:deltaT
! !
