GLXView subclass:#GLSphereDemoView1 
         instanceVariableNames:'angle redrawBlock deltaT rotLightMatrix sphere
                                lightPos light model material
                               '
         classVariableNames:''
         poolDictionaries:''
         category:'Views-3D'
!

GLSphereDemoView1 comment:'
a sphere and a light rotating around - without double buffer.
corresponds to a demo (in c) found in examples

start with:
    GLSphereDemoView1 start
'!

!GLSphereDemoView1 class methodsFor:'startup'!

start
    "check if display has rgb features"

    Display supportsRGB ifFalse:[
        self warn:'sorry, but this demo requires SGI hardware'
    ] ifTrue:[
        ^ super start
    ]
! !

!GLSphereDemoView1 class methodsFor:'defaults'!

defaultExtent
    ^ 200 @ 200
! !

!GLSphereDemoView1 methodsFor:'initialization'!

initialize
    super initialize.
    type := #rgbSingleBuffer.
    redrawBlock := [angle := angle + 3.0. self redraw].

    angle := 0.0.
    sphere := FloatArray with:0.0 with:0.0 with:0.0 with:50.0.
    deltaT := 0.05.
    rotLightMatrix := IdentityMatrix copy.

    lightPos := FloatArray withAll:#( 102.0 "POSITION"     0.0 0.0 1.0 0.0 
                                        0.0 "END" ).

    light :=    FloatArray withAll:#(   2.0 "AMBIENT"      0.1 0.1 0.1
                                      101.0 "LCOLOR"       0.5 1.0 1.0
                                      102.0 "POSITION"     90.0 90.0 150.0 0.0
                                        0.0 "END" ).


    model :=    FloatArray withAll:#(   2.0 "AMBIENT"      1.0 1.0 1.0
                                      201.0 "LOCALVIEWER"  0.0
                                        0.0 "END" ).

    material := FloatArray withAll:#(   5.0 "SHININESS"    30.0
                                        4.0 "SPECULAR"     0.2 0.2 0.2
                                        0.0 "END" ).

!

unrealize 
    Processor removeTimedBlock:redrawBlock.
    super unrealize.
!

realize
    super realize.
    Processor addTimedBlock:redrawBlock after:deltaT
! !

!GLSphereDemoView1 methodsFor:'drawing'!

redraw 
    self lmdef:#light      index:1 np:0 props:light.
    self lmdef:#material   index:1 np:0 props:material.
    self lmdef:#lightModel index:1 np:0 props:model.

    self mMode:#viewing.

    self perspectiveFovy:450 aspect:1.0 near:1.0 far:200.0.

    self lmbind:#material   index:1.
    self lmbind:#light0     index:1.
    self lmbind:#lightModel index:1.

    self colorRed:0 green:0 blue:0.
    self clear.

    self pushMatrix.
    self loadMatrix:IdentityMatrix.
    self rotate:6 axis:#x.
    self rotate:8 axis:#y.
    self multMatrix:rotLightMatrix.
    self lmdef:#light index:1 np:0 props:lightPos.
    self getMatrix:rotLightMatrix.
    self popMatrix.

    self pushMatrix.
    self translateX:0 y:0 z:-150.
    self colorRed:255 green:255 blue:255.
    self sphDraw:sphere.

    self popMatrix.

    Processor addTimedBlock:redrawBlock after:deltaT

! !
