'From Smalltalk/X, Version:2.6.5 on 13-Jun-1993 at 8:50:47'!

View subclass:#GLXView
         instanceVariableNames:'type'
         classVariableNames:'IdentityMatrix
                             Black Red Green Yellow Blue Magenta Cyan White
                             KnownObjects'
         poolDictionaries:''
         category:'Views-3D'
!

GLXView comment:'
This view (and therefore all subclasses) support GLX type of
3D rendering. This is a prototype implementation with just a
few things implemented; to show how things could be and what is
possible.

GLXViews will be useless on non IRIS workstations.
(however, things could be simulated here in theory ... :-)

written jun 93 by claus''
'!

!GLXView class methodsFor:'initialization'!

initialize
    super initialize.
    IdentityMatrix := FloatArray withAll:#(  1.0 0.0 0.0 0.0
                                             0.0 1.0 0.0 0.0
                                             0.0 0.0 1.0 0.0 
                                             0.0 0.0 0.0 1.0 ).
    Black := 0.
    Red := 1.
    Green := 2.
    Yellow := 3.
    Blue := 4.
    Magenta := 5.
    Cyan := 6.
    White := 7.

    KnownObjects isNil ifTrue:[
        KnownObjects := OrderedCollection new.
    ]
! !

!GLXView class methodsFor:'start'!

start
    |top v|

    (Display isKindOf:GLXWorkstation) ifFalse:[
        self warn:'sorry, but this view requires a display capable of GL drawing'.
        ^ nil
    ].
    top := StandardSystemView new.
    top extent:(self defaultExtent).
    top minExtent:(self defaultExtent).
    top maxExtent:(self defaultExtent).
    top label:'a GLXView'.
    v := self in:top.
    v origin:0@0 extent:1.0@1.0.
    top realize.
    ^ v
! !

!GLXView methodsFor:'initialization'!

initialize
    super initialize.
    type := #colorIndexSingleBuffer
! !

!GLXView methodsFor:'realization'!

create
    "redefined create, to make a GLX window instead of a normal
     one (should extract the createWindow from View and just
     redefine that on to get rid of all the other stuff here)"

    drawableId isNil ifTrue:[
        "make certain, superview is created also"

        superView notNil ifTrue:[
            superView id isNil ifTrue:[
                superView create
            ]
        ].

        "associate cursor/colors to device"

        viewBackground := viewBackground on:device.
        borderColor := borderColor on:device.
        shadowColor := shadowColor on:device.
        lightColor := lightColor on:device.
        cursor := cursor on:device.

        drawableId := device createGLXWindowFor:self 
                                        left:left top:top 
                                       width:width height:height
                                        type:type.

        extentChanged := false.
        originChanged := false.

        borderShape notNil ifTrue:[
            device setWindowBorderShape:(borderShape id) in:drawableId
        ].
        viewShape notNil ifTrue:[
            device setWindowShape:(viewShape id) in:drawableId
        ].
        backed notNil ifTrue:[
            device setBackingStore:backed in:drawableId
        ].
        saveUnder ifTrue:[
            device setSaveUnder:true in:drawableId
        ].

        font := font on:device.
"
        self inputOnly ifFalse:[
            self initGC
        ].
"
        self initializeMiddleButtonMenu.
        self initEvents.
        controller notNil ifTrue:[
            controller created:self
        ]
    ].

    "GLXView new realize"
!

destroy
    "view is about to be destroyed - unlink from GLX first"

    drawableId notNil ifTrue:[
        device unlinkGLXView:drawableId.
    ].
    super destroy
!

realize
    super realize.
    self enableEvent:#expose
!

recreate
    "recreate (i.e. tell X about me) after a snapin"

    drawableId isNil ifTrue:[
        "associate cursor/colors to device"

        super recreate.

        borderColor := borderColor on:device.
        shadowColor := shadowColor on:device.
        lightColor := lightColor on:device.

        foreground notNil ifTrue:[
            foreground := foreground on:device
        ].
        background notNil ifTrue:[
            background := background on:device
        ].
        paint notNil ifTrue:[
            paint := paint on:device
        ].
        cursor := cursor on:device.

        drawableId := device createGLXWindowFor:self 
                                           left:left top:top 
                                          width:width height:height
                                           type:type.

        extentChanged := false.
        originChanged := false.

        borderShape notNil ifTrue:[
            device setWindowBorderShape:(borderShape id) in:drawableId
        ].
        viewShape notNil ifTrue:[
            device setWindowShape:(viewShape id) in:drawableId
        ].
        backed notNil ifTrue:[
            device setBackingStore:backed in:drawableId
        ].
        saveUnder ifTrue:[
            device setSaveUnder:saveUnder in:drawableId
        ].

        font := font on:device.

        "XXX has to be changed: eventmasks are device specific -
         will not allow restart on another Workstation-type"

        device setEventMask:eventMask in:drawableId
    ]
! !

!GLXView methodsFor:'low level data transfer'!

n3f:arrayOf3FloatsWithNormal
    "load vertex normal"

    ^ device glxN3f:arrayOf3FloatsWithNormal in:drawableId
!

c3f:arrayOf3FloatsWithColor
    "load color vector"

    ^ device glxC3f:arrayOf3FloatsWithColor in:drawableId
!

v3f:arrayOf3FloatsWithVertex
    "load vertex"

    ^ device glxV3f:arrayOf3FloatsWithVertex in:drawableId
! !

!GLXView methodsFor:'low level objects'!

makeObject:id
    "start an object definition"

    ^ device glxMakeObject:id in:drawableId
!

closeObject
    "end an object definition"

    ^ device glxCloseObjectIn:drawableId
!

callObject:id
    "perform commands from an object"

    ^ device glxCallObject:id in:drawableId
! !

!GLXView methodsFor:'low level transformations'!

loadMatrix:arrayOf16Floats
    "load transformation matrix"

    ^ device glxLoadMatrix:arrayOf16Floats in:drawableId
!

multMatrix:arrayOf16Floats
    "multiplication matrix"

    ^ device glxMultMatrix:arrayOf16Floats in:drawableId
!

getMatrix:arrayOf16Floats
    "multiplication matrix"

    ^ device glxGetMatrix:arrayOf16Floats in:drawableId
!

perspectiveFovy:fovy aspect:aspect near:near far:far
    "define perspective projection"

    ^ device glxPerspectiveFovy:fovy aspect:aspect near:near far:far in:drawableId
!

translateX:x y:y z:z
    "translate current matrix"

    ^ device glxTranslateX:x y:y z:z  in:drawableId
!

scaleX:x y:y z:z
    "scale & mirror current matrix"

    ^ device glxScaleX:x y:y z:z in:drawableId
!

rotate:angle axis:axis
    "rotate the current matrix"

    ^ device glxRotate:angle axis:axis in:drawableId
!

rotateX:xAngle y:yAngle z:zAngle
    "rotate the current matrix"

    ^ device glxRotateX:xAngle y:yAngle z:zAngle in:drawableId
!

rotateX:angle
    "rotate the current matrix around x axis"

    ^ device glxRotateX:angle in:drawableId
!

rotateY:angle
    "rotate the current matrix around y axis"

    ^ device glxRotateY:angle in:drawableId
!

rotateZ:angle
    "rotate the current matrix around z axis"

    ^ device glxRotateZ:angle in:drawableId
!

lookatVx:vx vy:vy vz:vz px:px py:py pz:pz twist:twist
    "define viewing transformation"

    ^ device glxLookatVx:vx vy:vy vz:vz 
                      px:px py:py pz:pz 
                   twist:twist
                      in:drawableId
!

windowLeft:left right:right top:top bottom:bottom near:near far:far 
    "define viewing transformation"

    ^ device glxWindowLeft:left right:right 
                       top:top bottom:bottom 
                      near:near far:far 
                        in:drawableId
!

pushMatrix
    "push down transformation stack"

    ^ device glxPushmatrixIn:drawableId
!

popMatrix
    "pop transformation stack"

    ^ device glxPopmatrixIn:drawableId
! !

!GLXView methodsFor:'low level lights'!

lmdef:what index:index np:np props:props
    "define a material, light source or lighting model;
     props must be a FloatArray"

    ^ device glxLmdef:what index:index np:np props:props in:drawableId
!

lmbind:target index:index
    "select a material, lighyt or lighting model"

    ^ device glxLmbind:target index:index in:drawableId
! !

!GLXView methodsFor:'low level colors'!

color:index
    "set color, for non gouraud shading, we dont care if the
     argument is integer or float; otherwise, better results are
     expected with float values."

    ^ device glxColor:index in:drawableId
! 

colorRed:r green:g blue:b
    "set color, args must be integers"

    ^ device glxColorRed:r green:g blue:b in:drawableId
! !

!GLXView methodsFor:'low level polygons'!

move:a3DPoint
    "move to a position"

    ^ device glxMove:a3DPoint in:drawableId
!

moveX:x y:y z:z
    "move to a position"

    ^ device glxMoveX:x y:y z:z in:drawableId
!

drawX:x y:y z:z
    "draw to a position"

    ^ device glxDrawX:x y:y z:z in:drawableId
!

draw:a3DPoint
    "draw to a position"

    ^ device glxDraw:a3DPoint in:drawableId
!

rectangleOrigin:p1 corner:p2
    "draw a rectangle"

    ^ device glxRectX1:(p1 x) y1:(p1 y) x2:(p2 x) y2:(p2 y) in:drawableId
!

rectangleX1:x1 y1:y1 x2:x2 y2:y2
    "draw a rectangle"

    ^ device glxRectX1:x1 y1:y1 x2:x2 y2:y2 in:drawableId
!

beginPolygon 
    "start a polygon"

    ^ device glxBeginPolygonIn:drawableId
!

endPolygon 
    "end a polygon"

    ^ device glxEndPolygonIn:drawableId
! 

beginPoint 
    "start a point"

    ^ device glxBeginPointIn:drawableId
!

endPoint 
    "end a point"

    ^ device glxEndPointIn:drawableId
! 

beginClosedLine 
    "start a closed line"

    ^ device glxBeginClosedLineIn:drawableId
!

endClosedLine 
    "end a closed line"

    ^ device glxEndClosedLineIn:drawableId
! 

beginLine 
    "start a line group"

    ^ device glxBeginLineIn:drawableId
!

endLine 
    "end a line group"

    ^ device glxEndLineIn:drawableId
! 

beginTriangleMesh 
    "start a triangle mesh"

    ^ device glxBeginTriangleMeshIn:drawableId
!

endTriangleMesh 
    "end a triangle mesh"

    ^ device glxEndTriangleMeshIn:drawableId
! 

beginSurface 
    "start a NURBS surface def"

    ^ device glxBeginSurfaceIn:drawableId
!

endSurface
    "end a NURBS surface def"

    ^ device glxEndSurfaceIn:drawableId
! 

beginQuadrilateralStrip 
    "start a quadrilateral strip"

    ^ device glxBeginQuadrilateralStripIn:drawableId
!

endQuadrilateralStrip
    "end a quadrilateral strip"

    ^ device glxEndQuadrilateralStripIn:drawableId
! 

beginCurve 
    "start a NURBS curve def"

    ^ device glxBeginCurveIn:drawableId
!

endCurve
    "end a NURBS curve def"

    ^ device glxEndCurveIn:drawableId
! !

!GLXView methodsFor:'low level spheres'!

sphDraw:arrayOf4Floats
    "draw a sphere"

    ^ device glxSphDraw:arrayOf4Floats in:drawableId
! !

!GLXView methodsFor:'low level patches'!

defBasis:id mat:matrix
    ^ device glxDefBasis:id mat:matrix in:drawableId
!

patchCurvesU:u v:v
    ^ device glxPatchCurvesU:u v:v in:drawableId
!

patchBasisU:u v:v
    ^ device glxPatchBasisU:u v:v in:drawableId
!

patchPrecisionU:u v:v
    ^ device glxPatchPrecisionU:u v:v in:drawableId
!

patchX:arrayOf16XFloats y:arrayOf16YFloats z:arrayOf16ZFloats
    "draw a patch"

    ^ device glxPatchX:arrayOf16XFloats 
                     y:arrayOf16YFloats 
                     z:arrayOf16ZFloats in:drawableId
! !

!GLXView methodsFor:'low level misc'!

clear
    "clear to current color"

    ^ device glxClearIn:drawableId
!

zClear
    "clear z buffer"

    ^ device glxZClearIn:drawableId
!

backBuffer
    "switch to backbuffer"

    ^ device glxBackBufferIn:drawableId
!

frontBuffer
    "switch to frontbuffer"

    ^ device glxFrontBufferIn:drawableId
!

swapBuffers
    "swap double buffers"

    ^ device glxSwapBuffersIn:drawableId
!

rgbMode
    "set true color mode (no colormap)"

    ^ device glxRGBmodeIn:drawableId
!

doubleBuffer
    "set double buffer mode"

    ^ device glxDoubleBufferIn:drawableId
!

zBuffer:aBoolean
    "enable/disable z buffer mode"

    ^ device glxZbuffer:aBoolean in:drawableId
!

backFace:aBoolean
    "enable/disable backface"

    ^ device glxBackface:aBoolean in:drawableId
!

polyFill:aBoolean
    "enable/disable polygon filling"

    ^ device glxPolyFill:aBoolean in:drawableId
!

mMode:aSymbol 
    "enable/disable z buffer mode"

    ^ device glxMmode:aSymbol in:drawableId 
!

nMode:aSymbol 
    "set normal mode"

    ^ device glxNmode:aSymbol in:drawableId 
!

setConfig
    "must be sent after RGBmode, doubleBuffer etc. to have these
     changes really take effect. See GLX manual"

    ^ device glxGconfigIn:drawableId
! !

GLXView initialize!
