"       NAME            Paths
        AUTHOR          scaletti@uxc.cso.uiuc.edu (Kurt J. Hebel)
        FUNCTION        Reimplementation of the Path hierarchy
        ST-VERSION      2.2?
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1
        DATE    3 Jun 1987
"

AbstractPath subclass: #Line
        instanceVariableNames: 'beginPoint endPoint'
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Graphics-Paths'!
Line comment:
'A Line is made up of a beginning Point and an ending Point, in addition to the
 writing Form defined in the superclass.

 see example fileIn in the Demos directory'!

!Line methodsFor: 'point accessing'!

add: aPoint
        "Add this point to the end of the path."
        beginPoint isNil ifTrue: [beginPoint _ aPoint. ^ aPoint].
        endPoint isNil ifTrue: [endPoint _ aPoint. ^ aPoint].
        self indexOutOfRangeError.!

beginPoint
        "Answer the beginning point of the receiver."
        ^ beginPoint!

beginPoint: aPoint 
        "Set the beginning point of the receiver.  Answer the argument, aPoint."
        beginPoint _ aPoint.
        ^ aPoint!

endPoint
        "Answer the ending point of the receiver."
        ^ endPoint!

endPoint: aPoint 
        "Set the ending point of the receiver."
        endPoint _ aPoint.
        ^ aPoint!

pointAt: index
        "Return the point at the index given."
        index = 1 ifTrue: [^ beginPoint].
        index = 2 ifTrue: [^ endPoint].
        self indexOutOfRangeError.!

pointAt: index put: aPoint
        "Set the point at the index given."
        index = 1 ifTrue: [beginPoint _ aPoint. ^ aPoint].
        index = 2 ifTrue: [endPoint _ aPoint. ^ aPoint].
        self indexOutOfRangeError.!

points
        "Return the collection of points for this Path."
        ^ Array with: beginPoint with: endPoint!

points: aCollection
        "Set the collection of points for this Path."
        beginPoint _ nil. endPoint _ nil.
        aCollection do: [ :aPoint | self add: aPoint].! !

!Line methodsFor: 'transforming'!

scaleBy: aPoint 
        "Answer with a new Path scaled by aPoint.  Does not effect the current data
 in this Path."
        ^ self species
                from: beginPoint * aPoint
                to: endPoint * aPoint
                withForm: self form!

translateBy: aPoint 
        "Answer a new instance of Path whose elements are translated by aPoint.  Does
 not effect the elements of this Path."
        ^ self species
                from: beginPoint + aPoint
                to: endPoint + aPoint
                withForm: self form! !

!Line methodsFor: 'displaying'!

displayOn:aDisplayMedium at:aPoint clippingBox:clipRect rule:anInteger mask:aForm 
        "Display the receiver as a line connecting the beginning point with the ending
 point using the receiver's form at every point along the line.  Also, see comment
 in DisplayObject|displayOn:at:clippingBox:rule:mask:."

        (form isKindOf:DrawingPen) ifTrue:[
            aDisplayMedium lineWidth:form lineWidth.
            aDisplayMedium lineStyle:form lineStyle.
            aDisplayMedium capStyle:form capStyle.
            aDisplayMedium paint:form paint on:form dashPaint.
            aDisplayMedium displayLineFrom: (beginPoint + aPoint) rounded truncated
                                        to: (endPoint + aPoint) rounded truncated
        ] ifFalse:[
            aDisplayMedium
                displayLine: self form
                       from: beginPoint + aPoint
                       to: endPoint + aPoint
                       clippingBox: clipRect
                       rule: anInteger
                       mask: aForm
        ]
! !

!Line methodsFor: 'display box access'!

computeBoundingBox
        "Compute the smallest Rectangle that covers all of the points in the Path."

        | minPt maxPt |

        minPt _ beginPoint min: endPoint.
        maxPt _ beginPoint max: endPoint.
        ^ minPt corner: maxPt! !

!Line methodsFor: 'private'!

indexOutOfRangeError
        self error: 'Lines only have 2 points.  You tried to access more.'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!Line class methodsFor: 'instance creation'!

from: beginPoint to: endPoint withForm: aForm
        "Create a new Line with the given arguments."
        | newSelf | 

        newSelf _ self new.
        newSelf
                beginPoint: beginPoint;
                endPoint: endPoint;
                form: aForm.
        ^ newSelf! !


" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
