"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

DisplayObject subclass:#LogicObject
       instanceVariableNames:'origin
                              inputValues outputValues
                              oldOutputValues
                              changed changedPicture
                              name font id
                              delay'
       classVariableNames:''
       poolDictionaries:''
       category:'Interface-LogicTool'
!

LogicObject comment:'
 any gate

 $Header: /claus/smalltalk/st2.6/clients/LogicTool/RCS/LogicObj.st,v 1.1 94/01/13 00:45:42 claus Exp $
 written summer 89 by claus

 abstract superclass for all circuit elements; since all are gates
 with a little picture (icon) for it this is all done here.
 They only differ in how they evaluate their outputValues from the input
 values and where their pins are in the picture - these methods are
 defined in the specific subclasses (AndGate, OrGate etc.)"
'!


!LogicObject class methodsFor:'specification'!

delay
    ^ 10        "pico-seconds"
!

genericName
    ^ self name
!

dragOutline
    "if true, dragging is done by drawing outline only;
     if false, dragging is done by full draw (fast servers only)
     - can be redefined in subclasses"
    ^ true
!

icon
    ^ nil
!

iconFileName
    ^ self name , '.64'
!

iconExtent
    ^ 64 @ 64
!

numberOfInputs
    self subclassResponsibility
!

numberOfOutputs
    self subclassResponsibility
!

new
    |newGate|

    newGate := super new.
    newGate numberOfInputs:(self numberOfInputs).
    newGate numberOfOutputs:(self numberOfOutputs).
    newGate initialize.
    ^ newGate
! !

!LogicObject methodsFor:'initializing'!

initialize
    super initialize.
    changedPicture := false.
    changed := false
! !

!LogicObject methodsFor:'accessing'!

id
    ^ id
!

id:aNumber
    id := aNumber
!

name
    ^ name
!

name:aString
    name := aString.
    self recomputeFrame
!

font:aFont
    font := aFont.
    self recomputeFrame
!

changedPicture
    ^ changedPicture
!

unchangedPicture
    changedPicture := false
!

changed
    ^ changed
!

unchanged
    changed := false
!

icon
    ^ self class icon
!

numberOfInputs:aNumber
    inputValues := Array new:aNumber.
    inputValues atAllPut:true
!

numberOfOutputs:aNumber
    outputValues := Array new:aNumber.
    outputValues atAllPut:false.
    oldOutputValues := Array new:aNumber
!

numberOfInputs
    ^ inputValues size
!

numberOfOutputs
    ^ outputValues size
!

input:pin value:logicValue
    inputValues at:pin put:logicValue
!

output:pin
    ^ outputValues at:pin
!

hasFixedSize
    ^ true
!

origin
    ^ origin
!

origin:aPoint
    origin := aPoint.
    self recomputeFrame
!

moveTo:newOrigin
    origin := newOrigin.
    self recomputeFrame
!

recomputeFrame
    |icon iconWidth iconHeight textWidth textHeight myWidth myHeight|

    icon := self class icon.
    icon isNil ifTrue:[
        iconWidth := self width.
        iconHeight := self height
    ] ifFalse:[
        iconWidth := icon width.
        iconHeight := icon height
    ].
    textWidth := 0.
    textHeight := 0.
    font isNil ifFalse:[
        name isNil ifFalse:[
            textWidth := font widthOf:name.
            textHeight := font height
        ]
    ].
    myWidth := iconWidth max:textWidth.
    myHeight := iconHeight + 2 + textHeight.
    frame := Rectangle origin:origin extent:(myWidth @ myHeight)
!

isInput:aPoint
    "answer true, if aPoint is an input pin"

    ^ (self whichInput:aPoint) notNil
!

isOutput:aPoint
    "answer true, if aPoint is an output pin"

    ^ (self whichOutput:aPoint) notNil
!

pictureHeight
    |icon|

    icon := self class icon.
    icon isNil ifTrue:[
        ^ self height
    ].
    ^ icon height
!

inputPinLocation:inputNr
    "answer the x/y coordinate as point of an input"

    |delta|

    delta := self pictureHeight // (self numberOfInputs + 1).
    ^ (origin x) @ (origin y + (delta * inputNr))
!

outputPinLocation:inputNr
    "answer the x/y coordinate as point of an output"

    |delta|

    delta := self pictureHeight // (self numberOfOutputs + 1).
    ^ (origin x + self width - 1) @ (origin y + (delta * inputNr))
!

whichInput:aPoint

    "this code assumes, that all inputs are at the left of the picture,
     in height/(nInput+1) pixels space"

    |xpos ypos delta i n|

    delta := self pictureHeight // (self numberOfInputs + 1).
    xpos := origin x.
    ypos := origin y + delta.
    i := 1.
    n := self numberOfInputs.
    [i <= n] whileTrue:[
        (xpos == aPoint x) ifTrue:[
            (ypos == aPoint y) ifTrue:[
                ^ i
            ]
        ].
        ypos := ypos + delta.
        i := i + 1
    ].
    ^ nil
!

whichOutput:aPoint
    "this code assumes, that all outputs are at the right of the picture,
     in height/(nInput+1) pixels space"

    |xpos ypos delta i n|

    delta := self pictureHeight // (self numberOfOutputs + 1).
    xpos := origin x + self width - 1.
    ypos := origin y + delta.
    i := 1.
    n := self numberOfOutputs.
    [i <= n] whileTrue:[
        (xpos == aPoint x) ifTrue:[
            (ypos == aPoint y) ifTrue:[
                ^ i
            ]
        ].
        ypos := ypos + delta.
        i := i + 1
    ].
    ^ nil
!

inputPinsDo:aBlock
    "this code assumes, that all inputs are at the left of the picture,
     in height/(nInput+1) pixels space"

    |xpos ypos delta i n|

    delta := self pictureHeight // (self numberOfInputs + 1).
    xpos := origin x.
    ypos := origin y + delta.
    i := 1.
    n := self numberOfInputs.
    [i <= n] whileTrue:[
        aBlock value:(xpos @ ypos).
        ypos := ypos + delta.
        i := i + 1
    ]
!

outputPinsDo:aBlock
    "this code assumes, that all outputs are at the right of the picture,
     in height/(nOutput+1) pixels space"

    |xpos ypos delta i n|

    delta := self pictureHeight // (self numberOfOutputs + 1).
    xpos := origin x + self width - 1.
    ypos := origin y + delta.
    i := 1.
    n := self numberOfOutputs.
    [i <= n] whileTrue:[
        aBlock value:(xpos @ ypos).
        ypos := ypos + delta.
        i := i + 1
    ]
!

pinsDo:aBlock
    "evaluate aBlock for each pin"

    self inputPinsDo:aBlock.
    self outputPinsDo:aBlock
! !

!LogicObject methodsFor:'drawing'!

doDrawIn:aGC offset:anOffset
    |offs icon x y|

    offs := anOffset - aGC viewOrigin.
    icon := self icon.
    font isNil ifTrue:[
        font := aGC font
    ].
    font := font on:aGC device.
    x := origin x + offs x.
    y := origin y + offs y.
    aGC clearRectangleX:x y:y width:(frame width) height:(frame height).
    aGC displayOpaqueForm:icon x:x y:y.
    name notNil ifTrue:[
        aGC displayOpaqueString:name x:x
                                     y:(y + icon height + font ascent)
    ]
!

drawSelectedIn:aGC
    self drawSelectedIn:aGC offset:0@0
!

drawSelectedIn:aGC offset:anOffset
    "draw the receiver highlighted - this is usually redefined"

    aGC device hasColors ifTrue:[
        aGC paint:(Color red:100 green:0 blue:0).
        self doDrawIn:aGC offset:anOffset
    ] ifFalse:[
        self drawIn:aGC offset:anOffset.
        self drawOutlineIn:aGC offset:anOffset
    ]
!

drawIn:aGC
    self drawIn:aGC offset:0@0
!

drawIn:aGC offset:anOffset
    aGC paint:(Color black) on:(Color white).
    self doDrawIn:aGC offset:anOffset
! !

!LogicObject methodsFor:'simulation'!

click
    ^ self
!

freeze
    1 to:(self numberOfOutputs) do:[:index |
        oldOutputValues at:index put:(outputValues at:index)
    ]
!

evaluate
    self subclassResponsibility
! !

!LogicObject methodsFor:'storing'!

storeOn:aStream
    aStream nextPut:$(.
    aStream nextPutAll:(self class name).
    aStream nextPutAll:' new origin:'.  origin storeOn:aStream.
    aStream nextPutAll:'; name:'.  name storeOn:aStream.
    aStream nextPutAll:'; id:'.  id storeOn:aStream.
    aStream nextPutAll:'; yourself'.
    aStream nextPut:$)
! !
