"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Object subclass:#LogicTool
       instanceVariableNames:'topView logicView menu commandPanel
                              gridSize gridShown
                              openErrorBox createErrorBox
                              nameEnterBox subEnterBox
                              openBox saveBox currentFileName resources
                              '
       classVariableNames:''
       poolDictionaries:''
       category:'Interface-LogicTool'
!

LogicTool comment:'
 an interface to a logicView

 $Header: /claus/smalltalk/st2.6/clients/LogicTool/RCS/LogicTool.st,v 1.1 94/01/13 00:45:43 claus Exp $
 written summer 89 by claus
'!

!LogicTool class methodsFor:'defaults'!

listOfObjectClasses
    |list|
    list := OrderedCollection new.
    list add:Low.
    list add:High.
    list add:Input.
    list add:Output.
    list add:Inverter.
    list add:AndGate2.
    list add:NandGate2.
    list add:OrGate2.
    list add:NorGate2.
    list add:ExorGate2.
    list add:AndGate3.
    ^ list
! !

!LogicTool class methodsFor:'startup'!

startOn:aFileName
    "startup a LogicTool window, initialize contents from aFileName"

    ^ self new start openFile:aFileName

    "LogicTool startOn:'aFileName'"
!

start
    "startup a LogicTool window with empty contents"

    ^ self new start

    "LogicTool start"
! !

!LogicTool methodsFor:'startup'!

start
    |proto aString frame panelWidth|

    gridSize := 4.
    gridShown := false.
    resources := ResourcePack for:self class.


    topView := StandardSystemView
                 label:(self class name)
                 icon:(Form fromFile:(self class name , '.icon') resolution:100)
                 minExtent:(100 @ 100).

    menu := PullDownMenu in:topView.

    self initMenu.

    "create a proto button to get its dimension"

    proto := Button form:(Input icon).
    panelWidth := (proto width + 5) * 2 + 10.

    commandPanel := PanelView
                        origin:(5 @ (menu height + 5))
                        extent:[panelWidth
                                @
                                (topView height - menu height - 10)]
                        in:topView.
    proto := nil.
    commandPanel layout:#top.

    frame := HVScrollableView for:LogicView in:topView.
    frame origin:[(commandPanel left + commandPanel width + 5)
                  @
                  (menu height + 5)].
    frame extent:[(topView width - commandPanel width - 10)
                  @
                  (topView height - menu height - 10)].

    logicView := frame scrolledView.

    (self class listOfObjectClasses) do:[:elClass |
        Button form:(elClass icon)
             action:[logicView createObjectWithClass:elClass]
                 in:commandPanel
    ].

    openErrorBox := WarningBox new.
    createErrorBox := WarningBox new.

    openBox := FileSelectionBox
                    title:(resources string:'open file')
                    "pattern:'*.logic'"
                    action:[:fileName | self openFile:fileName].

    saveBox := FileSelectionBox
                    title:(resources string:'save file')
                    "pattern:'*.logic'"
                    action:[:fileName | self saveAs:fileName].

    nameEnterBox := EnterBox
                        title:(resources string:'enter name')
                        action:[:aString | self nameIt:aString].

    subEnterBox := EnterBox
                        title:(resources string:'select sub-circuit')
                        action:[:aString | self addSubCircuitFromFile:aString].
    topView open
! !

!LogicTool methodsFor:'menu definition'!

initMenu
    menu labels:(resources array:#('file' 
                                   'edit' 
"
                                   'form' 
"
                                   'simulation'
"
                                   'debugging'
"
                                  )).

    menu at:(resources string:'file')
            putLabels:(resources array:#('new' 
                                         'open ...' 
                                         'circuits ...'
                                         '-' 
                                         'save' 
                                         'save as ...' 
                                         '-' 
                                         'print'
                                         '-' 
                                         'quit'))
            selectors:#(doNew
                        doOpen
                        addSubCircuit
                        nil
                        doSave
                        doSaveAs
                        nil
                        doPrint
                        nil
                        finish)
             receiver:self.

    menu at:(resources string:'edit')
            putLabels:(resources array:#('copy' 
                                        'cut' 
                                        'paste'
                                        '-'
                                        'disconnect'
                                        '-'
                                        'group'
                                        '-'
                                        'name it'))
            selectors:#(copySelection
                        deleteSelection
                        pasteSelection
                        nil
                        disconnect
                        nil
                        group
                        nil
                        nameIt)
             receiver:self.

"
    menu at:(resources string:'form')
            putLabels:(resources array:#('grid on' 
                                         'grid size' 
                                         'show grid'))
            selectors:#(gridOn gridSize showGrid)
             receiver:self.
"
    menu at:(resources string:'simulation')
            putLabels:(resources array:#('start'
                                         'show values'))
            selectors:#(startSimulation showValues)
             receiver:self.

"
    menu at:(resources string:'debugging')
            putLabels:(resources array:#('logic-probe'
                                         '-'
                                         'logic-analyzer'))
            selectors:#(showProbe
                        nil
                        showAnalyzer)
             receiver:self.
"
! !

!LogicTool methodsFor:'user interaction'!

finish
    topView destroy.
    openErrorBox destroy.
    createErrorBox destroy.
    nameEnterBox destroy.
    subEnterBox destroy.
    openBox destroy.
    saveBox destroy
!

doNew
    logicView removeAll
!

openFile:aFileName
    |aStream|
    aStream := FileStream readonlyFileNamed:aFileName.
    aStream isNil ifTrue:[
        openErrorBox title:(resources string:'file %1 not found or not readable' with:aFileName).
        openErrorBox show
    ] ifFalse:[
        logicView fileInContentsFrom:aStream.
        aStream close
    ]
!

doOpen
    "self openFile:'.Draw'"
    openBox show
!

saveAs:aFileName
    |aStream|
    aStream := FileStream newFileNamed:aFileName.
    aStream isNil ifTrue:[
        createErrorBox title:(resources string:'cannot create file %1' with:aFileName).
        createErrorBox show
    ] ifFalse:[
        logicView storeContentsOn:aStream.
        aStream close
    ].
    currentFileName := aFileName
!

doSave
    currentFileName notNil ifTrue:[
        self saveAs:currentFileName
    ] ifFalse:[
        saveBox show
    ]
!

doSaveAs
    saveBox show
!

doPrint
    ^ self
!

showValues
    |simulationMenu|

    simulationMenu := menu menuAt:(resources string:'simulation').
    simulationMenu selectorAt:2 put:#hideValues.
    simulationMenu labelAt:2 put:(resources string:'hide values').
    logicView showValues
!

hideValues 
    |simulationMenu|

    simulationMenu := menu menuAt:(resources string:'simulation').
    simulationMenu selectorAt:2 put:#showValues.
    simulationMenu labelAt:2 put:(resources string:'show values').
    logicView  hideValues
!

startSimulation
    |simulationMenu|

    simulationMenu := menu menuAt:(resources string:'simulation').
    simulationMenu selectorAt:1 put:#stopSimulation.
    simulationMenu labelAt:1 put:(resources string:'stop').
    logicView startSimulation
!

stopSimulation
    |simulationMenu|

    simulationMenu := menu menuAt:(resources string:'simulation').
    simulationMenu selectorAt:1 put:#startSimulation.
    simulationMenu labelAt:1 put:(resources string:'start').
    logicView stopSimulation
!

showProbe
    |debuggingMenu|

    debuggingMenu := menu menuAt:(resources string:'debugging').
    debuggingMenu selectorAt:1 put:#hideProbe.
    logicView showProbe
!

hideProbe
    |debuggingMenu|

    debuggingMenu := menu menuAt:(resources string:'debugging').
    debuggingMenu selectorAt:1 put:#showProbe.
    logicView hideProbe
!

showAnalyzer
!

hideAnalyzer
!

gridOn
    |formMenu|

    formMenu := menu menuAt:(resources string:'form').
    formMenu labelAt:1 put:(resources string:'grid off').
    formMenu selectorAt:1 put:#gridOff.
    logicView grid:(gridSize @ gridSize)
!

gridOff
    |formMenu|

    formMenu := menu menuAt:(resources string:'form').
    formMenu labelAt:1 put:(resources string:'grid on').
    formMenu selectorAt:1 put:#gridOn.
    logicView grid:(1 @ 1)
!

showGrid
    |formMenu|

    formMenu := menu menuAt:(resources string:'form').
    formMenu labelAt:3 put:(resources string:'hide grid').
    formMenu selectorAt:3 put:#hideGrid.
    logicView showGrid.
    gridShown := true
!

hideGrid
    |formMenu|

    formMenu := menu menuAt:(resources string:'form').
    formMenu labelAt:3 put:(resources string:'show grid').
    formMenu selectorAt:3 put:#showGrid.
    logicView hideGrid.
    gridShown := false
!

gridSize
    (gridSize == 4) ifTrue:[
        gridSize := 8
    ] ifFalse:[
        gridSize := 4
    ].
    gridShown ifTrue:[
        logicView grid:(gridSize @ gridSize)
    ]
!

copySelection
    logicView copySelection
!

deleteSelection
    logicView deleteSelection
!

pasteSelection
    logicView pasteBuffer
!

nameIt:aName
    logicView nameSelection:aName
!

nameIt
    nameEnterBox show
!

addSubCircuitFromFile:aFileName
    |aStream aSubCircuit|
    aStream := FileStream readonlyFileNamed:aFileName.
    aStream isNil ifTrue:[
        openErrorBox title:(resources string:'file %1 not found or not readable' with:aFileName).
        openErrorBox show
    ] ifFalse:[
        aSubCircuit := aStream fileIn.
        aStream close.
        logicView add:aSubCircuit
    ]
!

addSubCircuit
    subEnterBox show
!

group
    logicView group
!

disconnect
    |editMenu|

    editMenu := menu menuAt:(resources string:'edit').
    editMenu labelAt:5 put:(resources string:'connect').
    editMenu selectorAt:5 put:#connect.
    logicView disconnect
!

connect
    |editMenu|

    editMenu := menu menuAt:(resources string:'edit').
    editMenu labelAt:5 put:(resources string:'disconnect').
    editMenu selectorAt:5 put:#disconnect.
    logicView connect
! !
