"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ObjectView subclass:#LogicView
       instanceVariableNames:'gates connections
                              createClass createdObject
                              startPoint
                              fromGate fromPin toPoint
                              disconnecting
                              warnBox 
                              simBlock simulating
                              showNames showValues
                              placeCursor connectCursor disconnectCursor'

       classVariableNames:''
       poolDictionaries:''
       category:'Interface-LogicTool'
!

LogicView comment:'
 a view to manipulate gates

 $Header: /claus/smalltalk/st2.6/clients/LogicTool/RCS/LogicView.st,v 1.1 94/01/13 00:45:44 claus Exp $
 written spring/summer 89 by claus
'!

!LogicView class methodsFor:'defaults'!

gridShown
    ^ false
!

hitDelta
    "when clicking an object, allow for hitDelta pixels around object;
     0 is exact; 4 is good for draw programs"
    ^ 4
! !

!LogicView methodsFor:'initializing'!

initialize
    super initialize.

    self initializeCursors.

    disconnecting := false.
    simulating := false.
    showNames := false.
    showValues := false.
    simBlock := [self simulate].
    gates := OrderedCollection new.
    connections := OrderedCollection new.

    self setupStandardActions.

    warnBox := WarningBox title:'connects 2 inputs'.

    connectCursor isNil ifTrue:[
        self class initializeCursors
    ].
    self cursor:connectCursor
!

setupStandardActions
    pressAction := [:pressPoint | self startConnectOrMove:pressPoint].
    shiftPressAction := [:pressPoint | self startSelectMoreOrMove:pressPoint].
    motionAction := [:movePoint | self doObjectMove:movePoint].
    releaseAction := [self endObjectMove].
    doublePressAction := [:pressPoint | self clickOnObject:pressPoint]
!

initializeCursors
    connectCursor := Cursor arrow.
    disconnectCursor := Cursor arrow.
    placeCursor := Cursor arrow.
    ^ true.
    connectCursor := 
        Cursor
         sourceForm:(Form fromFile:'solder.bit' resolution:100)
           maskForm:(Form fromFile:'solderM.bit' resolution:100)
            hotSpot:(0 @ 15).

    disconnectCursor := 
        Cursor
         sourceForm:(Form fromFile:'cutter.bit' resolution:100)
           maskForm:(Form fromFile:'cutterM.bit' resolution:100)
            hotSpot:(7 @ 3).

    placeCursor := 
        Cursor
         sourceForm:(Form fromFile:'place.bit' resolution:100)
           maskForm:(Form fromFile:'placeM.bit' resolution:100)
            hotSpot:(0 @ 0)
!

destroy
    self stopSimulation.
    super destroy
! !

!LogicView methodsFor:'connections'!

drawConnectionFrom:point1 to:point2
    self displayLineFromX:(point1 x) y:(point1 y) toX:(point2 x) y:(point2 y)
!

removeConnection:aConnection
    contents remove:aConnection.
    connections remove:aConnection.
    self redrawObjectsIn:(aConnection frame).
    self simulate
!

connectFrom:gate1 pin:pin1 to:gate2 pin:pin2
    |connection bad input output inputPin outputPin inputNr outputNr|
    "check if valid connection"
    (gate1 isInput:pin1) ifTrue:[
        input := gate1.
        inputPin := pin1.
        (gate2 isInput:pin2) ifTrue:[
            "connecting 2 inputs"
            warnBox title:'cannot connect 2 inputs'.
            warnBox showAtPointer.
            ^ self
        ].
        output := gate2.
        outputPin := pin2
    ] ifFalse:[
        output := gate1.
        outputPin := pin1.
        (gate2 isOutput:pin2) ifTrue:[
            "connecting 2 outputs"
            warnBox title:'cannot connect 2 outputs'.
            warnBox showAtPointer.
            ^ self
        ].
        input := gate2.
        inputPin := pin2
    ].
    "look if the input is already connected"
    inputNr := input whichInput:inputPin.
    outputNr := output whichOutput:outputPin.
    bad := false.
    self connectionsWith:input do:[:aConnection |
        (aConnection dstGate == input) ifTrue:[
            (aConnection dstPinNr == inputNr) ifTrue:[
                "input is already connected to another output"
                bad := true
            ]
        ]
    ].
    bad ifTrue:[
        warnBox title:'indirectly connects 2 outputs'.
        warnBox showAtPointer.
        ^ self
    ].
    connection := Connection from:output pin:outputNr to:input pin:inputNr.
    contents add:connection.
    connections add:connection.
    connection drawIn:self.
    self simulate
! !

!LogicView methodsFor:'selections'!

findGateAt:aPoint
    "find the first gate (by looking from front to back) which is hit by
     the argument, aPoint"

    |hdelta|

    hdelta := self class hitDelta.
    gates do:[:object |
        (object isHitBy:aPoint withDelta:hdelta) ifTrue: [
            ^ object
        ]
    ].
    ^ nil
!

pinHandleFor:aPoint
    |hsize centerX centerY|

    hsize := self class hitDelta.
    centerX := aPoint x.
    centerY := aPoint y.
    ^ Rectangle left:(centerX - hsize)
                 top:(centerY - hsize)
               right:(centerX + hsize)
              bottom:(centerY + hsize)
!

pin:pinPoint isHitBy:aPoint
    ^ (self pinHandleFor:pinPoint) containsPoint:aPoint
! !

!LogicView methodsFor:'interface to LogicTool'!

createObjectWithClass:aClass
    createClass := aClass.
    pressAction := [:pressPoint | self startCreate:pressPoint].
    shiftPressAction := pressAction.
    motionAction := [:movePoint | nil].
    releaseAction := [nil].
    self cursor:placeCursor
!

connect
    disconnecting := false.
    self cursor:connectCursor
!

disconnect
    disconnecting := true.
    self cursor:disconnectCursor
!

group
    |subCircuit minx miny|

    subCircuit := SubCircuit createFrom:selection.
    buffer := selection.
    self unselect.
    self forEach:buffer do:[:anObject |
        (anObject isKindOf:Connection) ifFalse:[
            minx isNil ifTrue:[
                minx := anObject origin x.
                miny := anObject origin y
            ] ifFalse:[
                minx := minx min:(anObject origin x).
                miny := miny min:(anObject origin y)
            ].
            self removeObject:anObject
        ]
    ].
    selection := nil.
    subCircuit origin:(minx @ miny).
    contents add:subCircuit.
    gates add:subCircuit.
    self select:subCircuit
!

showNames
    showNames := true.
    self redraw
!

dontShowNames
    showNames := false.
    self redraw
! !

!LogicView methodsFor:'select or move'!

moveObject:anObject to:newOrigin
    "change the position on anObject to newOrigin, aPoint"

    |sel oldFrame oldConnectionFrames connectionsWithGate|
    anObject notNil ifTrue:[
        anObject canBeMoved ifTrue:[
            (anObject origin = newOrigin) ifFalse:[
                oldFrame := self frameOf:anObject.
                connectionsWithGate := OrderedCollection new.
                oldConnectionFrames := OrderedCollection new.
                self connectionsWith:anObject do:[:aConnection |
                    connectionsWithGate add:aConnection.
                    oldConnectionFrames add:(aConnection frame)
                ].
                anObject moveTo:newOrigin.
                self connectionsWith:anObject do:[:aConnection |
                    aConnection recomputeFrame
                ].

                self redrawObjectsIn:oldFrame.
                oldConnectionFrames do:[:aFrame |
                    aFrame notNil ifTrue:[
                        self redrawObjectsIn:aFrame
                    ]
                ].
                (self isObscured:anObject) ifFalse:[
                    anObject drawIn:self
                ] ifTrue:[
                    self redrawObjectsIn:(self frameOf:anObject)
                ].
                connectionsWithGate do:[:aConnection |
                    aConnection notNil ifTrue:[
                        aConnection drawIn:self
                    ]
                ]
            ]
        ]
    ]
!

startCreate:aPoint
    startPoint := aPoint "grid:grid".
    createClass notNil ifTrue:[
        self unselect.
        createdObject := createClass new.
        createdObject origin:startPoint.
        motionAction := [:movePoint | self moveCreate:movePoint].
        releaseAction := [self endCreate].
        self assignIdFor:createdObject.
        contents add:createdObject.
        gates add:createdObject.
        createdObject drawIn:self
    ]
!

moveCreate:newOrigin
    "when creating, change the position of createdObject while button
     is down"

    |sel oldFrame oldConnectionFrames connectionsWithGate|
    createdObject notNil ifTrue:[
        (createdObject origin = newOrigin) ifFalse:[
            oldFrame := self frameOf:createdObject.
            createdObject moveTo:newOrigin.
            self redrawObjectsIn:oldFrame.
            (self isObscured:createdObject) ifFalse:[
                createdObject drawIn:self
            ] ifTrue:[
                self redrawObjectsIn:(self frameOf:createdObject)
            ]
        ]
    ]
!

endCreate
    disconnecting ifTrue:[
        self cursor:disconnectCursor
    ] ifFalse:[
        self cursor:connectCursor
    ].
    self setupStandardActions
!

clickOnObject:aPoint
    |aGate|

    aGate := self findGateAt:aPoint.
    aGate notNil ifTrue:[
        aGate click.
        connections do:[:aConnection |
            aConnection propagate
        ].
        self simulate
    ]
!

startConnectOrMove:aPoint
    |aGate aPin|

    "if connecting, do not consider connections"
    disconnecting ifTrue:[
        aGate := self findObjectAt:aPoint
    ] ifFalse:[
        aGate := self findGateAt:aPoint
    ].
    (aGate isKindOf:Connection) ifTrue:[
        disconnecting ifTrue:[
            self removeConnection:aGate
        ].
        ^ self
    ].

    "ok, this one hits a gate - look for i/o pins"
    aGate notNil ifTrue:[
        aGate pinsDo:[:thePin |
            (self pin:thePin isHitBy:aPoint) ifTrue:[
                aPin := thePin
            ]
        ]
    ].

    "if we found a pin, this starts a connect"
    aPin notNil ifTrue:[
        self startConnectFrom:aGate pin:aPin.
        ^ self
    ].

    "if we found a gate, this starts an objectMove"
    aGate notNil ifTrue:[
        self startObjectMove:aGate at:aPoint.
        ^ self
    ].
    "otherwise a multi-select"
    self startRectangleDrag:aPoint
!

startConnectFrom:aGate pin:aPin
    motionAction := [:movePoint | self doDragConnection:movePoint].
    releaseAction := [self endConnection].
    fromGate := aGate.
    fromPin := aPin
!

doDragConnection:aPoint
    |newPoint|
    newPoint := aPoint.
    newPoint x == lastButt x ifTrue:[
        newPoint y == lastButt y ifTrue:[^ self]
    ].
    fromPin notNil ifTrue:[
        toPoint isNil ifFalse:[
            "invert prev. drag"
            self xoring:[self drawConnectionFrom:fromPin to:toPoint]
        ].
        "invert new drag"
        toPoint := newPoint.
        self xoring:[self drawConnectionFrom:fromPin to:toPoint]
    ]
!

endConnection
    |toGate toPin|
    motionAction := [:movePoint | nil].
    releaseAction := [nil].
    fromPin notNil ifTrue:[
        toPoint notNil ifTrue:[
            "invert prev. drag"
            self xoring:[self drawConnectionFrom:fromPin to:toPoint].

            toGate := self findGateAt:toPoint.

            "ok, this one hits an object - look for i/o pins"
            toGate notNil ifTrue:[
                toGate pinsDo:[:thePin |
                    (self pin:thePin isHitBy:toPoint) ifTrue:[
                        toPin := thePin
                    ]
                ]
            ].
            "if we found a pin, do the connect"

            toPin notNil ifTrue:[
                self connectFrom:fromGate pin:fromPin to:toGate pin:toPin
            ]
        ].
        fromGate := nil.
        fromPin := nil.
        toPoint := nil
    ]
! !

!LogicView methodsFor:'misc'!

assignIdFor:aGate
    |gateClass genName number|
    gateClass := aGate class.
    genName := gateClass genericName.
    number := 0.
    gates notNil ifTrue:[
        gates do:[:anotherGate |
            (anotherGate class genericName = genName) ifTrue:[
                (anotherGate id > number) ifTrue:[
                    number := anotherGate id
                ]
            ]
        ]
    ].
    aGate id:(number + 1).
    aGate name:(genName) , '-' , (number + 1) printString.
    aGate font:font
!

findGateNamed:aString
    |theGate|

    gates notNil ifTrue:[
        gates do:[:aGate |
            (aGate name = aString) ifTrue:[
                theGate notNil ifTrue:[
                    self error:'duplicate name'
                ].
                theGate := aGate
            ]
        ]
    ].
    ^ theGate
!

nameSelection:aName
    self selectionDo:[:anObject |
        (anObject isKindOf:Connection) ifFalse:[
            anObject name:aName.
            anObject font:font.
            anObject drawIn:self
        ]
    ]
!

connectionsWith:aGate do:aBlock
    "evaluate aBlock for all connections from/to aGate"

    connections do:[:aConnection |
        aConnection notNil ifTrue:[
            (aConnection srcGate == aGate) ifTrue:[
                aBlock value:aConnection
            ] ifFalse:[
                (aConnection dstGate == aGate) ifTrue:[
                    aBlock value:aConnection
                ]
            ]
        ]
    ]
!

drawValuesOf:aGate
    |outputNr s p|

    self paint:Black.
    outputNr := 1.
    aGate outputPinsDo:[:aPinLocation |
        s := (aGate output:outputNr) ifTrue:['1'] ifFalse:['0'].
        p := aPinLocation - viewOrigin.
        self displayOpaqueString:s x:(p x - font width)
                                   y:(p y - font descent).
        outputNr := outputNr + 1
    ]
!

drawValues
    gates do:[:aGate |
        self drawValuesOf:aGate
    ]
! !

!LogicView methodsFor:'adding / removing'!

addObject:anObject
    "add the argument, anObject to the contents - with redraw"

    anObject notNil ifTrue:[
        (anObject isKindOf:Connection) ifTrue:[
            connections addLast:anObject
        ] ifFalse:[
            gates addLast:anObject
        ].
        super addObject:anObject
    ]
!

removeAll
    "remove all gates & connections"

    gates := OrderedCollection new.
    connections := OrderedCollection new.
    super removeAll
!

removeConnectionsWith:aGate
    "remove all connections from/to aGate"

    self connectionsWith:aGate do:[:aConnection |
        self removeConnection:aConnection
    ]
!

removeObject:anObject
    "remove the argument, anObject from the contents;
     if its a gate, also remove its connections"

    anObject notNil ifTrue:[
        (contents includes:anObject) ifTrue:[
            super removeObject:anObject
        ].
        (anObject isKindOf:Connection) ifTrue:[
            (connections includes:anObject) ifTrue:[
                connections remove:anObject
            ]
        ] ifFalse:[
            self removeConnectionsWith:anObject.
            gates remove:anObject
        ]
    ]
! !

!LogicView methodsFor:'simulation'!

showValues 
    showValues ifFalse:[
        showValues := true.
        self drawValues
    ]
!

hideValues
    showValues :=  false
!

startSimulation
    simulating ifFalse:[
        Processor addIdleBlock:simBlock.
        simulating := true
    ]
!

stopSimulation
    simulating ifTrue:[
        Processor removeIdleBlock:simBlock.
        simulating := false
    ]
!

simulate
    gates isNil ifTrue:[^ self].
    connections do:[:aConnection |
        aConnection propagate
    ].
    gates do:[:aGate |
        aGate evaluate.
        aGate changedPicture ifTrue:[
            aGate drawIn:self.
            aGate unchangedPicture
        ].
        aGate changed ifTrue:[
            showValues ifTrue:[
                self drawValuesOf:aGate
            ].
            aGate unchanged
        ]
    ]
! !

!LogicView methodsFor:'saving / restoring'!

storeContentsOn:aStream
    |excla|

    excla := aStream class chunkSeparator.
    "have to store gates first, then connections"
    self forEach:gates do:[:theObject |
        theObject storeOn:aStream.
        aStream nextPut:excla.
        aStream cr
    ].
    self forEach:connections do:[:theObject |
        theObject storeOn:aStream.
        aStream nextPut:excla.
        aStream cr
    ].
    aStream nextPut:excla
!

initializeFileInObject:newObject
    "since connections are filed out with named-gates but internally have
     links to the real gates, we have to transform here"

    |srcGate dstGate|

    (newObject isKindOf:Connection) ifTrue:[
        srcGate := self findGateNamed:(newObject srcGate).
        dstGate := self findGateNamed:(newObject dstGate).
        newObject srcGate:srcGate dstGate:dstGate
    ]
! !
