'From Smalltalk/X, Version:2.3.1 on 1-Sep-92 at 22:40:21'!

StandardSystemView subclass:#MailView
         instanceVariableNames:'buttonPanel textPanel 
                mailHeaderView mailDocuView mailTextView 
                mailText
                printButton saveButton deleteButton replyButton
                fullHeaderTextToggle rescanButton mailLabel 
                replyView
                replyButtonPanel replyEditView replySendButton replyAbortButton
                replyAddressLabel replyAddressField replySubjectField
                replyCCField replyAliasButton aliasBox
                checkBlock mailHandler editViewActive selectedLine
                showFullHeaderText waitCursor numberOfLetters
                checkInterval'
         classVariableNames:'MailIcon NoMailIcon LetterIcon'
         poolDictionaries:''
         category:'Interface-MailTool'
!

MailView comment:'
This mail interface is just a demo - there are many features missing.
'!

!MailView class methodsFor:'identification'!

version
    ^ '0.4'
! !

!MailView class methodsFor:'behavior'!

autoSelectNext
    "returning true here makes the delete function automatically
     select the next entry"

    ^ true
! !

!MailView class methodsFor:'instance creation'!

new
    MailIcon isNil ifTrue:[
        MailIcon := Form fromFile:'Mail.xbm' resolution:100.
        NoMailIcon := Form fromFile:'NoMail.xbm' resolution:100.
        LetterIcon := Form fromFile:'Letter.xbm' resolution:100
    ].

    ^ super
        label:'Mail'
        icon:NoMailIcon
        minExtent:(100 @ 100)
! !

!MailView methodsFor:'initialization'!

initialize
    |panelHeight myWidth myHeight headerHeight textHeight protoButton
     replySubjectLabel replyCCLabel activeReplyField
     enterGroup mm helper|

    super initialize.

    checkInterval := (resources at:'CHECK_INTERVAL' default:'30') asNumber.

    mm := self verticalIntegerPixelPerMillimeter.

    showFullHeaderText := false.
    waitCursor := Cursor wait.
    mailHandler := MailHandler new.

    numberOfLetters := mailHandler numberOfLetters.
"
    (numberOfLetters == 0) ifTrue:[
        mailLabel := Label form:NoMailIcon
    ] ifFalse:[
        mailLabel := Label form:MailIcon
    ].
"

    protoButton := (Button label:'dummy') action:[self doPrint].
    "
     this horizontalPanel contains the buttons ...
    "
    buttonPanel := HorizontalPanelView in:self.
    buttonPanel layout:#left.   "left adjusted is nicer ...     "
    panelHeight := ((protoButton height) "max:(mailLabel height)" ) + 10.
    buttonPanel origin:(0.0 @ 0.0) extent:(1.0 @ panelHeight).

"
    buttonPanel addSubView:mailLabel.
"

    "
     here are the buttons ...
    "
    printButton := (Button label:(resources string:'print')
                          action:[self doPrint]
                              in:buttonPanel) disable.

"
    saveButton := (Button label:(resources string:'save')
                         action:[self doSave]
                             in:buttonPanel).
"
    deleteButton := (Button label:(resources string:'delete')
                           action:[self doDelete]
                             in:buttonPanel) disable.

    replyButton := (Button label:(resources string:'send/reply')
                          action:[self doReply]
                             in:buttonPanel).

    fullHeaderTextToggle := Toggle label:(resources string:'full-header').
    fullHeaderTextToggle pressAction:[self fullHeaderText].
    fullHeaderTextToggle releaseAction:[self partialHeaderText].
    buttonPanel addSubView:fullHeaderTextToggle.

    rescanButton := (Button label:(resources string:'rescan mail')
                           action:[self doRescan]
                               in:buttonPanel).

    "
     a variable panel  for the other subviews ...
    "
    textPanel := VariableVerticalPanel in:self.
    textPanel origin:0.0 @ panelHeight corner:1.0@1.0.

    "
     top third contains a list of subject-lines ...
     this list is provided by the mailhandler
     (notice the floats in the origin/extent)
    "
    mailHeaderView := ScrollableView for:SelectionInListView in:textPanel.
    mailHeaderView origin:(0.0 @ 0.0) corner:(1.0 @ 0.25).
    mailHeaderView contents:(mailHandler headerLines).
    mailHeaderView action:[:lineNr | self showSelection:lineNr].

    "
     bottom 2-thirds will show the selected mail ...
     normal text is shown in a TextView, Multimedia mail (currently
     NeXT-mail) is shown in a DocumentView which is brought up when
     needed.
    "
    "
     because VariablePanel doesnt like adding & removing views,
     we put the two alternatively visible views into a helper, which
     is managed by the VariablePanel
    "
    helper := View origin:(0.0 @ 0.25) corner:(1.0 @ 1.0) in:textPanel.

    mailTextView := ScrollableView for:TextView in:helper.
    mailTextView origin:(0.0 @ 0.0) corner:(1.0 @ 1.0).

    "
     prepare for multimedia mail, by creating (invisible)
     documentView
    "
    mailDocuView := ScrollableView for:DocumentView in:helper.
    mailDocuView origin:(0.0 @ 0.0) corner:(1.0@1.0).
    mailDocuView hidden:true.

    editViewActive := false.
    activeReplyField := nil.

    replyView := StandardSystemView
                      extent:(600 @ 400)
                      label:'Send-Mail'
                      icon:LetterIcon
                      minExtent:(100 @ 100).

    "
     the reply view gets its own button panel
    "
    replyButtonPanel := HorizontalPanelView in:replyView.
    replyButtonPanel layout:#left.   "left adjusted is nicer ...     "
    replyButtonPanel origin:(0.0 @ 0.0)
                     extent:(1.0 @ (protoButton height + 10)).

    "
     and some stuff in it ...
    "
    replySendButton := Button label:(resources string:'send')
                             action:[self doSendReply]
                                 in:replyButtonPanel.

    replyAbortButton := Button label:(resources string:'cancel')
                              action:[self doAbortReply]
                                  in:replyButtonPanel.

    replyAliasButton := Button label:(resources string:'addresses')
                              action:[self doAddressesReply]
                                  in:replyButtonPanel.

    replyAddressLabel := Label in:replyView.
    replyAddressLabel origin:(0.0 @ (replyButtonPanel height)).
    replyAddressLabel label:'reply to:'.
    replyAddressLabel borderWidth:0.

    replySubjectLabel := Label in:replyView.
    replySubjectLabel origin:(0.0 @ (replyAddressLabel origin y
                                   + replyAddressLabel height
                                   + mm)).
    replySubjectLabel label:'subject :'.
    replySubjectLabel borderWidth:0.

    replyCCLabel := Label in:replyView.
    replyCCLabel origin:(0.0 @ (replySubjectLabel origin y
                              + replySubjectLabel height
                              + mm)).
    replyCCLabel label:'cc      :'.
    replyCCLabel borderWidth:0.

    enterGroup := EnterFieldGroup new.

    replyAddressField := EditField
                            origin:((replyAddressLabel origin x + replyAddressLabel width) 
                                    @ 
                                    (replyAddressLabel origin y))
                            extent:[(replyView width
                                     - replyAddressLabel width
                                     - mm)
                                    @ 
                                    (replyAddressLabel height)]
                                in:replyView.
    replyAddressField contents:''.
    replyAddressField disable.
    enterGroup add:replyAddressField.

    replySubjectField := EditField
                            origin:((replyAddressField origin x) 
                                    @ 
                                    (replySubjectLabel origin y))
                            extent:[(replyView width
                                     - replySubjectLabel width
                                     - mm)
                                    @ 
                                    (replySubjectLabel height)]
                                in:replyView.
    replySubjectField contents:''.
    replySubjectField disable.
    enterGroup add:replySubjectField.

    replyCCField := EditField
                            origin:((replyAddressField origin x) 
                                    @ 
                                    (replyCCLabel origin y))
                            extent:[(replyView width
                                     - replyCCLabel width
                                     - mm)
                                    @ 
                                    (replyCCLabel height)]
                                in:replyView.
    replyCCField contents:''.
    replyCCField disable.
    enterGroup add:replyCCField.

    replyEditView := ScrollableView for:MailReplyTextView in:replyView.
    replyEditView origin:[0 
                          @ 
                          (replyCCLabel origin y 
                           + replyCCLabel height
                           + mm + mm)]
                  extent:[replyView width
                          @
                          (replyView height
                           - mm - mm
                           - replyCCLabel origin y
                           - replyCCLabel heightIncludingBorder)].

    self checkMailIcon.

    ^ self
!

realize
    super realize.
    checkBlock := [self checkForMail].
    self checkForMail
! !

!MailView methodsFor:'private'!

checkMailIcon
    "set the icon according to basket empty-state"

    (numberOfLetters > 0) ifTrue:[
        "mailLabel form:MailIcon. "
        self icon:MailIcon
    ] ifFalse:[
        "mailLabel form:NoMailIcon. "
        self icon:NoMailIcon
    ]
!

checkForMail
    "this is sent every checkInterval secs to check for new mail"

    |currentNumberOfLetters|

    currentNumberOfLetters := mailHandler numberOfLetters.
    (currentNumberOfLetters ~~ numberOfLetters) ifTrue:[
        numberOfLetters := currentNumberOfLetters.
        self label:'Mail: reading new mail ...'.
        self checkMailIcon.
        self doRescan.
        self label:'Mail'
    ].
    "trigger next check in checkInterval seconds"
    Processor addTimedBlock:checkBlock after:checkInterval
! !

!MailView methodsFor:'user interaction'!

withWaitCursorDo:aBlock
    "evaluate aBlock while showing wait-cursor"

    |oldHeaderCursor oldTextCursor oldPanelCursor|

    oldHeaderCursor := mailHeaderView cursor.
    oldTextCursor := mailTextView cursor.
    oldPanelCursor := buttonPanel cursor.
    mailHeaderView cursor:waitCursor.
    mailTextView cursor:waitCursor.
    buttonPanel cursor:waitCursor.
    aBlock value.
    buttonPanel cursor:oldPanelCursor.
    mailHeaderView cursor:oldHeaderCursor.
    mailTextView cursor:oldTextCursor
!

showSelection:lineNr
    "user clicked on a header line - show selected mail in textView
     but ignore it, if he's currently editing a reply"

    |mailText document letter|

    editViewActive ifFalse: [
        (lineNr == 0) ifTrue:[
            printButton disable.
            deleteButton notNil ifTrue:[deleteButton disable].
            saveButton notNil ifTrue:[saveButton disable].
            selectedLine := nil.
            mailTextView contents:nil
        ] ifFalse:[
            printButton enable.
            deleteButton notNil ifTrue:[deleteButton enable].
            saveButton notNil ifTrue:[saveButton enable].
            selectedLine := lineNr.

            self withWaitCursorDo:[
                showFullHeaderText ifTrue:[
                    letter := mailHandler fullLetter:lineNr
                ] ifFalse:[
                    letter := mailHandler letter:lineNr
                ].
                letter notNil ifTrue:[
                    (letter type == #rtf) ifTrue:[
                        "its a NeXT rtf mail; 
                         need DocumentView to show contents of rtf stream..."
                        mailTextView unrealize.

                        mailDocuView contents:nil.
                        mailDocuView scrollToTop.
                        mailDocuView hidden:false; realize; raise.
                        mailDocuView fromRTFStream:(ReadStream on:letter contents).
                    ] ifFalse:[
                        (letter type ~~ #text) ifTrue:[
                            "not handled - use contents as text"
                            Transcript showCr:('unknown letter type: ' , letter type)
                        ].
                        "its a normal mail"
                        mailDocuView unrealize.

                        mailTextView contents:nil.
                        mailTextView scrollToTop.
                        mailTextView realize.
                        mailTextView contents:letter contents.
                    ]
                ]
            ]
        ]
    ]
!

fullHeaderText
    "user wants to see all header lines
     this includes all fields inserted by relay stations -"

    showFullHeaderText := true.
    selectedLine notNil ifTrue:[
        self showSelection:selectedLine
    ]
!

partialHeaderText
    "user wants to suppress full header lines
     Thats what I prefer - I'm not very interrested in all this relay info"

    showFullHeaderText := false.
    selectedLine notNil ifTrue:[
        self showSelection:selectedLine
    ]
!

doSave
    "user clicked on save-button
     much more work to be done here"

    selectedLine notNil ifTrue:[
        self warn:'gotcha - not implemented'
    ].
    ^ self
!

doPrint
    "user clicked on print-button"

    selectedLine notNil ifTrue:[
        self warn:'gotcha - not implemented'
    ].
    ^ self
!

doDelete
    "user clicked on delete-button
     delete current mail, and autoselect next mail in list
     if we reached the end of list, go back to previous mail"

    |headers|

    selectedLine notNil ifTrue:[
        deleteButton notNil ifTrue:[deleteButton disable].
        printButton notNil ifTrue:[printButton disable].
        saveButton notNil ifTrue:[saveButton disable].
        mailHeaderView selection:0.
        mailHandler deleteMail:selectedLine.
        numberOfLetters := mailHandler numberOfLetters.
        (numberOfLetters == 0) ifTrue:[
            "mailLabel form:NoMailIcon. "
            self icon:NoMailIcon
        ].
        headers := mailHandler headerLines.
        mailHeaderView contents:headers.
        self class autoSelectNext ifTrue:[
            selectedLine > (headers size) ifTrue:[
                selectedLine := selectedLine - 1
            ].
            mailHeaderView selection:selectedLine.
            self showSelection:selectedLine
        ] ifFalse:[
            selectedLine := nil.
            mailTextView contents:nil
        ]
    ].
    ^ self
!

doReply
    "user clicked on reply-button"

    |subjectLineIndex subject initialText destination|

    selectedLine notNil ifTrue:[
        destination := mailHandler originOf:selectedLine.
        subject := mailHandler subjectOf:selectedLine.
        subject notNil ifTrue:[
            (subject startsWith:'Re:') ifFalse:[
                subject := 'Re: ' , subject
            ]
        ].
    ].

    destination isNil ifTrue:[
        destination := 'name @ host'
    ].

    deleteButton notNil ifTrue:[deleteButton disable].
    replyButton notNil ifTrue:[replyButton disable].                        

    mailHeaderView disable.

    "
     now, make the replyView visible
    "
    replyAbortButton turnOff.

    replyView realize.

    selectedLine isNil ifTrue:[
        replyEditView middleButtonMenu disable:#pasteLetter
    ] ifFalse:[
        replyEditView middleButtonMenu enable:#pasteLetter.
        replyEditView originalLetter:(mailHandler letter:selectedLine) 
    ].

    "
     put receivers address into info-label
    "
    selectedLine notNil ifTrue:[
        replyAddressLabel label:'reply to:'
    ] ifFalse:[
        replyAddressLabel label:'send to :'
    ].

    replyAddressField contents:destination.
    subject isNil ifTrue:[
        selectedLine notNil ifTrue:[
            subject := 'Re: '
        ]
    ].
    replySubjectField contents:subject.

    "
     create initial text with subject and 
     mailer id - not my idea - but nice
    "
    initialText := Text new:1.
    initialText at:1 put:('X-Mailer: Smalltalk/X ('
                          , Smalltalk version
                          , ') MailTool ('
                          , self class version 
                          , '), of course').

    replyEditView contents:initialText.
    replyEditView cursorLine:3 col:1.

    editViewActive := true
!

doSendReply
    "in the reply view - user clicked send button"

    mailHandler sendMail:(replyEditView contents)
                 subject:(replySubjectField contents)
                      cc:(replyCCField contents)
                      to:(replyAddressField contents).

    self hideReplyView

!

doAbortReply
    "in the reply view - user clicked abort button"

    self hideReplyView
!

doAddressesReply
    "read & show address alias list"

    |s aliases line box c list|

    s := (OperatingSystem getHomeDirectory , '/.mailrc') asFilename readStream.
    aliases := Dictionary new.
    s notNil ifTrue:[
        [s atEnd] whileFalse:[
            line := s nextLine.
            line notNil ifTrue:[
                (line startsWith:'alias') ifTrue:[
                    c := line asCollectionOfWords.
                    aliases at:(c at:2) put:(c at:3).
                ]
            ]
        ].
        s close
    ].
    list := OrderedCollection new.
    aliases keysAndValuesDo:[:k :v |
        list add:k
    ].
    "
     launch a box with names
    "
    aliasBox isNil ifTrue:[
        aliasBox := ListSelectionBox new.
        aliasBox title:(resources string:'mail aliases').
    ].
    aliasBox list:list.
    aliasBox action:[:name |
        replyAddressField contents:(aliases at:name ifAbsent:[''])
    ].
    aliasBox showAtPointer
!

hideReplyView
    "make the reply view invisible"

    deleteButton notNil ifTrue:[deleteButton enable].
    replyButton notNil ifTrue:[replyButton enable].
    mailHeaderView enable.
    "
     make the replyView invisible
    "
    replyView unrealize.
    editViewActive := false
!

doRescan
    "user wants mail-file to be reread for new mail"

    self withWaitCursorDo:[
        |newHeaders oldSelection|

"
        mailHeaderView selection:nil.
        selectedLine := nil.
        mailTextView contents:nil.
"

        mailHandler scanMailFile.
        newHeaders := mailHandler headerLines.

        oldSelection := mailHeaderView selection.
        mailHeaderView setContents:newHeaders.
        mailHeaderView selection:oldSelection.
    ]
!

destroy
    "window manager wants MailView to vanish"

    self withWaitCursorDo:[
        Processor removeTimedBlock:checkBlock.
        mailHandler release.
    ].
    aliasBox notNil ifTrue:[aliasBox destroy].
    super destroy
! !
