'From Smalltalk/X, Version:1.5 on 4-Sep-91 at 18:41:13'!

StandardSystemView subclass:#MemoryMonitor
         instanceVariableNames:'halted delay myBlock myProcess oldData newData sumData
                                index org max min prevStringLen
                                grey'
         classVariableNames:''
         poolDictionaries:''
         category:'Interface-Debugger'
!

MemoryMonitor comment:'
Shows memory usage (oldspace + newspace). Stupid, but works.
'!

!MemoryMonitor class methodsFor:'startup'!

start
    |m|

    m := self origin:0 @ 0 extent:(200 @ 100).

    m label:'Memory Monitor'.
    m icon:(Form fromFile:'Monitor.icon' resolution:100).
    m minExtent:(100 @ 100).

    m open.
    ^ m

    "MemoryMonitor start"
! !

!MemoryMonitor methodsFor:'initialization'!

initialize
    super initialize.

    halted := false.
    delay := 0.5.
    ProcessorScheduler isPureEventDriven ifTrue:[
        myBlock := [self updateDisplay].
    ].
    oldData := Array new:1000.
    newData := Array new:1000.
    index := 1.
    org := font widthOf:'9999999'.
    max := ObjectMemory bytesUsed + 100000.
    min := ObjectMemory bytesUsed.
    prevStringLen := nil.
    viewBackground := Black

    "MemoryMonitor start"
!

realize
    super realize.
    self enableKeyEvents.
    myBlock notNil ifTrue:[
        Processor addTimedBlock:myBlock after:delay.
    ] ifFalse:[
        myProcess := [
            |d|

            [true] whileTrue:[
                (Delay forSeconds:delay) wait.
                self updateDisplay
            ]
        ] forkAt:4.
        myProcess name:'monitor update'
    ].
    grey := Color grey on:device.
! !

!MemoryMonitor methodsFor:'destroying'!

destroy
    myBlock notNil ifTrue:[
        Processor removeTimedBlock:myBlock.
    ] ifFalse:[
        myProcess terminate.
        myProcess := nil
    ].
    super destroy
! !

!MemoryMonitor methodsFor:'drawing'!

updateDisplay
    "update picture; trigger next update"

    |h hOld memUsed oldMemUsed newMemUsed x half scaleChange s thisStringLen|

    realized ifFalse:[^ self].
    oldMemUsed := ObjectMemory oldSpaceUsed.
    newMemUsed := ObjectMemory newSpaceUsed.
    memUsed := oldMemUsed + newMemUsed.
    oldData at:index put:oldMemUsed.
    newData at:index put:newMemUsed.

    h := (memUsed - min) * height // (max - min).
    hOld := (oldMemUsed - min) * height // (max - min).

    x := index - 1 + org.

    self paint:grey.
    self displayLineFromX:x y:(height - 1) toX:x y:(height - 1 - h).

    self paint:White.
    self displayLineFromX:x y:(height - 1) toX:x y:(height - 1 - hOld).

    s := (memUsed // 1024) printString , 'k'.
    thisStringLen := s size.
    (thisStringLen ~~ prevStringLen) ifTrue:[
        prevStringLen notNil ifTrue:[
            self displayOpaqueString:'       ' from:1 to:prevStringLen
                                   x:0 y:(height // 2 + font ascent)
        ]
    ].

    self paint:White on:Black.
    self displayOpaqueString:s x:0 y:(height // 2 + font ascent).
    prevStringLen := thisStringLen.

    (index >= (width - org)) ifTrue:[
        half := ((width - org) // 2) // 8 * 8.
        oldData replaceFrom:1 to:half with:oldData startingAt:(half + 1).
        newData replaceFrom:1 to:half with:newData startingAt:(half + 1).

        self copyFrom:self x:(half + org) y:0
                         toX:org y:0
                       width:(width - org - half) height:height.
        self clearRectangleX:(width - half "- org" "org + half") y:0 width:(width - org - half) height:height.
        index := index - half
    ] ifFalse:[
        index := index + 1
    ].

    scaleChange := false.
    (memUsed < min) ifTrue:[
        min := memUsed.
        scaleChange := true
    ].
    (memUsed > max) ifTrue:[
        max := memUsed.
        scaleChange := true
    ].
    scaleChange ifTrue:[
        self clear.
        self redraw
    ].
    myBlock notNil ifTrue:[
        Processor addTimedBlock:myBlock after:delay
    ].
!

redraw
    "redraw data"

    |h hOld memUsed oldMemUsed x half|

    realized ifFalse:[^ self].
    x := org.
    1 to:(index - 1) do:[:i |
        oldMemUsed := (oldData at:i).
        memUsed := oldMemUsed + (newData at:i).
        h := (((memUsed - min) asFloat / (max - min) asFloat) * height) asInteger.
        hOld := (((oldMemUsed - min) asFloat / (max - min) asFloat) * height) asInteger.

        self paint:grey.
        self displayLineFromX:x y:(height - 1) toX:x y:(height - 1 - h).

        self paint:White.
        self displayLineFromX:x y:(height - 1) toX:x y:(height - 1 - hOld).
        x := x + 1
    ].
    self paint:White.
    self displayString:(((max // 1024) printString) , 'k') x:0 y:(font ascent).
    self displayString:(((min // 1024) printString) , 'k') x:0 y:(height - font descent)
! !

!MemoryMonitor methodsFor:'events'!

keyPress:key x:x y:y
    key == $f ifTrue:[
        delay := delay / 2
    ].
    key == $s ifTrue:[
        delay := delay * 2
    ]
!

sizeChanged
    |nn no|

    ((width - org) == oldData size) ifTrue:[^ self].
    nn := Array new:width.
    no := Array new:width.
    (nn size > newData size) ifTrue:[
        nn replaceFrom:1 to:(newData size) with:newData.
        no replaceFrom:1 to:(oldData size) with:oldData
    ] ifFalse:[
        (index > nn size) ifTrue:[
            nn replaceFrom:1 to:(nn size) with:newData
                    startingAt:(index - nn size + 1 ).
            no replaceFrom:1 to:(no size) with:oldData
                    startingAt:(index - no size + 1 ).
            index := newData size - 1
        ] ifFalse:[
            nn replaceFrom:1 to:(nn size) with:newData.
            no replaceFrom:1 to:(no size) with:oldData
        ]
    ].
    newData := nn.
    oldData := no
! !
