'From Smalltalk/X, Version:2.3.1 on 6-Sep-92 at 15:15:51'!

NewsHandler subclass:#NNTPHandler
         instanceVariableNames:'lastGroup nntpConnection lastResponse
                                currentFirst currentLast sema'
         classVariableNames:'OKCANPOST OKNOPOST ERRACCESS
                             OKGROUP OKHEADER OKARTICLE OKSUBJECT OKLIST'
         poolDictionaries:''
         category:'Interface-NewsTool'
!

NNTPHandler comment:'
interface to news

%W% %E%
written june 93 by claus

'!

!NNTPHandler class methodsFor:'documentation'!

forMyself
"
    nntp-commands
        article <number>
        body <number>
        group <name>
        head <number>
        help
        last ???
        list
        newgroups ???
        newnews ???
        next
        post ???
        slave ???
        stat ???
        xhdr <number> <field> ???
"
! !

!NNTPHandler class methodsFor:'initialization'!

initialize
    super initialize.

    OKCANPOST := 200.
    OKNOPOST :=  0.  "XXX fill in"

    OKGROUP := 211.
    OKARTICLE := 220.
    OKHEADER := 221.
    OKLIST := 215.

    ERRACCESS := 0.  "XXX fill in"

    "NNTPHandler initialize"
! !

!NNTPHandler methodsFor:'misc'!

query:titleString
    |inp|

    [
        inp := DialogView request:titleString.
        inp = ''
    ] whileTrue:[            
            self warn:'Empty input is not allowed.
    Use ABORT instead.'
    ].
    ^inp

    "NNTPHandler new query:'foo ?'"
! !

!NNTPHandler methodsFor:'initialization'!

connect
    "setup connection to nntp-server on default host.
     The nntp hostname can come from either the shell environment ($NNTPSERVER)
     or be found in the global NNTPSERVER"

    |host|

    ('NNTPSERVER' knownAsSymbol and:[Smalltalk includesKey:'NNTPSERVER' asSymbol]) ifTrue:[
        host := Smalltalk at:'NNTPSERVER' asSymbol
    ] ifFalse:[
        host := OperatingSystem getEnvironment:'NNTPSERVER'.
    ].
    host isNil ifTrue:[
        host := self query:'Host where NNTP server is running:'.
        host isNil ifTrue:[
            ^false
        ]
    ].
    ^ self connectTo:host

    "Smalltalk at:#NNTPSERVER put:'slsv6bt'"
!

connectTo:aHost
    "setup connection to nntp-server at aHost"

    |response|

    nntpConnection := Socket connectTo:'nntp' on:aHost.
    nntpConnection isNil ifTrue:[
        self warn:'could not connect to NNTP server on ' , aHost.
        ^ false
    ].
    "nntpConnection buffered:true."
nntpConnection buffered:false. 

    response := self getResponseCode.
    response == OKNOPOST ifTrue:[
        self warn:'no posting allowed by nntpserver on ' , aHost
    ].
    response == ERRACCESS ifTrue:[
        self warn:'no access permitted by nntpserver on ', aHost.
        ^ false
    ].
    sema := Semaphore forMutualExclusion.
    ^ true

    "Socket connectTo:'nntp' on:'josef'"
    "Socket connectTo:'nntp' on:'slsv6bt'"
    "|n|
     n := NNTPHandler new.
     n connect.
     n articleRangeOf:'comp.lang.smalltalk'.
     n article:3200 in:'comp.lang.smalltalk'.
    "
!

release
    nntpConnection notNil ifTrue:[
        nntpConnection close.
    ].
    lastGroup := nil.
    sema := nil.
    super release
! !

!NNTPHandler  methodsFor:'misc'!

getResponseCode
    |code|

    lastResponse := nntpConnection nextLine.
    lastResponse notNil ifTrue:[
"        lastResponse printNewline.     "
        code := Number readFromString:lastResponse       
    ].
    ^ code
!

sendRequest:what
    nntpConnection nextPutLine:what; synchronizeOutput.
    ^ self getResponseCode
!

selectGroup:aGroupName
    |response s|

    aGroupName = lastGroup ifTrue:[
        ^ true
    ].
    nntpConnection isNil ifTrue:[
        ^ false
    ].
    self flushSubjectCache.

    currentFirst := 0.
    currentLast := 0.    

    response := self sendRequest:('GROUP ' , aGroupName).
    response == OKGROUP ifFalse:[
        ^ false
    ].
    s := ReadStream on:lastResponse.
    Number readFrom:s.      "skip"
    Number readFrom:s.      "skip"
    currentFirst := Number readFrom:s.    
    currentLast:= Number readFrom:s.    
    lastInCurrentGroup := currentLast.
    firstInCurrentGroup := currentFirst.
    lastGroup := aGroupName.
    ^ true
!

withTextResponseDo:aBlock
    |line|

    line := nntpConnection nextLine.
    [line notNil and:[line ~= '.']] whileTrue:[
        aBlock value:line.
        line := nntpConnection nextLine
    ]
!

getTextResponseFor:request expecting:expectedReply
    "send a request, expect a reply and return the received text"

    |text response|

    response := self sendRequest:request.
    expectedReply notNil ifTrue:[
        response == expectedReply ifFalse:[
            ^ nil
        ]
    ].

    text := (Text new:500) grow:0.
    self withTextResponseDo:[:line |
        text add:line
    ].
    ^ text
!

getTextResponseFor:request inGroup:group expecting:expectedReply
    |text|

    sema critical:[
        (group isNil or:[self selectGroup:group]) ifTrue:[
            text := self getTextResponseFor:request expecting:expectedReply 
        ].
    ].
    ^ text
!

helpInfo
    "return helptext from server"

    ^ self getTextResponseFor:'HELP' inGroup:nil expecting:nil
!

newsGroups
    "return a list of all newsgroups"

    |response|

    listOfNewsGroups isNil ifTrue:[
        nntpConnection isNil ifTrue:[
            ^ nil
        ].
        listOfNewsGroups := OrderedCollection new.
        articleCounts := Dictionary new.
        articleRanges := Dictionary new.
        moderatedFlags := Dictionary new.

        response := self sendRequest:('LIST').
        response == OKLIST ifTrue:[
            Transcript showCr:'reading active ...'.
            self withTextResponseDo:[:line |
                self processActiveLine:line
            ]
        ].
        Transcript showCr:'done.'
    ].
    ^ listOfNewsGroups
!

list
    "return list from server"

    ^ self getTextResponseFor:'LIST' inGroup:nil expecting:nil
!

article:aNumber in:aGroup
    "return an article in a newsgroup"

    ^ self getTextResponseFor:'ARTICLE ' , aNumber printString inGroup:aGroup expecting:OKARTICLE
!

withArticle:aNumber in:aGroup do:aBlock
    "helper for extraction methods"

    |response|

    sema critical:[
        (self selectGroup:aGroup) ifTrue:[
            response := self sendRequest:('ARTICLE ' , aNumber printString).
            response == OKARTICLE ifTrue:[
                self withTextResponseDo:aBlock
            ]
        ]
    ]
!

withArticleHeaderOf:aNumber in:aGroup do:aBlock
    "helper for extraction methods"

    |response|

    sema critical:[
        (self selectGroup:aGroup) ifTrue:[
            response := self sendRequest:('HEAD ' , aNumber printString).
            response == OKHEADER ifTrue:[
                self withTextResponseDo:aBlock
            ]
        ]
    ]
!

articleSubjectOf:aNumber in:aGroup
    |subject response|

    subject := self cachedSubjectOf:aNumber.
    subject notNil ifTrue:[^ subject].

    sema critical:[
        (self selectGroup:aGroup) ifTrue:[
            response := self sendRequest:('HEAD ' , aNumber printString).
            response == OKHEADER ifTrue:[
                subject := '<no subject>'.
                self withTextResponseDo:[:line |
                    (line startsWith:'Subject: ') ifTrue:[
                        subject := line copyFrom:10
                    ].
                ].
                self cacheSubject:subject of:aNumber.
            ].
        ].
    ].
    ^ subject
!

XXarticleSubjectOf:aNumber in:aGroup
    "return the article subject of an article in a newsgroup.
     this does not work and I dont have any documentation on the
     XHDR request at the moment ..."

    ^ self getTextResponseFor:'XHDR ' , aNumber printString , ' Subject' inGroup:aGroup expecting:OKHEADER
!

articleFromOf:aNumber in:aGroupName
    |from  response|


    self withArticleHeaderOf:aNumber in:aGroupName do:[:line |
        (line startsWith:'From: ') ifTrue:[
            from := line copyFrom:7 
        ].
        (line startsWith:'Subject:') ifTrue:[
            self cacheSubject:(line copyFrom:10) of:aNumber.
        ].
    ].

    ^ from
!

XXarticleFromOf:aNumber in:aGroup
    "return the articles sender of an article in a newsgroup.
     this does not work and I dont have any documentation on the
     XHDR request at the moment ..."

    ^ self getTextResponseFor:'XHDR ' , aNumber printString , ' From' inGroup:aGroup expecting:OKHEADER
!

articleHeaderSummaryOf:aNumber in:aNewsGroupName
    "return the article header of an article in a newsgroup"

    |line index subject from numberString maxNumber
     digits range index2|

    from := self articleFromOf:aNumber in:aNewsGroupName.
    from isNil ifTrue:[^ nil].

    subject := self articleSubjectOf:aNumber in:aNewsGroupName.
    subject isNil ifTrue:[
        subject := '<no subject>'
    ].

    "look for better from ..."
    index := from indexOf:$(.
    (index ~~ 0) ifTrue:[
        index2 := from indexOf:$) startingAt:(index + 1).
        (index2 ~~ 0) ifTrue:[
            from := from copyFrom:index to:index2
        ]
    ].

    maxNumber := (self articleRangeOf:aNewsGroupName) last.
    digits := maxNumber printString size.
    numberString := '   ' , (aNumber printString).
    numberString := numberString copyFrom:(numberString size - digits).
    ^ numberString , ' ' , subject,  ' ' , from
!

articleArchiveNameOf:aNumber in:aNewsGroupName
    "return the articles archive file name of an article in a newsgroup"

    |archive|

    archive := nil.
    self withArticle:aNumber in:aNewsGroupName do:[:line |
        ((line startsWith:'Archive-Name: ') or:[line startsWith:'Archive-name: ']) ifTrue:[
            archive := line copyFrom:15
        ]
    ].

    ^ archive
!

articlesIn:aNewsGroupName do:aBlock
    "evaluate aBlock for each article number in given newsgroup"

    (self articleRangeOf:aNewsGroupName) do:aBlock
!

numberOfArticlesIn:aGroup
    |range count|

    articleCounts isNil ifTrue:[
        articleCounts := Dictionary new
    ].
    count := articleCounts at:aGroup ifAbsent:[nil].
    count isNil ifTrue:[
        range := self articleRangeOf:aGroup.
        count := range size.
        articleCounts at:aGroup put:count.
    ].
    ^ count
!

articleRangeOf:aNewsGroupName
    "return range of article numbers in given newsgroup"

    |dir index range article|

    articleRanges isNil ifTrue:[
        articleRanges := Dictionary new
    ].
    range := articleRanges at:aNewsGroupName ifAbsent:[nil].
    range isNil ifTrue:[
        (self selectGroup:aNewsGroupName) ifFalse:[^ NumberSet new].
        currentLast == 0 ifTrue:[
            range := NumberSet new
        ] ifFalse:[
            range := NumberSet with:(currentFirst to:currentLast)
        ].
        articleRanges at:aNewsGroupName put:range
    ].

    "been here before .. range already known"
    ^ range
! !

NNTPHandler initialize!
