'From Smalltalk/X, Version:2.3.1 on 6-Sep-92 at 15:15:51'!

Object subclass:#NewsHandler
         instanceVariableNames:'listOfNewsGroups articleCounts moderatedFlags
                readRanges articleRanges subscribedFlags
                articleStream listOfSubscribedGroups hashLists lastGroup
                lastIndex cachedSubjects
                currentGroup firstInCurrentGroup lastInCurrentGroup'
         classVariableNames:''
         poolDictionaries:''
         category:'Interface-NewsTool'
!

NewsHandler comment:'
interface to news

%W% %E%
written summer 90 by claus

I have put all interface to the news-system into this class,
so that this code is not spread around;
should be enhanced to allow for much more functionality


'!

!NewsHandler class methodsFor:'defaults'!

oldSpoolDirPath
    ^ '/usr/spool/oldnews'
!

spoolDirPath
    "return the path of your spool directory.
     rubbish - should come from a configuration file ..."

    ^ '/usr/spool/news'
!

activeFilePath
    "on most machines this is usr/lib/news/active; on some,
     its in usr/local/lib/news. Check for local first; this should
     work ...
     rubbish - should come from a configuration file ..."

    (OperatingSystem isReadable:'/usr/local/lib/news/active') ifTrue:[
        ^ '/usr/local/lib/news/active'
    ].
    ^ '/usr/lib/news/active'
! !

!NewsHandler class methodsFor:'instance creation'!

new
    "answer a new newshandler for news"

    ^ self basicNew initialize
! !

!NewsHandler methodsFor:'initialization'!

initialize
    hashLists := Array new:513.
    lastGroup := nil
!

release
    articleStream notNil ifTrue:[
        articleStream close.
        articleStream := nil
    ].
    articleRanges := nil.
    articleCounts := nil
! !


!NewsHandler methodsFor:'private'!

flushSubjectCache
    currentGroup := nil.
    cachedSubjects := nil
!    

cachedSubjectOf:article
    cachedSubjects notNil ifTrue:[
        ^ cachedSubjects at:(article - firstInCurrentGroup + 1).
    ].
    ^ nil
!

cacheSubject:aString of:article
    cachedSubjects isNil ifTrue:[
        cachedSubjects := Array new:(lastInCurrentGroup - firstInCurrentGroup + 1).
    ].
    cachedSubjects at:(article - firstInCurrentGroup + 1) put:aString.
! !

!NewsHandler methodsFor:'misc'!

newsGroups
    "return a list of all newsgroups"

    |dir top stream|

    listOfNewsGroups isNil ifTrue:[
        top := self class spoolDirPath.
        dir := FileDirectory directoryNamed:top.
        dir isNil ifTrue:[
            self notify:'no news directory'.
            ^ nil
        ].
        dir := nil.

        listOfNewsGroups := OrderedCollection new.
        articleCounts := Dictionary new.
        articleRanges := Dictionary new.
        moderatedFlags := Dictionary new.

        stream := FileStream readonlyFileNamed:self class activeFilePath.
        stream isNil ifTrue:[
            self notify:'no active file'.
            ^ nil
        ].
        self readActiveFileFrom:stream
    ].
    ^ listOfNewsGroups

    "(NewsHandler new) newsGroups"
!

numberOfArticlesIn:aGroup
    "return the number of articles in given newsgroup"

    |count|

    count := articleCounts at:aGroup ifAbsent:[nil].
    count isNil ifTrue:[
        self newsGroups.
        count := articleCounts at:aGroup ifAbsent:[0]
    ].
    ^ count
!

articleHeaderSummaryOf:aNumber in:aNewsGroupName
    "return the article header of an article in a newsgroup"

    |line index subject from numberString path file maxNumber
     digits range index2|

    (aNewsGroupName ~= currentGroup) ifTrue:[self flushSubjectCache].

    path := (self groupNameToPath:aNewsGroupName) , '/' , (aNumber printString).
    file := FileStream readonlyFileNamed:path.
    file isNil ifTrue:[
        ^ nil "'-- article not readable --'  "
    ].
    subject := '<no subject>'.
    from := '(?)'.

    index := file peekForLineStartingWithAny:#('Subject: ' 'From: ').
    index notNil ifTrue:[
        line := file nextLine.
        (index == 1) ifTrue:[
            subject := line copyFrom:10.
            line := file peekForLineStartingWith:'From:'.
            line notNil ifTrue:[
                from := line copyFrom:7
            ]
        ] ifFalse:[
            from := line copyFrom:7.
            line := file peekForLineStartingWith:'Subject:'.
            line notNil ifTrue:[
                subject := line copyFrom:10
            ]
        ]
    ].

    file close.

    range := self articleRangeOf:aNewsGroupName.
    firstInCurrentGroup := range first.
    lastInCurrentGroup := range last.
    self cacheSubject:subject of:aNumber.

    "look for better from ..."
    index := from indexOf:$(.
    (index ~~ 0) ifTrue:[
        index2 := from indexOf:$) startingAt:(index + 1).
        (index2 ~~ 0) ifTrue:[
            from := from copyFrom:index to:index2
        ]
    ].
    maxNumber := (self articleRangeOf:aNewsGroupName) last.
    digits := maxNumber printString size.
    numberString := '   ' , (aNumber printString).
    numberString := numberString copyFrom:(numberString size - digits).
    ^ numberString , ' ' , subject,  ' ' , from
!

article:aNumber in:aNewsGroupName
    "return the article in a newsgroup"

    |text path|

    articleStream notNil ifTrue:[articleStream close].
    path := (self groupNameToPath:aNewsGroupName) , '/' , (aNumber printString).
    articleStream := FileStream readonlyFileNamed:path.
    articleStream isNil ifTrue:[
        ^ '-- article not available --'
    ].
    text := FileText of:articleStream.
    ^ text
!

subscribedGroups
    "answer a list of all subscribed newsgroups"

    listOfSubscribedGroups isNil ifTrue:[self readNewsrc].
    ^ listOfSubscribedGroups

    "(NewsHandler new) subscribedGroups"
!

isSubscribed:aNewsGroupName
    "answer true if a given newsgroup is subscribed"

    listOfSubscribedGroups isNil ifTrue:[
        self readNewsrc
    ].
    ^ subscribedFlags at:aNewsGroupName ifAbsent:[false]
!

readRangeOf:aNewsGroupName
    "return a set of read articles in a given newsgroup"

    |range|

    listOfSubscribedGroups isNil ifTrue:[self readNewsrc].

    range := readRanges at:aNewsGroupName ifAbsent:[nil].

    "here, we avaluate the lazy range"
    (range isMemberOf:String) ifTrue:[
        range := NumberSet readFrom:(ReadStream on:range).
        readRanges at:aNewsGroupName put:range
    ].
    range isNil ifTrue:[
        range := NumberSet new.
        readRanges at:aNewsGroupName put:range
    ].
    ^ range
!

subscribe:aBoolean to:aNewsGroupName
    "un/subscribe a newsGroup"

    listOfSubscribedGroups isNil ifTrue:[self readNewsrc].
    (subscribedFlags at:aNewsGroupName ifAbsent:[false]) == aBoolean ifFalse:[
        aBoolean ifTrue:[
            (listOfSubscribedGroups includes:aNewsGroupName) ifFalse:[
                listOfSubscribedGroups add:aNewsGroupName
            ]
        ] ifFalse:[
            (listOfSubscribedGroups includes:aNewsGroupName) ifTrue:[
                listOfSubscribedGroups remove:aNewsGroupName
            ]
        ].
        subscribedFlags at:aNewsGroupName put:aBoolean
    ]
!

numberOfUnreadArticlesIn:aNewsGroupName
    "return the number of unread articles in given newsgroup"

    |count|

    listOfNewsGroups isNil ifTrue:[self newsGroups].
    (self numberOfArticlesIn:aNewsGroupName) == 0 ifTrue:[^ 0].
    count := 0.
    self unreadArticlesIn:aNewsGroupName do:[:dummy |
        count := count + 1
    ].
    ^ count
!

articlesIn:aNewsGroupName
    "return a list of all article numbers in given newsgroup"

    |list|

    list := OrderedCollection new.
    self articlesIn:aNewsGroupName do:[:anEntry |
        list add:anEntry
    ].
    ^ list
!

articlesIn:aNewsGroupName do:aBlock
    "evaluate aBlock for each article number in given newsgroup"

    |dir range article|

    listOfNewsGroups isNil ifTrue:[self newsGroups].

    range := articleRanges at:aNewsGroupName ifAbsent:[nil].
    range isNil ifTrue:[
        range := NumberSet new.
        dir := FileDirectory directoryNamed:(self groupNameToPath:aNewsGroupName).
        dir isNil ifFalse:[
            dir do:[:anEntry |
                ((dir typeOf:anEntry) == #regular) ifTrue:[
                    article := Number readFromString:anEntry.
                    aBlock value:article.
                    range add:article
                ]
            ]
        ].
        articleRanges at:aNewsGroupName put:range.
        ^ self
    ].

    "been here before .. range already known"

    range do:aBlock
!

unreadArticlesIn:aNewsGroupName
    "return a list of all unread article file names in given newsgroup"

    |list|

    list := OrderedCollection new.
    self unreadArticlesIn:aNewsGroupName do:[:anEntry |
        list add:anEntry
    ].
    ^ list
!

unreadArticlesIn:aNewsGroupName do:aBlock
    "evaluate aBlock for each unread article file name in given newsgroup"

    |range|

    range := self readRangeOf:aNewsGroupName.
    self articlesIn:aNewsGroupName do:[:anEntry |
        range isNil ifTrue:[
            aBlock value:anEntry
        ] ifFalse:[
            (range includes:anEntry) ifFalse:[
                aBlock value:anEntry
            ]
        ]
    ]
!

articleRangeOf:aNewsGroupName
    "return range of article numbers in given newsgroup"

    |dir range article|

    listOfNewsGroups isNil ifTrue:[self newsGroups].
    range := articleRanges at:aNewsGroupName ifAbsent:[nil].
    range isNil ifTrue:[
        range := NumberSet new.
        dir := FileDirectory directoryNamed:(self groupNameToPath:aNewsGroupName).
        dir isNil ifFalse:[
            dir do:[:anEntry |
                ((dir typeOf:anEntry) == #regular) ifTrue:[
                    article := Number readFromString:anEntry.
                    range add:article
                ]
            ]
        ].
        articleRanges at:aNewsGroupName put:range
    ].

    "been here before .. range already known"

    ^ range
!

withArticleHeaderOf:aNumber in:aNewsGroupName do:aBlock
    "helper for extraction methods"

    |line path file|

    path := (self groupNameToPath:aNewsGroupName) , '/' , (aNumber printString).
    file := FileStream readonlyFileNamed:path.
    file isNil ifTrue:[^ nil].

    [file atEnd] whileFalse:[
        line := file nextLine.
        line notNil ifTrue:[
            aBlock value:line
        ]
    ].

    file close
!

articleSubjectOf:aNumber in:aNewsGroupName
    "return the article subject of an article in a newsgroup"

    |line subject path file range|

    (aNewsGroupName ~= currentGroup) ifTrue:[self flushSubjectCache].

    subject := self cachedSubjectOf:aNumber.
    subject notNil ifTrue:[^ subject].

    subject := '<no subject>'.

    self withArticleHeaderOf:aNumber in:aNewsGroupName do:[:line |
        (line startsWith:'Subject: ') ifTrue:[
            subject := line copyFrom:10
        ]
    ].

    self cacheSubject:subject of:aNumber.

    ^ subject
!

articleAuthorOf:aNumber in:aNewsGroupName
    "return the article author of an article in a newsgroup"

    |line from index|

    from := '(?)'.
    self withArticleHeaderOf:aNumber in:aNewsGroupName do:[:line |
        (line startsWith:'From: ') ifTrue:[
            from := line copyFrom:7
        ]
    ].

    "look for better from ..."
    index := from indexOf:$(.
    (index ~~ 0) ifTrue:[
        from := from copyFrom:index to:(from indexOf:$) startingAt:index)
    ].
    ^ from
!

articleArchiveNameOf:aNumber in:aNewsGroupName
    "return the articles archive file name of an article in a newsgroup"

    |archive|

    archive := nil.
    self withArticleHeaderOf:aNumber in:aNewsGroupName do:[:line |
        ((line startsWith:'Archive-Name: ') or:[line startsWith:'Archive-name: ']) ifTrue:[
            archive := line copyFrom:15
        ]
    ].

    ^ archive
!

articleMessageIDOf:aNumber in:aNewsGroupName
    "return the articles messageID of an article in a newsgroup"

    |id|

    id := nil.
    self withArticleHeaderOf:aNumber in:aNewsGroupName do:[:line |
        (line startsWith:'Message-ID: ') ifTrue:[
            id := line copyFrom:13
        ]
    ].

    ^ id
!

markArticleRead:articleNumber inGroup:aGroupName
    "add article to read-range of aGroupName"

    |range|

    range := self readRangeOf:aGroupName.
    range add:articleNumber
!

markReadAllArticlesInGroup:aGroupName
    "add all articles in aGroupName to read-range"

    |range|

    range := self readRangeOf:aGroupName.
    self unreadArticlesIn:aGroupName do:[:article |
        range add:article
    ]
!

markArticleUnread:articleNumber inGroup:aGroupName
    "add article to read-range of aGroupName"

    |range|

    range := self readRangeOf:aGroupName.
    range remove:articleNumber

!

markUnreadAllArticlesInGroup:aGroupName
    "remove all articles in aGroupName to read-range"

    self readRangeOf:aGroupName put:nil

!

readRangeOf:aNewsGroupName put:aRange
    "set the set of read articles in a given newsgroup"

    listOfSubscribedGroups isNil ifTrue:[self readNewsrc].
    readRanges at:aNewsGroupName put:aRange
!

anyArticlesIn:aNewsGroupName
    "return true, if there are any articles in given newsgroup"

    ^ (self numberOfArticlesIn:aNewsGroupName) ~~ 0
!

anyUnreadArticlesIn:aNewsGroupName
    "return true, if there are any unread articles in given newsgroup"

    |range readRange|

    range := self articleRangeOf:aNewsGroupName.
    (range size == 0) ifTrue:[^ false].

    readRange := self readRangeOf:aNewsGroupName.
    range do:[:anEntry |
        (readRange includes:anEntry) ifFalse:[^ true]
    ].
    ^ false
!

updateNewsrc
    "rewrite .newsrc file"

    |stream name range|

    name := OperatingSystem getHomeDirectory , '/.newsrc'.

    stream := FileStream newFileNamed:(name ,'.new').

    stream isNil ifTrue:[
        self notify:'cannot update .newsrc'.
        ^ self
    ].
    listOfNewsGroups isNil ifTrue:[
        self notify:'no .newsrc update'.
        ^ self
    ].
    listOfNewsGroups do:[:group|
        stream nextPutAll:group.
        (subscribedFlags at:group ifAbsent:[false]) ifTrue:[
            stream nextPut:$:
        ] ifFalse:[
            stream nextPut:$!!
        ].
        stream space.
        range := readRanges at:group ifAbsent:[nil].
        range notNil ifTrue:[
            (range isMemberOf:String) ifTrue:[
                stream nextPutAll:range
            ] ifFalse:[
                range printOn:stream
            ]
        ].
        stream cr
    ].
    stream close.
    OperatingSystem executeCommand:('mv ' , name , ' ' , name , '.old').
    OperatingSystem executeCommand:('mv ' , name , '.new ' , name)
! !

!NewsHandler methodsFor:'private'!

readNewsrc
    "read .newsrc, fill in flags of all subscribed newsgroups"

    |inStream newsRC|

    listOfNewsGroups isNil ifTrue:[self newsGroups].
    listOfNewsGroups isNil ifTrue:[
        self notify:'no news'.
        ^ self
    ].

    newsRC := OperatingSystem getHomeDirectory , '/.newsrc'.
    inStream := FileStream readonlyFileNamed:newsRC.
    inStream isNil ifTrue:[
        self notify:'no .newsrc in home directory'.
        ^ self
    ].
    listOfSubscribedGroups := OrderedCollection new.
    subscribedFlags := Dictionary new.
    readRanges := Dictionary new.
    self readSubscribedGroupsFrom:inStream.
    inStream close
!

relativePathNameToGroup:aPathName
    "answer the newsgroup name for a given path (relative to /usr/spool/news)"

    ^ (aPathName copy) replaceAll:$/ by:$. 
!

groupNameToPath:aNewsGroupName
    "answer the news - path name for a given newsgroup"

    |relPath|

    relPath := aNewsGroupName copy replaceAll:$. by:$/.
    ^ (self class spoolDirPath) , '/', relPath

!

findGroupsAndCountsOf:aDir into:groups and:counts
    "recursively traverses news-spool directory; in the way
     count articles and append all subgroup names to groups and 
     corresponding article counts to counts.
     and fill hashList with table index positions.
    -- obsolete ---"

    |articleCount hasSubdirs subGroups subCounts myPath relPath groupName
     hashKey|

    articleCount := 0.
    hasSubdirs := false.

    subGroups := OrderedCollection new.
    subCounts := OrderedCollection new.

    aDir do:[:aName |
        (aName startsWith:'.') ifFalse:[
            (aDir isDirectory:aName) ifTrue:[
                "a subdirectory"
                hasSubdirs := true.
                self findGroupsAndCountsOf:(FileDirectory directoryNamed:aName in:aDir)
                                      into:subGroups and:subCounts
            ] ifFalse:[
                (aName at:1) isDigit ifTrue:[
                    "an article"
                    articleCount := articleCount + 1
                ]
            ]
        ]
    ].
    "if there where any articles, append my name"
    (articleCount == 0) ifFalse:[
        myPath := aDir pathName.
        (myPath = self class spoolDirPath) ifFalse:[
            relPath := myPath copyFrom:(self class spoolDirPath size + 2).  "start after slash"
            groupName := self relativePathNameToGroup:relPath.

            groups add:groupName.
            counts add:articleCount.

            self addHash:(groups size) forGroup:groupName
        ]
    ].
    hasSubdirs ifTrue:[
        groups addAll:subGroups.
        counts addAll:subCounts
    ]
!

readSubscribedGroupsFrom:aStream
    "read .newsrc file"

    |line index group range subscribed|

Transcript showCr:'reading .newsrc'.

    listOfNewsGroups isNil ifTrue:[
        listOfNewsGroups := OrderedCollection new.
        articleCounts := Dictionary new.
        articleRanges := Dictionary new.
        moderatedFlags := Dictionary new.
    ].

    [aStream atEnd] whileFalse:[
        line := aStream nextLine.
        line notNil ifTrue:[
            index := line indexOf:$:.
            (index == 0) ifTrue:[
                index := line indexOf:$!!.
                (index == 0) ifTrue:[
                    Transcript show:'invalid line in .newsrc: '.
                    Transcript showCr:line
                ] ifFalse:[
                    "group!! something - unsubscribed"
                    subscribed := false
                ]
            ] ifFalse:[
                "group: something - subscribed"
                subscribed := true
            ].
            (index == 0) ifFalse:[
                group := line copyFrom:1 to:(index - 1).
                range := line copyFrom:(index + 1).
                ((range size == 0) or:[range isBlank]) ifTrue:[
                    range := nil
                ].
                index := self indexOfGroup:group.
                (index == 0) ifTrue:[
                    Transcript showCr:('nonexisting group: ' , group , ' in .newsrc.').

                    "its in newsrc - but not in news (no articles)"
                    listOfNewsGroups add:group.
                    "articleCounts at:group put:0.      "
                    "articleRanges at:group put:0.      "
                    subscribedFlags at:group put:subscribed.
                    "readRanges at:group put:0. "
                    moderatedFlags at:group put:true.  "just to make shure he wont post"

                    self addHash:(listOfNewsGroups size) forGroup:group
                ] ifFalse:[
                    "already there - check for double entry in newsrc"
                    (subscribedFlags at:group ifAbsent:[false]) == subscribed ifTrue:[
                        subscribedFlags at:group put:subscribed
                    ] ifFalse:[
                        Transcript showCr:('multiple entry for: ' , group , ' in .newsrc.').
                        subscribed ifTrue:[
                            subscribedFlags at:group put:true
                        ]
                    ].
                    (readRanges at:group ifAbsent:[nil]) notNil ifTrue:[
                        self error:'oops readrange'
                    ].
                    readRanges at:group put:range
                ].
                subscribed ifTrue:[listOfSubscribedGroups add:group]
            ]
        ]
    ].
Transcript showCr:'done'
!

hashCodeForString:aString
    |len|

    len := aString size.
     ^ ((len bitShift:9) +
        ((aString at:1) asciiValue bitShift:1) +
        (aString at:2) asciiValue) \\ 513 + 1
!

indexOfGroup:aGroupString
    "return the index of the group aGroupString.
     this index is used to access the various tables"

    |hashKey list count index|

    "keep last group - there are often other accesses to same
     group immediately following ..."

    listOfNewsGroups isNil ifTrue:[^ 0].

    (lastGroup = aGroupString) ifTrue:[
        ^ lastIndex
    ].

    hashKey := self hashCodeForString:aGroupString.
    list := hashLists at:hashKey.
    list isNil ifTrue:[^ 0].

    count := 1.
    list do:[:link |
        index := link value.
        ((listOfNewsGroups at:index) = aGroupString) ifTrue:[
"            Transcript showCr:(count printString).  "
            lastGroup := aGroupString.
            lastIndex := index.
            ^ index
        ].
        count := count + 1
    ].

    ^ 0
!

addHash:index forGroup:aGroupName
    |hashKey|

    hashKey := self hashCodeForString:aGroupName.
    (hashLists at:hashKey) isNil ifTrue:[
       hashLists at:hashKey put:(LinkedList new)
    ].
    (hashLists at:hashKey) add:(ValueLink new value:index)
!

readActiveFileFrom:aStream
    |line index start group nArticles nextLocalArticle mode moderated available|

Transcript showCr:'reading active'.
    [aStream atEnd] whileFalse:[
        line := aStream nextLine.
        line notNil ifTrue:[
            self processActiveLine:line
        ]
    ].
Transcript showCr:'done'
!

processActiveLine:line
    |index start group nArticles nextLocalArticle mode moderated available|

    index := line indexOfSeparatorStartingAt:1.
    (index == 0) ifTrue:[
        self notify:('corrupted active file: ' , line)
    ] ifFalse:[
        group := line copyFrom:1 to:(index - 1).

        start := index + 1.
        index := line indexOfSeparatorStartingAt:start.
        (index == 0) ifTrue:[
            self notify:('corrupted active file: ' , line)
        ] ifFalse:[
            nArticles := Number readFromString:(line copyFrom:start to:(index - 1)).

            start := index + 1.
            index := line indexOfSeparatorStartingAt:start.
            (index == 0) ifTrue:[
                self notify:('corrupted active file: ' , line)
            ] ifFalse:[
                nextLocalArticle := Number readFromString:(line copyFrom:start to:(index - 1)).

                start := index + 1.
                mode := line copyFrom:start.

                (mode = 'm') ifTrue:[
                    moderated := true.
                    available := true
                ] ifFalse:[
                    moderated := false.
                    (mode = 'y') ifTrue:[
                        available := true
                    ] ifFalse:[
                        available := false
                    ]
                ].
                available ifTrue:[
                    listOfNewsGroups add:group.
                    articleCounts at:group put:nArticles.
                    articleRanges at:group put:nil.
                    moderatedFlags at:group put:moderated.

                    self addHash:(listOfNewsGroups size) forGroup:group
                ]
            ]
        ]
    ]
! !
