"
 COPYRIGHT (c) 1990 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ModalBox subclass:#Notifier
       instanceVariableNames:'labelField okButton okAction panel'
       classVariableNames:''
       poolDictionaries:''
       category:'Views-DialogBoxes'
!

Notifier comment:'

COPYRIGHT (c) 1990 by Claus Gittinger
              All Rights Reserved

this class implements a pop-up box to notify user. 

$Header: /claus/smalltalk/st2.6/libwidg/RCS/Notifier.st,v 1.4 93/12/11 01:47:06 claus Exp $
written Feb 90 by claus
'!

!Notifier class methodsFor:'defaults'!

defaultExtent
    ^ 300 @ 200
! !

!Notifier class methodsFor:'instance creation'!

title:titleString
    "create a Notifier with title:aTitleString"

    ^ (self new) title:titleString

    "(Notifier title:'hello') showAtPointer"
!

title:titleString okText:okText action:aBlock
    "create a Notifier; define its title, text in ok-Button and
     action"

    ^ (self new) title:titleString okText:okText okAction:aBlock

    "(Notifier title:'hello' okText:'ok' action:nil) showAtPointer"
! !

!Notifier methodsFor:'initialization'!

initialize
    |innerWidth|

    super initialize.

    innerWidth := width - (ViewSpacing * 2).

    labelField := Label label:'' in:self.
    labelField borderWidth:0.
    labelField origin:(ViewSpacing @ ViewSpacing)
               extent:(innerWidth @ labelField height).

    panel := HorizontalPanelView
                origin:[ViewSpacing @ (height - (font height * 2) - ViewSpacing - (borderWidth * 2))]
                extent:[(width - (ViewSpacing * 2)) @ (font height * 2 + (borderWidth * 2))]
                    in:self.
    panel borderWidth:0.

    okButton := Button label:(resources at:'ok')
                      action:[okButton turnOffWithoutRedraw.
                              self okPressed] 
                          in:panel

    "self notify:'hello world'"
!

initEvents
    super initEvents.
    self enableKeyEvents
! !

!Notifier methodsFor:'realization'!

positionOffset
    "return the delta, by which the box should be displayed
     from the mouse pointer. Value returned here makes
     okButton appear under the cursor"

    ^ (okButton originRelativeTo:self) + (okButton extent // 2)
! !

!Notifier methodsFor:'accessing'!

font:aFont
    "pass font down to label"

    aFont ~= labelField font ifTrue:[
        labelField font:aFont.
        self resize
    ]
!

title:aString
    "set the Notifiers title"

    aString ~= labelField label ifTrue:[
        labelField label:aString.
        labelField resize.
        self resize
    ]
!

okText:aString
    "set the text to be displayed in the ok-button"

    aString ~= okButton label ifTrue:[
        okButton label:aString.
        okButton resize.
        self resize
    ]
!

okAction:aBlock
    "set the action to be performed when user presses ok-button"

    okAction := aBlock
!

title:titleString okText:okString okAction:aBlock
    "convenient function to set all relevant stuff"

    ((titleString ~= labelField label) or:[okString ~= okButton label]) ifTrue:[
        labelField label:titleString.
        okButton label:okString.
        labelField resize.
        okButton resize.
        self resize
    ].
    okAction := aBlock.
! !

!Notifier methodsFor:'private'!

resize
    "resize myself to make everything visible"

    |wWanted hWanted|

    wWanted := (labelField width max:okButton width) + (ViewSpacing * 2).
    hWanted := ViewSpacing
               + labelField height
               + (ViewSpacing * 4)
               + okButton height
               + ViewSpacing.
    self extent:(wWanted @ hWanted)
! !

!Notifier methodsFor:'user interaction'!

okPressed
    "user pressed ok button - hide myself and,
     if there is any action defined, evaluate it"

    self hide.
    okAction notNil ifTrue:[
        okAction value
    ]
! !
