"       NAME            Paths
        AUTHOR          scaletti@uxc.cso.uiuc.edu (Kurt J. Hebel)
        FUNCTION        Reimplementation of the Path hierarchy
        ST-VERSION      2.2?
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1
        DATE    3 Jun 1987
"

AbstractPath subclass: #Path
        instanceVariableNames: 'collectionOfPoints'
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Graphics-Paths'!
Path comment:
'Path defines a path that includes an arbitrary collection of points.  In addition
 to the protocol defined in the superclass, PointPaths allow access to the collection
 of points.  The instance variable ''collectionOfPoints'' can be any kind of
 Collection.  This class uses the do: message to iterate through the collection
 of points, and simply displays a copy of its form at each of the given points.'!

!Path methodsFor: 'point accessing'!

add: aPoint 
        "Add aPoint to the receiver's collection of points in the Path."
        ^ collectionOfPoints add: aPoint!

points
        "Return the collection of points for this Path."
        ^ collectionOfPoints!

points: aCollection
        "Set the collection of points for this Path."
        collectionOfPoints _ aCollection.! !

!Path methodsFor: 'transforming'!

scaleBy: aPoint 
        "Answer with a new Path of the same class scaled by aPoint.  Does not effect
         the current data in this Path."
        | newPath |

        newPath _ self species on:
                (collectionOfPoints collect: [ :point | point * aPoint]).
        newPath form: self form.
        ^ newPath!

translateBy: aPoint 
        "Answer a new instance of Path whose elements are translated by aPoint.  Does
         not effect the elements of this Path."
        | newPath |

        newPath _ self species on:
                (collectionOfPoints collect: [ :point | point + aPoint]).
        newPath form: self form.
        ^ newPath! !

!Path methodsFor: 'displaying'!

displayOn:aDisplayMedium at:aDisplayPoint clippingBox:clipRectangle rule:ruleInteger mask: aForm 
        "Display this PointPath as a copy of its form at each point in its collection
         of points.  Also, see comment in DisplayObject|displayOn:at:clippingBox:rule:mask:."

        | displayForm |

        displayForm _ self form.
        collectionOfPoints do: [ :point | 
                displayForm
                        displayOn: aDisplayMedium
                        at: point + aDisplayPoint
                        clippingBox: clipRectangle
                        rule: ruleInteger
                        mask: aForm].! !

!Path methodsFor: 'display box access'!

computeBoundingBox
        "Compute the smallest Rectangle that covers all of the points in the Path."

        | minPt maxPt |

        maxPt _ collectionOfPoints at: 1.
        minPt _ maxPt.
        collectionOfPoints do: [ :aPoint |
                minPt _ minPt min: aPoint.
                maxPt _ maxPt max: aPoint].
        ^ minPt corner: maxPt! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!Path class methodsFor: 'instance creation'!

new
        "Create a new PointPath with its collection set to a new OrderedCollection."

        ^ super new points: OrderedCollection new!

new: size
        "Create a new PointPath with its collection set to a new OrderedCollection of
         the given size."
        ^ super new points: (OrderedCollection new: size)!

on: aCollection
        "Create a new PointPath with the argument as its collection of points."
        ^ super new points: aCollection! !


" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
