"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

MenuView subclass:#PatternMenu
       instanceVariableNames:'patterns nrows ncols
                              patternSize space layout windowRatio inSet'
       classVariableNames:''
       poolDictionaries:''
       category:'Applications-DrawTool'
!

PatternMenu comment:'

COPYRIGHT (c) 1989-93 by Claus Gittinger
              All Rights Reserved

a menu for fill- and line-patterns

$Header: /claus/smalltalk/st2.6/clients/DrawTool/RCS/PattMenu.st,v 1.2 94/01/13 00:42:47 claus Exp $
written fall 89 by claus
'!

!PatternMenu class methodsFor:'defaults'!

defaultLayout
    "specifies default layout; either verticalFirst or horizontalFirst"
    ^ #verticalFirst
!

defaultSpace
    "returns space between patterns"
    ^ 1
!

defaultPatternSize
    "returns default pattern size"

    "take 32x32 on a 100 dpi display"

    ^ (32 * Display horizontalPixelPerInch / 100) rounded asInteger 
      @
      (32 * Display verticalPixelPerInch / 100) rounded asInteger 
!

defaultWindowRatio
    "height shalt be 3 times the width"
    ^ 1 @ 3
! !

!PatternMenu class methodsFor:'instance creation'!

patterns:patterns selectors:selectors receiver:anObject for:aTopMenu
    |newMenu|
    newMenu := self in:(aTopMenu superView).
    newMenu patterns:patterns.
    newMenu selectors:selectors.
    newMenu receiver:anObject.
    ^newMenu
!

patterns:patterns selectors:selectors args:args receiver:anObject for:aTopMenu
    |newMenu|
    newMenu := self in:(aTopMenu superView).
    newMenu patterns:patterns.
    newMenu selectors:selectors.
    newMenu args:args.
    newMenu receiver:anObject.
    ^newMenu
! !

!PatternMenu methodsFor:'initialization'!

initialize
    super initialize.
    View3D ifTrue:[
        inSet := 4
    ] ifFalse:[
        inSet := 0
    ].
    layout := self class defaultLayout.
    windowRatio := self class defaultWindowRatio.
    space := self class defaultSpace.
    patternSize := self class defaultPatternSize
! !

!PatternMenu methodsFor:'accessing'!

patterns
    ^ patterns
!

patterns:collectionOfPatterns
    patterns := collectionOfPatterns.
    enableFlags := Array new:(patterns size).
    enableFlags atAllPut:true.
    self recomputeSize
!

isValidSelection:aNumber
    aNumber isNil ifTrue:[^ false].
    (aNumber < 1) ifTrue:[^ false].
    (aNumber > patterns size) ifTrue:[^ false].
    ^ true
! !

!PatternMenu methodsFor:'private'!

recomputeSize
    "from the ratio, compute number of row/cols needed"
    |factor nr nc w h nPattern|

    nr := windowRatio y.
    nc := windowRatio x.
    nPattern := patterns size.
    factor := nPattern // (nr * nc).
    factor := factor sqrt asInteger.
    nrows := nr * factor.
    ncols := nc * factor.
    (nrows * ncols) < nPattern ifTrue:[
        ncols := ncols + 1.
        nrows := nPattern // ncols.
        (nrows * ncols) < nPattern ifTrue:[
            nrows := nrows + 1
        ]
    ].
    w := ((patternSize x + inSet + inSet) * ncols) + ((ncols - 1) * space).
    h := ((patternSize y + inSet + inSet) * nrows) + ((nrows - 1) * space).
    self width:w height:h
!

positionToSelectionX:xpos y:ypos
    |index row col|

    (ypos < 0 or:[ypos >= height]) ifTrue:[^ nil].
    (xpos < 0 or:[xpos >= width]) ifTrue:[^ nil].

    col := xpos // ((patternSize x + inSet + inSet) + space).
    row := ypos // ((patternSize y + inSet + inSet) + space).
    (layout == #verticalFirst) ifTrue:[
        index := nrows * col + row
    ] ifFalse:[
        index := ncols * row + col
    ].
    index := index + 1.
    (index > patterns size) ifTrue:[
        ^ nil
    ].
    ^ index
! !

!PatternMenu methodsFor:'drawing'!

show:aPattern atX:xp y:yp selected:selected
    |w h patternWidth patternHeight oldFg oldBg oldPaint|

    patternWidth := patternSize x.
    patternHeight := patternSize y.
    w := patternWidth + inSet + inSet.
    h := patternHeight + inSet + inSet.
    aPattern notNil ifTrue:[
        "this is slow on stupid servers ...
        self fillRectangleX:xp + inSet
                          y:yp + inSet
                      width:patternWidth
                     height:patternHeight
                       with:aPattern
        "
        (aPattern isKindOf:Form) ifTrue:[
            self foreground:Black background:White.
            self copyFrom:aPattern x:0 y:0
                      toX:xp + inSet y:yp + inSet
                    width:patternWidth
                   height:patternHeight
        ] ifFalse:[
            self fillRectangleX:xp + inSet
                              y:yp + inSet
                          width:patternWidth
                         height:patternHeight
                           with:aPattern
        ]
    ].
    View3D ifTrue:[
        oldFg := foreground.
        oldBg := background.
        oldPaint := paint.
        selected ifTrue:[
            self drawEdgesForX:xp y:yp width:w height:h level:-2
        ] ifFalse:[
            self drawEdgesForX:xp y:yp width:w height:h level:2
        ].
        self foreground:oldFg background:oldBg.
        paint := nil.
        self paint:oldPaint
    ] ifFalse:[
        selected ifTrue:[
            self displayRectangleX:xp y:yp width:w height:h.
            self displayRectangleX:xp + 1 y:yp + 1 width:w - 2 height:h - 2.
            self displayRectangleX:xp + 2 y:yp + 2 width:w - 4 height:h - 4
        ]
    ]
!

redrawPattern:patternNr
    |row col p xpos ypos|

    p := patternNr - 1.
    (layout == #verticalFirst) ifTrue:[
        col := p // nrows.
        row := p - (col * nrows)
    ] ifFalse:[
        row := p // ncols.
        col := p - (row * ncols)
    ].
    xpos := (patternSize x + inSet + inSet + space) * col.
    ypos := (patternSize y + inSet + inSet + space) * row.

    self     show:(patterns at:patternNr) 
              atX:xpos 
                y:ypos
         selected:(patternNr == selection)
!

redrawElement:aNumber
    self redrawPattern:aNumber
!

redraw
    |row col xpos ypos dx dy|

    shown ifFalse:[^ self].
    self backingStore:true.    "next time will be fast"
    View3D ifFalse:[
        "draw a grid"
        xpos := patternSize x.
        ypos := 0.
        1 to:(ncols - 1) do:[:index |
            1 to:space do:[:index2 |
                self displayLineFromX:xpos y:0 toX:xpos y:height.
                xpos := xpos + 1
            ].
            xpos := xpos + patternSize x
        ].
        xpos := 0.
        ypos := patternSize y.
        1 to:(nrows - 1) do:[:index |
            1 to:space do:[:index2 |
                self displayLineFromX:0 y:ypos toX:width y:ypos.
                ypos := ypos + 1
            ].
            ypos := ypos + patternSize y
        ]
    ].

    row := 0.
    col := 0.
    xpos := 0.
    ypos := 0.
    dx := patternSize x + inSet + inSet + space.
    dy := patternSize y + inSet + inSet + space.
    1 to:(patterns size) do:[:index |
        self   show:(patterns at:index)
                atX:xpos
                  y:ypos
           selected:(index == selection).

        (layout == #verticalFirst) ifTrue:[
            row := row + 1.
            (row == nrows) ifTrue:[
                row := 0.
                ypos := 0.
                col := col + 1.
                xpos := xpos + dx
            ] ifFalse:[
                ypos := ypos + dy
            ]
        ] ifFalse:[
            col := col + 1.
            (col == ncols) ifTrue:[
                col := 0.
                xpos := 0.
                row := row + 1.
                ypos := ypos + dy
            ] ifFalse:[
                xpos := xpos + dx
            ]
        ]
    ]
!

redrawX:x y:y width:w height:h
    self redraw
! !
