'From Smalltalk/X, Version:1.4 on 24-Jul-91 at 14:22:39'!

"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

BitBlt subclass:#Pen
       instanceVariableNames:'locationX locationY direction isDown dX dY'
       classVariableNames:''
       poolDictionaries:''
       category:'Graphics-Support'
!

Pen comment:'
%W% %E%
'!

!Pen methodsFor:'initialization'!

initialize
    super initialize.
    self defaultNib:1.
    self home.
    self east.
    self down
! !

!Pen methodsFor:'accessing'!

direction
    ^ direction
!

location
    ^ locationX @ locationY
! !

!Pen methodsFor:'pen up/down'!

down
    isDown := true
!

up
    isDown := false
! !

!Pen methodsFor:'moving'!

place:aPoint
    locationX := aPoint x.
    locationY := aPoint y
!

home
    self place:(destForm center)
!

direction:aNumber
    "0 is east; 90 is south"

    direction := aNumber.
    direction >= 360 ifTrue:[
        direction := direction - (((direction / 360) asInteger) * 360)
    ].
    direction < 0 ifTrue:[
        direction := direction - (((direction / 360) asInteger) * 360).
        direction := direction + 360
    ].
    "avoid rounding errors and allow integer arithmetic for common cases"
    direction == 0 ifTrue:[
        dX := 1. dY := 0
    ] ifFalse:[
        direction == 90 ifTrue:[
            dX := 0. dY := 1
        ] ifFalse: [
            direction == 180 ifTrue:[
                dX := -1. dY := 0
            ] ifFalse: [
                direction == 270 ifTrue:[
                    dX := 0. dY := -1
                ] ifFalse: [
                    dX := direction degreesToRadians cos.
                    dY := direction degreesToRadians sin
                ]
            ]
        ]
    ]
!

turn:aNumber
    self direction:(direction + aNumber)
!

north
    direction := 270.
    dX := 0.0.
    dY := -1.0
!

east
    direction := 0.
    dX := 1.0.
    dY := 0.0
!

south
    direction := 90.
    dX := 0.0.
    dY := 1.0
!

west
    direction := 180.
    dX := -1.0.
    dY := 0.0
!

go:distance
    |newX newY|

    isDown ifTrue:[
        newX := (dX * distance) + locationX.
        newY := (dY * distance) + locationY.

        self drawFromX:(locationX asInteger) y:(locationY asInteger)
                   toX:(newX asInteger) y:(newY asInteger).

        locationX := newX.
        locationY := newY
    ] ifFalse:[
        locationX := (dX * distance) + locationX.
        locationY := (dY * distance) + locationY
    ]
!

goto:newLocation
    isDown ifTrue:[
        self drawFromX:(locationX asInteger) y:(locationY asInteger)
                   toX:(newLocation x asInteger) y:(newLocation y asInteger)
    ].
    locationX := newLocation x.
    locationY := newLocation y
! !

!Pen methodsFor:'geometric designs'!

spiral:n angle:a

    1 to:n do:[ :i |
        self go:i.
        self turn:a
    ]

    "(Pen new) spiral:300 angle:89"
    "(Pen new) spiral:300 angle:121"
    "(Pen new) spiral:300 angle:59"
!

dragon:n

    (n == 0) ifTrue:[
        self go:10
    ] ifFalse:[
        n > 0 ifTrue:[
            self dragon: n - 1.
            self turn:90.
            self dragon:1 - n
        ] ifFalse:[
            self dragon: -1 - n.
            self turn: -90.
            self dragon: 1 + n
        ]
    ]
    "(Pen new) dragon:10"
!

hilbert:n side:s
    |a m negA negM|

    n == 0 ifTrue:[
        self turn:180
    ] ifFalse: [
        n > 0 ifTrue:[
            a := 90.
            negA := -90.
            m := n - 1
        ] ifFalse:[
            a := -90.
            negA := 90.
            m := n + 1
        ].
        negM := 0 - m.
        self turn:a.
        self hilbert:negM side:s.
        self turn:a.
        self go:s.
        self hilbert:m side:s.
        self turn:negA.
        self go:s.
        self turn:negA.
        self hilbert:m side:s.
        self go:s.
        self turn:a.
        self hilbert:negM side:s.
        self turn:a
    ]
    "(Pen new) hilbert:5 side:15"
!

koch:n side:s

    n == 0 ifTrue:[
        self go:s
    ] ifFalse: [
        self koch:(n - 1) side:s.
        self turn:-45.
        self koch:(n - 1) side:s.
        self turn:90.
        self koch:(n - 1) side:s.
        self turn:-45.
        self koch:(n - 1) side:s
    ]
    "(Pen new) koch:5 side:4"
!

mandala:n radius:r
    |px py points deltaAngle angle|

    deltaAngle := 360.0 / n.
    angle := 0.0.
    points := Array new:n.
    1 to:n do:[:index |
        px := (angle degreesToRadians cos * r) asInteger + locationX.
        py := (angle degreesToRadians sin * r) asInteger + locationY.
        points at:index put:(px @ py).
        angle := angle + deltaAngle
    ].
    1 to:n do:[:thisIndex |
        thisIndex to:n do:[:otherIndex |
            self place:(points at:thisIndex).
            self goto:(points at:otherIndex)
        ]
    ]
    "(Pen new) mandala:20 radius:300"
!

cCurve:level side:sideLength
    "draw a c-curve with the specified level and side length"

    level == 0 ifTrue:[^ self go:sideLength].
    self cCurve:level - 1 side:sideLength. self turn:90.
    self cCurve:level - 1 side:sideLength. self turn:-90

    "(Pen new) turn:-90; cCurve:10 side:4"
! !

!Pen class methodsFor:'examples'!

example
    "draws a spiral"
    |bic|
    bic := Pen new.
    "bic mask:(Form grey)."
    bic defaultNib:4.
    "bic combinationRule:Form under."
    1 to:50 do:[:i | bic go:i*4. bic turn:89]
    "Pen example"
! !
