"
 COPYRIGHT (c) 1993 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

IdentitySet subclass:#SignalSet
         instanceVariableNames:''
         classVariableNames:'SetOfAnySignal'
         poolDictionaries:''
         category:'Kernel-Exceptions'
!

SignalSet comment:'

COPYRIGHT (c) 1993 by Claus Gittinger
              All Rights Reserved

$Header: /claus/smalltalk/st2.6/libbasic/RCS/SignalSet.st,v 1.5 94/01/16 03:46:19 claus Exp $
'!

!SignalSet class methodsFor:'documentation'!

documentation
"
    SignalSet allows catching of multiple signals. A SignalSet consists of
    a number of signals and also implements the #handle:do: method just as
    signals do. However, any signal from the SignalSet will, if signalled, lead
    into the handler.
    There is also a special signalSet, which can be used to catch any
    signal in the system.

    For more detail, see comment in Signal and examples in doc/coding.
"
! !

!SignalSet class methodsFor:'instance creation'!

anySignal
    "return a special signalSet for catching any signal"

    SetOfAnySignal isNil ifTrue:[
        SetOfAnySignal := self basicNew
    ].
    ^ SetOfAnySignal
! !

!SignalSet methodsFor:'save evaluation'!

handle:handleBlock do:aBlock
    "evaluate the argument, aBlock.
     If any of the signals in the receiver is raised during evaluation,
     evaluate the handleBlock passing it an Exception argument.
     The handler may decide how to react to the signal by sending
     a corresponding message to the exception (see there).
     If the signal is not raised, return the value of evaluating
     aBlock."

     ^ aBlock value  "the real logic is in raise/Exception"
!

catch:aBlock
     "evaluate the argument, aBlock; return false.
      If any of the signals in the receiver is raised during evaluation, 
      abort the evaluation and return true. 
      With the special anySignal, evaluation can be performed totally save 
      from signals 
      - but who (beside radical c++ fans) would do that ?"

      |raiseOccurred|

      raiseOccurred := false.
      self handle:[:ex | raiseOccurred := true. ex return] do:aBlock.
      ^ raiseOccurred

      "SignalSet anySignal catch:[
          (#(1 2 3 4) at:5) / 0.0
       ]"
! !

!SignalSet methodsFor:'queries'!

includes:aSignal
    "return true, if the receiver contains the argument, aSignal.
     The special any-Set includes every signal."

    (self == SetOfAnySignal) ifTrue:[
        ^ aSignal isKindOf:Signal
    ].
    ^ super includes:aSignal
! !
