"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

View subclass:#StandardSystemView
       instanceVariableNames:'label icon iconView iconLabel
                              minExtent maxExtent'
       classVariableNames:   ''
       poolDictionaries:''
       category:'Views-Basic'
!

StandardSystemView comment:'

COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

I represent topViews i.e. those views which have a title-label,
an icon etc.

$Header: /claus/smalltalk/st2.6/libview/RCS/StdSysV.st,v 1.7 94/01/08 17:16:02 claus Exp $
written spring/summer 89 by claus
'!

!StandardSystemView class methodsFor:'defaults'!

defaultExtent
    "topviews extent is (0.6 @ 0.6) of screen by default"

    ^ (Display width // 3 * 2) @ (Display height // 3 * 2)
! !

!StandardSystemView class methodsFor:'startup'!

start
    "create, realize the view - this topview and all its subviews will
     run as a separate process with its own windowGroup"

    ^ self new open "realize"
! !

!StandardSystemView class methodsFor:'instance creation'!

model:aModel label:aLabel minimumSize:minExtent
    |newView|

    newView := self origin:nil 
                    extent:minExtent
                     label:aLabel 
                      icon:nil
                 minExtent:minExtent
                 maxExtent:nil.
    newView model:aModel.
    newView controller:(self defaultController new view:newView).
    ^ newView
!

label:aLabel
    "create a new topView and define its label"

    ^ self origin:nil extent:nil
           label:aLabel icon:nil
           minExtent:nil maxExtent:nil
!

label:aLabel minExtent:anExtent
    ^ self origin:nil extent:nil
           label:aLabel icon:nil
           minExtent:anExtent maxExtent:nil
!

label:aLabel icon:aForm
    ^ self origin:nil extent:nil
           label:aLabel icon:aForm
           minExtent:nil maxExtent:nil
!

label:aLabel icon:aForm minExtent:anExtent
    ^ self origin:nil extent:nil
           label:aLabel icon:aForm
           minExtent:anExtent maxExtent:nil
!

label:aLabel icon:aForm minExtent:minExtent maxExtent:maxExtent
    ^ self origin:nil extent:nil
           label:aLabel icon:aForm
           minExtent:minExtent maxExtent:maxExtent
!

origin:anOrigin extent:anExtent label:aLabel
    ^ self origin:anOrigin extent:anExtent
           label:aLabel icon:nil
           minExtent:nil maxExtent:nil
!

origin:anOrigin label:aLabel icon:aForm
                minExtent:minExtent maxExtent:maxExtent
    ^ self origin:anOrigin extent:nil
           label:aLabel icon:aForm
           minExtent:minExtent maxExtent:maxExtent
!

extent:anExtent label:aLabel
                minExtent:minExtent
    ^ self origin:nil extent:anExtent
           label:aLabel icon:nil
           minExtent:minExtent maxExtent:nil
!

extent:anExtent label:aLabel icon:aForm
    ^ self origin:nil extent:anExtent
           label:aLabel icon:aForm
           minExtent:nil maxExtent:nil
!

extent:anExtent label:aLabel icon:aForm
                minExtent:minExtent
    ^ self origin:nil extent:anExtent
           label:aLabel icon:aForm
           minExtent:minExtent maxExtent:nil
!

extent:anExtent label:aLabel icon:aForm
                minExtent:minExtent maxExtent:maxExtent
    ^ self origin:nil extent:anExtent
           label:aLabel icon:aForm
           minExtent:minExtent maxExtent:maxExtent
! !

!StandardSystemView methodsFor:'initialization'!

initialize
    |c|

    super initialize.

    self addToCurrentProject.

    c := device center.
    left := c x - (width // 2).
    top := c y - (height // 2).
    borderWidth := 2. "- notice: many window managers ignore this"
    label := 'aView'.
    minExtent := 10 @ 10.
    maxExtent := (device width) @ (device height)
!

defaultController
    "for ST-80 compatibility only - not used in ST/X"

    ^ StandardSystemController
!

addToCurrentProject
    "add the receiver (a topview) to the current projects set-of-views.
     (If there is a current project)"

    |p|

    (Project notNil and:[(p := Project current) notNil]) ifTrue:[
        p addView: self
    ]
!

reinitialize
    "reopen the receiver if if was visible before.
     This is called right after snapIn; Notice, that all instance variables
     (such as shown, realized etc.) are left-over from the time the snapout
     was done. Remap the receiver, if it was mapped at snapout time"

    |myController|

    "if I have already been reinited - return"
    drawableId notNil ifTrue:[
        ^ self
    ].

    "have to kludge with the controller 
     - otherwise its startup performs unwanted actions ..."

    myController := controller.
    controller := nil.

    "physically create the view & subviews"
    self recreate.

    "if I was mapped, do it again"
    realized ifTrue:[
        "if it was iconified, try to remap iconified"
        shown ifFalse:[
            device mapView:self id:drawableId iconified:true
                       atX:left y:top width:width height:height
        ] ifTrue:[
            device mapView:self id:drawableId iconified:false
                       atX:left y:top width:width height:height
        ].

        "and restart the window-group process"
        windowGroup notNil ifTrue:[
            windowGroup restart
        ]
    ].

    "restore controller"
    controller := myController
!

recreate
    "recreate the view after a snap-in"

    icon := self convertedIcon.
    super recreate.

    iconView notNil ifTrue:[
        iconView create.
        device setWindowIconWindow:iconView in:drawableId
    ] ifFalse:[
        (icon notNil and:[icon id notNil]) ifTrue:[
            device setWindowIcon:icon in:drawableId
        ].
    ].

    iconLabel notNil ifTrue:[
        device setIconName:iconLabel in:drawableId
    ]
!

reAdjustGeometry
    "when we come up on a smaller display, 
     make certain, that the receiver is visible"

    |dX dY|

    dX := (device horizontalPixelPerMillimeter * 20) rounded.
    dY := (device verticalPixelPerMillimeter * 20) rounded.

    ((self left > (device width - dX)) or:[
      self top > (device height - dY)]) ifTrue:[
        'moving view into visible area' errorPrintNewline.
        self origin:(device width - dX) @ (device height - dY)
    ]
! !

!StandardSystemView methodsFor:'destroying'!

destroy
    super destroy.
    Project current notNil ifTrue:[
        Project current removeView:self
    ].
! !

!StandardSystemView methodsFor:'private'!

convertedIcon
    "make certain, that the icon is a b&w bitmap;
     do so by converting if appropriate.
     Will add a device supportsDeepIcons and only convert when needed;
     for now (since there are only Xdisplays) we always have to convert."

    |deviceIcon|

    icon isNil ifTrue:[^ nil].

    icon depth ~~ 1 ifTrue:[
        deviceIcon := icon asMonochromeFormOn:device.
    ] ifFalse:[
        deviceIcon := icon
    ].
    deviceIcon notNil ifTrue:[
        deviceIcon := deviceIcon on:device
    ].
    ^ deviceIcon
! !

!StandardSystemView methodsFor:'realization'!

create
    "create - make certain that icon is available"

    icon := self convertedIcon.
    super create.

    iconView notNil ifTrue:[
        iconView create.
        device setWindowIconWindow:iconView in:drawableId
    ].
    iconLabel notNil ifTrue:[
        device setIconName:iconLabel in:drawableId
    ]
!

openWithExtent:anExtent
    "set extent and open - for ST-80 compatibility"

    self extent:anExtent.
    self open
! !

!StandardSystemView methodsFor:'accessing'!

heightIncludingBorder
    "return the views overall-height"

    ^ height
!

widthIncludingBorder
    "return the views overall-width"

    ^ width
!

label
    "return the views name in the title area"

    ^ label
!

label:aString
    "define the views name in the windows title area"

    label := aString.
    drawableId notNil ifTrue: [
        device setWindowName:aString in:drawableId
    ]
!

iconLabel
    "return the name displayed in the icon"

    ^ iconLabel
!

iconLabel:aString
    "define the name to be displayed in the icon"

    iconLabel := aString.
    drawableId notNil ifTrue:[
        device setIconName:aString in:drawableId
    ]
!

name
    "return the topViews label"

    ^ label
!

icon
    "return the form defined as icon"

    ^ icon
!

icon:aForm
    "define the form (bitmap) used as icon"

    |invertedIcon i|

    icon := aForm.
    icon notNil ifTrue:[
        drawableId notNil ifTrue:[
            icon depth ~~ 1 ifTrue:[
                icon := icon asMonochromeFormOn:device.
            ].
            "icons assume 1s as black - invert icon if the device thinks different"
            (device depth == 1 and:[device whitepixel ~~ 0]) ifTrue:[
                i := icon on:device.
                i notNil ifTrue:[
                    invertedIcon := Form width:icon width height:icon height on:device.
                    invertedIcon function:#copy.
                    invertedIcon foreground:Color noColor background:Color allColor.
                    invertedIcon copyFrom:i x:0 y:0 toX:0 y:0 width:icon width height:icon height.
                    i := invertedIcon.
                ]
            ] ifFalse:[
                i := icon on:device.
            ].
            (i notNil and:[i id notNil]) ifTrue:[
                device setWindowIcon:i in:drawableId
            ]
        ]
    ]
!

iconView
    "return the view used as icon-view"

    ^ iconView
!

iconView:aView
    "specify the view to be used as icon"

    iconView := aView.
    drawableId notNil ifTrue:[
        aView create.
        device setWindowIconWindow:aView in:drawableId
    ]
!

iconName:aString
    "this method will vanish soon ... - for backward compatibility"

    self iconLabel:aString
!

maximumSize:anExtent
    "same as maxExtent: for ST-80 compatibility"

    ^ self maxExtent:anExtent
!

minimumSize:anExtent
    "same as minExtent: for ST-80 compatibility"

    ^ self minExtent:anExtent
!

minExtent
    "return the views minimum allowed extent"

    ^ minExtent
!

minExtent:min
    "define the minimum extent the view may have -
     depends on good-will of window manager"

    minExtent := min.
    (width notNil and:[height notNil]) ifTrue:[
        ((width < (minExtent x)) or:
         [height < (minExtent y)]) ifTrue: [
            self extent:minExtent
        ]
    ]
!

maxExtent
    "return the views maximum allowed extent"

    ^ maxExtent
!

maxExtent:max
    "define the maximum extent the view may have -
     depends on good-will of window manager"

    maxExtent := max.
    (width notNil and:[height notNil]) ifTrue:[
        ((width > (maxExtent x)) or:
         [height > (maxExtent y)]) ifTrue: [
            self extent:maxExtent
        ]
    ]
! !
