"
 COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Object subclass:#Stream
       instanceVariableNames:''
       classVariableNames:''
       poolDictionaries:''
       category:'Streams'
!

Stream comment:'

COPYRIGHT (c) 1989 by Claus Gittinger
              All Rights Reserved

$Header: /claus/smalltalk/st2.6/libbasic/RCS/Stream.st,v 1.5 94/02/05 12:24:58 claus Exp $
'!

!Stream class methodsFor:'instance creation'!

new
    "report an error - Streams are created using on:-messages"

    ^ self error:'Streams cannot be created with new'
! !

!Stream methodsFor:'accessing'!

contents
    "return the entire contents of the stream
     - we do not know here how to do it, it must be redefined in subclass"

    ^ self subclassResponsibility
! !

!Stream methodsFor:'reading'!

next
    "return the next element of the stream
     - we do not know here how to do it, it must be redefined in subclass"

    ^ self subclassResponsibility
!

next:count
    "return the next count elements of the stream as a Collection."

    |coll|

    coll := self contentsSpecies new:count.
    1 to:count do:[:index |
        coll at:index put:(self next)
    ].
    ^ coll

    "(ReadStream on:#(1 2 3 4 5)) next:3"
    "(ReadStream on:'hello') next:3"
!

nextMatchFor:anObject
    "read an element from the receiver, return true if it was equal to
     the argument, anObject"

    ^ (self next = anObject)

    "
     |s|
     s := ReadStream on:#(1 2 3 4 5 6 7 8).
     s nextMatchFor:2
    "
    "
     |s|
     s := ReadStream on:#(1 2 3 4 5 6 7 8).
     s nextMatchFor:2.
     s nextMatchFor:2
    "
!

skip:count 
    "skip count objects, return the receiver"

    "dont know how to unread ..."
    count < 0 ifTrue:[
        ^ self error:'stream is not positionable'
    ].
    count timesRepeat:self next

    "
     |s|
     s := ReadStream on:#(1 2 3 4 5 6 7 8).
     s skip:4.
     s next
    "
!

skipFor:anObject
    "skip all objects up-to and including anObject; return the element after anObject."

    (self skipThrough:anObject) notNil ifTrue:[
        ^ self next
    ].
    ^ nil

    "
     |s|
     s := ReadStream on:#(1 2 3 4 5 6 7 8).
     s skipFor:4.
    "
!

skipThrough:anObject
    "skip all objects up-to and including anObject. 
     The next read operation will return the element after anObject."

    |nextElement|

    [self atEnd] whileFalse:[
        nextElement := self next.
        (nextElement = anObject) ifTrue: [ ^ self ]
    ].
    ^ nil

    "
     |s|
     s := ReadStream on:#(1 2 3 4 5 6 7 8).
     s skipThrough:4.
     s next
    "
! !

!Stream methodsFor:'writing'!

nextPut:anObject
    "put the argument, anObject onto the receiver
     - we do not know here how to do it, it must be redefined in subclass"

    ^ self subclassResponsibility
!

next:count put:anObject
    "put the argument, anObject count times onto the receiver"

    count timesRepeat:[self nextPut:anObject].
    ^ anObject

    "
     |s|

     s := WriteStream on:#().
     s nextPut:1.
     s next:5 put:2.
     s nextPut:3.
     s contents
    "
!

nextPutAll:aCollection
    "put all elements of the argument, aCollection onto the receiver"

    aCollection do:[:element |
        self nextPut:element
    ].
    ^ aCollection

    "
     |s|

     s := WriteStream on:#().
     s nextPutAll:(1 to:5).
     s nextPutAll:#('one' 'two' 'three').
     s contents
    "
    "
     |s|

     s := WriteStream on:(String new).
     s nextPutAll:($a to:$f).
     s nextPutAll:'one '; 
       nextPutAll:'two ';
       nextPutAll:'three'.
     s contents
    "
!

nextPutAll:aCollection startingAt:first to:last
    "put the elements with index from first to last
     of the argument, aCollection onto the receiver"

    aCollection from:first to:last do:[:element |
        self nextPut:element
    ].
    ^ aCollection

    "
     |s|

     s := WriteStream on:#().
     s nextPutAll:#('one' 'two' 'three' 'four' 'five') startingAt:2 to:4.
     s contents
    "
!

endEntry
    "ignored here - for compatibility with Transcript"

    ^ self
!

show:aString
    "put all elements of the argument, aString onto the receiver;
     this makes streams somewhat compatible to TextCollectors and
     allows you to say: Smalltalk at:#Transcript put:Stdout"

    ^ self nextPutAll:aString printString
!

showCr:aString
    "put all elements of the argument, aString onto the receiver;
     and append a newline.
     For compatibility with TextCollectors"

    self show:aString.
    self cr
!

cr
    "append a carriage-return to the stream"

    self nextPut:(Character cr)
!

tab
    "append a tab-character to the stream"

    self nextPut:(Character tab)
!

crTab
    "append a carriage-return followed by a tab to the stream"

    self nextPut:(Character cr).
    self nextPut:(Character tab)
!

space
    "append a space character to the receiver-stream"

    self nextPut:(Character space)
!

spaces:count
    "append count space-characters to the receiver-stream"

    self next:count put:(Character space)
!

ff
    "append a form-feed (new-pagee) to the receiver-stream"

    self nextPut:(Character ff)
! !

!Stream methodsFor:'closing'!

close
    "close the stream - nothing done here.
     Added for compatibility with external streams."

    ^ self
! !

!Stream methodsFor:'testing'!

atEnd
    "return true if the end of the stream has been reached;
     - we do not know here how to do it, it must be redefined in subclass"

    ^ self subclassResponsibility
! !

!Stream methodsFor:'enumerating'!

do:aBlock
    "evaluate the argument, aBlock for all remaining elements,
     up to the end of the stream"

    [self atEnd] whileFalse:[
        aBlock value:(self next)
    ]

    "
     |s|
     s := ReadStream on:#(1 2 3 4 5 6 7 8 9).
     s next.
     s next.
     s do:[:element | Transcript showCr:element]
    "
! !

!Stream methodsFor:'private'!

contentsSpecies
    "this should return the class of which an instance is
     returned by the #contents method. It is redefinable in subclasses"

    "return Array here - since the abstract Stream has no idea
     of the underlying collection class"

    ^ Array
! !

!Stream methodsFor:'queries'!

isStream
    "return true, if the receiver is some kind of Stream."

    ^ true
!

lineLength
    "this is just a layout hint for prettyprinting functions
     - for compatibility with TextCollectors"

    ^ 80
! !

!Stream methodsFor: 'nonhomogeneous accessing'!

nextNumber:n 
        "Return the next n bytes as a positive Integer; bytes are taken msb-first."

        | s i 
          val "{ Class: SmallInteger }" |

	"claus: this method is central in binaryStorage -
	 therefore it has been tuned a bit"

	n == 1 ifTrue:[
	    ^ self next
	].
	n == 2 ifTrue:[
	    val := self next.
	    val := (val bitShift:8) + self next.
	    ^ val
	].
	n == 3 ifTrue:[
	    val := self next.
	    val := (val bitShift:8) + self next.
	    val := (val bitShift:8) + self next.
	    ^ val
	].
	n == 4 ifTrue:[
	    val := self next.
	    val <= 16r3F ifTrue:[
	        val := (val bitShift:8) + self next.
	        val := (val bitShift:8) + self next.
	        val := (val bitShift:8) + self next.
	        ^ val
	    ].
	    "sorry, but need a largeInteger"
	    s := val.
	    s := (s bitShift:8) + self next.
	    s := (s bitShift:8) + self next.
	    s := (s * 256) + self next.
	    ^ s
	].

        s := 0.
        1 to: n do: [:j | 
	    s := s * 256 + self next
	].
        ^ s truncated
!

nextNumber:n put:v 
        "Append to the receiver the argument, v, which is a positive Integer,
         as the next n bytes. Bytes are written msb first. 
         Possibly pad with leading zeros."

        | vlen i |

	"claus: this method is central in binaryStorage -
	 therefore it has been tuned a bit"

	n == 1 ifTrue:[
	    v class == SmallInteger ifTrue:[
	        (v between:0 and:16rFF) ifTrue:[
		    self nextPut:v.
		    ^ self
	        ].
	    ].
	].
        n == 2 ifTrue:[
	    v class == SmallInteger ifTrue:[
                (v between:0 and:16rFFFF) ifTrue:[
                    self nextPut:(v bitShift:-8).
                    self nextPut:(v bitAnd:16rFF).
                    ^ self
                ].
            ].
        ].
        n == 3 ifTrue:[
	    v class == SmallInteger ifTrue:[
                (v between:0 and:16rFFFFFF) ifTrue:[
                    self nextPut:((v bitShift:-16) bitAnd:16rFF).
                    self nextPut:((v bitShift:-8) bitAnd:16rFF).
                    self nextPut:(v bitAnd:16rFF).
                    ^ self
                ].
            ].
        ].
        n == 4 ifTrue:[
	    v class == SmallInteger ifTrue:[
                (v >= 0) ifTrue:[
                    self nextPut:((v bitShift:-24) bitAnd:16rFF).
                    self nextPut:((v bitShift:-16) bitAnd:16rFF).
                    self nextPut:((v bitShift:-8) bitAnd:16rFF).
                    self nextPut:(v bitAnd:16rFF).
                    ^ self
                ].
            ].
        ].

        n < (vlen := v digitLength) ifTrue: [
	    self error: 'number too big'
	].

        "pad with leading zeros"
        i := n.
        [i > vlen] whileTrue: [
            self nextPut: 0. 
            i := i - 1
        ].
        i = 1 ifTrue: [
	    ^ self nextPut: v
        ].
        [i > 0] whileTrue: [
	    self nextPut: (v digitAt: i). 
	    i := i - 1
	]
! !
