'From Smalltalk/X, Version:2.3.1 on 16-Aug-92 at 20:55:12'!

Object subclass:#TetrisBlock
         instanceVariableNames:'slots origin sideLength color'
         classVariableNames:'RandomStream'
         poolDictionaries:''
         category:'Games-Tetris'
!

TetrisBlock comment:'
 I represent the blocks falling down ...

 types are:  0 -> xxxx

             1 -> xx
                  xx

             2 -> xxx
                    x

             3 -> xxx
                  x  

             4 -> xx 
                   xx

             5 ->  xx
                  xx 

             6 -> xxx
                   x 

 @(#)TBlock.st  1.1 92/02/03
 written spring 91 by claus
'!

!TetrisBlock class methodsFor:'instance creation'!

newWithSize:l
    "generate a new random TetrisBlock"
    |rnd newBlock|

    RandomStream isNil ifTrue:[
        RandomStream := Random new
    ].
    rnd := -1.
    [(rnd < 0) or:[rnd > 6]] whileTrue:[
        rnd := self nextRandomNumber
    ].
    newBlock := self basicNew initialize.
    newBlock setType:rnd.
    newBlock sideLength:l.
    ^ newBlock
! !

!TetrisBlock class methodsFor:'private'!

nextRandomNumber
    |rnd|

    rnd := RandomStream next.   "0 < rnd < 1"
    rnd := rnd * 7.             "0 < rnd < 7"
    rnd := rnd asInteger.       "0 .. 6"
    ^ rnd
! !

!TetrisBlock methodsFor:'accessing'!

slots
    ^ slots
!

sideLength:aNumber
    sideLength := aNumber
!

color:aColor
    color := aColor
!

origin
    ^ origin
!

origin:aPoint
    origin := aPoint
! !

!TetrisBlock methodsFor:'visible moving'!

moveDownIn:aView
    self clearIn:aView.
    self moveDown.
    self drawIn:aView
!

moveLeftIn:aView
    self clearIn:aView.
    self moveLeft.
    self drawIn:aView
!

moveRightIn:aView
    self clearIn:aView.
    self moveRight.
    self drawIn:aView
!

rotateLeftIn:aView
    self clearIn:aView.
    self rotateLeft.
    self drawIn:aView
!

rotateRightIn:aView
    self clearIn:aView.
    self rotateRight.
    self drawIn:aView
! !

!TetrisBlock methodsFor:'drawing'!

clearIn:aView
    |orgX orgY|

    aView paint:(aView viewBackground).
    orgX := origin x.
    orgY := origin y.
    slots do:[:aPoint |
        aView fillRectangleX:(orgX + aPoint x) * sideLength
                           y:(orgY + aPoint y) * sideLength
                       width:sideLength
                      height:sideLength
    ]
!

drawIn:aView
    |side orgX orgY x y|

    side := sideLength - 2.
    orgX := origin x.
    orgY := origin y.
    aView paint:color.
    slots do:[:aPoint |
        x := (orgX + aPoint x) * sideLength.
        y := (orgY + aPoint y) * sideLength.
        aView fillRectangleX:x + 1
                           y:y + 1
                       width:side
                      height:side.
    ].
    View3D ifTrue:[
        slots do:[:aPoint |
            x := (orgX + aPoint x) * sideLength.
            y := (orgY + aPoint y) * sideLength.
            aView drawEdgesForX:(x + 1) y:(y + 1)
                          width:(side) height:(side) level:2
        ]
    ]
! !

!TetrisBlock methodsFor:'private'!

initialize
    slots := Array new:4.
    origin := 0@0
!

setType:type
    (type == 0) ifTrue:[
        "0/0  1/0  2/0  3/0"
        slots at:1 put:(0 @ 0).
        slots at:2 put:(1 @ 0).
        slots at:3 put:(2 @ 0).
        slots at:4 put:(3 @ 0).
        ^ self 
    ].
    (type == 1) ifTrue:[
        "0/0  1/0  
         0/1  1/1"
        slots at:1 put:(0 @ 0).
        slots at:2 put:(1 @ 0).
        slots at:3 put:(0 @ 1).
        slots at:4 put:(1 @ 1).
        ^ self 
    ].
    (type == 2) ifTrue:[
        "0/0  1/0  2/0
                   2/1"
        slots at:1 put:(0 @ 0).
        slots at:2 put:(1 @ 0).
        slots at:3 put:(2 @ 0).
        slots at:4 put:(2 @ 1).
        ^ self 
    ].
    (type == 3) ifTrue:[
        "0/0  1/0  2/0
         0/1"
        slots at:1 put:(0 @ 0).
        slots at:2 put:(1 @ 0).
        slots at:3 put:(2 @ 0).
        slots at:4 put:(0 @ 1).
        ^ self 
    ].
    (type == 4) ifTrue:[
        "0/0  1/0  
              1/1  2/1"
        slots at:1 put:(0 @ 0).
        slots at:2 put:(1 @ 0).
        slots at:3 put:(1 @ 1).
        slots at:4 put:(2 @ 1).
        ^ self 
    ].
    (type == 5) ifTrue:[
        "     1/0  2/0
         0/1  1/1     "
        slots at:1 put:(1 @ 0).
        slots at:2 put:(2 @ 0).
        slots at:3 put:(0 @ 1).
        slots at:4 put:(1 @ 1).
        ^ self 
    ].
    (type == 6) ifTrue:[
        "0/0  1/0  2/0
              1/1     "
        slots at:1 put:(0 @ 0).
        slots at:2 put:(1 @ 0).
        slots at:3 put:(2 @ 0).
        slots at:4 put:(1 @ 1).
        ^ self 
    ].
    self error:'unsupported type'
! !

!TetrisBlock methodsFor:'queries'!

hits:aPoint
    slots do:[:p |
        ((p + origin) = aPoint) ifTrue:[^ true]
    ].
    ^ false
!

maxX
    |max x|

    max := -9999.
    slots do:[:aPoint |
        x := aPoint x.
        (x > max) ifTrue:[
            max := x
        ]
    ].
    ^ max
!

maxY
    |max y|

    max := -9999.
    slots do:[:aPoint |
        y := aPoint y.
        y > max ifTrue:[
            max := y
        ]
    ].
    ^ max
!

minX
    |min x|

    min := 9999.
    slots do:[:aPoint |
        x := aPoint x.
        x < min ifTrue:[
            min := x
        ]
    ].
    ^ min
!

minY
    |min y|

    min := 9999.
    slots do:[:aPoint |
        y := aPoint y.
        y < min ifTrue:[
            min := y
        ]
    ].
    ^ min
! !

!TetrisBlock methodsFor:'manipulation'!

moveDown
    origin y:(origin y + 1)
!

moveUp
    origin y:(origin y - 1)
!

moveLeft
    origin x:(origin x - 1)
!

moveRight
    origin x:(origin x + 1)
!

rotateLeft
    |oldPoint newPoint|

    1 to:(slots size) do:[:index |
        oldPoint := slots at:index.
        newPoint := (oldPoint y) @ (oldPoint x negated).
        slots at:index put:newPoint
    ]
!

rotateRight
    |oldPoint newPoint|

    1 to:(slots size) do:[:index |
        oldPoint := slots at:index.
        newPoint := (oldPoint y negated) @ (oldPoint x).
        slots at:index put:newPoint
    ]
! !
