ObjectView subclass:#TreeView
       instanceVariableNames:'topNode xIndent fgColor bgColor maxY maxX assoc'
       classVariableNames:''
       poolDictionaries:''
       category:'Demos-Graphics'
!

TreeView comment:'
 an abstract view for trees showing ObjectView usage. Also
 another example for object oriented programming and inheritance:
 subclasses share much of the code.

 See ClassTreeView and WindowTreeView for a specific example.

 start with:
     XXXView start

 %W% %E%
 written spring 91 by claus
'!

!TreeView class methodsFor:'startup'!

start
    |aDemo topView v|

    topView := StandardSystemView 
                    label:'Tree hierarchy'
                    icon:(Form fromFile:'Demo.icon' resolution:100).
    v  := HVScrollableView for:self in:topView.
    v origin:(0 @ 0) extent:(1.0 @ 1.0).
    topView open

    "ClassTreeView start"
    "WindowTreeView start"
! !

!TreeView methodsFor:'initialization'!

initialize
    super initialize.

    xIndent := (self horizontalPixelPerMillimeter:10) rounded asInteger.
    pressAction := [:aPoint | self click:aPoint].
    fgColor := Black.
    bgColor := White.
    self updateTree.
!

initializeMiddleButtonMenu
    "to be redefined in subclasses"
! !

!TreeView methodsFor:'generating the class-tree picture'!

updateTree
    assoc := IdentityDictionary new.
    self removeAll.
    maxX := 0.
    maxY := 0.
    self addChildrenToTreeFrom:topNode 
                          xPos:(self horizontalPixelPerMillimeter:5) rounded asInteger
                          yPos:(self verticalPixelPerMillimeter:5) rounded asInteger.
!

addChildrenToTreeFrom:anElement xPos:xPos yPos:yPos
    |subclassNames newObject yNext|

    newObject := self addElementToTree:anElement xPos:xPos yPos:yPos.
    yNext := yPos + newObject frame height.

    self childrenOf:anElement do:[:aChild |
        yNext := self addChildrenToTreeFrom:aChild
                                       xPos:(xPos + xIndent)
                                       yPos:yNext
    ].
    ^ yNext
!

addElementToTree:anElement xPos:xPos yPos:yPos
    |newObject|

    newObject := DrawText new.
    newObject text:(self textFor:anElement).
    newObject origin:(xPos @ yPos).
    newObject foreground:(self colorFor:anElement) background:bgColor.
    newObject fillPattern:1.

    maxY := maxY max:(yPos + newObject height).
    maxX := maxX max:(xPos + newObject width).
    self add:newObject.
    assoc at:newObject put:anElement.
    ^ newObject
!

childrenOf:anElement do:aBlock
    "should be redefined for your type of elements"

    ^ nil
!

textFor:anElement
    "should return a string representing the element"

    ^ anElement printString
!

colorFor:anElement
    ^ Black
! !

!TreeView methodsFor:'user interaction'!

click:aPoint
    |anObject|

    anObject := self findObjectAtVisible:aPoint.
    (anObject ~~ selection) ifTrue:[
        self unselect.
        anObject notNil ifTrue:[
            self select:anObject
        ]
    ]
! !

!TreeView methodsFor:'support'!

withSelectedElementDo:aBlock
    |class|

    selection isNil ifFalse:[
        aBlock value:(assoc at:selection)
    ]
! !

!TreeView methodsFor:'drawing'!

showSelected:anObject
    "show an object as selected"

    |fg bg|

    fg := anObject foreground.
    bg := anObject background.

    anObject foreground:bg background:fg.
    anObject drawIn:self.
    anObject foreground:fg background:bg 
! !

!TreeView methodsFor:'queries'!

widthOfContents
    "sent by scrollbar to query contents size"

    ^ maxX
!

heightOfContents
    "sent by scrollbar to query contents size"

    ^ maxY
! !
