'From Smalltalk/X, Version:2.0 on 5-Mar-92 at 14:16:20'!

StandardSystemView subclass:#VocPanel
         instanceVariableNames:'levelView pause playing recording recordBlock playBlock
                dataBuffer recordStream playStream inStream outStream red 
                prevMax prevX playButton recordButton blockSize timeDelta
                nBytes fileName'
         classVariableNames:''
         poolDictionaries:''
         category:'Applications-Sound'
!

!VocPanel class methodsFor:'defaults'!

defaultExtent
    ^ (Display horizontalPixelPerMillimeter * 60) asInteger
      @
      (Display verticalPixelPerMillimeter * 25) asInteger
! !

!VocPanel class methodsFor:'startup'!

new
    |newPanel|

    newPanel := super label:'Sound Panel'.
    newPanel icon:(Form fromFile:'play.xbm' resolution:100).
    ^ newPanel

    "VocPanel start"
!

startOn:aFileName
    |newPanel|

    newPanel := super label:aFileName.
    newPanel icon:(Form fromFile:'play.xbm' resolution:100).
    newPanel fileName:aFileName.
    newPanel realize.
    ^ newPanel

    "VocPanel startOn:'../fileIn/sounds/rooster.snd'"
    "VocPanel startOn:'../fileIn/sounds/sample.snd'"
! !

!VocPanel methodsFor:'accessing'!

fileName:aFileName
    fileName := aFileName
! !

!VocPanel methodsFor:'user interaction'!

pauseOn
    pause := true
!

stop
    playing ifTrue:[
        self stopPlay
    ].
    recording ifTrue:[
        self stopRecord
    ]
!

pauseOff
    pause := false.
    playing ifTrue:[
        device addTimedBlock:playBlock after:timeDelta 
    ]
!

play
    self stop.

    inStream := FileStream readonlyFileNamed:fileName.
    inStream isNil ifTrue:[
        self notify:'cannot read sound file'.
        ^ self
    ].
    playStream := SoundStream writing.
    playStream isNil ifTrue:[
        self notify:'cannot open sound-stream' 
    ] ifFalse:[
        playing := true.
        dataBuffer := ByteArray new:blockSize.
        nBytes := -1.
        prevMax := 128.
        prevX := 0.
        device addTimedBlock:playBlock after:timeDelta
    ]
!

record
    self stop.
    fileName := 'recording'.
    outStream := FileStream newFileNamed:fileName.
    outStream isNil ifTrue:[
        self notify:'cannot create output file'.
        recordButton turnOff.
        ^ self
    ].
    recordStream := SoundStream reading.
    recordStream isNil ifTrue:[
        self notify:'cannot open sound-stream'.
        recordButton turnOff
    ] ifFalse:[
        recording := true.
        dataBuffer := ByteArray new:blockSize.
        prevMax := 128.
        prevX := 0.
        device addTimedBlock:recordBlock after:timeDelta
    ]
! !

!VocPanel methodsFor:'initializing'!

initialize
    |panel1 panel2 pauseToggle stopButton passiveFG passiveBG ppmm|

    super initialize.

    fileName := 'recording'.

    recording := false.
    playing := false.
    pause := true.
    blockSize := SoundStream blockSize.
    timeDelta := 1.0 / (SoundStream defaultSampleRate) * blockSize * 0.9.
"timeDelta := 0.0001.   "
    recordBlock := [self recordData].
    playBlock := [self playData].

    panel1 := HorizontalPanelView origin:(0.0 @ 0.0)
                                  extent:(1.0 @ 0.5)
                                      in:self.
    panel2 := HorizontalPanelView origin:(0.0 @ 0.5)
                                  extent:(1.0 @ 0.5)
                                      in:self.
    device hasColors ifTrue:[
        red := Color name:'red'
    ].
    red isNil ifTrue:[
        red := White
    ].

    recordButton := Toggle label:'record' in:panel2.
    device hasColors ifTrue:[
        recordButton foregroundColor:red.
        recordButton enteredForegroundColor:red.
        recordButton activeBackgroundColor:red
    ].
    recordButton pressAction:[
        "
        device hasColors ifTrue:[
            recordButton foreground:White background:red
        ]. 
        "
        playButton turnOff. 
        self record
    ].
    recordButton releaseAction:[
        "
        recordButton foregroundColor:passiveFG 
                     backgroundColor:passiveBG. 
        "
        self stopRecord
    ].

    pauseToggle := Toggle label:'pause' in:panel2.
    pauseToggle pressAction:[self pauseOn]. 
    pauseToggle releaseAction:[self pauseOff].
    pauseToggle turnOn.

    stopButton := Button label:'stop' action:[playButton turnOff.
                                              recordButton turnOff.
                                              self stop] in:panel2.
    playButton := Toggle label:'play' in:panel2.
    playButton pressAction:[recordButton turnOff. self play].
    playButton releaseAction:[self stopPlay].

    ppmm := device horizontalPixelPerMillimeter.
    levelView := View extent:((width // 4 * 3) @ (ppmm * 9) asInteger)
                          in:panel1.
    levelView viewBackground:Black

    "VocPanel start"
! !

!VocPanel methodsFor:'record & play'!

recordData
    |x max count absLevel innerWidth innerHeight|

    recording ifFalse:[
        self stopRecord.
        ^ self
    ].

    device addTimedBlock:recordBlock after:timeDelta. 
    count := recordStream nextBytes:blockSize into:dataBuffer.

    pause ifFalse:[
        outStream nextPutBytes:count from:dataBuffer
    ].

    max := dataBuffer maximumValue.
    (max < 0) ifTrue:[^ self].

    absLevel := levelView level abs.
    innerWidth := levelView width - absLevel - absLevel.
    innerHeight := levelView height - absLevel - absLevel.

    (max > prevMax) ifTrue:[
        (max == 255) ifTrue:[
            levelView foreground:red.
            levelView fillRectangleX:absLevel y:absLevel
                               width:innerWidth
                              height:innerHeight.
            prevX := innerWidth
        ] ifFalse:[
            x := ((max - 127) * innerWidth) bitShift:-7.
            levelView foreground:White.
            levelView fillRectangleX:(absLevel + prevX) y:absLevel
                               width:(x - prevX)
                              height:innerHeight.
            prevX := x
        ]
    ] ifFalse:[
        (max < prevMax) ifTrue:[
            x := ((max - 127) * innerWidth) bitShift:-7.
            levelView foreground:Black.
            levelView fillRectangleX:(absLevel + x) y:absLevel
                               width:(prevX - x) 
                              height:innerHeight.

            (prevMax == 255) ifTrue:[
                levelView foreground:White.
                levelView fillRectangleX:absLevel y:absLevel
                                   width:x 
                                  height:innerHeight
            ].
            prevX := x
        ]
    ].
    device synchronizeOutput.
    prevMax := max
!

playData
    |x max count absLevel innerWidth innerHeight|

    pause ifTrue:[^ self].

    device addTimedBlock:playBlock after:timeDelta. 
    (nBytes == -1) ifTrue:[
        nBytes := inStream nextBytes:blockSize into:dataBuffer
    ].

    (nBytes == 0) ifTrue:[
        self stopPlay.
        ^ self
    ].

    playStream nextPutBytes:nBytes from:dataBuffer.

    max := dataBuffer maximumValue.
    (max < 0) ifTrue:[^ self].

    absLevel := levelView level abs.
    innerWidth := levelView width - absLevel - absLevel.
    innerHeight := levelView height - absLevel - absLevel.

    (max > prevMax) ifTrue:[
        (max == 255) ifTrue:[
            levelView foreground:red.
            levelView fillRectangleX:absLevel y:absLevel
                               width:innerWidth
                              height:innerHeight.
            prevX := innerWidth
        ] ifFalse:[
            x := ((max - 127) * innerWidth) bitShift:-7.
            levelView foreground:White.
            levelView fillRectangleX:(absLevel + prevX) y:absLevel
                               width:(x - prevX)
                              height:innerHeight.
            prevX := x
        ]
    ] ifFalse:[
        (max < prevMax) ifTrue:[
            x := ((max - 127) * innerWidth) bitShift:-7.
            levelView foreground:Black.
            levelView fillRectangleX:(absLevel + x) y:absLevel
                               width:(prevX - x) 
                              height:innerHeight.

            (prevMax == 255) ifTrue:[
                levelView foreground:White.
                levelView fillRectangleX:absLevel y:absLevel
                                   width:x 
                                  height:innerHeight
            ].
            prevX := x
        ]
    ].
    prevMax := max.

    nBytes := inStream nextBytes:blockSize into:dataBuffer
!

stopRecord
    recording ifTrue:[
        recording := false.
        recordStream close.
        outStream close.
        device removeTimedBlock:recordBlock.
        self clearLevelDisplay.
        recordButton turnOff
    ]
!

stopPlay
    playing ifTrue:[
        playing := false.
        playStream close.
        inStream close.
        device removeTimedBlock:playBlock.
        self clearLevelDisplay.
        playButton turnOff
    ]
!

clearLevelDisplay
    |absLevel innerWidth innerHeight|

    absLevel := levelView level abs.
    innerWidth := levelView width - absLevel - absLevel.
    innerHeight := levelView height - absLevel - absLevel.

    levelView foreground:Black.
    levelView fillRectangleX:absLevel y:absLevel
                        width:innerWidth
                       height:innerHeight
! !
