'From Smalltalk/X, Version:2.6.3 on 25-Mar-1993 at 16:32:16'!

View subclass:#VocView
         instanceVariableNames:'data dataFile xOrg yOrg displayMode zoom selectionStart
                selectionEnd clickStart clickEnd fileName acceptAction queryBox
                sampleRate'
         classVariableNames:''
         poolDictionaries:''
         category:'Applications-Sound'
!

VocView comment:'
This view can display sampled audio sound; its used by VocBrowser.
(Voc comes from the first implementation, which knew how to handle
soundblaster .voc files. In the meanwhile, NeXTs .snd and SUNs .au
files are also supported. Currently it can only handle 8-bit mono
sounds.)
'!

!VocView methodsFor:'redrawing'!

redraw
    |startX endX|

    startX := 0.
    endX := width - 1.
    dataFile notNil ifTrue:[
        self redrawFileX:startX toX:endX
    ] ifFalse:[
        self redrawDataX:startX toX:endX
    ].
    self invertSelection
!

invertSelection
    |oldFg l r|

    (selectionStart notNil and:[selectionEnd notNil]) ifTrue:[
        l := selectionStart - xOrg + 1.
        (l < 0) ifTrue:[l := 0].
        (l >= width) ifTrue:[^ self].

        r := selectionEnd - xOrg + 1.
        (r < 0) ifTrue:[^ self].
        (r >= width) ifTrue:[r := width - 1].

        (r >= l) ifTrue:[
            oldFg := foreground.
            self foreground:(foreground bitXor:background).
            self xoring:[
                self fillRectangleX:l y:0 width:(r - l) height:height
            ].
            self foreground:oldFg
        ]
    ]
!

redrawX:x y:y width:w height:h
    dataFile notNil ifTrue:[
        self redrawFileX:x toX:(x + w - 1)
    ] ifFalse:[
        self redrawDataX:x toX:(x + w - 1)
    ].
    self invertSelection
!

redrawDataX:startX toX:endX
    |x y0 start end preVal prevX val count ndata prevVal y w index step xStep|

    y0 := self height // 2 - 128.
    x := startX.
    ndata := data size.
    w := endX - startX + 1.
    (zoom < 1) ifTrue:[
        step := (1 / zoom) asInteger.
        xStep := 1
    ] ifFalse:[
        step := 1.
        xStep := zoom asInteger
    ].

    self paint:viewBackground.
    self fillRectangleX:startX y:0 width:(endX - startX + 1) height:height.
    self paint:(Color white).

    (ndata == 0) ifTrue:[
        count := 0
    ] ifFalse:[
        start := (xOrg + startX) min:ndata.
        end := (xOrg + endX) min:ndata.
        count := end - start + 1.
        (displayMode == #vector) ifTrue:[
            (start == 1) ifTrue:[
                prevVal := 128
            ] ifFalse:[
                prevVal := data at:(start - 1)
            ].
            prevX := x - xStep.
            index := start.
            start to:end do:[:i |
                val := data at:index.
                self displayLineFromX:prevX y:(y0 + prevVal)
                                  toX:x     y:(y0 + val).
                prevVal := val.
                prevX := x.
                x := x + xStep.
                index := index + step
            ]
        ] ifFalse:[
            prevVal := data at:start.
            prevX := x.
            index := start.
            start to:end do:[:i |
                val := data at:index.
                (val == prevVal) ifFalse:[
                    y := y0 + prevVal.
                    self displayLineFromX:prevX y:y
                                      toX:x     y:y.
                    prevX := x.
                    prevVal := val
                ].
                x := x + xStep.
                index := index + step
            ].
            self displayLineFromX:prevX y:(y0 + prevVal)
                              toX:(x - zoom) y:(y0 + prevVal)

        ]
    ].
    count < w ifTrue:[
        self displayLineFromX:x y:(y0 + 128)
                          toX:w y:(y0 + 128)
    ].
    self paint:(Color red).
    self displayLineFromX:startX y:(y0 + 128)
                      toX:endX y:(y0 + 128)
!

redrawFileX:startX toX:endX
    |oldPos y0 bytes val prevVal limit count rest x w|

    oldPos := dataFile position.
    limit := dataFile size.
    dataFile position:xOrg.
    bytes := ByteArray new:width.
    rest := limit - xOrg.
    w := endX - startX + 1.
    (rest < w) ifTrue:[
        count := rest.
        count < 0 ifTrue:[
            count := 0
        ]
    ] ifFalse:[
        count := w
    ].
    dataFile nextBytes:count into:bytes.
    dataFile position:oldPos.

    y0 := self height // 2 - 128.
    x := 0.
    prevVal := 128.
    self paint:viewBackground.
    self fillRectangleX:startX y:0 width:(endX - startX + 1) height:height.
    self paint:(Color white).
    (displayMode == #vector) ifTrue:[
        1 to:count do:[:index |
            val := bytes at:index.
            self displayLineFromX:(x - zoom) y:(y0 + prevVal)
                              toX:x y:(y0 + val).
            prevVal := val.
            x := x + zoom
        ]
    ] ifFalse:[
        1 to:count do:[:index |
            val := bytes at:index.
            self displayPointX:x y:(y0 + val).
            prevVal := val.
            x := x + 1
        ]
    ].
    count < w ifTrue:[
        self displayLineFromX:x y:(y0 + 128) toX:width y:(y0 + 128)
    ].
    self paint:(Color red).
    self displayLineFromX:startX y:(y0 + 128) toX:endX y:(y0 + 128)
! !

!VocView methodsFor:'scrolling'!

scrollToLeft
    self positionToStart
!

scrollLeft

    |oldOrg|

    oldOrg := xOrg.
    xOrg := xOrg - (width // 4 // zoom).
    xOrg < 1 ifTrue:[
        xOrg := 1
    ].
    (oldOrg ~~ xOrg) ifTrue:[
        self redraw.
        self originChanged:(xOrg - oldOrg)
    ]
!

scrollRight
    |delta|

    delta := (width // 4) // zoom.
    xOrg := xOrg + delta.
    self redraw.
    self originChanged:delta
!

scrollHorizontalToPercent:percent

    |oldOrg fullSize|

    oldOrg := xOrg.
    fullSize := self widthOfContents asFloat.
    xOrg := (fullSize * percent / 100.0) asInteger.
    xOrg > fullSize ifTrue:[
        xOrg := fullSize - width
    ].
    (xOrg < 1) ifTrue:[
        xOrg := 1
    ].
    (oldOrg ~~ xOrg) ifTrue:[
        self redraw.
        self originChanged:(xOrg - oldOrg)
    ]
!

scrollDown
    ^ self
!

scrollUp
    self originChanged:0
!

scrollVerticalToPercent:percent

    ^ self
! !

!VocView methodsFor:'initializing'!

initialize
    super initialize.
    displayMode := #vector.
    viewBackground := Black.
    foreground := White.
    xOrg := 1.
    yOrg := 0.
    zoom := 1
!

initializeMiddleButtonMenu
    |labels|

    Language == #german ifTrue:[
        labels := #('kopieren'
                    'loeschen'
                    'einfuegen'
                    '-'
                    'verstaerken'
                    '-'
                    'wiedergabe'
                    'aufnahme'
                    '-'
                    'anzeigemodus'
                    '-'
                    'zoom in'
                    'zoom out'
                    '-'
                    'zum anfang'
                    'zum ende'
                    '-'
                    'uebernehmen')
    ] ifFalse:[
        labels := #('copy'
                    'cut'
                    'paste'
                    '-'
                    'amplify'
                    '-'
                    'play'
                    'record'
                    '-'
                    'display mode'
                    '-'
                    'zoom in'
                    'zoom out'
                    '-'
                    'to start'
                    'to end'
                    '-'
                    'accept')
    ].
    self
        middleButtonMenu:(PopUpMenu 
                                labels:labels
                             selectors:#(copy
                                         cut
                                         paste
                                         nil
                                         amplify
                                         nil
                                         play
                                         record
                                         nil
                                         switchDisplayMode
                                         nil
                                         zoomIn
                                         zoomOut
                                         nil
                                         positionToStart
                                         positionToEnd
                                         nil
                                         accept)
                                  receiver:self
                                       for:self)
!

create
    super create.
    self enableButtonEvents.
    self enableButtonMotionEvents
! !

!VocView methodsFor:'accessing'!

contents:aCollection
    "set the contents from the collection"

    |wasEmpty|

    wasEmpty := data isNil and:[dataFile isNil].
    dataFile notNil ifTrue:[
        dataFile close.
        dataFile := nil
    ].
    data := aCollection.
    xOrg := 1.
    (data isNil and:[wasEmpty]) ifFalse:[
        self contentsChanged.
        self redraw
    ]
!

contentsFromFile:fileName in:aDirectory
    "set the contents from the voc-file fileName"

    |size bytes|

    dataFile notNil ifTrue:[
        dataFile close
    ].
    data := nil.
    xOrg := 1.
    dataFile := FileStream readonlyFileNamed:fileName in:aDirectory.
    dataFile notNil ifTrue:[
        "check format"
        bytes := ByteArray new:4.
        dataFile nextBytes:4 into:bytes.
        (bytes = #(16r2e 16r73 16r6e 16r64)) ifTrue:[
            "NeXT sound file or SUN au file"
            self contentsFromNeXTFileStream:dataFile
        ] ifFalse:[
            "take as raw-data"
            sampleRate := 12000.
            dataFile position:1.
            size := dataFile size.
            data := ByteArray new:size.
            dataFile nextBytes:size into:data
        ].
        dataFile close.
        dataFile := nil.
        self redraw
    ].
    self contentsChanged
!

fromSoundStream:aStream
    "NeXT or SUN sound header is

#define SND_MAGIC ((int)0x2e736e64)

#define SND_FORMAT_UNSPECIFIED          (0)
#define SND_FORMAT_MULAW_8              (1)
#define SND_FORMAT_LINEAR_8             (2)
#define SND_FORMAT_LINEAR_16            (3)
#define SND_FORMAT_LINEAR_24            (4)
#define SND_FORMAT_LINEAR_32            (5)
#define SND_FORMAT_FLOAT                (6)
#define SND_FORMAT_DOUBLE               (7)
#define SND_FORMAT_INDIRECT             (8)
#define SND_FORMAT_NESTED               (9)
#define SND_FORMAT_DSP_CORE             (10)
#define SND_FORMAT_DSP_DATA_8           (11)
#define SND_FORMAT_DSP_DATA_16          (12)
#define SND_FORMAT_DSP_DATA_24          (13)
#define SND_FORMAT_DSP_DATA_32          (14)
#define SND_FORMAT_DISPLAY              (16)
#define SND_FORMAT_MULAW_SQUELCH        (17)
#define SND_FORMAT_EMPHASIZED           (18)
#define SND_FORMAT_COMPRESSED           (19)
#define SND_FORMAT_COMPRESSED_EMPHASIZED (20)
#define SND_FORMAT_DSP_COMMANDS         (21)
#define SND_FORMAT_DSP_COMMANDS_SAMPLES (22)

/*
 * Sampling rates directly supported in hardware.
 */
#define SND_RATE_CODEC          (8012.8210513)
#define SND_RATE_LOW            (22050.0)
#define SND_RATE_HIGH           (44100.0)
"
    |magic dataLocation dataSize dataFormat
     samplingRate channelCount info|

    magic := aStream nextLong.
    dataLocation := aStream nextLong.
    dataSize := aStream nextLong.
    samplingRate := aStream nextLong.
    channelCount := aStream nextLong.
    info := String new:4.
    aStream nextBytes:4 into:info
!

heightOfContents
    "assume 8-bit sample data"

    ^ 256
!

widthOfContents

    dataFile notNil ifTrue:[
        ^ dataFile size * zoom
    ].
    data notNil ifTrue:[
        ^ data size * zoom
    ].
    ^ 0
!

xOriginOfContents
    ^ xOrg - 1

!

yOriginOfContents

    |org|

    org := 256 - height.
    org < 0 ifTrue:[^ 0].
    ^ org // 2
!

modified
    ^ false
!

contents
    "return the data collection"

    ^ data
!

acceptAction:aBlock
    "set the action to be performed on accept"

    acceptAction := aBlock
!

modified:aBoolean
    ^ self

!

contentsFromNeXTFileStream:dataFile
    "set the contents from the FileStream dataFile,
     which must have NeXT/SUN Sound-file format.
     The data is converted (currently) to 8bit mono,
     since this is what all sound devices support"

    |size bytes offset format nChannels valueBytes nSamples
     val monoValue|

    bytes := ByteArray new:4.
    dataFile position:5.

    dataFile nextBytes:4 into:bytes.
    offset := bytes at:1.
    offset := (offset * 256) + (bytes at:2).
    offset := (offset * 256) + (bytes at:3).
    offset := (offset * 256) + (bytes at:4).

    dataFile nextBytes:4 into:bytes.
    size := bytes at:1.
    size := (size * 256) + (bytes at:2).
    size := (size * 256) + (bytes at:3).
    size := (size * 256) + (bytes at:4).

    dataFile nextBytes:4 into:bytes.
    (bytes = #(0 0 0 1)) ifTrue:[
        "sun au; linear-8"
        format := #signedLinear8
    ] ifFalse:[
        (bytes = #(0 0 0 2)) ifTrue:[
            "NeXT linear-8"
            format := #linear8
        ] ifFalse:[
            (bytes = #(0 0 0 3)) ifTrue:[
                "NeXT linear-16"
                format := #linear16
            ] ifFalse:[
                self notify:'unsupported Soundfile data format'.
                self halt. ^ self
            ]
        ]
    ].
    dataFile nextBytes:4 into:bytes.
    sampleRate := bytes at:1.
    sampleRate := (sampleRate * 256) + (bytes at:2).
    sampleRate := (sampleRate * 256) + (bytes at:3).
    sampleRate := (sampleRate * 256) + (bytes at:4).

    dataFile nextBytes:4 into:bytes.
    nChannels := bytes at:1.
    nChannels := (nChannels * 256) + (bytes at:2).
    nChannels := (nChannels * 256) + (bytes at:3).
    nChannels := (nChannels * 256) + (bytes at:4).

Transcript showCr:('Sound file:').
Transcript showCr:(' format: ' , format).
Transcript showCr:(' size: ' , size printString).
Transcript showCr:(' offset: ' , offset printString).
Transcript showCr:(' rate: ' , sampleRate printString).
Transcript showCr:(' channels: ' , nChannels printString).

    dataFile position:offset.

    (format == #linear16) ifTrue:[
        nSamples := size // 2 // nChannels.
        data := ByteArray new:nSamples.
        (nChannels == 2) ifTrue:[
            "make it mono"
            1 to:nSamples do:[:index |
                dataFile nextBytes:4 into:bytes.
                monoValue := ((bytes at:1) + (bytes at:3)) // 2.
                data at:index put:((monoValue + 128) bitAnd:16rFF)
            ]
        ] ifFalse:[
            1 to:nSamples do:[:index |
                monoValue := 0.
                1 to:nChannels do:[:chan |
                    dataFile nextBytes:2 into:bytes.
                    monoValue := monoValue + (bytes at:1)
                ].
                monoValue := monoValue // nChannels.
                data at:index put:((monoValue + 128) bitAnd:16rFF)
            ]
        ]
    ].

    (format == #signedLinear8) ifTrue:[
        nSamples := size // nChannels.
        data := ByteArray new:nSamples.
        nChannels == 2 ifTrue:[
            "make mono"
            1 to:nSamples do:[:index |
                monoValue := dataFile nextByte.
                monoValue := monoValue + dataFile nextByte.
                monoValue := monoValue // 2.
                data at:index put:((monoValue) bitAnd:16rFF) - 127
            ]
        ] ifFalse:[
            1 to:nSamples do:[:index |
                monoValue := dataFile nextByte.
                monoValue := monoValue // nChannels.
                data at:index put:monoValue
            ]
        ]
    ].

    (format == #linear8) ifTrue:[
        nSamples := size // nChannels.
        data := ByteArray new:nSamples.
        nChannels == 2 ifTrue:[
            "make mono"
            1 to:nSamples do:[:index |
                monoValue := dataFile nextByte.
                monoValue := monoValue + dataFile nextByte.
                monoValue := monoValue // 2.
                data at:index put:((monoValue + 128) bitAnd:16rFF)
            ]
        ] ifFalse:[
            1 to:nSamples do:[:index |
                monoValue := dataFile nextByte.
                data at:index put:((monoValue + 128) bitAnd:16rFF)
            ]
        ]
    ]
! !

!VocView methodsFor:'user interaction'!

positionToStart
    |oldOrg|

    oldOrg := xOrg.
    xOrg := 1.
    (oldOrg ~~ xOrg) ifTrue:[
        self redraw.
        self originChanged:(xOrg - oldOrg)
    ]
!

positionToEnd
    |oldOrg|

    oldOrg := xOrg.

    dataFile notNil ifTrue:[
        xOrg := dataFile size "- (width // 2)".
        xOrg < 1 ifTrue:[
            xOrg := 1
        ]
    ] ifFalse:[
        xOrg := 1.
        data notNil ifTrue:[
            xOrg := data size "- (width // 2)".
            xOrg < 1 ifTrue:[
                xOrg := 1
            ]
        ]
    ].
    (oldOrg ~~ xOrg) ifTrue:[
        self redraw.
        self originChanged:(xOrg - oldOrg)
    ]
!

switchDisplayMode
    (displayMode == #vector) ifTrue:[
        displayMode := #point
    ] ifFalse:[
        displayMode := #vector
    ].
    self redraw
!

sizeChanged
    super sizeChanged.
    self originChanged:0
!

zoomIn
    zoom := zoom * 2.
    self contentsChanged.
    self redraw
!

zoomOut
    zoom := zoom / 2.
    self contentsChanged.
    self redraw
!

cut
    |start end ndata|

    (selectionStart notNil and:[selectionEnd notNil]) ifTrue:[
        start := selectionStart.
        end := selectionEnd.
        self unselect.
        start isNil ifTrue:[^ self].
        (start > data size) ifTrue:[^ self].
        (end > data size) ifTrue:[end := data size].

        ndata := ByteArray new:(data size - (end - start + 1)).
        ndata replaceFrom:1 to:(start - 1) with:data startingAt:1.
        ndata replaceFrom:start to:(ndata size) with:data startingAt:(end + 1).
        data := ndata.
        self contentsChanged.
        self redraw
    ]
!

copy
    (selectionStart notNil and:[selectionEnd notNil]) ifTrue:[
        self invertSelection.
        selectionStart := nil
    ]
!

paste
    ^ self
!

accept
    acceptAction notNil ifTrue:[
        acceptAction value:data
    ]
!

play
    "play data on sound-device"

    |outStream count offset junkSize n|

    junkSize := SoundStream blockSize.

    outStream := SoundStream writing.
    outStream isNil ifTrue:[
        self notify:'cannot open sound device'.
        ^ self
    ].
    count := data size.
    offset := 1.
    [count > 0] whileTrue:[
        n := junkSize min:count.
        outStream nextPutBytes:n from:data startingAt:offset.
        offset := offset + n.
        count := count - n
    ].

    outStream synchronizeOutput.
    outStream close
!

record
    "record data from sound-device - just a test,
     records 5 seconds. Should change to use VocPanel."

    |tmpFile inStream buffer count|

    tmpFile := FileStream newFileNamed:'recording'.
    tmpFile isNil ifTrue:[
        self showAlert:'cannot create temp file'.
        ^ self
    ].

    buffer := ByteArray new:4096.

    inStream := SoundStream reading.
    inStream isNil ifTrue:[
        self showAlert:'cannot open sound device'.
        ^ self
    ].
    1 to:5 do:[:i |
        inStream nextBytes:4096 into:buffer.
        tmpFile nextPutBytes:4096 from:buffer
    ].
    inStream close.
    tmpFile close
!

amplify
    "expand data - ask for factor first"

    |gain|

    queryBox isNil ifTrue:[
        queryBox := EnterBox new
    ].
    queryBox initialText:''.
    (Language == #german) ifTrue:[
        queryBox title:'verstaerkung:'.
        queryBox okText:'ok'.
        queryBox abortText:'Abbruch'
    ] ifFalse:[
        queryBox title:'gain:'.
        queryBox okText:'ok'.
        queryBox abortText:'abort'
    ].
    queryBox action:[:gainString | 
                         gain := Number readFromString:gainString.
                         gain notNil ifTrue:[
                             self doAmplify:gain
                         ]
                    ].
    queryBox showAtPointer
! !

!VocView methodsFor:'events'!

leftButtonPressX:x y:y
    "mouse-click - prepare for selection change"

    self unselect.
    clickStart := x + xOrg.
    clickEnd := x + xOrg.
    selectionStart := clickStart.
    selectionEnd := nil
!

buttonMotion:butt x:x y:y
    "mouse-move change selection"

    |l r oldFg xn xl xr|

    xn := x + xOrg.
    xn == clickEnd ifTrue:[^ self].
    (xn > clickEnd) ifTrue:[
        r := xn.
        l := clickEnd + 1
    ] ifFalse:[
        r := clickEnd.
        l := xn + 1
    ].

    oldFg := foreground.
    self foreground:(foreground bitXor:background).
    self xoring:[
        xl := l - xOrg.
        (xl < 0) ifTrue:[xl := 0].
        xr := r - xOrg.
        (xr < 0) ifTrue:[xr := 0].
        (xr > width) ifTrue:[xr := width].
        self fillRectangleX:xl y:0 width:(xr - xl + 1) height:height
    ].
    self foreground:oldFg.
    clickEnd := xn
!

buttonRelease:button x:x y:y
    "mouse-release - end selection"

    selectionStart notNil ifTrue:[
        clickEnd notNil ifTrue:[
            selectionEnd := clickEnd + xOrg
        ]
    ]
!

leftButtonShiftPressX:x y:y
    "mouse-click - add to selection"

    self buttonMotion:1 x:x y:y
! !

!VocView methodsFor:'selections'!

unselect
    self invertSelection.
    selectionStart := nil.
    selectionEnd := nil
! !

!VocView methodsFor:'private'!

doAmplify:gain
    |oldValue newValue|

    1 to:(data size) do:[:index |
        oldValue := (data at:index) - 128.
        newValue := (oldValue * gain) rounded asInteger + 128.
        (newValue < 0) ifTrue:[
            newValue := 0
        ] ifFalse:[
            (newValue > 255) ifTrue:[
                newValue := 255
            ]
        ].
        data at:index put:newValue
    ].
    self redraw
! !
