TreeGraphView subclass:#WindowTreeView
       instanceVariableNames:'showAll'
       classVariableNames:''
       poolDictionaries:''
       category:'Demos-Graphics'
!

WindowTreeView comment:'
 a small application showing an ObjectView / TreeView use

 must fileIn TreeView.st, TreeGraphView.st first,
 then start with:
     WindowTreeView start

 %W% %E%
 written spring 91 by claus
'!

!WindowTreeView class methodsFor:'startup'!

start
    |aDemo topView v|

    topView := StandardSystemView 
                    label:'Window hierarchy - known views'
                    icon:(Form fromFile:'Demo.icon' resolution:100).
    v  := HVScrollableView for:self in:topView.
    v origin:(0 @ 0) extent:(1.0 @ 1.0).

    topView open

    "WindowTreeView start"
! !

!WindowTreeView methodsFor:'initialization'!

initialize
    topNode := nil.
    showAll := false.
    super initialize.
!

initializeMiddleButtonMenu
    self middleButtonMenu:(PopUpMenu
                                labels:(resources array:#(
                                            'update'
                                            '-'
                                            'inspect'
                                            '-'
                                            'raise'
                                            'lower'
                                            '-'
                                            'destroy'
                                           )
                                       )
                             selectors:#(doUpdate
                                         nil
                                         doInspect
                                         nil
                                         doRaise
                                         doLower
                                         nil
                                         doDestroy
                                        )
                                receiver:self
                                     for:self)
! !

!WindowTreeView methodsFor:'generating the class-tree picture'!

addChildrenToTreeFrom:aView xPos:xPos yPos:yPos
    "special - we have a forest here ..."

    |newObject yNext yPrev|

    yNext := yPos.
    aView isNil ifTrue:[
        Display allViewsDo:[:v |
            v superView isNil ifTrue:[
                "its a top-view"

                (v respondsTo:#subViews) ifTrue:[
                    "a real view, dont show if its a modal or
                     popup view, except when currently visible"
                    (showAll or:[v realized or:[v createOnTop not]]) ifTrue:[
                        "dont show ShadowViews, popups etc."
                        yPrev := yNext.
                        yNext := self addChildrenToTreeFrom:v father:nil xPos:xPos yPos:yNext first:false.

                        "anything added ?"
                        yNext ~~ yPrev ifTrue:[
                            "create a gap"
                             yNext := yNext + (device verticalPixelPerMillimeter*10) rounded.
                        ]
                    ]
                ]
            ]
        ]
    ].
    ^ yNext
!

addChildrenToTreeFrom:anElement father:father xPos:xPos yPos:yPos first:isFirst
    ^ super addChildrenToTreeFrom:anElement father:father xPos:xPos yPos:yPos first:false
!

childrenOf:aView do:aBlock
    |subviews|

    (aView respondsTo:#subViews) ifFalse:[
        "not a real view"

        ^ self
    ].

    showAll ifFalse:[
        "dont show ShadowViews, popups etc."

        aView createOnTop ifTrue:[
            ^ self
        ]
    ].

    subviews := aView subViews.
    subviews notNil ifTrue:[
        subviews do:aBlock
    ]
!

textFor:aView
    (aView isKindOf:StandardSystemView) ifTrue:[
        ^ (aView class name , ' (' , aView label , ')')
    ].
    ^ aView class name
!

colorFor:aView
    (aView realized and:[
     aView topView realized and:[
     aView topView shown]]) ifTrue:[
        device hasGreyscales ifTrue:[
            ^ Color red
        ].
        ^ Color veryDarkGrey
    ].
    ^ Black
! !

!WindowTreeView methodsFor:'user interaction'!

doUpdate
    self updateTree.
    self contentsChanged
!

doInspect
    self withSelectedElementDo:[:theView |
         theView inspect
    ]
!

doRaise
    self withSelectedElementDo:[:theView |
         theView topView raise
    ]
!

doLower
    self withSelectedElementDo:[:theView |
         theView topView lower
    ]
!

doDestroy
    self withSelectedElementDo:[:theView |
        theView destroy.
        self updateTree
    ]
! !
