"
 COPYRIGHT (c) 1994 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ImageReader subclass:#XPMReader
         instanceVariableNames:''
         classVariableNames:''
         poolDictionaries:''
         category:'Graphics-Support'
!

XPMReader comment:'

COPYRIGHT (c) 1994 by Claus Gittinger
              All Rights Reserved

$Header$
'!

!XPMReader class methodsFor:'documentation'!

documentation
"
    this class provides methods for loading x-pixmap-file (xpm) images.
    These images are used (in X) for palette images (see ctwm or hp-vue for a lot of them).
    The code here is a hack - it may not work for all images (it works for the testfiles
    I got here)
"
! !

!XPMReader class methodsFor:'testing'!

isValidImageFile:aFileName
    "return true, if aFileName contains an x-bitmap-file image"

    |line inStream index1 index2 keyword|

    inStream := self streamReadingFile:aFileName.
    inStream isNil ifTrue:[^ false].

    line := inStream nextLine.
    inStream close.
    line isNil ifTrue:[^ false].
    (line startsWith:'/* XPM') ifFalse:[^ false].
    ^ true

    "XPMReader isValidImageFile:'bitmaps/magtape.xpm'" 
    "XPMReader isValidImageFile:'bitmaps/ljet.xpm'" 
! !

!XPMReader methodsFor:'reading from file'!

colorNameFrom:aStream
    |s|

    aStream peek == $# ifTrue:[
        aStream next.
        s := '#'.
        [aStream peek isAlphaNumeric] whileTrue:[
            s := s copyWith:aStream next
        ].
        ^ s
    ].
    ^ aStream nextWord
!

fromFile:aFileName
    |line 
     srcIndex "{ Class: SmallInteger }"
     dstIndex "{ Class: SmallInteger }"
     colorName colorMapSize redMap greenMap blueMap
     unknownUse xlation s bitsPerPixel|

    inStream := self class streamReadingFile:aFileName.
    inStream isNil ifTrue:[
        'file open error' printNewline.
        ^ nil
    ].

    line := inStream nextLine.
    (line notNil and:[line startsWith:'/* XPM']) ifFalse:[
        'format error (expected XPM)' printNewline.
        inStream close.
        ^ nil
    ].

    line := inStream nextLine.
    (line notNil and:[line startsWith:'static char']) ifFalse:[
        'format error (expected static char)' printNewline.
        inStream close.
        ^ nil
    ].
    line := inStream nextLine.
    [line notNil and:[line startsWith:'/*']] whileTrue:[
        line := inStream nextLine.
    ].
    (line notNil and:[line startsWith:'"']) ifFalse:[
        'format error (expected "ww hh nn mm)' printNewline.
        inStream close.
        ^ nil
    ].
    s := ReadStream on:line.
    s next.  "skip quote"
    width := Integer readFrom:s.
    height := Integer readFrom:s.
    colorMapSize := Integer readFrom:s.
    unknownUse := Integer readFrom:s.

    xlation := Array new:256.

    redMap := Array new:colorMapSize.
    greenMap := Array new:colorMapSize.
    blueMap := Array new:colorMapSize.
    colorMap := Array with:redMap with:greenMap with:blueMap.

    1 to:colorMapSize do:[:colorIndex |
        |index line color|

        line := inStream nextLine.
        [line notNil and:[line startsWith:'/*']] whileTrue:[
            line := inStream nextLine.
        ].
        (line notNil and:[line startsWith:'"']) ifFalse:[
            'format error (expected color spec)' printNewline.
            inStream close.
            ^ nil
        ].
        s := ReadStream on:line.
        s next. "skip quote"
        index := s next asciiValue.
        xlation at:index put:colorIndex - 1.

        s skipSeparatorsExceptCR.
        (s next ~~ $s) ifTrue:[
            'expected ''s''' printNewline
        ].
        s skipSeparatorsExceptCR.
        s nextWord.     "color"
        s skipSeparatorsExceptCR.
        (s next ~~ $m) ifTrue:[
            'expected ''m''' printNewline
        ].
        s skipSeparatorsExceptCR.
        s nextWord.
        s skipSeparatorsExceptCR.
        (s next ~~ $c) ifTrue:[
            'expected ''c''' printNewline
        ].
        s skipSeparatorsExceptCR.
        colorName := self colorNameFrom:s.
        ((colorName = 'none') or:[colorName = 'None']) ifTrue:[
            "mhmh must add mask to Image-instances soon ..."
            color := Color white
        ] ifFalse:[
            color := Color name:colorName.
        ].
        redMap at:colorIndex put:(color red * 255 // 100).
        greenMap at:colorIndex put:(color green * 255 // 100).
        blueMap at:colorIndex put:(color blue * 255 // 100).
    ].

    "actually, could make it an image with less depth most of the time ..."

"
    bitsPerPixel := ((colorMapSize - 1) log:2) truncated + 1.
"
    bitsPerPixel := 8.
    data := ByteArray new:(width * height).

    dstIndex := 1.
    1 to:height do:[:row |
        line := inStream nextLine.
        [line notNil and:[line startsWith:'/*']] whileTrue:[
            line := inStream nextLine.
        ].
        (line notNil and:[line startsWith:'"']) ifFalse:[
            'format error (expected pixels)' printNewline.
            inStream close.
            ^ nil
        ].
        srcIndex := 2.
        1 to: width do:[:col |
            |char|

            char := line at:srcIndex.
            data at:dstIndex put:(xlation at:char asciiValue).
            srcIndex := srcIndex + 1.
            dstIndex := dstIndex + 1
        ]
    ].

    photometric := #palette.
    samplesPerPixel := 1.
    bitsPerSample := Array with:bitsPerPixel.

    "XPMReader fromFile:'bitmaps/magtape.xpm'" 
    "XPMReader fromFile:'bitmaps/pixmap.xpm'" 
    "XPMReader fromFile:'bitmaps/ljet.xpm'" 
! !
